@@ -2,7 +2,7 @@ push_textdomain("tribes")
 
 dirname = path.dirname (__file__)
 
-descriptions:new_militarysite_type {
+wl.Descriptions():new_militarysite_type {
    name = "amazons_observation_tower",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext ("amazons_building", "Observation Tower"),
@@ -23,7 +23,7 @@ descriptions:new_militarysite_type {
    max_soldiers = 5,
    heal_per_second = 150,
    conquers = 9,
-   prefer_heroes = false,
+   prefer_heroes = true,
 
    messages = {
       -- TRANSLATORS: Message sent by an Amazon military site
