@@ -2,17 +2,20 @@ push_textdomain("tribes")
 
 dirname = path.dirname(__file__)
 
-descriptions:new_militarysite_type {
+wl.Descriptions():new_militarysite_type {
    name = "empire_castle",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext("empire_building", "Castle"),
    icon = dirname .. "menu.png",
    size = "big",
 
-   animations = {
+   animation_directory = dirname,
+   spritesheets = {
       idle = {
-         pictures = path.list_files(dirname .. "idle_??.png"),
-         hotspot = { 94, 106 }
+         frames = 1,
+         columns = 1,
+         rows = 1,
+         hotspot = { 98, 114 }
       }
    },
 
