@@ -2,7 +2,7 @@ push_textdomain("tribes")
 
 dirname = path.dirname(__file__)
 
-descriptions:new_productionsite_type {
+wl.Descriptions():new_productionsite_type {
    name = "atlanteans_scouts_house",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext("atlanteans_building", "Scout’s House"),
@@ -17,15 +17,20 @@ descriptions:new_productionsite_type {
       log = 1
    },
 
+   animation_directory = dirname,
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle_??.png"),
-         hotspot = { 41, 44 },
+         hotspot = { 44, 49 },
       },
+   },
+
+   spritesheets = {
       build = {
-         pictures = path.list_files(dirname .. "build_??.png"),
-         hotspot = { 41, 44 },
-      }
+         frames = 3,
+         columns = 3,
+         rows = 1,
+         hotspot = { 44, 49 },
+      },
    },
 
    aihints = {},
@@ -42,7 +47,7 @@ descriptions:new_productionsite_type {
    programs = {
       main = {
          -- TRANSLATORS: Completed/Skipped/Did not start scouting because ...
-         descname = _"scouting",
+         descname = _("scouting"),
          actions = {
             "consume=smoked_fish",
             "sleep=duration:30s",
@@ -53,7 +58,7 @@ descriptions:new_productionsite_type {
          }
       },
       targeted_scouting = {
-         descname = _"scouting",
+         descname = _("scouting"),
          actions = {
             "consume=smoked_fish,atlanteans_bread",
             "callworker=targeted_scouting"
