@@ -2,7 +2,7 @@ push_textdomain("tribes")
 
 dirname = path.dirname(__file__)
 
-descriptions:new_warehouse_type {
+wl.Descriptions():new_warehouse_type {
    name = "barbarians_warehouse",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext("barbarians_building", "Warehouse"),
@@ -24,13 +24,19 @@ descriptions:new_warehouse_type {
    },
 
    animation_directory = dirname,
-   animations = {
+   spritesheets = {
       idle = {
-         hotspot = { 60, 78 },
+         frames = 1,
+         rows = 1,
+         columns = 1,
+         hotspot = { 60, 78 }
       },
       build = {
-         hotspot = { 60, 78 },
-      }
+         frames = 4,
+         rows = 2,
+         columns = 2,
+         hotspot = { 60, 78 }
+      },
    },
 
    aihints = {},
