@@ -2,7 +2,7 @@ push_textdomain("tribes")
 
 dirname = path.dirname(__file__)
 
-descriptions:new_ware_type {
+wl.Descriptions():new_ware_type {
    name = "liana",
    -- TRANSLATORS: This is a ware name used in lists of wares. Used to make ropes.
    descname = pgettext("ware", "Liana"),
