@@ -2,7 +2,7 @@ push_textdomain("tribes")
 
 dirname = path.dirname(__file__)
 
-descriptions:new_worker_type {
+wl.Descriptions():new_worker_type {
    name = "empire_miner_master",
    -- TRANSLATORS: This is a worker name used in lists of workers
    descname = pgettext("empire_worker", "Master Miner"),
@@ -14,17 +14,26 @@ descriptions:new_worker_type {
       idle = {
          hotspot = { 7, 22 }
       },
+   },
+
+   spritesheets ={
       walk = {
-         hotspot = { 11, 22 },
          fps = 10,
-         directional = true
+         frames = 20,
+         rows = 5,
+         columns = 4,
+         directional = true,
+         hotspot = { 11, 22 }
       },
       walkload = {
-         hotspot = { 10, 22 },
          fps = 10,
-         directional = true
-      }
-   }
+         frames = 20,
+         rows = 5,
+         columns = 4,
+         directional = true,
+         hotspot = { 10, 22 }
+      },
+   },
 }
 
 pop_textdomain()
