@@ -4,12 +4,13 @@ dirname = path.dirname(__file__)
 
 wl.Descriptions():new_immovable_type{
    name = "bar-ruin00",
-   descname = _ "Ruin",
+   descname = _("Ruin"),
    size = "small",
    programs = {},
+   animation_directory = dirname,
+   icon = dirname .. "menu.png",
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle.png"),
          hotspot = { 34, 32 },
       },
    }
