@@ -4,12 +4,12 @@ dirname = path.dirname(__file__)
 
 wl.Descriptions():new_immovable_type{
    name = "snowman",
-   descname = _ "Snowman",
+   descname = _("Snowman"),
    size = "none",
    programs = {},
+   animation_directory = dirname,
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle.png"),
          hotspot = { 9, 24 },
       },
    }
