@@ -5,12 +5,12 @@ dirname = path.dirname(__file__)
 wl.Descriptions():new_immovable_type{
    name = "track_winter",
    -- TRANSLATORS: This track is made of footprints in the snow
-   descname = _ "Track",
+   descname = _("Track"),
    size = "none",
    programs = {},
+   animation_directory = dirname,
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle.png"),
          hotspot = { 30, 15 },
       },
    }
