@@ -4,12 +4,12 @@ dirname = path.dirname(__file__)
 
 wl.Descriptions():new_immovable_type{
    name = "cactus1",
-   descname = _ "Cactus",
+   descname = _("Cactus"),
    size = "none",
    programs = {},
+   animation_directory = dirname,
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle.png"),
          hotspot = { 10, 71 },
       },
    }
