@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2021 by the Widelands Development Team
+ * Copyright (C) 2009-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -108,6 +107,7 @@ enum class SchedulerTaskId : uint8_t {
 	kManagementUpdate,
 	kUpdateStats,
 	kWarehouseFlagDist,
+	kDiplomacy,
 	kUnset
 };
 
@@ -122,16 +122,16 @@ const std::vector<std::vector<int8_t>> neuron_curves = {
 
 // TODO(tiborb): this should be replaced by command line switch
 constexpr int kFNeuronBitSize = 32;
-constexpr int kMutationRatePosition = 42;
 // This is expiration time for distance from a flag to nearest warehouse
-constexpr Duration kFlagDistanceExpirationPeriod = Duration(120 * 1000);
+constexpr Duration kFlagDistanceExpirationPeriod(120 * 1000);
 // If the distance of flag-warehouse was not updated for this time, we presume that the flag
 // does not exist anymore and remove it
-constexpr Duration kOldFlagRemoveTime = Duration(5 * 60 * 1000);
+constexpr Duration kOldFlagRemoveTime(5 * 60 * 1000);
 
 constexpr uint32_t kNoField = std::numeric_limits<uint32_t>::max();
 
-constexpr uint16_t kFarButReachable = 1000;
+constexpr uint16_t kWhFarButReachable = 250;
+constexpr uint16_t kWhNotReachable = 500;
 
 struct CheckStepRoadAI {
 	CheckStepRoadAI(Widelands::Player* const pl, uint8_t const mc, bool const oe);
@@ -360,6 +360,10 @@ struct ImmovableAttribute {
 struct BuildableField {
 	explicit BuildableField(const Widelands::FCoords& fc);
 
+	// Preventing unintended copy construction (good idea?)
+	BuildableField(const BuildableField& temp_obj) = delete;
+	BuildableField& operator=(const BuildableField& temp_obj) = delete;
+
 	Widelands::FCoords coords;
 
 	Time field_info_expiration;
@@ -368,6 +372,8 @@ struct BuildableField {
 	bool enemy_nearby;
 	bool enemy_accessible_;
 
+	bool invalidated;
+
 	bool enemy_wh_nearby;
 	uint16_t unowned_land_nearby;
 	uint16_t enemy_owned_land_nearby;
@@ -392,10 +398,12 @@ struct BuildableField {
 	uint8_t space_consumers_nearby;
 	uint8_t rangers_nearby;
 	// to manage the military better following variables exists:
-	// capacity of nearby buildings:
+	// future soldier capacity of own nearby militarysites:
 	int16_t area_military_capacity;
+	int16_t future_area_military_capacity;
 	// distance to near buldings:
 	int16_t military_loneliness;
+	int16_t future_military_loneliness;
 	// count of military buildings in construction
 	// when making decision on new mulitary buildings it considers also
 	// unowned fields and mines, but this information is not quite correct as there
@@ -408,15 +416,15 @@ struct BuildableField {
 	int16_t ally_military_presence;
 	// stationed (manned) military buildings nearby
 	int16_t military_stationed;
-	// unconnected buildings nearby
-	bool unconnected_nearby;
+	uint32_t average_flag_dist_to_wh;
 	int16_t military_unstationed;
 	int16_t own_non_military_nearby;
 	bool defense_msite_allowed;
 	ExtendedBool is_portspace;
 	bool port_nearby;  // to increase priority if a port is nearby,
 	// especially for new colonies
-	ExtendedBool portspace_nearby;  // special fields intended for ports
+	ExtendedBool portspace_nearby;    // special fields intended for ports
+	ExtendedBool shipyard_preferred;  // special fields intended for shipyards
 	int32_t max_buildcap_nearby;
 	// It is not necessary to check resources (stones, fish...) too frequently as they do not change
 	// fast. This stores the time of the last check.
@@ -470,7 +478,7 @@ struct BuildingObserver {
 
 	int32_t total_count() const;
 	AiModeBuildings aimode_limit_status() const;
-	bool buildable(const Widelands::Player& p);
+	bool buildable(const Widelands::Player& p) const;
 
 	// Convenience functions for is_what
 	bool is(BuildingAttribute) const;
@@ -643,18 +651,20 @@ private:
 
 constexpr int kNeuronWeightLimit = 100;
 constexpr size_t kNeuronMaxPosition = 20;
-constexpr size_t kSecondParentProbability = 50;
+// This defines probability of taking DNA from second parent
+// Probably it would best to remove the 'crossbreeding' concept altogether
+constexpr size_t kSecondParentProbability = 500;
 
 // A bunch of parameters used for trainig AI (for calculation of fitness function result)
 constexpr int16_t kCurrentLandDivider = 2;
 constexpr int16_t kLandDeltaMultiplier = 1;
-constexpr int16_t kBonus = 1000;
-constexpr int16_t kAttackersMultiplier = 1;
-constexpr int16_t kAttackBonus = 100;
+constexpr int16_t kAttackersBonus = 3;
+constexpr int16_t kAttackBonus = 100;  // Bonus if the AI attacked at least once
 constexpr int16_t kTrainedSoldiersScore = 250;
 constexpr int16_t kConqueredWhBonus = 300;
-constexpr int16_t kStrengthMultiplier = 30;
+constexpr int16_t kStrengthMultiplier = 15;
 constexpr int16_t kPSitesRatioMultiplier = 1;
+constexpr int32_t kShipBonus = 1000;  // Bonus for each completed ship
 
 struct Neuron {
 	static int clip_weight_to_range(int w) {
@@ -728,7 +738,9 @@ struct ManagementData {
 	            int16_t trained_soldiers,
 	            uint16_t strength,
 	            uint32_t existing_ps,
-	            const Time& first_iron_mine_time);
+	            const Time& first_iron_mine_time,
+	            uint16_t ships_count,
+	            uint16_t finished_mine_types);
 	void dump_data(Widelands::PlayerNumber);
 	uint16_t new_neuron_id() {
 		++next_neuron_id;
@@ -748,9 +760,9 @@ struct ManagementData {
 		ai_training_mode_ = true;
 	}
 
-	int16_t get_military_number_at(uint8_t);
-	void set_military_number_at(uint8_t, int16_t);
-	MutatingIntensity do_mutate(uint8_t, int16_t);
+	int16_t get_military_number_at(uint8_t) const;
+	void set_military_number_at(uint8_t, int16_t) const;
+	MutatingIntensity do_mutate(bool, int16_t);
 	int8_t shift_weight_value(int8_t, bool = true);
 	void test_consistency(bool = false);
 
@@ -792,6 +804,9 @@ struct SchedulerTask {
 	uint8_t priority;
 	// used only for debug purposes
 	std::string descr;
+	uint32_t call_count;
+	double total_exec_time_ms;
+	double max_exec_time_ms;
 };
 
 // List of blocked fields with block time, with some accompanying functions
@@ -826,7 +841,7 @@ private:
 		uint32_t old_players_power = 0U;
 		uint32_t old60_players_power = 0U;
 		uint32_t players_casualities = 0U;
-		Time last_time_seen = Time(0);
+		Time last_time_seen = Time();  // never seen
 		uint32_t players_land = 0U;
 		uint32_t old_players_land = 0U;
 		uint32_t old60_players_land = 0U;
@@ -924,7 +939,7 @@ public:
 	                  uint16_t distance,
 	                  const Time& gametime,
 	                  uint32_t nearest_warehouse);
-	int16_t get_distance(uint32_t flag_coords, const Time& gametime, uint32_t* nw);
+	int16_t get_wh_distance(uint32_t flag_coords, const Time& gametime, uint32_t* nw);
 	void set_road_built(uint32_t coords_hash, const Time& gametime);
 	bool is_road_prohibited(uint32_t coords_hash, const Time& gametime);
 	uint16_t count() const;
