@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2021 by the Widelands Development Team
+ * Copyright (C) 2009-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -127,9 +126,36 @@ bool DefaultAI::marine_main_decisions(const Time& gametime) {
 	uint16_t ports_count = 0;
 	uint16_t shipyards_count = 0;
 	uint16_t expeditions_in_prep = 0;
+	uint16_t expeditions_ready = 0;
 	uint16_t expeditions_in_progress = 0;
 	bool idle_shipyard_stocked = false;
 
+	// goes over productionsites and gets status of shipyards
+	for (const ProductionSiteObserver& ps_obs : productionsites) {
+		if (ps_obs.bo->is(BuildingAttribute::kShipyard)) {
+			++shipyards_count;
+
+			// In very rare situation, we might have non-seafaring map but there is a shipyard
+			if (!map_allows_seafaring_) {
+				verb_log_dbg_time(
+				   game().get_gametime(),
+				   "  %1d: we have a shipyard in a non seafaring economy, dismantling it...\n",
+				   player_number());
+				if (!ps_obs.site->get_economy(Widelands::wwWORKER)->warehouses().empty()) {
+					game().send_player_dismantle(*ps_obs.site, true);
+				} else {
+					game().send_player_bulldoze(*ps_obs.site);
+				}
+				return false;
+			}
+		}
+	}
+
+	// If non-seafaring economy, no sense to go on with this function
+	if (!map_allows_seafaring_) {
+		return false;
+	}
+
 	// goes over all warehouses (these includes ports)
 	for (const WarehouseSiteObserver& wh_obs : warehousesites) {
 		if (wh_obs.bo->is(BuildingAttribute::kPort)) {
@@ -138,25 +164,36 @@ bool DefaultAI::marine_main_decisions(const Time& gametime) {
 				if (pd->expedition_started()) {
 					++expeditions_in_prep;
 				}
+				if (pd->is_expedition_ready()) {
+					++expeditions_ready;
+				}
 			}
 		}
 	}
 
-	// goes over productionsites and gets status of shipyards
+	// iterates over all ships
+	for (const ShipObserver& observer : allships) {
+		if (observer.ship->state_is_expedition()) {
+			++expeditions_in_progress;
+		}
+	}
+
+	assert(allships.size() >= expeditions_in_progress);
+	bool ship_free = allships.size() - expeditions_in_progress > 0;
+
+	/* Now we decide whether we have enough ships or need to build another:
+	 * - We always need at least one ship in transport mode
+	 * - We want at least as many free ships as we have ports
+	 * - If ships utilization is too high
+	 */
+	const bool need_ship =
+	   ports_count > 0 && shipyards_count > 0 && basic_economy_established &&
+	   (!ship_free || persistent_data->ships_utilization > 5000 ||
+	    static_cast<int>(allships.size()) - ports_count - expeditions_in_progress < 0);
+
+	// goes over productionsites finds shipyards and configures them
 	for (const ProductionSiteObserver& ps_obs : productionsites) {
 		if (ps_obs.bo->is(BuildingAttribute::kShipyard)) {
-			++shipyards_count;
-
-			// In very rare situation, we might have non-seafaring map but the shipyard is working
-			if (!map_allows_seafaring_ && !ps_obs.site->is_stopped()) {
-				verb_log_dbg_time(
-				   game().get_gametime(),
-				   "  %1d: we have working shipyard in a non seafaring ecoomy, stopping it...\n",
-				   player_number());
-				game().send_player_start_stop_building(*ps_obs.site);
-				return false;
-			}
-
 			// counting stocks
 			uint8_t stocked_wares = 0;
 			std::vector<Widelands::InputQueue*> const inputqueues = ps_obs.site->inputqueues();
@@ -168,78 +205,31 @@ bool DefaultAI::marine_main_decisions(const Time& gametime) {
 			if (stocked_wares == 16 && ps_obs.site->is_stopped() && ps_obs.site->can_start_working()) {
 				idle_shipyard_stocked = true;
 			}
-		}
-	}
 
-	// If non-seafaring economy, no sense to go on with this function
-	if (!map_allows_seafaring_) {
-		return false;
-	}
-
-	// and now over ships
-	for (const ShipObserver& observer : allships) {
-		if (observer.ship->state_is_expedition()) {
-			++expeditions_in_progress;
-		}
-	}
-
-	assert(allships.size() >= expeditions_in_progress);
-	bool ship_free = allships.size() - expeditions_in_progress > 0;
-
-	enum class FleetStatus : uint8_t { kNeedShip = 0, kEnoughShips = 1, kDoNothing = 2 };
-
-	// now we decide whether we have enough ships or need to build another
-	// three values: kDoNothing, kNeedShip, kEnoughShips
-	FleetStatus enough_ships = FleetStatus::kDoNothing;
-	if (ports_count > 0 && shipyards_count > 0 && idle_shipyard_stocked) {
-
-		if (!basic_economy_established) {
-			enough_ships = FleetStatus::kEnoughShips;
-			// we always need at least one ship in transport mode
-		} else if (!ship_free) {
-			enough_ships = FleetStatus::kNeedShip;
-
-			// we want at least as many free ships as we have ports
-		} else if (int(allships.size()) - ports_count - expeditions_in_progress < 0) {
-			enough_ships = FleetStatus::kNeedShip;
-
-			// if ships utilization is too high
-		} else if (persistent_data->ships_utilization > 5000) {
-			enough_ships = FleetStatus::kNeedShip;
-
-		} else {
-			enough_ships = FleetStatus::kEnoughShips;
-		}
-	}
-
-	// building a ship? if yes, find a shipyard and order it to build a ship
-	if (enough_ships == FleetStatus::kNeedShip) {
-
-		for (const ProductionSiteObserver& ps_obs : productionsites) {
-			if (ps_obs.bo->is(BuildingAttribute::kShipyard) && ps_obs.site->can_start_working() &&
-			    ps_obs.site->is_stopped()) {
-				// make sure it is fully stocked
-				// counting stocks
-				uint8_t stocked_wares = 0;
-				std::vector<Widelands::InputQueue*> const inputqueues = ps_obs.site->inputqueues();
-				for (Widelands::InputQueue* queue : inputqueues) {
-					if (queue->get_type() == Widelands::wwWARE) {
-						stocked_wares += queue->get_filled();
+			if (expeditions_ready > 0 && idle_shipyard_stocked && allships.size() - ports_count == 0) {
+				if (potential_wrong_shipyard_) {
+					if (!ps_obs.site->get_economy(Widelands::wwWORKER)->warehouses().empty()) {
+						game().send_player_dismantle(*ps_obs.site, true);
+					} else {
+						game().send_player_bulldoze(*ps_obs.site);
 					}
+					game().send_player_sink_ship(*allships.back().ship);
+					allships.pop_back();
+					potential_wrong_shipyard_ = false;
+					return true;
 				}
-				if (stocked_wares < 16) {
-					continue;
-				}
-
+				potential_wrong_shipyard_ = true;
+				return false;
+			}
+			if (need_ship && idle_shipyard_stocked) {
 				game().send_player_start_stop_building(*ps_obs.site);
-				return true;
 			}
 		}
 	}
 
 	// starting an expedition? if yes, find a port and order it to start an expedition
 	if (ports_count > 0 && expeditions_in_progress == 0 && expeditions_in_prep == 0 &&
-	    persistent_data->no_more_expeditions == kFalse && ship_free && basic_economy_established) {
+	    !persistent_data->no_more_expeditions && ship_free && basic_economy_established) {
 
 		// we need to find a port
 		for (const WarehouseSiteObserver& wh_obs : warehousesites) {
@@ -417,7 +407,7 @@ void DefaultAI::check_ship_in_expedition(ShipObserver& so, const Time& gametime)
 		                  so.ship->get_position().y);
 
 		// In case there is no port left to get back to, continue exploring
-		if (!so.ship->get_fleet() || !so.ship->get_fleet()->has_ports()) {
+		if ((so.ship->get_fleet() == nullptr) || !so.ship->get_fleet()->has_ports()) {
 			verb_log_dbg_time(
 			   gametime, "%d: %s at %3dx%3d: END OF EXPEDITION without port, continue exploring\n", pn,
 			   so.ship->get_shipname().c_str(), so.ship->get_position().x, so.ship->get_position().y);
@@ -453,7 +443,7 @@ void DefaultAI::check_ship_in_expedition(ShipObserver& so, const Time& gametime)
 // this is called whenever we gain ownership of a Ship
 void DefaultAI::gain_ship(Widelands::Ship& ship, NewShip type) {
 
-	allships.push_back(ShipObserver());
+	allships.emplace_back();
 	allships.back().ship = &ship;
 	allships.back().island_circ_direction = randomExploreDirection();
 
@@ -476,8 +466,8 @@ void DefaultAI::gain_ship(Widelands::Ship& ship, NewShip type) {
 }
 
 Widelands::IslandExploreDirection DefaultAI::randomExploreDirection() {
-	return std::rand() % 20 < 10 ? Widelands::IslandExploreDirection::kClockwise :  // NOLINT
-	          Widelands::IslandExploreDirection::kCounterClockwise;
+	return RNG::static_rand(20) < 10 ? Widelands::IslandExploreDirection::kClockwise :
+                                      Widelands::IslandExploreDirection::kCounterClockwise;
 }
 
 // this is called whenever ship received a notification that requires
@@ -507,7 +497,7 @@ void DefaultAI::expedition_management(ShipObserver& so) {
 		                  so.ship->get_position().y, spot_score);
 
 		// we make a decision based on the score value and random
-		if (std::rand() % 8 < spot_score) {  // NOLINT
+		if (RNG::static_rand(8) < spot_score) {
 			// we build a port here
 			game().send_player_ship_construct_port(*so.ship, so.ship->exp_port_spaces().front());
 			so.last_command_time = gametime;
@@ -579,7 +569,7 @@ bool DefaultAI::attempt_escape(ShipObserver& so) {
 
 		for (int8_t i = 0; i < 30; ++i) {
 			map.get_neighbour(tmp_coords, dir, &tmp_coords);
-			if (!(map.get_fcoords(tmp_coords).field->nodecaps() & Widelands::MOVECAPS_SWIM)) {
+			if ((map.get_fcoords(tmp_coords).field->nodecaps() & Widelands::MOVECAPS_SWIM) == 0) {
 				break;
 			}
 			if (i <= 4) {  // Four fields from the ship is too close for "open sea"
@@ -605,15 +595,15 @@ bool DefaultAI::attempt_escape(ShipObserver& so) {
 	assert(possible_directions.size() >= new_teritory_directions.size());
 
 	// If only open sea (no unexplored sea) is found, we don't always divert the ship
-	if (new_teritory_directions.empty() && std::rand() % 100 < 80) {  // NOLINT
+	if (new_teritory_directions.empty() && RNG::static_rand(100) < 80) {
 		return false;
 	}
 
 	if (!possible_directions.empty() || !new_teritory_directions.empty()) {
 		const Widelands::Direction direction =
 		   !new_teritory_directions.empty() ?
-		      new_teritory_directions.at(std::rand() % new_teritory_directions.size()) :  // NOLINT
-		      possible_directions.at(std::rand() % possible_directions.size());           // NOLINT
+            new_teritory_directions.at(RNG::static_rand(new_teritory_directions.size())) :
+            possible_directions.at(RNG::static_rand(possible_directions.size()));
 		game().send_player_ship_scouting_direction(
 		   *so.ship, static_cast<Widelands::WalkingDir>(direction));
 
