@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,28 +12,27 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "base/scoped_timer.h"
 
 #include <SDL_timer.h>
-#include <boost/format.hpp>
 
 #include "base/log.h"
+#include "base/string.h"
 
-ScopedTimer::ScopedTimer(const std::string& message, bool v) : message_(message), only_verbose_(v) {
+ScopedTimer::ScopedTimer(const std::string& message, bool only_verbose)
+   : message_(message), only_verbose_(only_verbose) {
 	startime_ = SDL_GetTicks();
 	lasttime_ = startime_;
 }
 
 ScopedTimer::~ScopedTimer() {
-	if (!only_verbose_ || g_verbose) {
+	if ((!only_verbose_ || g_verbose) && !message_.empty()) {
 		uint32_t ms_in_existance = SDL_GetTicks() - startime_;
-		const std::string logmessage = (boost::format(message_) % ms_in_existance).str();
-		log_info("%s\n", logmessage.c_str());
+		log_info("%s", format(message_, ms_in_existance).c_str());
 	}
 }
 
