@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -25,7 +24,7 @@
 constexpr uint16_t kWidelandsCopyrightStart = 2002;
 // If the following line is changed, the corresponding regex in 'utils/update_copyright.py' will
 // also need updating.
-constexpr uint16_t kWidelandsCopyrightEnd = 2021;
+constexpr uint16_t kWidelandsCopyrightEnd = 2022;
 
 /// \return the build id which is automagically created from the revision number
 /// or the VERSION file
@@ -35,4 +34,8 @@ const std::string& build_id();
 /// or to a default value)
 const std::string& build_type();
 
+/// \return the build id and type as one complete string, which is set
+/// during compile time (either manually or to a default value)
+const std::string& build_ver_details();
+
 #endif  // end of include guard: WL_BUILD_INFO_H
