@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2021 by the Widelands Development Team
+ * Copyright (C) 2004-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -68,12 +67,12 @@ Flag::Flag()
  * since die() always calls cleanup() first.
  */
 Flag::~Flag() {
-	if (ware_filled_) {
+	if (ware_filled_ != 0) {
 		log_warn("Flag: ouch! wares left\n");
 	}
 	delete[] wares_;
 
-	if (building_) {
+	if (building_ != nullptr) {
 		log_warn("Flag: ouch! building left\n");
 	}
 
@@ -82,7 +81,7 @@ Flag::~Flag() {
 	}
 
 	for (const RoadBase* const road : roads_) {
-		if (road) {
+		if (road != nullptr) {
 			log_warn("Flag: ouch! road left\n");
 		}
 	}
@@ -128,6 +127,8 @@ Flag::Flag(EditorGameBase& egbase,
      wares_(new PendingWare[ware_capacity_]),
      always_call_for_flag_(nullptr),
      act_pending_(false) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	std::fill(std::begin(roads_), std::end(roads_), nullptr);
 
 	set_owner(owning_player);
@@ -137,31 +138,31 @@ Flag::Flag(EditorGameBase& egbase,
 	upcast(RoadBase, road, egbase.map().get_immovable(coords));
 	upcast(Game, game, &egbase);
 
-	if (game) {
-		if (ware_eco) {
+	if (game != nullptr) {
+		if (ware_eco != nullptr) {
 			// We're saveloading
 			ware_eco->add_flag(*this);
 		} else {
 			//  we split a road, or a new, standalone flag is created
-			(road ? road->get_economy(wwWARE) : owning_player->create_economy(wwWARE))
+			(road != nullptr ? road->get_economy(wwWARE) : owning_player->create_economy(wwWARE))
 			   ->add_flag(*this);
 		}
-		if (worker_eco) {
+		if (worker_eco != nullptr) {
 			// We're saveloading
 			worker_eco->add_flag(*this);
 		} else {
 			//  we split a road, or a new, standalone flag is created
-			(road ? road->get_economy(wwWORKER) : owning_player->create_economy(wwWORKER))
+			(road != nullptr ? road->get_economy(wwWORKER) : owning_player->create_economy(wwWORKER))
 			   ->add_flag(*this);
 		}
-		if (road && !ware_eco && !worker_eco) {
+		if ((road != nullptr) && (ware_eco == nullptr) && (worker_eco == nullptr)) {
 			road->presplit(*game, coords);
 		}
 	}
 
 	init(egbase);
 
-	if (!ware_eco && !worker_eco && road && game) {
+	if ((ware_eco == nullptr) && (worker_eco == nullptr) && (road != nullptr) && (game != nullptr)) {
 		road->postsplit(*game, *this);
 	}
 }
@@ -200,18 +201,18 @@ void Flag::set_economy(Economy* const e, WareWorker type) {
 		}
 	}
 
-	if (building_) {
+	if (building_ != nullptr) {
 		building_->set_economy(e, type);
 	}
 
 	for (const FlagJob& temp_job : flag_jobs_) {
-		if (temp_job.request && temp_job.request->get_type() == type) {
+		if ((temp_job.request != nullptr) && temp_job.request->get_type() == type) {
 			temp_job.request->set_economy(e);
 		}
 	}
 
 	for (RoadBase* const road : roads_) {
-		if (road) {
+		if (road != nullptr) {
 			road->set_economy(e, type);
 		}
 	}
@@ -274,7 +275,7 @@ void Flag::detach_road(int32_t const dir) {
 /**
  * \return all positions we occupy on the map. For a Flag, this is only one.
  */
-BaseImmovable::PositionList Flag::get_positions(const EditorGameBase&) const {
+BaseImmovable::PositionList Flag::get_positions(const EditorGameBase& /* egbase */) const {
 	PositionList rv;
 	rv.push_back(position_);
 	return rv;
@@ -285,7 +286,7 @@ BaseImmovable::PositionList Flag::get_positions(const EditorGameBase&) const {
  */
 void Flag::get_neighbours(WareWorker type, RoutingNodeNeighbours& neighbours) {
 	for (RoadBase* const road : roads_) {
-		if (!road) {
+		if (road == nullptr) {
 			continue;
 		}
 
@@ -311,7 +312,7 @@ void Flag::get_neighbours(WareWorker type, RoutingNodeNeighbours& neighbours) {
 		neighbours.push_back(n);
 	}
 
-	if (building_ && building_->descr().get_isport()) {
+	if ((building_ != nullptr) && building_->descr().get_isport()) {
 		Warehouse* wh = dynamic_cast<Warehouse*>(building_);
 		if (PortDock* pd = wh->get_portdock()) {
 			pd->add_neighbours(neighbours);
@@ -324,7 +325,7 @@ void Flag::get_neighbours(WareWorker type, RoutingNodeNeighbours& neighbours) {
  */
 RoadBase* Flag::get_roadbase(Flag& flag) {
 	for (RoadBase* const road : roads_) {
-		if (road) {
+		if (road != nullptr) {
 			if (&road->get_flag(RoadBase::FlagStart) == &flag ||
 			    &road->get_flag(RoadBase::FlagEnd) == &flag) {
 				return road;
@@ -333,7 +334,7 @@ RoadBase* Flag::get_roadbase(Flag& flag) {
 	}
 	return nullptr;
 }
-Road* Flag::get_road(Flag& flag) {
+Road* Flag::get_road(Flag& flag) const {
 	for (int8_t i = WalkingDir::FIRST_DIRECTION; i <= WalkingDir::LAST_DIRECTION; ++i) {
 		if (Road* const road = get_road(i)) {
 			if (&road->get_flag(RoadBase::FlagStart) == &flag ||
@@ -346,13 +347,13 @@ Road* Flag::get_road(Flag& flag) {
 }
 
 Road* Flag::get_road(uint8_t const dir) const {
-	if (roads_[dir - 1] && Road::is_road_descr(&roads_[dir - 1]->descr())) {
+	if ((roads_[dir - 1] != nullptr) && Road::is_road_descr(&roads_[dir - 1]->descr())) {
 		return dynamic_cast<Road*>(roads_[dir - 1]);
 	}
 	return nullptr;
 }
 Waterway* Flag::get_waterway(uint8_t const dir) const {
-	if (roads_[dir - 1] && Waterway::is_waterway_descr(&roads_[dir - 1]->descr())) {
+	if ((roads_[dir - 1] != nullptr) && Waterway::is_waterway_descr(&roads_[dir - 1]->descr())) {
 		return dynamic_cast<Waterway*>(roads_[dir - 1]);
 	}
 	return nullptr;
@@ -395,7 +396,7 @@ uint8_t Flag::nr_of_waterways() const {
 }
 
 bool Flag::is_dead_end() const {
-	if (get_building()) {
+	if (get_building() != nullptr) {
 		return false;
 	}
 	Flag const* first_other_flag = nullptr;
@@ -404,7 +405,7 @@ bool Flag::is_dead_end() const {
 		if (RoadBase* const road = get_roadbase(road_id)) {
 			Flag& start = road->get_flag(RoadBase::FlagStart);
 			Flag& other = this == &start ? road->get_flag(RoadBase::FlagEnd) : start;
-			if (first_other_flag) {
+			if (first_other_flag != nullptr) {
 				if (&other != first_other_flag) {
 					return false;
 				}
@@ -429,14 +430,14 @@ bool Flag::has_capacity() const {
  *
  * The capacity queue is a simple FIFO queue.
  */
-void Flag::wait_for_capacity(Game&, Worker& bob) {
+void Flag::wait_for_capacity(Game& /* game */, Worker& bob) {
 	capacity_wait_.push_back(&bob);
 }
 
 /**
  * Remove the worker from the list of workers waiting for free capacity.
  */
-void Flag::skip_wait_for_capacity(Game&, Worker& w) {
+void Flag::skip_wait_for_capacity(Game& /* game */, Worker& w) {
 	CapacityWaitQueue::iterator const it =
 	   std::find(capacity_wait_.begin(), capacity_wait_.end(), &w);
 	if (it != capacity_wait_.end()) {
@@ -455,7 +456,7 @@ void Flag::add_ware(EditorGameBase& egbase, WareInstance& ware) {
 	pi.priority = 0;
 
 	Transfer* trans = ware.get_transfer();
-	if (trans) {
+	if (trans != nullptr) {
 		uint32_t trans_steps = trans->get_steps_left();
 		if (trans_steps < 3) {
 			pi.priority = 2;
@@ -464,7 +465,7 @@ void Flag::add_ware(EditorGameBase& egbase, WareInstance& ware) {
 		}
 
 		Request* req = trans->get_request();
-		if (req) {
+		if (req != nullptr) {
 			pi.priority = pi.priority + req->get_normalized_transfer_priority();
 		}
 	}
@@ -482,7 +483,7 @@ void Flag::add_ware(EditorGameBase& egbase, WareInstance& ware) {
  * \note Due to fetch_from_flag() semantics, this function makes no sense
  * for a  building destination.
  */
-bool Flag::has_pending_ware(Game&, Flag& dest) {
+bool Flag::has_pending_ware(Game& /* game */, Flag& dest) {
 	for (int32_t i = 0; i < ware_filled_; ++i) {
 		if (!wares_[i].pending) {
 			continue;
@@ -503,7 +504,7 @@ bool Flag::has_pending_ware(Game&, Flag& dest) {
  * ware. Ware with highest transfer priority is chosen.
  * \return true if an ware is actually waiting for the carrier.
  */
-bool Flag::ack_pickup(Game&, Flag& destflag) {
+bool Flag::ack_pickup(Game& /* game */, Flag& destflag) {
 	int32_t highest_pri = -1;
 	int32_t i_pri = -1;
 
@@ -575,7 +576,7 @@ void Flag::wake_up_capacity_queue(Game& game) {
 	while (!capacity_wait_.empty()) {
 		Worker* const w = capacity_wait_.front().get(game);
 		capacity_wait_.pop_front();
-		if (w && w->wakeup_flag_capacity(game, *this)) {
+		if ((w != nullptr) && w->wakeup_flag_capacity(game, *this)) {
 			break;
 		}
 	}
@@ -612,7 +613,7 @@ WareInstance* Flag::fetch_pending_ware(Game& game, PlayerImmovable& dest) {
 	memmove(&wares_[best_index], &wares_[best_index + 1],
 	        sizeof(wares_[0]) * (ware_filled_ - best_index));
 
-	ware->set_location(game, nullptr);
+	ware->set_location(game, nullptr);  // Ware has no location while in transit
 
 	// wake up capacity wait queue
 	wake_up_capacity_queue(game);
@@ -625,7 +626,7 @@ WareInstance* Flag::fetch_pending_ware(Game& game, PlayerImmovable& dest) {
  */
 void Flag::propagate_promoted_road(Road* const promoted_road) {
 	// Abort if flag has a building attached to it
-	if (building_) {
+	if (building_ != nullptr) {
 		return;
 	}
 
@@ -633,7 +634,7 @@ void Flag::propagate_promoted_road(Road* const promoted_road) {
 	int32_t sum = 0;
 	for (int8_t i = WalkingDir::FIRST_DIRECTION; i <= WalkingDir::LAST_DIRECTION; ++i) {
 		Road* const road = get_road(i);
-		if (road && road != promoted_road) {
+		if ((road != nullptr) && road != promoted_road) {
 			sum += kRoadMaxWallet + road->wallet() * road->wallet();
 		}
 	}
@@ -641,7 +642,7 @@ void Flag::propagate_promoted_road(Road* const promoted_road) {
 	// Distribute propagation coins in a smart way
 	for (int8_t i = WalkingDir::FIRST_DIRECTION; i <= WalkingDir::LAST_DIRECTION; ++i) {
 		Road* const road = get_road(i);
-		if (road && !road->is_busy()) {
+		if ((road != nullptr) && !road->is_busy()) {
 			road->add_to_wallet(0.5 * (kRoadMaxWallet - road->wallet()) *
 			                    (kRoadMaxWallet + road->wallet() * road->wallet()) / sum);
 		}
@@ -727,7 +728,7 @@ void Flag::call_carrier(Game& game, WareInstance& ware, PlayerImmovable* const n
 		pi = &wares_[i];
 
 		// Deal with the non-moving case quickly
-		if (!nextstep) {
+		if (nextstep == nullptr) {
 			pi->nextstep = nullptr;
 			pi->pending = true;
 			return;
@@ -762,7 +763,7 @@ void Flag::call_carrier(Game& game, WareInstance& ware, PlayerImmovable* const n
 			Flag* other;
 			RoadBase::FlagId flagid;
 
-			if (!road) {
+			if (road == nullptr) {
 				continue;
 			}
 
@@ -832,20 +833,20 @@ void Flag::cleanup(EditorGameBase& egbase) {
 		flag_jobs_.erase(flag_jobs_.begin());
 	}
 
-	while (ware_filled_) {
+	while (ware_filled_ != 0) {
 		WareInstance& ware = *wares_[--ware_filled_].ware;
 
 		ware.set_location(egbase, nullptr);
 		ware.destroy(egbase);
 	}
 
-	if (building_) {
+	if (building_ != nullptr) {
 		building_->remove(egbase);  //  immediate death
 		assert(!building_);
 	}
 
 	for (RoadBase* rb : roads_) {
-		if (rb) {
+		if (rb != nullptr) {
 			rb->remove(egbase);  //  immediate death
 		}
 	}
@@ -868,7 +869,7 @@ void Flag::cleanup(EditorGameBase& egbase) {
 }
 
 void Flag::draw(const Time& gametime,
-                const InfoToDraw,
+                const InfoToDraw /*info_to_draw*/,
                 const Vector2f& field_on_dst,
                 const Coords& coords,
                 float scale,
@@ -904,7 +905,7 @@ void Flag::draw(const Time& gametime,
  * when a player removes a flag.
  */
 void Flag::destroy(EditorGameBase& egbase) {
-	if (building_) {
+	if (building_ != nullptr) {
 		building_->destroy(egbase);
 		assert(!building_);
 	}
@@ -912,7 +913,7 @@ void Flag::destroy(EditorGameBase& egbase) {
 	PlayerImmovable::destroy(egbase);
 }
 
-void Flag::receive_worker(Game&, Worker&) {
+void Flag::receive_worker(Game& /* game */, Worker& /*worker*/) {
 	// Callback when a requested scout arrives.
 	// He knows what to do next by himself, nothing to do for us currently.
 }
@@ -925,7 +926,7 @@ void Flag::do_schedule_act(Game& game, const Duration& d) {
 	schedule_act(game, d);
 }
 
-void Flag::act(Game& game, uint32_t) {
+void Flag::act(Game& game, uint32_t /*data*/) {
 	assert(act_pending_);
 	act_pending_ = false;
 
@@ -935,8 +936,8 @@ void Flag::act(Game& game, uint32_t) {
 		if (it->type == FlagJob::Type::kScout) {
 			ProductionSite* ps = get_economy(wwWORKER)->find_closest_occupied_productionsite(
 			   *this, owner().tribe().scouts_house(), true);
-			if (ps) {
-				Worker* worker = ps->working_positions()[0].worker.get(game);
+			if (ps != nullptr) {
+				Worker* worker = ps->working_positions()->at(0).worker.get(game);
 				assert(worker);
 				if (!worker->top_state().objvar1.is_set() && worker->get_location(game) == ps &&
 				    !ps->has_forced_state()) {
@@ -993,8 +994,11 @@ void Flag::add_flag_job(Game& game, const FlagJob::Type t) {
  * This function is called when one of the flag job workers arrives on
  * the flag. Give him his job.
  */
-void Flag::flag_job_request_callback(
-   Game& game, Request& rq, DescriptionIndex, Worker* const w, PlayerImmovable& target) {
+void Flag::flag_job_request_callback(Game& game,
+                                     Request& rq,
+                                     DescriptionIndex /* index */,
+                                     Worker* const w,
+                                     PlayerImmovable& target) {
 	Flag& flag = dynamic_cast<Flag&>(target);
 
 	assert(w);
@@ -1019,7 +1023,7 @@ void Flag::log_general_info(const Widelands::EditorGameBase& egbase) const {
 
 	Widelands::PlayerImmovable::log_general_info(egbase);
 
-	if (ware_filled_) {
+	if (ware_filled_ != 0) {
 		molog(egbase.get_gametime(), "Wares at flag:\n");
 		for (int i = 0; i < ware_filled_; ++i) {
 			PendingWare& pi = wares_[i];
