@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2021 by the Widelands Development Team
+ * Copyright (C) 2011-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -117,11 +116,11 @@ struct StepEvalFindShipFleet {
 	}
 	int32_t stepcost(
 	   Map& /* map */, FCoords from, int32_t fromcost, WalkingDir /* dir */, FCoords to) const {
-		if (!(to.field->nodecaps() & (MOVECAPS_SWIM | MOVECAPS_WALK))) {
+		if ((to.field->nodecaps() & (MOVECAPS_SWIM | MOVECAPS_WALK)) == 0) {
 			return -1;
 		}
 
-		if (!(from.field->nodecaps() & MOVECAPS_SWIM)) {
+		if ((from.field->nodecaps() & MOVECAPS_SWIM) == 0) {
 			// We are allowed to land on and leave the shore,
 			// but not in the middle of a path
 			if (fromcost > 0) {
@@ -219,13 +218,7 @@ bool ShipFleet::merge(EditorGameBase& egbase, ShipFleet* other) {
 
 	uint32_t old_nrports = ports_.size();
 	ports_.insert(ports_.end(), other->ports_.begin(), other->ports_.end());
-	portpaths_.resize((ports_.size() * (ports_.size() - 1)) / 2);
-
-	for (uint32_t j = 1; j < other->ports_.size(); ++j) {
-		for (uint32_t i = 0; i < j; ++i) {
-			portpath(old_nrports + i, old_nrports + j) = other->portpath(i, j);
-		}
-	}
+	port_paths_.insert(other->port_paths_.begin(), other->port_paths_.end());
 
 	for (uint32_t idx = old_nrports; idx < ports_.size(); ++idx) {
 		ports_[idx]->set_fleet(this);
@@ -236,7 +229,7 @@ bool ShipFleet::merge(EditorGameBase& egbase, ShipFleet* other) {
 	}
 
 	other->ports_.clear();
-	other->portpaths_.clear();
+	other->port_paths_.clear();
 	other->remove(egbase);
 
 	update(egbase);
@@ -273,7 +266,7 @@ void ShipFleet::cleanup(EditorGameBase& egbase) {
 			Economy::check_split(base, pd->base_flag(), wwWORKER);
 		}
 	}
-	portpaths_.clear();
+	port_paths_.clear();
 
 	while (!ships_.empty()) {
 		Ship* ship = ships_.back();
@@ -286,14 +279,16 @@ void ShipFleet::cleanup(EditorGameBase& egbase) {
 
 ShipFleet::PortPath& ShipFleet::portpath(uint32_t i, uint32_t j) {
 	assert(i < j);
-
-	return portpaths_[((j - 1) * j) / 2 + i];
+	assert(j < ports_.size());
+	// This creates a default-constructed portpath if it did not yet exist.
+	return port_paths_[std::make_pair(ports_[i]->serial(), ports_[j]->serial())];
 }
 
 const ShipFleet::PortPath& ShipFleet::portpath(uint32_t i, uint32_t j) const {
 	assert(i < j);
-
-	return portpaths_[((j - 1) * j) / 2 + i];
+	assert(j < ports_.size());
+	// This throws an exception if the port path was never computed yet.
+	return port_paths_.at(std::make_pair(ports_[i]->serial(), ports_[j]->serial()));
 }
 
 ShipFleet::PortPath& ShipFleet::portpath_bidir(uint32_t i, uint32_t j, bool& reverse) {
@@ -467,7 +462,7 @@ struct StepEvalFindPorts {
 
 	int32_t stepcost(
 	   const Map& map, FCoords from, int32_t /* fromcost */, WalkingDir dir, FCoords to) const {
-		if (!(to.field->nodecaps() & MOVECAPS_SWIM)) {
+		if ((to.field->nodecaps() & MOVECAPS_SWIM) == 0) {
 			return -1;
 		}
 		return map.calc_bidi_cost(from, dir);
@@ -514,7 +509,7 @@ void ShipFleet::connect_port(EditorGameBase& egbase, uint32_t idx) {
 	FCoords cur;
 	while (!se.targets.empty() && astar.step(cur, cost)) {
 		BaseImmovable* imm = cur.field->get_immovable();
-		if (!imm || imm->descr().type() != MapObjectType::PORTDOCK) {
+		if ((imm == nullptr) || imm->descr().type() != MapObjectType::PORTDOCK) {
 			continue;
 		}
 
@@ -523,7 +518,7 @@ void ShipFleet::connect_port(EditorGameBase& egbase, uint32_t idx) {
 				continue;
 			}
 
-			if (pd->get_fleet() && pd->get_fleet() != this) {
+			if ((pd->get_fleet() != nullptr) && pd->get_fleet() != this) {
 				log_err_time(egbase.get_gametime(),
 				             "ShipFleet::connect_port: different fleets despite reachability\n");
 				continue;
@@ -576,7 +571,6 @@ void ShipFleet::add_port(EditorGameBase& egbase, PortDock* port) {
 		}
 	}
 
-	portpaths_.resize((ports_.size() * (ports_.size() - 1)) / 2);
 	if (upcast(Game, g, &egbase)) {
 		schedule_.port_added(*g, *port);
 	}
@@ -584,20 +578,18 @@ void ShipFleet::add_port(EditorGameBase& egbase, PortDock* port) {
 }
 
 void ShipFleet::remove_port(EditorGameBase& egbase, PortDock* port) {
-	std::vector<PortDock*>::iterator it = std::find(ports_.begin(), ports_.end(), port);
-	if (it != ports_.end()) {
-		uint32_t gap = it - ports_.begin();
-		for (uint32_t i = 0; i < gap; ++i) {
-			portpath(i, gap) = portpath(i, ports_.size() - 1);
+	for (auto it = port_paths_.begin(); it != port_paths_.end();) {
+		if (it->first.first == port->serial() || it->first.second == port->serial()) {
+			it = port_paths_.erase(it);
+		} else {
+			++it;
 		}
-		for (uint32_t i = gap + 1; i < ports_.size() - 1; ++i) {
-			portpath(gap, i) = portpath(i, ports_.size() - 1);
-			if (portpath(gap, i).path) {
-				portpath(gap, i).path->reverse();
-			}
+	}
+	{
+		std::vector<PortDock*>::iterator it = std::find(ports_.begin(), ports_.end(), port);
+		if (it != ports_.end()) {
+			ports_.erase(it);
 		}
-		ports_.erase(it);
-		portpaths_.resize((ports_.size() * (ports_.size() - 1)) / 2);
 	}
 	port->set_fleet(nullptr);
 
@@ -675,7 +667,7 @@ PortDock* ShipFleet::get_arbitrary_dock() const {
  * Trigger an update of ship scheduling
  */
 void ShipFleet::update(EditorGameBase& egbase) {
-	if (act_pending_ || !serial()) {
+	if (act_pending_ || (serial() == 0u)) {
 		return;
 	}
 
@@ -690,7 +682,7 @@ void ShipFleet::update(EditorGameBase& egbase) {
  *
  * @note Do not call this directly; instead, trigger it via @ref update
  */
-void ShipFleet::act(Game& game, uint32_t) {
+void ShipFleet::act(Game& game, uint32_t /*data*/) {
 	assert(act_pending_);
 	act_pending_ = false;
 
@@ -740,7 +732,7 @@ void ShipFleet::Loader::load(FileRead& fr) {
 		ports_[i] = fr.unsigned_32();
 	}
 
-	fleet.act_pending_ = fr.unsigned_8();
+	fleet.act_pending_ = (fr.unsigned_8() != 0u);
 	fleet.schedule_.load(fr);
 }
 
@@ -764,8 +756,6 @@ void ShipFleet::Loader::load_pointers() {
 		fleet.ports_.back()->set_fleet(&fleet);
 	}
 
-	fleet.portpaths_.resize((fleet.ports_.size() * (fleet.ports_.size() - 1)) / 2);
-
 	fleet.act_pending_ = save_act_pending;
 
 	fleet.schedule_.load_pointers(map_object_loader);
@@ -794,13 +784,13 @@ MapObject::Loader* ShipFleet::load(EditorGameBase& egbase, MapObjectLoader& mol,
 		const uint8_t packet_version = fr.unsigned_8();
 		if (packet_version == kCurrentPacketVersion) {
 			PlayerNumber owner_number = fr.unsigned_8();
-			if (!owner_number || owner_number > egbase.map().get_nrplayers()) {
+			if ((owner_number == 0u) || owner_number > egbase.map().get_nrplayers()) {
 				throw GameDataError("owner number is %u but there are only %u players", owner_number,
 				                    egbase.map().get_nrplayers());
 			}
 
 			Player* owner = egbase.get_player(owner_number);
-			if (!owner) {
+			if (owner == nullptr) {
 				throw GameDataError("owning player %u does not exist", owner_number);
 			}
 			loader->init(egbase, mol, *(new ShipFleet(owner)));
@@ -819,7 +809,7 @@ void ShipFleet::save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw)
 	fw.unsigned_8(HeaderShipFleet);
 	fw.unsigned_8(kCurrentPacketVersion);
 
-	fw.unsigned_8(owner_->player_number());
+	fw.unsigned_8(owner_.load()->player_number());
 
 	MapObject::save(egbase, mos, fw);
 
@@ -832,7 +822,7 @@ void ShipFleet::save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw)
 		fw.unsigned_32(mos.get_object_file_index(*temp_port));
 	}
 
-	fw.unsigned_8(act_pending_);
+	fw.unsigned_8(static_cast<uint8_t>(act_pending_));
 
 	schedule_.save(egbase, mos, fw);
 }
