@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2021 by the Widelands Development Team
+ * Copyright (C) 2007-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,14 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
-#include <boost/test/unit_test.hpp>
-
 #include "base/macros.h"
+#include "base/test.h"
 #include "economy/flag.h"
 #include "economy/iroute.h"
 #include "economy/itransport_cost_calculator.h"
@@ -28,10 +26,6 @@
 #include "logic/map_objects/tribes/wareworker.h"
 #include "logic/widelands_geometry.h"
 
-// Triggered by BOOST_AUTO_TEST_CASE
-CLANG_DIAG_OFF("-Wdisabled-macro-expansion")
-CLANG_DIAG_OFF("-Wused-but-marked-unused")
-
 /******************/
 /* Helper classes */
 /******************/
@@ -65,7 +59,8 @@ public:
 		return position_;
 	}
 
-	void get_neighbours(Widelands::WareWorker type, Widelands::RoutingNodeNeighbours&) override;
+	void get_neighbours(Widelands::WareWorker type,
+	                    Widelands::RoutingNodeNeighbours& /*n*/) override;
 
 	// test functionality
 	bool all_members_zeroed() const;
@@ -88,8 +83,9 @@ void TestingRoutingNode::get_neighbours(Widelands::WareWorker type,
 	}
 }
 bool TestingRoutingNode::all_members_zeroed() const {
-	bool integers_zero = !mpf_cycle_ware && !mpf_realcost_ware && !mpf_estimate_ware &&
-	                     !mpf_cycle_worker && !mpf_realcost_worker && !mpf_estimate_worker;
+	bool integers_zero = (mpf_cycle_ware == 0u) && (mpf_realcost_ware == 0) &&
+	                     (mpf_estimate_ware == 0) && (mpf_cycle_worker == 0u) &&
+	                     (mpf_realcost_worker == 0) && (mpf_estimate_worker == 0);
 	bool pointers_zero = (mpf_backlink_ware == nullptr) && (mpf_backlink_worker == nullptr);
 
 	return pointers_zero && integers_zero;
@@ -109,7 +105,7 @@ class TestingRoute : public Widelands::IRoute {
 public:
 	using Nodes = std::vector<Widelands::RoutingNode*>;
 
-	void init(int32_t) override {
+	void init(int32_t /* cost */) override {
 		nodes.clear();
 	}
 	void insert_as_first(Widelands::RoutingNode* node) override {
@@ -172,7 +168,7 @@ private:
 /*************************************************************************/
 /*                                 TESTS                                 */
 /*************************************************************************/
-BOOST_AUTO_TEST_SUITE(Routing)
+TESTSUITE_START(Routing)
 
 // {{{ TestingRoutingNode Test Cases
 /*
@@ -180,13 +176,13 @@ BOOST_AUTO_TEST_SUITE(Routing)
  * are sure it works in the other tests
  *
  */
-BOOST_AUTO_TEST_CASE(testingnode_creation) {
+TESTCASE(testingnode_creation) {
 	TestingRoutingNode d0;
 	TestingRoutingNode d1(0, Widelands::Coords(15, 0));
 
-	BOOST_CHECK_EQUAL(d0.get_position().y, d1.get_position().y);
-	BOOST_CHECK_EQUAL(d0.get_position().x, 0);
-	BOOST_CHECK_EQUAL(d1.get_position().x, 15);
+	check_equal(d0.get_position().y, d1.get_position().y);
+	check_equal(d0.get_position().x, 0);
+	check_equal(d1.get_position().x, 15);
 }
 struct TestingNodeDefaultNodesFixture {
 	TestingNodeDefaultNodesFixture() {
@@ -209,17 +205,20 @@ struct TestingNodeDefaultNodesFixture {
 	TestingRoutingNode* d0;
 	TestingRoutingNode* d1;
 };
-BOOST_FIXTURE_TEST_CASE(testingnode_neighbour_attaching, TestingNodeDefaultNodesFixture) {
-	d0->add_neighbour(d1);
+TESTCASE(testingnode_neighbour_attaching) {
+	TestingNodeDefaultNodesFixture f;
+	f.d0->add_neighbour(f.d1);
 
-	BOOST_CHECK_EQUAL(d0->get_neighbour(0), d1);
+	check_equal(f.d0->get_neighbour(0), f.d1);
 }
-BOOST_FIXTURE_TEST_CASE(testingnode_illegalneighbour_access, TestingNodeDefaultNodesFixture) {
+TESTCASE(testingnode_illegalneighbour_access) {
+	TestingNodeDefaultNodesFixture f;
 	try {
-		d0->get_neighbour(0);
-		BOOST_ERROR("BadAccess not thrown");
+		f.d0->get_neighbour(0);
 	} catch (BadAccess&) {
+		return;
 	}
+	throw wexception("BadAccess not thrown");
 }
 // }}} End of TestingRoutingNode Test cases
 
@@ -227,10 +226,10 @@ BOOST_FIXTURE_TEST_CASE(testingnode_illegalneighbour_access, TestingNodeDefaultN
 /*
  * Now test the routing nodes functionality
  */
-BOOST_AUTO_TEST_CASE(RoutingNode_InitializeMemberVariables) {
+TESTCASE(RoutingNode_InitializeMemberVariables) {
 	TestingRoutingNode d0(0, Widelands::Coords(15, 0));
 
-	BOOST_CHECK(d0.all_members_zeroed());
+	check_equal(true, d0.all_members_zeroed());
 }
 
 struct SimpleRouterFixture {
@@ -251,12 +250,12 @@ struct SimpleRouterFixture {
 	 * Callback for the incredibly rare case that the \ref Router pathfinding
 	 * cycle wraps around.
 	 */
-	void reset() {
-		if (d0) {
+	void reset() const {
+		if (d0 != nullptr) {
 			d0->reset_path_finding_cycle(Widelands::wwWARE);
 			d0->reset_path_finding_cycle(Widelands::wwWORKER);
 		}
-		if (d1) {
+		if (d1 != nullptr) {
 			d1->reset_path_finding_cycle(Widelands::wwWARE);
 			d1->reset_path_finding_cycle(Widelands::wwWORKER);
 		}
@@ -270,99 +269,105 @@ struct SimpleRouterFixture {
 };
 
 // {{{ TestingRoute Testing
-BOOST_AUTO_TEST_CASE(TestingRoute_emptyatstart) {
+TESTCASE(TestingRoute_emptyatstart) {
 	TestingRoute r;
-	BOOST_CHECK_EQUAL(r.get_length(), 0);
+	check_equal(r.get_length(), 0);
 }
-BOOST_AUTO_TEST_CASE(TestingRoute_addnode) {
+TESTCASE(TestingRoute_addnode) {
 	TestingRoute r;
 	TestingRoutingNode d;
 	r.insert_as_first(&d);
 
-	BOOST_CHECK_EQUAL(r.get_length(), 1);
+	check_equal(r.get_length(), 1);
 }
-BOOST_FIXTURE_TEST_CASE(TestingRoute_hasnode, SimpleRouterFixture) {
-	BOOST_CHECK_EQUAL(route.has_node(d0), false);
-	BOOST_CHECK_EQUAL(route.has_node(d1), false);
-	route.insert_as_first(d0);
-	BOOST_CHECK_EQUAL(route.has_node(d0), true);
-	BOOST_CHECK_EQUAL(route.has_node(d1), false);
-	route.insert_as_first(d1);
-	BOOST_CHECK_EQUAL(route.has_node(d0), true);
-	BOOST_CHECK_EQUAL(route.has_node(d1), true);
+TESTCASE(TestingRoute_hasnode) {
+	SimpleRouterFixture f;
+	check_equal(f.route.has_node(f.d0), false);
+	check_equal(f.route.has_node(f.d1), false);
+	f.route.insert_as_first(f.d0);
+	check_equal(f.route.has_node(f.d0), true);
+	check_equal(f.route.has_node(f.d1), false);
+	f.route.insert_as_first(f.d1);
+	check_equal(f.route.has_node(f.d0), true);
+	check_equal(f.route.has_node(f.d1), true);
 }
-BOOST_FIXTURE_TEST_CASE(TestingRoute_haschain, SimpleRouterFixture) {
+TESTCASE(TestingRoute_haschain) {
+	SimpleRouterFixture f;
 	std::vector<Widelands::RoutingNode*> chain;
 
-	chain.push_back(d0);
-	route.insert_as_first(d0);
-	BOOST_CHECK_EQUAL(route.has_chain(chain), true);
+	chain.push_back(f.d0);
+	f.route.insert_as_first(f.d0);
+	check_equal(f.route.has_chain(chain), true);
 
-	route.insert_as_first(d1);
-	chain.insert(chain.begin(), d1);
-	BOOST_CHECK_EQUAL(route.has_chain(chain), true);
+	f.route.insert_as_first(f.d1);
+	chain.insert(chain.begin(), f.d1);
+	check_equal(f.route.has_chain(chain), true);
 }
-BOOST_FIXTURE_TEST_CASE(TestingRoute_chainisunidirectional, SimpleRouterFixture) {
+TESTCASE(TestingRoute_chainisunidirectional) {
+	SimpleRouterFixture f;
 	std::vector<Widelands::RoutingNode*> chain;
 	// Chains are unidirectional. Check that chain.clear();
-	chain.push_back(d0);
-	chain.push_back(d1);
-	route.insert_as_first(d0);
-	route.insert_as_first(d1);
-	BOOST_CHECK_EQUAL(route.has_chain(chain), false);
+	chain.push_back(f.d0);
+	chain.push_back(f.d1);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(f.d1);
+	check_equal(f.route.has_chain(chain), false);
 }
-BOOST_FIXTURE_TEST_CASE(TestingRoute_haschain_checksubchain, SimpleRouterFixture) {
+TESTCASE(TestingRoute_haschain_checksubchain) {
+	SimpleRouterFixture f;
 	// Do not get confused when a partial chain is found
 	TestingRoutingNode d;
 	std::vector<Widelands::RoutingNode*> chain;
 
 	chain.push_back(&d);
-	chain.push_back(d0);
-	chain.push_back(d1);
+	chain.push_back(f.d0);
+	chain.push_back(f.d1);
 
-	route.insert_as_first(&d);
-	route.insert_as_first(d0);
-	route.insert_as_first(&d);  // d -> d0 -> d
-	BOOST_CHECK_EQUAL(route.has_chain(chain), false);
-	route.insert_as_first(d1);
-	route.insert_as_first(d0);
-	route.insert_as_first(&d);
-	route.insert_as_first(d0);
-	route.insert_as_first(d1);  // d1 d0 d d0 d1 d d0 d
-	BOOST_CHECK_EQUAL(route.has_chain(chain), true);
+	f.route.insert_as_first(&d);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(&d);  // d -> d0 -> d
+	check_equal(f.route.has_chain(chain), false);
+	f.route.insert_as_first(f.d1);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(&d);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(f.d1);  // d1 d0 d d0 d1 d d0 d
+	check_equal(f.route.has_chain(chain), true);
 }
-BOOST_FIXTURE_TEST_CASE(TestingRoute_haschain_checksubchain_endisnotstart, SimpleRouterFixture) {
+TESTCASE(TestingRoute_haschain_checksubchain_endisnotstart) {
+	SimpleRouterFixture f;
 	// Do not get confused when a partial chain is found
 	TestingRoutingNode d;
 	std::vector<Widelands::RoutingNode*> chain;
 
 	chain.push_back(&d);
-	chain.push_back(d0);
-	chain.push_back(d1);
+	chain.push_back(f.d0);
+	chain.push_back(f.d1);
 
-	route.insert_as_first(&d);
-	route.insert_as_first(d0);
-	route.insert_as_first(d0);
-	BOOST_CHECK_EQUAL(route.has_chain(chain), false);
-	route.insert_as_first(d0);
-	route.insert_as_first(d0);
-	route.insert_as_first(&d);
-	route.insert_as_first(d0);
-	BOOST_CHECK_EQUAL(route.has_chain(chain), false);
-	route.insert_as_first(d1);
-	route.insert_as_first(d0);
-	route.insert_as_first(&d);
-	route.insert_as_first(d0);
-	route.insert_as_first(d1);
-	BOOST_CHECK_EQUAL(route.has_chain(chain), true);
+	f.route.insert_as_first(&d);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(f.d0);
+	check_equal(f.route.has_chain(chain), false);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(&d);
+	f.route.insert_as_first(f.d0);
+	check_equal(f.route.has_chain(chain), false);
+	f.route.insert_as_first(f.d1);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(&d);
+	f.route.insert_as_first(f.d0);
+	f.route.insert_as_first(f.d1);
+	check_equal(f.route.has_chain(chain), true);
 }
-BOOST_FIXTURE_TEST_CASE(TestingRoute_init, SimpleRouterFixture) {
-	route.insert_as_first(d0);
-	BOOST_CHECK_EQUAL(route.get_length(), 1);
-	route.init(0);
-	BOOST_CHECK_EQUAL(route.get_length(), 0);
-	route.insert_as_first(d1);
-	BOOST_CHECK_EQUAL(route.get_length(), 1);
+TESTCASE(TestingRoute_init) {
+	SimpleRouterFixture f;
+	f.route.insert_as_first(f.d0);
+	check_equal(f.route.get_length(), 1);
+	f.route.init(0);
+	check_equal(f.route.get_length(), 0);
+	f.route.insert_as_first(f.d1);
+	check_equal(f.route.get_length(), 1);
 }
 // }}}
 
@@ -373,18 +378,20 @@ BOOST_FIXTURE_TEST_CASE(TestingRoute_init, SimpleRouterFixture) {
 /****************/
 /* SIMPLE TESTS */
 /****************/
-BOOST_FIXTURE_TEST_CASE(router_findroute_seperatedNodes_exceptFail, SimpleRouterFixture) {
-	bool rval = r.find_route(*d0, *d1, &route, Widelands::wwWORKER, -1, cc);
+TESTCASE(router_findroute_seperatedNodes_exceptFail) {
+	SimpleRouterFixture f;
+	bool rval = f.r.find_route(*f.d0, *f.d1, &f.route, Widelands::wwWORKER, -1, f.cc);
 
-	BOOST_CHECK_EQUAL(rval, false);
+	check_equal(rval, false);
 }
-BOOST_FIXTURE_TEST_CASE(router_findroute_connectedNodes_exceptSuccess, SimpleRouterFixture) {
-	d0->add_neighbour(d1);
-	d1->add_neighbour(d0);
+TESTCASE(router_findroute_connectedNodes_exceptSuccess) {
+	SimpleRouterFixture f;
+	f.d0->add_neighbour(f.d1);
+	f.d1->add_neighbour(f.d0);
 
-	bool rval = r.find_route(*d0, *d1, &route, Widelands::wwWORKER, -1, cc);
+	bool rval = f.r.find_route(*f.d0, *f.d1, &f.route, Widelands::wwWORKER, -1, f.cc);
 
-	BOOST_CHECK_EQUAL(rval, true);
+	check_equal(rval, true);
 }
 
 struct ComplexRouterFixture {
@@ -409,7 +416,7 @@ struct ComplexRouterFixture {
 	 */
 	TestingRoutingNode* new_node_w_neighbour(TestingRoutingNode* const d,
 	                                         const Widelands::Coords& pos = Widelands::Coords(0, 0),
-	                                         int32_t = 1,
+	                                         int32_t /* unused */ = 1,
 	                                         int32_t const waitcost = 0) {
 		TestingRoutingNode* dnew = new TestingRoutingNode(waitcost, pos);
 
@@ -502,48 +509,50 @@ struct ComplexRouterFixture {
 	TestingTransportCostCalculator cc;
 };
 
-BOOST_FIXTURE_TEST_CASE(triangle_test, ComplexRouterFixture) {
-	add_triangle(d0);
+TESTCASE(triangle_test) {
+	ComplexRouterFixture f;
+	f.add_triangle(f.d0);
 
-	BOOST_CHECK_EQUAL(nodes.size(), 3);
+	check_equal(f.nodes.size(), 3);
 }
 
-BOOST_FIXTURE_TEST_CASE(find_long_route, ComplexRouterFixture) {
-	Nodes chain;
+TESTCASE(find_long_route) {
+	ComplexRouterFixture f;
+	ComplexRouterFixture::Nodes chain;
 
-	TestingRoutingNode* d5 = add_chain(5, d0, &chain);
+	TestingRoutingNode* d5 = f.add_chain(5, f.d0, &chain);
 
-	new_node_w_neighbour(d0);
+	f.new_node_w_neighbour(f.d0);
 
-	bool rval = r.find_route(*d0, *d5, &route, Widelands::wwWORKER, -1, cc);
+	bool rval = f.r.find_route(*f.d0, *d5, &f.route, Widelands::wwWORKER, -1, f.cc);
 
-	BOOST_CHECK_EQUAL(rval, true);
+	check_equal(rval, true);
 
-	add_dead_end(dynamic_cast<TestingRoutingNode*>(chain[0]));
-	add_dead_end(dynamic_cast<TestingRoutingNode*>(chain[3]));
-	add_dead_end(dynamic_cast<TestingRoutingNode*>(chain[5]));
+	f.add_dead_end(dynamic_cast<TestingRoutingNode*>(chain[0]));
+	f.add_dead_end(dynamic_cast<TestingRoutingNode*>(chain[3]));
+	f.add_dead_end(dynamic_cast<TestingRoutingNode*>(chain[5]));
 
-	BOOST_CHECK(route.has_chain(chain));
+	check_equal(true, f.route.has_chain(chain));
 
 	// directly connect d0 -> d5
-	d0->add_neighbour(d5);
+	f.d0->add_neighbour(d5);
 
-	rval = r.find_route(*d0, *d5, &route, Widelands::wwWORKER, -1, cc);
+	rval = f.r.find_route(*f.d0, *d5, &f.route, Widelands::wwWORKER, -1, f.cc);
 
-	BOOST_CHECK_EQUAL(rval, true);
+	check_equal(rval, true);
 
 	chain.clear();
-	chain.push_back(d0);
+	chain.push_back(f.d0);
 	chain.push_back(d5);
 
-	BOOST_CHECK(route.has_chain(chain));
+	check_equal(true, f.route.has_chain(chain));
 }
 
 /*************************************************************************/
 /*                            Distance routing                           */
 /*************************************************************************/
 struct DistanceRoutingFixture : public ComplexRouterFixture {
-	DistanceRoutingFixture() : ComplexRouterFixture() {
+	DistanceRoutingFixture() {
 		// node is connected through a long and a short path
 		// start d1 end
 		start = d0;
@@ -563,50 +572,52 @@ struct DistanceRoutingFixture : public ComplexRouterFixture {
 	TestingRoutingNode* end;
 	TestingRoutingNode *d1, *d2, *d3, *d4, *d5;
 };
-BOOST_FIXTURE_TEST_CASE(priced_routing, DistanceRoutingFixture) {
-	Nodes chain;
-	chain.push_back(start);
-	chain.push_back(d1);
-	chain.push_back(end);
+TESTCASE(priced_routing) {
+	DistanceRoutingFixture f;
+	DistanceRoutingFixture::Nodes chain;
+	chain.push_back(f.start);
+	chain.push_back(f.d1);
+	chain.push_back(f.end);
 
-	bool rval = r.find_route(*start, *end, &route, Widelands::wwWORKER, -1, cc);
+	bool rval = f.r.find_route(*f.start, *f.end, &f.route, Widelands::wwWORKER, -1, f.cc);
 
-	BOOST_CHECK(rval);
-	BOOST_CHECK(route.has_chain(chain));
+	check_equal(true, rval);
+	check_equal(true, f.route.has_chain(chain));
 
 	// Make the middle node on the short path very expensive
-	d1->set_waitcost(8);
+	f.d1->set_waitcost(8);
 
 	// Same result without wait
-	rval = r.find_route(*start, *end, &route, Widelands::wwWORKER, -1, cc);
-	BOOST_CHECK(rval);
-	BOOST_CHECK(route.has_chain(chain));
+	rval = f.r.find_route(*f.start, *f.end, &f.route, Widelands::wwWORKER, -1, f.cc);
+	check_equal(true, rval);
+	check_equal(true, f.route.has_chain(chain));
 
 	// For wares, we now take the long route
-	rval = r.find_route(*start, *end, &route, Widelands::wwWARE, -1, cc);
+	rval = f.r.find_route(*f.start, *f.end, &f.route, Widelands::wwWARE, -1, f.cc);
 
 	chain.clear();
-	chain.push_back(start);
-	chain.push_back(d2);
-	chain.push_back(d3);
-	chain.push_back(d4);
-	chain.push_back(d5);
-	chain.push_back(end);
+	chain.push_back(f.start);
+	chain.push_back(f.d2);
+	chain.push_back(f.d3);
+	chain.push_back(f.d4);
+	chain.push_back(f.d5);
+	chain.push_back(f.end);
 
-	BOOST_CHECK(rval);
+	check_equal(true, rval);
 
-	BOOST_CHECK(route.has_chain(chain));
+	check_equal(true, f.route.has_chain(chain));
 }
-BOOST_FIXTURE_TEST_CASE(cutoff, DistanceRoutingFixture) {
-	Nodes chain;
+TESTCASE(cutoff) {
+	DistanceRoutingFixture f;
+	DistanceRoutingFixture::Nodes chain;
 
-	Widelands::RoutingNode* end_node = add_chain(4, d0, &chain);
+	Widelands::RoutingNode* end_node = f.add_chain(4, f.d0, &chain);
 
-	bool rval = r.find_route(*d0, *end_node, &route, Widelands::wwWORKER, 1000, cc);
+	bool rval = f.r.find_route(*f.d0, *end_node, &f.route, Widelands::wwWORKER, 1000, f.cc);
 
-	BOOST_CHECK_EQUAL(rval, false);
+	check_equal(rval, false);
 }
 
 // }}}
 
-BOOST_AUTO_TEST_SUITE_END()
+TESTSUITE_END()
