@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -34,6 +33,7 @@
 #include "editor/tools/increase_resources_tool.h"
 #include "editor/tools/set_port_space_tool.h"
 #include "editor/tools/set_terrain_tool.h"
+#include "editor/tools/tool_conf.h"
 #include "editor/ui_menus/help.h"
 #include "editor/ui_menus/main_menu_load_map.h"
 #include "editor/ui_menus/main_menu_map_options.h"
@@ -48,6 +48,7 @@
 #include "editor/ui_menus/tool_place_immovable_options_menu.h"
 #include "editor/ui_menus/tool_resize_options_menu.h"
 #include "editor/ui_menus/tool_set_terrain_options_menu.h"
+#include "editor/ui_menus/tool_toolhistory_options_menu.h"
 #include "editor/ui_menus/toolsize_menu.h"
 #include "graphic/mouse_cursor.h"
 #include "graphic/playercolor.h"
@@ -68,6 +69,7 @@
 #include "sound/sound_handler.h"
 #include "ui_basic/messagebox.h"
 #include "ui_basic/progresswindow.h"
+#include "wlapplication_mousewheel_options.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_base.h"
 #include "wui/toolbar.h"
@@ -91,7 +93,7 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
                as_tooltip_text_with_hotkey(
                   /** TRANSLATORS: Title for the main menu button in the editor */
                   _("Main Menu"),
-                  shortcut_string_for(KeyboardShortcut::kEditorMenu),
+                  shortcut_string_for(KeyboardShortcut::kEditorMenu, true),
                   UI::PanelStyle::kWui),
                UI::DropdownType::kPictorialMenu,
                UI::PanelStyle::kWui,
@@ -106,11 +108,12 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
                as_tooltip_text_with_hotkey(
                   /** TRANSLATORS: Title for the tool menu button in the editor */
                   _("Tools"),
-                  shortcut_string_for(KeyboardShortcut::kEditorTools),
+                  shortcut_string_for(KeyboardShortcut::kEditorTools, true),
                   UI::PanelStyle::kWui),
                UI::DropdownType::kPictorialMenu,
                UI::PanelStyle::kWui,
-               UI::ButtonStyle::kWuiPrimary),
+               UI::ButtonStyle::kWuiPrimary,
+               [this](ToolMenuEntry t) { tool_menu_selected(t); }),
      showhidemenu_(toolbar(),
                    "dropdown_menu_showhide",
                    0,
@@ -123,10 +126,11 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
                    _("Show / Hide"),
                    UI::DropdownType::kPictorialMenu,
                    UI::PanelStyle::kWui,
-                   UI::ButtonStyle::kWuiPrimary),
+                   UI::ButtonStyle::kWuiPrimary,
+                   [this](ShowHideEntry t) { showhide_menu_selected(t); }),
      undo_(nullptr),
      redo_(nullptr),
-     tools_(new Tools(e.map())),
+     tools_(new Tools(*this, e.map())),
      history_(nullptr)  // history needs the undo/redo buttons
 {
 	add_main_menu();
@@ -145,33 +149,38 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
 
 	toolbar()->add_space(15);
 
-	undo_ = add_toolbar_button(
-	   "wui/editor/menus/undo", "undo",
-	   as_tooltip_text_with_hotkey(
-	      _("Undo"), shortcut_string_for(KeyboardShortcut::kEditorUndo), UI::PanelStyle::kWui));
-	redo_ = add_toolbar_button(
-	   "wui/editor/menus/redo", "redo",
-	   as_tooltip_text_with_hotkey(
-	      _("Redo"), shortcut_string_for(KeyboardShortcut::kEditorRedo), UI::PanelStyle::kWui));
+	undo_ =
+	   add_toolbar_button("wui/editor/menus/undo", "undo",
+	                      as_tooltip_text_with_hotkey(
+	                         _("Undo"), shortcut_string_for(KeyboardShortcut::kEditorUndo, true),
+	                         UI::PanelStyle::kWui));
+	redo_ =
+	   add_toolbar_button("wui/editor/menus/redo", "redo",
+	                      as_tooltip_text_with_hotkey(
+	                         _("Redo"), shortcut_string_for(KeyboardShortcut::kEditorRedo, true),
+	                         UI::PanelStyle::kWui));
 
-	history_.reset(new EditorHistory(*undo_, *redo_));
+	history_.reset(new EditorHistory(*this, *undo_, *redo_));
 
 	undo_->sigclicked.connect([this] { history_->undo_action(); });
 	redo_->sigclicked.connect([this] { history_->redo_action(); });
 
 	toolbar()->add_space(15);
 
-	add_toolbar_button("ui_basic/menu_help", "help",
-	                   as_tooltip_text_with_hotkey(
-	                      _("Help"), shortcut_string_for(KeyboardShortcut::kCommonEncyclopedia),
-	                      UI::PanelStyle::kWui),
-	                   &menu_windows_.help, true);
+	add_toolbar_button(
+	   "ui_basic/menu_help", "help",
+	   as_tooltip_text_with_hotkey(_("Help"),
+	                               shortcut_string_for(KeyboardShortcut::kCommonEncyclopedia, true),
+	                               UI::PanelStyle::kWui),
+	   &menu_windows_.help, true);
 	menu_windows_.help.open_window = [this] {
 		new EditorHelp(*this, menu_windows_.help, &egbase().lua());
 	};
 
 	finalize_toolbar();
 
+	set_display_flags(EditorInteractive::dfShowResources | EditorInteractive::dfShowImmovables |
+	                  EditorInteractive::dfShowBobs | EditorInteractive::dfShowGrid);
 #ifndef NDEBUG
 	set_display_flag(InteractiveBase::dfDebug, true);
 #else
@@ -181,6 +190,8 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
 	map_view()->field_clicked.connect([this](const Widelands::NodeAndTriangle<>& node_and_triangle) {
 		map_clicked(node_and_triangle, false);
 	});
+
+	initialization_complete();
 }
 
 void EditorInteractive::add_main_menu() {
@@ -205,7 +216,7 @@ void EditorInteractive::add_main_menu() {
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("Load Map"), MainMenuEntry::kLoadMap,
 	              g_image_cache->get("images/wui/editor/menus/load_map.png"), false, "",
-	              shortcut_string_for(KeyboardShortcut::kEditorLoad));
+	              shortcut_string_for(KeyboardShortcut::kEditorLoad, false));
 
 	menu_windows_.savemap.open_window = [this] {
 		new MainMenuSaveMap(*this, menu_windows_.savemap, menu_windows_.mapoptions);
@@ -213,7 +224,7 @@ void EditorInteractive::add_main_menu() {
 	/** TRANSLATORS: An entry in the editor's main menu */
 	mainmenu_.add(_("Save Map"), MainMenuEntry::kSaveMap,
 	              g_image_cache->get("images/wui/editor/menus/save_map.png"), false, "",
-	              shortcut_string_for(KeyboardShortcut::kEditorSave));
+	              shortcut_string_for(KeyboardShortcut::kEditorSave, false));
 
 	menu_windows_.mapoptions.open_window = [this] {
 		new MainMenuMapOptions(*this, menu_windows_.mapoptions);
@@ -320,12 +331,13 @@ void EditorInteractive::add_tool_menu() {
 	tool_windows_.players.open_window = [this] {
 		new EditorPlayerMenu(*this, tools()->set_starting_pos, tool_windows_.players);
 	};
+
 	/** TRANSLATORS: An entry in the editor's tool menu */
 	toolmenu_.add(_("Players"), ToolMenuEntry::kPlayers,
 	              g_image_cache->get("images/wui/editor/tools/players.png"), false,
 	              /** TRANSLATORS: Tooltip for the map size tool in the editor */
 	              _("Set number of players and their names, tribes and starting positions"),
-	              shortcut_string_for(KeyboardShortcut::kEditorPlayers));
+	              shortcut_string_for(KeyboardShortcut::kEditorPlayers, false));
 
 	/** TRANSLATORS: An entry in the editor's tool menu */
 	toolmenu_.add(_("Map origin"), ToolMenuEntry::kMapOrigin,
@@ -348,7 +360,18 @@ void EditorInteractive::add_tool_menu() {
 	              g_image_cache->get("images/wui/editor/fsel_editor_info.png"), false,
 	              /** TRANSLATORS: Tooltip for the map information tool in the editor */
 	              _("Click on a field to show information about it"),
-	              shortcut_string_for(KeyboardShortcut::kEditorInfo));
+	              shortcut_string_for(KeyboardShortcut::kEditorInfo, false));
+
+	/** TRANSLATORS: An entry in the editor's tool menu */
+	toolmenu_.add(_("Tool History"), ToolMenuEntry::kToolHistory,
+	              g_image_cache->get("images/wui/editor/fsel_editor_toolhistory.png"), false,
+	              /** TRANSLATORS: Tooltip for the tool history tool in the editor */
+	              _("Restore previous tool settings"),
+	              shortcut_string_for(KeyboardShortcut::kEditorToolHistory, false));
+	tool_windows_.toolhistory.open_window = [this] {
+		new EditorToolhistoryOptionsMenu(*this, tools()->tool_history, tool_windows_.toolhistory);
+	};
+
 	toolmenu_.selected.connect([this] { tool_menu_selected(toolmenu_.get_selected()); });
 	toolbar()->add(&toolmenu_);
 }
@@ -388,6 +411,9 @@ void EditorInteractive::tool_menu_selected(ToolMenuEntry entry) {
 	case ToolMenuEntry::kFieldInfo:
 		select_tool(tools()->info, EditorTool::First);
 		break;
+	case ToolMenuEntry::kToolHistory:
+		tool_windows_.toolhistory.toggle();
+		break;
 	}
 	toolmenu_.toggle();
 }
@@ -412,31 +438,43 @@ void EditorInteractive::rebuild_showhide_menu() {
 	showhidemenu_.add(buildhelp() ? _("Hide Building Spaces") : _("Show Building Spaces"),
 	                  ShowHideEntry::kBuildingSpaces,
 	                  g_image_cache->get("images/wui/menus/toggle_buildhelp.png"), false, "",
-	                  shortcut_string_for(KeyboardShortcut::kCommonBuildhelp));
+	                  shortcut_string_for(KeyboardShortcut::kCommonBuildhelp, false));
+
+	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether to show maximum
+	 * building spaces that will be available if all immovables (trees, rocks, etc.) are removed */
+	showhidemenu_.add(get_display_flag(dfShowMaximumBuildhelp) ? _("Hide Maximum Building Spaces") :
+                                                                _("Show Maximum Building Spaces"),
+	                  ShowHideEntry::kMaximumBuildingSpaces,
+	                  g_image_cache->get("images/wui/menus/toggle_maxbuild.png"), false,
+	                  _("Toggle whether to show maximum building spaces that will be available if "
+	                    "all immovables (trees, rocks, etc.) are removed"),
+	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideMaximumBuildhelp, false));
 
 	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether the map grid is shown
 	 */
-	showhidemenu_.add(draw_grid_ ? _("Hide Grid") : _("Show Grid"), ShowHideEntry::kGrid,
+	showhidemenu_.add(get_display_flag(dfShowGrid) ? _("Hide Grid") : _("Show Grid"),
+	                  ShowHideEntry::kGrid,
 	                  g_image_cache->get("images/wui/menus/menu_toggle_grid.png"), false, "",
-	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideGrid));
+	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideGrid, false));
 
-	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether immovables (trees,
-	 * rocks etc.) are shown */
-	showhidemenu_.add(draw_immovables_ ? _("Hide Immovables") : _("Show Immovables"),
-	                  ShowHideEntry::kImmovables,
-	                  g_image_cache->get("images/wui/menus/toggle_immovables.png"), false, "",
-	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideImmovables));
+	showhidemenu_.add(
+	   /** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether immovables
+	    *  (trees, rocks etc.) are shown */
+	   get_display_flag(dfShowImmovables) ? _("Hide Immovables") : _("Show Immovables"),
+	   ShowHideEntry::kImmovables, g_image_cache->get("images/wui/menus/toggle_immovables.png"),
+	   false, "", shortcut_string_for(KeyboardShortcut::kEditorShowhideImmovables, false));
 
 	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether animals are shown */
-	showhidemenu_.add(draw_bobs_ ? _("Hide Animals") : _("Show Animals"), ShowHideEntry::kAnimals,
+	showhidemenu_.add(get_display_flag(dfShowBobs) ? _("Hide Animals") : _("Show Animals"),
+	                  ShowHideEntry::kAnimals,
 	                  g_image_cache->get("images/wui/menus/toggle_bobs.png"), false, "",
-	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideCritters));
+	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideCritters, false));
 
 	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether resources are shown */
-	showhidemenu_.add(draw_resources_ ? _("Hide Resources") : _("Show Resources"),
+	showhidemenu_.add(get_display_flag(dfShowResources) ? _("Hide Resources") : _("Show Resources"),
 	                  ShowHideEntry::kResources,
 	                  g_image_cache->get("images/wui/menus/toggle_resources.png"), false, "",
-	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideResources));
+	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideResources, false));
 
 	showhidemenu_.select(last_selection);
 }
@@ -446,6 +484,9 @@ void EditorInteractive::showhide_menu_selected(ShowHideEntry entry) {
 	case ShowHideEntry::kBuildingSpaces: {
 		toggle_buildhelp();
 	} break;
+	case ShowHideEntry::kMaximumBuildingSpaces: {
+		toggle_maximum_buildhelp();
+	} break;
 	case ShowHideEntry::kGrid: {
 		toggle_grid();
 	} break;
@@ -471,7 +512,7 @@ void EditorInteractive::load(const std::string& filename) {
 	cleanup_for_load();
 
 	std::unique_ptr<Widelands::MapLoader> ml(map->get_correct_loader(filename));
-	if (!ml.get()) {
+	if (!ml) {
 		throw WLWarning(
 		   _("Unsupported Format"),
 		   _("Widelands could not load the file \"%s\". The file format seems to be incompatible."),
@@ -496,6 +537,11 @@ void EditorInteractive::load(const std::string& filename) {
 }
 
 void EditorInteractive::cleanup_for_load() {
+	if (cleaning_up_) {
+		return;
+	}
+	cleaning_up_ = true;
+
 	// TODO(unknown): get rid of cleanup_for_load, it tends to be very messy
 	// Instead, delete and re-create the egbase.
 	// TODO(Nordfriese): …and then we can get rid of delete_world_and_tribes() as well
@@ -503,19 +549,21 @@ void EditorInteractive::cleanup_for_load() {
 	// This is needed so add-ons are configured correctly if current
 	// and previous map had different world add-on settings
 	egbase().delete_world_and_tribes();
+
+	cleaning_up_ = false;
 }
 
 /// Called just before the editor starts, after postload, init and gfxload.
 void EditorInteractive::start() {
 	// Run the editor initialization script, if any
 	try {
-		g_sh->change_music("ingame", 1000);
+		g_sh->change_music(Songset::kIngame, 1000);
 		egbase().lua().run_script("map:scripting/editor_init.lua");
 	} catch (LuaScriptNotExistingError&) {
 		// do nothing.
 	}
 	map_changed(MapWas::kReplaced);
-	if (registry_to_open_) {
+	if (registry_to_open_ != nullptr) {
 		registry_to_open_->create();
 		registry_to_open_ = nullptr;
 	}
@@ -538,7 +586,7 @@ void EditorInteractive::think() {
 
 void EditorInteractive::exit() {
 	if (need_save_) {
-		if (SDL_GetModState() & KMOD_CTRL) {
+		if ((SDL_GetModState() & KMOD_CTRL) != 0) {
 			end_modal<UI::Panel::Returncodes>(UI::Panel::Returncodes::kBack);
 		} else {
 			UI::WLMessageBox mmb(this, UI::WindowStyle::kWui, _("Unsaved Map"),
@@ -549,15 +597,45 @@ void EditorInteractive::exit() {
 			}
 		}
 	}
-	g_sh->change_music("menu", 200);
+	g_sh->change_music(Songset::kMenu, 200);
 	end_modal<UI::Panel::Returncodes>(UI::Panel::Returncodes::kBack);
 }
 
 void EditorInteractive::map_clicked(const Widelands::NodeAndTriangle<>& node_and_triangle,
                                     const bool should_draw) {
-	history_->do_action(tools_->current(), tools_->use_tool, *egbase().mutable_map(),
-	                    node_and_triangle, *this, should_draw);
+
+	EditorTool& current_tool = tools_->current();
+	EditorTool::ToolIndex subtool_idx = tools_->use_tool;
+
+	history_->do_action(
+	   current_tool, subtool_idx, *egbase().mutable_map(), node_and_triangle, should_draw);
+
 	set_need_save(true);
+
+	if (!tool_settings_changed_) {
+		return;
+	}
+
+	ToolConf conf;
+	if (current_tool.save_configuration(conf)) {
+		if (tools()->tool_history.add_configuration(conf)) {
+			update_tool_history_window();
+		}
+	}
+
+	tool_settings_changed_ = false;
+}
+
+void EditorInteractive::update_tool_history_window() {
+	UI::UniqueWindow* window = get_open_tool_window(WindowID::ToolHistory);
+	if (window == nullptr) {
+		return;
+	}
+
+	EditorToolhistoryOptionsMenu* toolhistory_window =
+	   dynamic_cast<EditorToolhistoryOptionsMenu*>(window);
+
+	toolhistory_window->update();
 }
 
 bool EditorInteractive::handle_mouserelease(uint8_t btn, int32_t x, int32_t y) {
@@ -576,7 +654,8 @@ bool EditorInteractive::handle_mousepress(uint8_t btn, int32_t x, int32_t y) {
 
 void EditorInteractive::draw(RenderTarget& dst) {
 	const auto& ebase = egbase();
-	auto* fields_to_draw = map_view()->draw_terrain(ebase, nullptr, Workareas(), draw_grid_, &dst);
+	auto* fields_to_draw =
+	   map_view()->draw_terrain(ebase, nullptr, Workareas(), get_display_flag(dfShowGrid), &dst);
 
 	const float scale = 1.f / map_view()->view().zoom;
 	const Time& gametime = ebase.get_gametime();
@@ -612,7 +691,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 
 	for (size_t idx = 0; idx < fields_to_draw->size(); ++idx) {
 		const FieldsToDraw::Field& field = fields_to_draw->at(idx);
-		if (draw_immovables_) {
+		if (get_display_flag(dfShowImmovables)) {
 			Widelands::BaseImmovable* const imm = field.fcoords.field->get_immovable();
 			if (imm != nullptr && imm->get_positions(ebase).front() == field.fcoords) {
 				imm->draw(
@@ -620,8 +699,8 @@ void EditorInteractive::draw(RenderTarget& dst) {
 			}
 		}
 
-		if (draw_bobs_) {
-			for (Widelands::Bob* bob = field.fcoords.field->get_first_bob(); bob;
+		if (get_display_flag(dfShowBobs)) {
+			for (Widelands::Bob* bob = field.fcoords.field->get_first_bob(); bob != nullptr;
 			     bob = bob->get_next_bob()) {
 				bob->draw(
 				   ebase, InfoToDraw::kNone, field.rendertarget_pixel, field.fcoords, scale, &dst);
@@ -630,7 +709,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 
 		// Draw resource overlay.
 		uint8_t const amount = field.fcoords.field->get_resources_amount();
-		if (draw_resources_ && amount > 0) {
+		if (get_display_flag(dfShowResources) && amount > 0) {
 			const std::string& immname = ebase.descriptions()
 			                                .get_resource_descr(field.fcoords.field->get_resources())
 			                                ->editor_image(amount);
@@ -641,12 +720,38 @@ void EditorInteractive::draw(RenderTarget& dst) {
 			}
 		}
 
-		// Draw build help.
-		if (buildhelp()) {
-			const auto* overlay =
-			   get_buildhelp_overlay(tools_->current().nodecaps_for_buildhelp(field.fcoords, ebase));
+		const Widelands::NodeCaps nodecaps =
+		   tools_->current().nodecaps_for_buildhelp(field.fcoords, ebase);
+		const Widelands::NodeCaps maxcaps =
+		   tools_->current().maxcaps_for_buildhelp(field.fcoords, ebase);
+		// Draw build help for maximum building spaces.
+		bool buildhelp_drawn = false;
+		if (get_display_flag(dfShowMaximumBuildhelp)) {
+			const auto* overlay = get_buildhelp_overlay(maxcaps);
 			if (overlay != nullptr) {
-				blit_field_overlay(&dst, field, overlay->pic, overlay->hotspot, scale);
+				float opacity = 1.0f;
+				if (nodecaps != maxcaps) {
+					opacity = 0.6f;
+				} else {
+					buildhelp_drawn = true;
+				}
+				blit_field_overlay(&dst, field, overlay->pic, overlay->hotspot, scale, opacity);
+			}
+		}
+
+		// Draw build help for actual building spaces.
+		if (buildhelp() && !buildhelp_drawn) {
+			const auto* overlay = get_buildhelp_overlay(nodecaps);
+			if (overlay != nullptr) {
+				// If maximum cap buildhelp is also on and is different,
+				// and current space is medium (ie. maximum is big or port),
+				// then scale down a bit to make the maximum more visible.
+				const float scaling =
+				   get_display_flag(dfShowMaximumBuildhelp) &&
+				         ((nodecaps & Widelands::BUILDCAPS_SIZEMASK) == Widelands::BUILDCAPS_MEDIUM) ?
+                  0.9f :
+                  1.0f;
+				blit_field_overlay(&dst, field, overlay->pic, overlay->hotspot, scale * scaling);
 			}
 		}
 
@@ -675,7 +780,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 				const FieldsToDraw::Field& brn = fields_to_draw->at(field.brn_index);
 				const FieldsToDraw::Field& bln = fields_to_draw->at(field.bln_index);
 				if (selected_triangles.count(
-				       Widelands::TCoords<>(field.fcoords, Widelands::TriangleIndex::R))) {
+				       Widelands::TCoords<>(field.fcoords, Widelands::TriangleIndex::R)) != 0u) {
 					const Vector2i tripos((field.rendertarget_pixel.x + rn.rendertarget_pixel.x +
 					                       brn.rendertarget_pixel.x) /
 					                         3,
@@ -687,7 +792,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 					   &dst, tripos, pic, Vector2i(pic->width() / 2, pic->height() / 2), scale, 1.f);
 				}
 				if (selected_triangles.count(
-				       Widelands::TCoords<>(field.fcoords, Widelands::TriangleIndex::D))) {
+				       Widelands::TCoords<>(field.fcoords, Widelands::TriangleIndex::D)) != 0u) {
 					const Vector2i tripos((field.rendertarget_pixel.x + bln.rendertarget_pixel.x +
 					                       brn.rendertarget_pixel.x) /
 					                         3,
@@ -706,8 +811,8 @@ void EditorInteractive::draw(RenderTarget& dst) {
 /// Needed to get freehand painting tools (hold down mouse and move to edit).
 void EditorInteractive::set_sel_pos(Widelands::NodeAndTriangle<> const sel) {
 	bool const target_changed = tools_->current().operates_on_triangles() ?
-	                               sel.triangle != get_sel_pos().triangle :
-	                               sel.node != get_sel_pos().node;
+                                  sel.triangle != get_sel_pos().triangle :
+                                  sel.node != get_sel_pos().node;
 	InteractiveBase::set_sel_pos(sel);
 	if (target_changed && is_painting_) {
 		map_clicked(sel, true);
@@ -730,28 +835,33 @@ void EditorInteractive::stop_painting() {
 	is_painting_ = false;
 }
 
-bool EditorInteractive::player_hears_field(const Widelands::Coords&) const {
+bool EditorInteractive::player_hears_field(const Widelands::Coords& /*coords*/) const {
 	return true;
 }
 
 void EditorInteractive::toggle_resources() {
-	draw_resources_ = !draw_resources_;
-	rebuild_showhide_menu();
+	set_display_flag(
+	   EditorInteractive::dfShowResources, !get_display_flag(EditorInteractive::dfShowResources));
 }
 
 void EditorInteractive::toggle_immovables() {
-	draw_immovables_ = !draw_immovables_;
-	rebuild_showhide_menu();
+	set_display_flag(
+	   EditorInteractive::dfShowImmovables, !get_display_flag(EditorInteractive::dfShowImmovables));
 }
 
 void EditorInteractive::toggle_bobs() {
-	draw_bobs_ = !draw_bobs_;
-	rebuild_showhide_menu();
+	set_display_flag(
+	   EditorInteractive::dfShowBobs, !get_display_flag(EditorInteractive::dfShowBobs));
 }
 
 void EditorInteractive::toggle_grid() {
-	draw_grid_ = !draw_grid_;
-	rebuild_showhide_menu();
+	set_display_flag(
+	   EditorInteractive::dfShowGrid, !get_display_flag(EditorInteractive::dfShowGrid));
+}
+
+void EditorInteractive::toggle_maximum_buildhelp() {
+	set_display_flag(EditorInteractive::dfShowMaximumBuildhelp,
+	                 !get_display_flag(EditorInteractive::dfShowMaximumBuildhelp));
 }
 
 bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
@@ -800,6 +910,10 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 			toggle_resources();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorShowhideMaximumBuildhelp, code)) {
+			toggle_maximum_buildhelp();
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kEditorUndo, code)) {
 			history_->undo_action();
 			return true;
@@ -808,6 +922,10 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 			history_->redo_action();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorToolHistory, code)) {
+			tool_windows_.toolhistory.toggle();
+			return true;
+		}
 
 		for (int i = 0; i < 10; ++i) {
 			if (matches_shortcut(static_cast<KeyboardShortcut>(
@@ -866,6 +984,17 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 	return InteractiveBase::handle_key(down, code);
 }
 
+bool EditorInteractive::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
+	int32_t change =
+	   get_mousewheel_change(MousewheelHandlerConfigID::kEditorToolsize, x, y, modstate);
+	if (change == 0) {
+		return false;
+	}
+	set_sel_radius_and_update_menu(
+	   std::max(0, std::min(static_cast<int32_t>(get_sel_radius()) + change, MAX_TOOL_AREA)));
+	return true;
+}
+
 void EditorInteractive::select_tool(EditorTool& primary, EditorTool::ToolIndex const which) {
 	if (which == EditorTool::First && &primary != tools_->current_pointer) {
 		if (primary.has_size_one()) {
@@ -891,6 +1020,7 @@ void EditorInteractive::select_tool(EditorTool& primary, EditorTool::ToolIndex c
 		unset_sel_picture();
 	}
 	set_sel_triangles(primary.operates_on_triangles());
+	tool_settings_changed_ = true;
 }
 
 void EditorInteractive::run_editor(UI::Panel* error_message_parent,
@@ -904,7 +1034,7 @@ void EditorInteractive::run_editor(UI::Panel* error_message_parent,
 		        "  FATAL EXCEPTION in editor: %s\n"
 		        "##############################\n",
 		        e.what());
-		if (!error_message_parent) {
+		if (error_message_parent == nullptr) {
 			return;
 		}
 		// Note: We don't necessarily want a bug report here, but the wording must
@@ -912,13 +1042,12 @@ void EditorInteractive::run_editor(UI::Panel* error_message_parent,
 		// during winter time freeze. We can consider rephrasing it after v1.0.
 		UI::WLMessageBox m(
 		   error_message_parent, UI::WindowStyle::kFsMenu, _("Error"),
-		   (boost::format(
-		       _("An error has occured. The error message is:\n\n%1$s\n\nPlease report "
-		         "this problem to help us improve Widelands. You will find related messages in the "
-		         "standard output (stdout.txt on Windows). You are using build %2$s "
-		         "(%3$s).\nPlease add this information to your report.")) %
-		    e.what() % build_id() % build_type())
-		      .str(),
+		   format(
+		      _("An error has occured. The error message is:\n\n%1$s\n\nPlease report "
+		        "this problem to help us improve Widelands. You will find related messages in the "
+		        "standard output (stdout.txt on Windows). You are using version %2$s.\n"
+		        "Please add this information to your report."),
+		      e.what(), build_ver_details()),
 		   UI::WLMessageBox::MBoxType::kOk);
 		m.run<UI::Panel::Returncodes>();
 	}
@@ -933,7 +1062,7 @@ void EditorInteractive::do_run_editor(const EditorInteractive::Init init,
 
 	// We need to disable non-world add-ons in the editor
 	for (auto it = egbase.enabled_addons().begin(); it != egbase.enabled_addons().end();) {
-		if (it->category != AddOns::AddOnCategory::kWorld) {
+		if ((*it)->category != AddOns::AddOnCategory::kWorld) {
 			it = egbase.enabled_addons().erase(it);
 		} else {
 			++it;
@@ -948,11 +1077,9 @@ void EditorInteractive::do_run_editor(const EditorInteractive::Init init,
 			throw wexception("EditorInteractive::run_editor: Empty map file name");
 		}
 
-		Notifications::publish(
-		   UI::NoteLoadingMessage((boost::format(_("Loading map “%s”…")) % filename).str()));
+		Notifications::publish(UI::NoteLoadingMessage(format(_("Loading map “%s”…"), filename)));
 		eia.load(filename);
 
-		egbase.postload_addons();
 		egbase.postload();
 		eia.start();
 		if (!script_to_run.empty()) {
@@ -967,7 +1094,6 @@ void EditorInteractive::do_run_editor(const EditorInteractive::Init init,
 			throw wexception("EditorInteractive::run_editor: Script given when none was expected");
 		}
 
-		egbase.postload_addons();
 		egbase.postload();
 
 		egbase.mutable_map()->create_empty_map(
@@ -1006,7 +1132,7 @@ void EditorInteractive::load_world_units(EditorInteractive* eia,
 	verb_log_info("┏━ Loading world\n");
 	ScopedTimer timer("┗━ took %ums", true);
 
-	if (eia) {
+	if (eia != nullptr) {
 		// In order to ensure that items created by add-ons are properly
 		// removed from the editor's object selection menus, we clear
 		// and repopulate these menus every time the world is reloaded.
@@ -1023,7 +1149,7 @@ void EditorInteractive::load_world_units(EditorInteractive* eia,
 			// Category because it will load all the map objects we are interested in
 			std::unique_ptr<EditorCategory> c(
 			   new EditorCategory(*category_table, type, *descriptions));
-			if (eia) {
+			if (eia != nullptr) {
 				eia->editor_categories_[type].push_back(std::move(c));
 			}
 		}
@@ -1047,10 +1173,10 @@ void EditorInteractive::load_world_units(EditorInteractive* eia,
 	verb_log_info("┃    Resources");
 	load_resources(*table);
 
-	for (const AddOns::AddOnInfo& info : egbase.enabled_addons()) {
-		if (info.category == AddOns::AddOnCategory::kWorld) {
-			verb_log_info("┃    Add-On ‘%s’", info.internal_name.c_str());
-			table = egbase.lua().run_script(kAddOnDir + '/' + info.internal_name + "/editor.lua");
+	for (const auto& info : egbase.enabled_addons()) {
+		if (info->category == AddOns::AddOnCategory::kWorld) {
+			verb_log_info("┃    Add-On ‘%s’", info->internal_name.c_str());
+			table = egbase.lua().run_script(kAddOnDir + '/' + info->internal_name + "/editor.lua");
 			load_category(*table, "critters", Widelands::MapObjectType::CRITTER);
 			load_category(*table, "immovables", Widelands::MapObjectType::IMMOVABLE);
 			load_category(*table, "terrains", Widelands::MapObjectType::TERRAIN);
@@ -1062,11 +1188,11 @@ void EditorInteractive::load_world_units(EditorInteractive* eia,
 void EditorInteractive::map_changed(const MapWas& action) {
 	switch (action) {
 	case MapWas::kReplaced:
-		history_.reset(new EditorHistory(*undo_, *redo_));
+		history_.reset(new EditorHistory(*this, *undo_, *redo_));
 		undo_->set_enabled(false);
 		redo_->set_enabled(false);
 
-		tools_.reset(new Tools(egbase().map()));
+		tools_.reset(new Tools(*this, egbase().map()));
 		select_tool(tools_->info, EditorTool::First);
 		set_sel_radius(0);
 
@@ -1074,7 +1200,7 @@ void EditorInteractive::map_changed(const MapWas& action) {
 		show_buildhelp(true);
 
 		// Close all windows.
-		for (Panel* child = get_first_child(); child; child = child->get_next_sibling()) {
+		for (Panel* child = get_first_child(); child != nullptr; child = child->get_next_sibling()) {
 			if (dynamic_cast<UI::Window*>(child) != nullptr) {
 				child->die();
 			}
@@ -1106,3 +1232,76 @@ EditorInteractive::editor_categories(Widelands::MapObjectType type) const {
 	assert(editor_categories_.count(type) == 1);
 	return editor_categories_.at(type);
 }
+
+EditorHistory& EditorInteractive::history() {
+	return *history_;
+}
+
+/**
+ * Restores tool settings in a tool's window from a configuration.
+ * Opens the window, if it's not already open.
+ **/
+void EditorInteractive::restore_tool_configuration(const ToolConf& conf) {
+	assert(conf.primary != nullptr);
+
+	EditorTool& primary = *conf.primary;
+	primary.load_configuration(conf);
+	select_tool(primary, EditorTool::First);
+
+	UI::UniqueWindow* window = get_open_tool_window(primary.get_window_id());
+	if (window == nullptr) {
+		UI::UniqueWindow::Registry& registry = get_registry_for_window(primary.get_window_id());
+		registry.create();
+
+		window = get_open_tool_window(primary.get_window_id());
+	}
+
+	if (window != nullptr) {
+		dynamic_cast<EditorToolOptionsMenu*>(window)->update_window();
+		window->focus();
+	}
+}
+
+/**
+ * Returns open window for the given window id, if the window is open.
+ * Otherwise returns nullptr.
+ **/
+UI::UniqueWindow* EditorInteractive::get_open_tool_window(WindowID window_id) {
+	const UI::UniqueWindow::Registry& window_registry = get_registry_for_window(window_id);
+
+	return window_registry.window;
+}
+
+/**
+ * Returns window registry for window id.
+ **/
+UI::UniqueWindow::Registry& EditorInteractive::get_registry_for_window(WindowID window_id) {
+
+	switch (window_id) {
+	case WindowID::ToolHistory:
+		return tool_windows_.toolhistory;
+	case WindowID::ChangeHeight:
+		return tool_windows_.height;
+	case WindowID::NoiseHeight:
+		return tool_windows_.noiseheight;
+	case WindowID::Terrain:
+		return tool_windows_.terrain;
+	case WindowID::Immovables:
+		return tool_windows_.immovables;
+	case WindowID::Critters:
+		return tool_windows_.critters;
+	case WindowID::ChangeResources:
+		return tool_windows_.resources;
+	case WindowID::Resize:
+		return tool_windows_.resizemap;
+	case WindowID::Unset:
+		break;
+	}
+
+	NEVER_HERE();
+}
+
+void EditorInteractive::set_sel_radius(const uint32_t n) {
+	InteractiveBase::set_sel_radius(n);
+	tool_settings_changed_ = true;
+}
