@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,14 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "editor/ui_menus/main_menu_map_options.h"
 
 #include "base/i18n.h"
+#include "base/string.h"
 #include "editor/editorinteractive.h"
 #include "graphic/font_handler.h"
 #include "graphic/text_layout.h"
@@ -141,7 +141,7 @@ UI::Dropdown<Widelands::PlayerNumber>* SuggestedTeamsEntry::create_dropdown(size
 		if (dd_index >= team_.size()) {
 			assert(dd_index == team_.size());
 			team_.push_back(Widelands::SuggestedTeam());
-			buttons_.push_back({});
+			buttons_.emplace_back();
 		}
 		team_[dd_index].push_back(player);
 		buttons_[dd_index].push_back(create_button(player));
@@ -208,6 +208,8 @@ void SuggestedTeamsEntry::update() {
 	}
 
 	layout();
+
+	initialization_complete();
 }
 
 constexpr uint16_t kMaxRecommendedWaterwayLengthLimit = 20;
@@ -424,7 +426,7 @@ MainMenuMapOptions::MainMenuMapOptions(EditorInteractive& parent, Registry& regi
 	const unsigned nr_players = eia().egbase().map().get_nrplayers();
 	teams_box_.add(new UI::Textarea(
 	   &teams_box_, UI::PanelStyle::kWui, UI::FontStyle::kWuiLabel, 0, 0, max_w_, labelh_,
-	   (boost::format(ngettext("%u Player", "%u Players", nr_players)) % nr_players).str()));
+	   format(ngettext("%u Player", "%u Players", nr_players), nr_players)));
 	teams_box_.add_space(padding_);
 	teams_box_.add(new UI::Textarea(&teams_box_, UI::PanelStyle::kWui, UI::FontStyle::kWuiLabel, 0,
 	                                0, max_w_, labelh_, _("Suggested Teams:")));
@@ -477,6 +479,8 @@ MainMenuMapOptions::MainMenuMapOptions(EditorInteractive& parent, Registry& regi
 	name_.focus();
 	center_to_parent();
 	move_to_top();
+
+	initialization_complete();
 }
 
 void MainMenuMapOptions::update_waterway_length_warning() {
@@ -484,9 +488,8 @@ void MainMenuMapOptions::update_waterway_length_warning() {
 	if (len > kMaxRecommendedWaterwayLengthLimit) {
 		waterway_length_warning_->set_icon(g_image_cache->get("images/ui_basic/stop.png"));
 		waterway_length_warning_->set_tooltip(
-		   (boost::format(_("It is not recommended to permit waterway lengths greater than %u")) %
-		    kMaxRecommendedWaterwayLengthLimit)
-		      .str());
+		   format(_("It is not recommended to permit waterway lengths greater than %u"),
+		          kMaxRecommendedWaterwayLengthLimit));
 	} else {
 		waterway_length_warning_->set_icon(nullptr);
 		waterway_length_warning_->set_tooltip("");
@@ -501,7 +504,7 @@ void MainMenuMapOptions::update() {
 	const Widelands::Map& map = eia().egbase().map();
 	author_.set_text(map.get_author());
 	name_.set_text(map.get_name());
-	size_.set_text((boost::format(_("Size: %1% x %2%")) % map.get_width() % map.get_height()).str());
+	size_.set_text(format(_("Size: %1% x %2%"), map.get_width(), map.get_height()));
 	descr_->set_text(map.get_description());
 	hint_->set_text(map.get_hint());
 	waterway_length_box_->set_value(map.get_waterway_max_length());
@@ -512,7 +515,7 @@ void MainMenuMapOptions::update() {
 		tag.second->set_state(tags.count(tag.first) > 0);
 	}
 
-	balancing_dropdown_.select(tags.count("balanced") ? "balanced" : "unbalanced");
+	balancing_dropdown_.select(tags.count("balanced") != 0u ? "balanced" : "unbalanced");
 
 	theme_dropdown_.select(map.get_background_theme());
 	if (!theme_dropdown_.has_selection()) {
@@ -559,6 +562,14 @@ void MainMenuMapOptions::clicked_cancel() {
 	registry_.destroy();
 }
 
+bool MainMenuMapOptions::handle_key(bool down, SDL_Keysym code) {
+	if (down && code.sym == SDLK_RETURN) {
+		clicked_ok();
+		return true;
+	}
+	return UI::UniqueWindow::handle_key(down, code);
+}
+
 /*
  * Add a tag to the checkboxes
  */
