@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -65,7 +64,7 @@ MainMenuNewRandomMapPanel::MainMenuNewRandomMapPanel(
               UI::SpinBox::Units::kNone,
               UI::SpinBox::Type::kSmall),
      // World + Resources
-     current_world_(std::rand() % Widelands::Map::kOldWorldNames.size()),  // NOLINT
+     current_world_(RNG::static_rand(Widelands::Map::kOldWorldNames.size())),
      resource_amounts_({
         /** TRANSLATORS: Amount of resources in the random map generator in the editor */
         _("Low"),
@@ -165,7 +164,7 @@ MainMenuNewRandomMapPanel::MainMenuNewRandomMapPanel(
                 0,
                 inner_w / 3,
                 mountains_label_.get_h(),
-                (boost::format(_("%i %%")) % mountainsval_).str(),
+                format(_("%i %%"), mountainsval_),
                 UI::Align::kCenter),
      island_mode_(this, panel_style_, Vector2i::zero(), _("Island mode")),
      // Geeky stuff
@@ -215,7 +214,7 @@ MainMenuNewRandomMapPanel::MainMenuNewRandomMapPanel(
 	world_.selected.connect([this]() {
 		current_world_ = world_.get_selected();
 		if (current_world_ == static_cast<int>(Widelands::Map::kOldWorldNames.size())) {
-			current_world_ = std::rand() % Widelands::Map::kOldWorldNames.size();  // NOLINT
+			current_world_ = RNG::static_rand(Widelands::Map::kOldWorldNames.size());
 		}
 		nr_edit_box_changed();
 	});
@@ -415,7 +414,7 @@ void MainMenuNewRandomMapPanel::normalize_landmass(ButtonId clicked_button) {
 	water_.set_value(waterval_);
 	land_.set_value(landval_);
 	wasteland_.set_value(wastelandval_);
-	mountains_.set_text((boost::format(_("%i %%")) % mountainsval_).str());
+	mountains_.set_text(format(_("%i %%"), mountainsval_));
 }
 
 void MainMenuNewRandomMapPanel::select_terrains_distribution() {
@@ -447,9 +446,9 @@ void MainMenuNewRandomMapPanel::select_terrains_distribution() {
 	case TerrainDistribution::kRandom: {
 		// Decide the values randomly (within reasonable intervals)
 
-		waterval_ = 5 + 5 * (std::rand() % 7);       // [ 5, 35], NOLINT
-		landval_ = 15 + 5 * (std::rand() % 6);       // [15, 40], NOLINT
-		mountainsval_ = 10 + 5 * (std::rand() % 6);  // [10, 35], NOLINT
+		waterval_ = 5 + 5 * RNG::static_rand(7);       // [ 5, 35]
+		landval_ = 15 + 5 * RNG::static_rand(6);       // [15, 40]
+		mountainsval_ = 10 + 5 * RNG::static_rand(6);  // [10, 35]
 
 		unsigned sum = waterval_ + landval_ + mountainsval_;
 		assert(sum % 5 == 0);
@@ -492,7 +491,7 @@ void MainMenuNewRandomMapPanel::select_terrains_distribution() {
 bool MainMenuNewRandomMapPanel::do_generate_map(Widelands::EditorGameBase& egbase,
                                                 EditorInteractive* eia,
                                                 SinglePlayerGameSettingsProvider* sp) {
-	if (ok_button_) {
+	if (ok_button_ != nullptr) {
 		ok_button_->set_enabled(false);
 		cancel_button_->set_enabled(false);
 	}
@@ -507,7 +506,7 @@ bool MainMenuNewRandomMapPanel::do_generate_map(Widelands::EditorGameBase& egbas
 	// map currently. And if add-ons modify the default units, they may no longer satisfy
 	// the map generator's assumptions. It is probably safer to just disable them all.
 
-	if (eia) {
+	if (eia != nullptr) {
 		egbase.create_loader_ui({"editor"}, true, "", editor_splash_image());
 		eia->cleanup_for_load();
 
@@ -570,7 +569,7 @@ bool MainMenuNewRandomMapPanel::do_generate_map(Widelands::EditorGameBase& egbas
 
 	map->recalc_whole_map(egbase);
 	map->set_background_theme(map_info.world_name);
-	if (eia) {
+	if (eia != nullptr) {
 		eia->map_changed(EditorInteractive::MapWas::kReplaced);
 		egbase.remove_loader_ui();
 
@@ -603,13 +602,13 @@ bool MainMenuNewRandomMapPanel::do_generate_map(Widelands::EditorGameBase& egbas
 		if (result) {
 			// Initialize with some good default values
 
-			const unsigned plnum = std::rand() % nr_players;  // NOLINT
+			const unsigned plnum = RNG::static_rand(nr_players);
 
 			map->set_name(_("Random Map"));
 			map->set_author(_("The Widelands Random Map Generator"));
 			map->set_description(
 			   _("This map was generated automatically by the Widelands Random Map Generator."));
-			map->set_waterway_max_length((std::rand() % 5) * (std::rand() % 6));  // NOLINT
+			map->set_waterway_max_length(RNG::static_rand(5) * RNG::static_rand(6));
 
 			sp->set_map("", "", map_info.world_name, "", nr_players, false);
 			sp->set_scenario(false);
@@ -619,13 +618,12 @@ bool MainMenuNewRandomMapPanel::do_generate_map(Widelands::EditorGameBase& egbas
 
 			for (unsigned p = 0; p < nr_players; ++p) {
 				sp->set_player_name(
-				   p, p == plnum ? _("Player") :
-				                   (boost::format(_("Computer %u")) % (p > plnum ? p : p + 1)).str());
+				   p, p == plnum ? _("Player") : format(_("Computer %u"), (p > plnum ? p : p + 1)));
 				sp->set_player_tribe(p, "", true);
 				sp->set_player_team(p, p == plnum ? 0 : 1);
 				sp->set_player_init(p, 0);
 			}
-		} else if (ok_button_) {
+		} else if (ok_button_ != nullptr) {
 			ok_button_->set_enabled(true);
 			cancel_button_->set_enabled(true);
 		}
@@ -646,7 +644,7 @@ void MainMenuNewRandomMapPanel::id_edit_box_changed() {
 	}
 
 	if (!Widelands::UniqueRandomMapInfo::set_from_id_string(map_info, str, world_names)) {
-		if (ok_button_) {
+		if (ok_button_ != nullptr) {
 			ok_button_->set_enabled(false);
 		}
 	} else {
@@ -678,7 +676,7 @@ void MainMenuNewRandomMapPanel::id_edit_box_changed() {
 		// Update other values in UI as well
 		button_clicked(ButtonId::kNone);
 
-		if (ok_button_) {
+		if (ok_button_ != nullptr) {
 			ok_button_->set_enabled(true);
 		}
 	}
@@ -703,14 +701,14 @@ void MainMenuNewRandomMapPanel::nr_edit_box_changed() {
 
 			map_id_edit_.set_text(id_string);
 
-			if (ok_button_) {
+			if (ok_button_ != nullptr) {
 				ok_button_->set_enabled(true);
 			}
-		} else if (ok_button_) {
+		} else if (ok_button_ != nullptr) {
 			ok_button_->set_enabled(false);
 		}
 	} catch (...) {
-		if (ok_button_) {
+		if (ok_button_ != nullptr) {
 			ok_button_->set_enabled(false);
 		}
 	}
@@ -771,6 +769,7 @@ MainMenuNewRandomMap::MainMenuNewRandomMap(UI::Panel& parent,
 
 	set_center_panel(&box_);
 	center_to_parent();
+	initialization_complete();
 }
 
 void MainMenuNewRandomMap::clicked_ok() {
@@ -783,7 +782,6 @@ bool MainMenuNewRandomMap::handle_key(bool down, SDL_Keysym code) {
 	if (down) {
 		switch (code.sym) {
 		case SDLK_RETURN:
-		case SDLK_KP_ENTER:
 			clicked_ok();
 			return true;
 		default:
