@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,16 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #ifndef WL_EDITOR_UI_MENUS_TOOLSIZE_MENU_H
 #define WL_EDITOR_UI_MENUS_TOOLSIZE_MENU_H
 
-#include "ui_basic/button.h"
-#include "ui_basic/textarea.h"
+#include "ui_basic/spinbox.h"
 #include "ui_basic/unique_window.h"
 
 class EditorInteractive;
@@ -37,12 +35,10 @@ struct EditorToolsizeMenu : public UI::UniqueWindow {
 
 private:
 	EditorInteractive& eia() const;
-	void decrease_radius();
-	void increase_radius();
-
-	UI::Textarea textarea_;
-	UI::Button increase_, decrease_;
+	UI::SpinBox spinbox_;
 	uint32_t value_;
+	void changed();
+	static constexpr uint32_t kMargin = 5;
 };
 
 #endif  // end of include guard: WL_EDITOR_UI_MENUS_TOOLSIZE_MENU_H
