@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 by the Widelands Development Team
+ * Copyright (C) 2017-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -155,8 +154,7 @@ StyleManager::StyleManager() {
 	scrollbarstyles_.clear();
 
 	LuaInterface lua;
-	std::unique_ptr<LuaTable> table(
-	   lua.run_script((boost::format("%1%init.lua") % template_dir()).str()));
+	std::unique_ptr<LuaTable> table(lua.run_script(format("%1%init.lua", template_dir())));
 
 	// Buttons
 	std::unique_ptr<LuaTable> element_table = table->get_table("buttons");
@@ -383,10 +381,7 @@ const RGBColor& StyleManager::minimap_icon_frame() const {
 }
 
 std::string StyleManager::color_tag(const std::string& text, const RGBColor& color) {
-	static boost::format f("<font color=%s>%s</font>");
-	f % color.hex_value();
-	f % text;
-	return f.str();
+	return format("<font color=%s>%s</font>", color.hex_value(), text);
 }
 
 // Fill the maps
