@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -618,10 +617,11 @@ std::string make_ligatures(const char* input) {
 		} else if (kArabicFinalChars.count(c) == 1) {  // All Arabic characters have a final form
 			try {
 				// Skip diacritics for position analysis
-				for (int k = i - 2; k >= 0 && kArabicDiacritics.count(previous); --k) {
+				for (int k = i - 2; k >= 0 && (kArabicDiacritics.count(previous) != 0u); --k) {
 					previous = parseme.charAt(k);
 				}
-				for (int k = i + 2; k < (parseme.length() - 1) && kArabicDiacritics.count(next); ++k) {
+				for (int k = i + 2; k < (parseme.length() - 1) && (kArabicDiacritics.count(next) != 0u);
+				     ++k) {
 					next = parseme.charAt(k);
 				}
 
@@ -632,7 +632,7 @@ std::string make_ligatures(const char* input) {
 					--i;
 					previous = (i > 0) ? parseme.charAt(i - 1) : not_a_character;
 					// Skip diacritics for position analysis
-					for (int k = i - 2; k >= 0 && kArabicDiacritics.count(previous); --k) {
+					for (int k = i - 2; k >= 0 && (kArabicDiacritics.count(previous) != 0u); --k) {
 						previous = parseme.charAt(k);
 					}
 				}
