@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -23,8 +22,6 @@
 #include <memory>
 #include <queue>
 
-#include <boost/algorithm/string.hpp>
-
 #include "base/i18n.h"
 #include "base/log.h"
 #include "base/macros.h"
@@ -56,10 +53,10 @@ namespace {
  * It is used by the renderer after the tags have been parsed.
  */
 void replace_entities(std::string* text) {
-	boost::replace_all(*text, "&gt;", ">");
-	boost::replace_all(*text, "&lt;", "<");
-	boost::replace_all(*text, "&nbsp;", " ");
-	boost::replace_all(*text, "&amp;", "&");  // Must be performed last
+	replace_all(*text, "&gt;", ">");
+	replace_all(*text, "&lt;", "<");
+	replace_all(*text, "&nbsp;", " ");
+	replace_all(*text, "&amp;", "&");  // Must be performed last
 }
 }  // namespace
 
@@ -115,7 +112,7 @@ public:
 	FontCache() = default;
 	~FontCache();
 
-	IFont& get_font(NodeStyle* style);
+	IFont& get_font(NodeStyle* ns);
 
 private:
 	struct FontDescr {
@@ -148,8 +145,8 @@ IFont& FontCache::get_font(NodeStyle* ns) {
 	} else if (ns->font_face == "sans") {
 		ns->font_face = ns->fontset->sans();
 	}
-	const bool is_bold = ns->font_style & IFont::BOLD;
-	const bool is_italic = ns->font_style & IFont::ITALIC;
+	const bool is_bold = (ns->font_style & IFont::BOLD) != 0;
+	const bool is_italic = (ns->font_style & IFont::ITALIC) != 0;
 	if (is_bold && is_italic) {
 		if (ns->font_face == ns->fontset->condensed() ||
 		    ns->font_face == ns->fontset->condensed_bold() ||
@@ -238,7 +235,7 @@ public:
 	virtual bool is_expanding() const {
 		return false;
 	}
-	virtual void set_w(uint16_t) {
+	virtual void set_w(uint16_t /* w */) {
 	}  // Only, when is_expanding
 
 	virtual const std::vector<Reference> get_references() {
@@ -287,9 +284,7 @@ protected:
 		const int maximum_size = g_gr->max_texture_size_for_font_rendering();
 		if (check_w > maximum_size || check_h > maximum_size) {
 			const std::string error_message =
-			   (boost::format("Texture (%d, %d) too big! Maximum size is %d.") % check_w % check_h %
-			    maximum_size)
-			      .str();
+			   format("Texture (%d, %d) too big! Maximum size is %d.", check_w, check_h, maximum_size);
 			log_err("%s\n", error_message.c_str());
 			throw TextureTooBig(error_message);
 		}
@@ -317,7 +312,7 @@ public:
 	}
 	virtual ~Layout() = default;
 
-	uint16_t height() {
+	uint16_t height() const {
 		return h_;
 	}
 	uint16_t
@@ -326,7 +321,7 @@ public:
 private:
 	bool calculate_line_width(uint16_t* x, uint16_t* w, uint16_t lineheight);
 	uint16_t fit_line(uint16_t w,
-	                  const Borders&,
+	                  const Borders& /*p*/,
 	                  std::vector<std::shared_ptr<RenderNode>>* rv,
 	                  bool trim_spaces);
 
@@ -618,7 +613,7 @@ uint16_t Layout::fit_nodes(std::vector<std::shared_ptr<RenderNode>>* rv,
  */
 class TextNode : public RenderNode {
 public:
-	TextNode(FontCache& font, NodeStyle&, const std::string& txt);
+	TextNode(FontCache& font, NodeStyle& /*ns*/, const std::string& txt);
 	~TextNode() override = default;
 
 	std::string debug_info() const override {
@@ -676,7 +671,7 @@ uint16_t TextNode::hotspot_y() const {
 		// (UChar32, char32At, TTF_GlyphIsProvided32, TTF_GlyphMetrics32)
 		UChar codepoint = unicode_txt.charAt(i);
 		int maxy;
-		if (TTF_GlyphIsProvided(font_.get_ttf_font(), codepoint) &&
+		if ((TTF_GlyphIsProvided(font_.get_ttf_font(), codepoint) != 0) &&
 		    TTF_GlyphMetrics(
 		       font_.get_ttf_font(), codepoint, nullptr, nullptr, nullptr, &maxy, nullptr) == 0) {
 			ascent = std::max(ascent, maxy);
@@ -714,7 +709,7 @@ public:
 		return "ft";
 	}
 
-	std::shared_ptr<UI::RenderedText> render(TextureCache*) override;
+	std::shared_ptr<UI::RenderedText> render(TextureCache* /*texture_cache*/) override;
 
 	bool is_expanding() const override {
 		return is_expanding_;
@@ -729,9 +724,8 @@ private:
 std::shared_ptr<UI::RenderedText> FillingTextNode::render(TextureCache* texture_cache) {
 	std::shared_ptr<UI::RenderedText> rendered_text(new UI::RenderedText());
 	const std::string hash =
-	   (boost::format("rt:fill:%s:%s:%i:%i:%i:%s") % txt_ % nodestyle_.font_color.hex_value() %
-	    nodestyle_.font_style % width() % height() % (is_expanding_ ? "e" : "f"))
-	      .str();
+	   format("rt:fill:%s:%s:%i:%i:%i:%s", txt_, nodestyle_.font_color.hex_value(),
+	          nodestyle_.font_style, width(), height(), (is_expanding_ ? "e" : "f"));
 
 	std::shared_ptr<const Image> rendered_image = texture_cache->get(hash);
 	if (rendered_image == nullptr) {
@@ -740,8 +734,7 @@ std::shared_ptr<UI::RenderedText> FillingTextNode::render(TextureCache* texture_
 		auto texture = std::make_shared<Texture>(width(), height());
 		for (uint16_t curx = 0; curx < w_; curx += ttf->width()) {
 			Rectf srcrect(0.f, 0.f, std::min<int>(ttf->width(), w_ - curx), h_);
-			texture->blit(
-			   Rectf(curx, 0, srcrect.w, srcrect.h), *ttf.get(), srcrect, 1., BlendMode::Copy);
+			texture->blit(Rectf(curx, 0, srcrect.w, srcrect.h), *ttf, srcrect, 1., BlendMode::Copy);
 		}
 		rendered_image = texture_cache->insert(hash, std::move(texture));
 	}
@@ -771,7 +764,7 @@ public:
 	std::shared_ptr<UI::RenderedText> render(TextureCache* texture_cache) override {
 		if (show_spaces_) {
 			std::shared_ptr<UI::RenderedText> rendered_text(new UI::RenderedText());
-			const std::string hash = (boost::format("rt:wsp:%i:%i") % width() % height()).str();
+			const std::string hash = format("rt:wsp:%i:%i", width(), height());
 			std::shared_ptr<const Image> rendered_image = texture_cache->get(hash);
 			if (rendered_image == nullptr) {
 				auto texture = std::make_shared<Texture>(width(), height());
@@ -849,9 +842,8 @@ public:
 	}
 	std::shared_ptr<UI::RenderedText> render(TextureCache* texture_cache) override {
 		std::shared_ptr<UI::RenderedText> rendered_text(new UI::RenderedText());
-		const std::string hash = (boost::format("rt:sp:%s:%i:%i:%s") % filename_ % width() %
-		                          height() % (is_expanding_ ? "e" : "f"))
-		                            .str();
+		const std::string hash =
+		   format("rt:sp:%s:%i:%i:%s", filename_, width(), height(), (is_expanding_ ? "e" : "f"));
 
 		std::shared_ptr<const Image> rendered_image = texture_cache->get(hash);
 		if (rendered_image == nullptr) {
@@ -905,7 +897,6 @@ class DivTagRenderNode : public RenderNode {
 public:
 	explicit DivTagRenderNode(const NodeStyle& ns)
 	   : RenderNode(ns),
-	     desired_width_(),
 	     w_(0),
 	     h_(0),
 	     background_color_(0, 0, 0),
@@ -1023,7 +1014,7 @@ public:
 	              bool use_playercolor)
 	   : RenderNode(ns),
 	     image_(use_playercolor ? playercolor_image(color, image_filename) :
-	                              g_image_cache->get(image_filename)),
+                                 g_image_cache->get(image_filename)),
 	     filename_(image_filename),
 	     scale_(scale),
 	     color_(color),
@@ -1034,7 +1025,6 @@ public:
 	ImgRenderNode(NodeStyle& ns, const Image* image)
 	   : RenderNode(ns),
 	     image_(image),
-	     filename_(""),
 	     scale_(1.0),
 	     color_(RGBColor(0, 0, 0)),
 	     use_playercolor_(false) {
@@ -1073,9 +1063,9 @@ std::shared_ptr<UI::RenderedText> ImgRenderNode::render(TextureCache* texture_ca
 		rendered_text->rects.push_back(
 		   std::unique_ptr<UI::RenderedRect>(new UI::RenderedRect(image_)));
 	} else {
-		const std::string hash = (boost::format("rt:img:%s:%s:%i:%i") % filename_ %
-		                          (use_playercolor_ ? color_.hex_value() : "") % width() % height())
-		                            .str();
+		const std::string hash =
+		   format("rt:img:%s:%s:%i:%i", filename_, (use_playercolor_ ? color_.hex_value() : ""),
+		          width(), height());
 		std::shared_ptr<const Image> rendered_image = texture_cache->get(hash);
 		if (rendered_image == nullptr) {
 			auto texture = std::make_shared<Texture>(width(), height());
@@ -1120,7 +1110,7 @@ public:
 
 	virtual void enter() {
 	}
-	virtual void emit_nodes(std::vector<std::shared_ptr<RenderNode>>&);
+	virtual void emit_nodes(std::vector<std::shared_ptr<RenderNode>>& /*nodes*/);
 
 private:
 	void make_text_nodes(const std::string& txt,
@@ -1225,7 +1215,7 @@ void TagHandler::make_text_nodes(const std::string& txt,
 
 void TagHandler::emit_nodes(std::vector<std::shared_ptr<RenderNode>>& nodes) {
 	for (Child* c : tag_.children()) {
-		if (c->tag) {
+		if (c->tag != nullptr) {
 			std::unique_ptr<TagHandler> th(create_taghandler(
 			   *c->tag, font_cache_, nodestyle_, image_cache_, renderer_style_, fontsets_));
 			th->enter();
@@ -1322,7 +1312,7 @@ public:
 		if (!nodes.empty()) {
 			nodes.push_back(std::shared_ptr<RenderNode>(new NewlineNode(nodestyle_)));
 		}
-		if (indent_) {
+		if (indent_ != 0u) {
 			nodes.push_back(std::shared_ptr<RenderNode>(new SpaceNode(nodestyle_, indent_)));
 		}
 		TagHandler::emit_nodes(nodes);
@@ -1460,7 +1450,7 @@ public:
 			} else {
 				node.reset(new SpaceNode(nodestyle_, 0, 0, true));
 			}
-			if (background_image_) {
+			if (background_image_ != nullptr) {
 				node->set_background(background_image_, image_filename_);
 			}
 			nodes.push_back(node);
@@ -1515,7 +1505,8 @@ public:
 	 * previous line when their final position is calculated in Layout::fit_nodes().
 	 */
 	void enter() override {
-		Borders padding, margin;
+		Borders padding;
+		Borders margin;
 
 		handle_unique_attributes();
 		const AttrMap& a = tag_.attrs();
@@ -1571,7 +1562,7 @@ public:
 			if (n->width() >= INFINITE_WIDTH) {
 				continue;
 			}
-			if (width_first_subnode >= INFINITE_WIDTH && n->width()) {
+			if (width_first_subnode >= INFINITE_WIDTH && (n->width() != 0u)) {
 				width_first_subnode = n->width() + padding.left + padding.right;
 			}
 			widest_subnode = std::max<int>(widest_subnode, n->width() + padding.left + padding.right);
@@ -1604,7 +1595,7 @@ public:
 			}
 			break;
 		default:
-			if (!w_) {
+			if (w_ == 0u) {
 				w_ = widest_subnode;
 			}
 			// Else do nothing
@@ -1663,7 +1654,7 @@ public:
 			std::string width_string = a["width"].get_string();
 			if (width_string == "*") {
 				render_node_->set_desired_width(DesiredWidth(INFINITE_WIDTH, WidthUnit::kFill));
-			} else if (boost::algorithm::ends_with(width_string, "%")) {
+			} else if (ends_with(width_string, "%")) {
 				width_string = width_string.substr(0, width_string.length() - 1);
 				uint8_t width_percent = strtol(width_string.c_str(), nullptr, 10);
 				if (width_percent > 100) {
@@ -1769,9 +1760,7 @@ TagHandler* create_taghandler(Tag& tag,
 	TagHandlerMap::iterator i = map.find(tag.name());
 	if (i == map.end()) {
 		throw RenderError(
-		   (boost::format("No Tag handler for %s. This is a bug, please submit a report.") %
-		    tag.name())
-		      .str());
+		   format("No Tag handler for %s. This is a bug, please submit a report.", tag.name()));
 	}
 	return i->second(tag, fc, ns, image_cache, renderer_style, fontsets);
 }
@@ -1797,7 +1786,7 @@ std::shared_ptr<RenderNode>
 Renderer::layout(const std::string& text, uint16_t width, bool is_rtl, const TagSet& allowed_tags) {
 	std::unique_ptr<Tag> rt(parser_->parse(text, allowed_tags));
 
-	if (!width) {
+	if (width == 0u) {
 		width = INFINITE_WIDTH;
 	}
 
