@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 // this originally comes from Return to the Shadows (http://www.rtts.org/)
@@ -29,11 +28,6 @@
 #include <cstdio>
 #endif
 
-// We have to add Boost to this block to make codecheck happy
-#include <boost/algorithm/string/predicate.hpp>
-#include <boost/algorithm/string/replace.hpp>
-#include <boost/format.hpp>
-#include <boost/lexical_cast.hpp>
 #ifdef _WIN32
 #include <direct.h>
 #include <windows.h>
@@ -48,8 +42,8 @@
 
 #include "base/i18n.h"
 #include "base/log.h"
+#include "base/string.h"
 #include "config.h"
-#include "graphic/text_layout.h"
 #include "io/filesystem/disk_filesystem.h"
 #include "io/filesystem/filesystem_exceptions.h"
 #include "io/filesystem/layered_filesystem.h"
@@ -65,23 +59,16 @@
 #define PATH_MAX MAX_PATH
 #endif
 
+/* Quickfix for bug https://github.com/widelands/widelands/issues/5614:
+ * Most systems specify PATH_MAX to be the maximum number of characters in a file path.
+ * Systems without a limit (or which don't care about standards) may neglect to define this symbol.
+ * On such systems, simply use an arbitrary value that is high enough for some very long paths.
+ */
+#ifndef PATH_MAX
+#define PATH_MAX 0x10000
+#endif
+
 namespace {
-// Characters that are allowed in filenames, but not at the beginning
-const std::vector<std::string> illegal_filename_starting_characters{
-   ".", "-",
-   " ",  // Keep the blank last
-};
-
-// Characters that are disallowed anywhere in a filename
-// No potential file separators or other potentially illegal characters
-// https://msdn.microsoft.com/en-us/library/windows/desktop/aa365247(v=vs.85).aspx
-// http://www.linfo.org/file_name.html
-// https://support.apple.com/en-us/HT202808
-// We can't just regex for word & digit characters here because of non-Latin scripts.
-const std::vector<std::string> illegal_filename_characters{
-   "<", ">", ":", "\"", "|", "?", "*", "/", "\\",
-};
-
 /// A class that makes iteration over filename_?.* templates easy. It is much faster than using
 /// regex.
 class NumberGlob {
@@ -106,7 +93,7 @@ private:
  */
 NumberGlob::NumberGlob(const std::string& file_template) : template_(file_template), current_(0) {
 	int nchars = count(file_template.begin(), file_template.end(), '?');
-	format_ = "%0" + boost::lexical_cast<std::string>(nchars) + "i";
+	format_ = "%0" + as_string(nchars) + "i";
 
 	max_ = 1;
 	for (int i = 0; i < nchars; ++i) {
@@ -121,11 +108,9 @@ bool NumberGlob::next(std::string* s) {
 		return false;
 	}
 
-	if (max_) {
-		*s = boost::replace_last_copy(
-		   template_, to_replace_, (boost::format(format_) % current_).str());
-	} else {
-		*s = template_;
+	*s = template_;
+	if (max_ != 0u) {
+		replace_last(*s, to_replace_, format(format_, current_));
 	}
 	++current_;
 	return true;
@@ -146,7 +131,7 @@ bool FileSystem::is_path_absolute(const std::string& path) const {
 	if (path_size == root_size) {
 		return path == root_;
 	}
-	if (path.compare(0, root_.size(), root_)) {
+	if (path.compare(0, root_.size(), root_) != 0) {
 		return false;
 	}
 
@@ -205,8 +190,13 @@ std::string FileSystem::fix_cross_file(const std::string& path) const {
 // static
 std::string FileSystem::get_working_directory() {
 	char cwd[PATH_MAX + 1];
+#ifdef _MSC_VER
+	// getcwd is deprecated on MSVC
+	char* const result = _getcwd(cwd, PATH_MAX);
+#else
 	char* const result = getcwd(cwd, PATH_MAX);
-	if (!result) {
+#endif
+	if (result == nullptr) {
 		throw FileError("FileSystem::get_working_directory()", "widelands", "can not run getcwd");
 	}
 
@@ -222,58 +212,6 @@ char FileSystem::file_separator() {
 #endif
 }
 
-bool FileSystem::is_legal_filename(const std::string& filename) {
-	if (filename.empty()) {
-		return false;
-	}
-	for (const std::string& illegal_start : illegal_filename_starting_characters) {
-		if (boost::starts_with(filename, illegal_start)) {
-			return false;
-		}
-	}
-	for (const std::string& illegal_char : illegal_filename_characters) {
-		if (boost::contains(filename, illegal_char)) {
-			return false;
-		}
-	}
-	return true;
-}
-
-std::string FileSystem::illegal_filename_tooltip() {
-	std::vector<std::string> starting_characters;
-	for (const std::string& character : illegal_filename_starting_characters) {
-		if (character == " ") {
-			/** TRANSLATORS: Part of tooltip entry for characters in illegal filenames. replaces tha
-			 * blank space in a list of illegal characters */
-			starting_characters.push_back(pgettext("illegal_filename_characters", "blank space"));
-		} else {
-			starting_characters.push_back(character);
-		}
-	}
-	const std::string illegal_start(as_listitem(
-	   /** TRANSLATORS: Tooltip entry for characters in illegal filenames.
-	    *  %s is a list of illegal characters */
-	   (boost::format(pgettext("illegal_filename_characters", "%s at the start of the filename")) %
-	    richtext_escape(i18n::localize_list(starting_characters, i18n::ConcatenateWith::OR)))
-	      .str(),
-	   UI::FontStyle::kWuiMessageParagraph));
-
-	const std::string illegal(as_listitem(
-	   /** TRANSLATORS: Tooltip entry for characters in illegal filenames.
-	    * %s is a list of illegal characters */
-	   (boost::format(pgettext("illegal_filename_characters", "%s anywhere in the filename")) %
-	    richtext_escape(i18n::localize_list(illegal_filename_characters, i18n::ConcatenateWith::OR)))
-	      .str(),
-	   UI::FontStyle::kWuiMessageParagraph));
-
-	return (boost::format("%s%s%s") %
-	        /** TRANSLATORS: Tooltip header for characters in illegal filenames.
-	         * This is followed by a list of bullet points */
-	        pgettext("illegal_filename_characters", "The following characters are not allowed:") %
-	        illegal_start % illegal)
-	   .str();
-}
-
 // TODO(unknown): Write homedir detection for non-getenv-systems
 std::string FileSystem::get_homedir() {
 	std::string homedir;
@@ -394,8 +332,8 @@ std::vector<std::string> FileSystem::get_xdgdatadirs() {
 	environment_char = getenv("XDG_DATA_DIRS");
 #endif
 	std::string environment(environment_char == nullptr || *environment_char == 0 ?
-	                           "/usr/local/share:/usr/share" :
-	                           environment_char);
+                              "/usr/local/share:/usr/share" :
+                              environment_char);
 
 	// https://stackoverflow.com/a/14266139
 	std::string token;
@@ -532,7 +470,7 @@ std::string FileSystem::canonicalize_name(const std::string& path) const {
 				continue;
 			}
 			// Remove double dot and the preceding component (if any)
-			else if (*str == '.' && *(str + 1) == '\0') {
+			if (*str == '.' && *(str + 1) == '\0') {
 				if (i != components.begin()) {
 #ifdef _WIN32
 					// On windows don't remove driveletter in this error condition
@@ -605,13 +543,12 @@ std::string FileSystem::filename_ext(const std::string& f) {
 
 	if (std::string::npos == ext_start) {
 		return "";
-	} else {
-		return f.substr(ext_start);
 	}
+	return f.substr(ext_start);
 }
 
 std::string FileSystem::filename_without_ext(const char* const p) {
-	std::string fname(p ? FileSystem::fs_filename(p) : "");
+	std::string fname(p != nullptr ? FileSystem::fs_filename(p) : "");
 	std::string ext(FileSystem::filename_ext(fname));
 	return fname.substr(0, fname.length() - ext.length());
 }
