@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2021 by the Widelands Development Team
+ * Copyright (C) 2007-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,10 +12,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
-#define BOOST_TEST_MODULE FileSystemTests
-#include <boost/test/unit_test.hpp>
+#include "base/test.h"
+TEST_EXECUTABLE(filesystem, false)
