@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -23,10 +22,9 @@
 #include <cstdlib>
 #include <cstring>
 
-#include <boost/algorithm/string.hpp>
-
 #include "base/i18n.h"
 #include "base/log.h"
+#include "base/string.h"
 #include "base/wexception.h"
 #include "build_info.h"
 #include "io/fileread.h"
@@ -95,7 +93,7 @@ void Section::Value::mark_used() {
 int32_t Section::Value::get_int() const {
 	char* endp;
 	int64_t const i = strtol(value_.get(), &endp, 0);
-	if (*endp) {
+	if (*endp != 0) {
 		throw wexception("%s: '%s' is not an integer", get_name(), get_untranslated_string());
 	}
 	int32_t const result = static_cast<int32_t>(i);
@@ -109,7 +107,7 @@ int32_t Section::Value::get_int() const {
 uint32_t Section::Value::get_natural() const {
 	char* endp;
 	int64_t i = strtoll(value_.get(), &endp, 0);
-	if (*endp || i < 0) {
+	if ((*endp != 0) || i < 0) {
 		throw wexception("%s: '%s' is not natural", get_name(), get_untranslated_string());
 	}
 	return i;
@@ -118,7 +116,7 @@ uint32_t Section::Value::get_natural() const {
 uint32_t Section::Value::get_positive() const {
 	char* endp;
 	int64_t i = strtoll(value_.get(), &endp, 0);
-	if (*endp || i < 1) {
+	if ((*endp != 0) || i < 1) {
 		throw wexception("%s: '%s' is not positive", get_name(), get_untranslated_string());
 	}
 	return i;
@@ -126,12 +124,12 @@ uint32_t Section::Value::get_positive() const {
 
 bool Section::Value::get_bool() const {
 	for (char const* word : trueWords) {
-		if (boost::iequals(value_.get(), word)) {
+		if (iequals(value_.get(), word)) {
 			return true;
 		}
 	}
 	for (char const* word : falseWords) {
-		if (boost::iequals(value_.get(), word)) {
+		if (iequals(value_.get(), word)) {
 			return false;
 		}
 	}
@@ -147,7 +145,7 @@ Vector2i Section::Value::get_point() const {
 	char* endp = value_.get();
 	int64_t const x = strtol(endp, &endp, 0);
 	int64_t const y = strtol(endp, &endp, 0);
-	if (*endp) {
+	if (*endp != 0) {
 		throw wexception("%s: '%s' is not a Vector2i", get_name(), get_untranslated_string());
 	}
 	if (x > std::numeric_limits<int32_t>::max()) {
@@ -227,7 +225,7 @@ void Section::check_used() const {
 
 bool Section::has_val(char const* const name) const {
 	for (const Value& temp_value : values_) {
-		if (boost::iequals(temp_value.get_name(), name)) {
+		if (iequals(temp_value.get_name(), name)) {
 			return true;
 		}
 	}
@@ -243,7 +241,7 @@ bool Section::has_val(char const* const name) const {
  */
 Section::Value* Section::get_val(char const* const name) {
 	for (Value& value : values_) {
-		if (boost::iequals(value.get_name(), name)) {
+		if (iequals(value.get_name(), name)) {
 			value.mark_used();
 			return &value;
 		}
@@ -261,7 +259,7 @@ Section::Value* Section::get_val(char const* const name) {
 Section::Value* Section::get_next_val(char const* const name) {
 	for (Value& value : values_) {
 		if (!value.is_used()) {
-			if (!name || boost::iequals(value.get_name(), name)) {
+			if ((name == nullptr) || iequals(value.get_name(), name)) {
 				value.mark_used();
 				return &value;
 			}
@@ -272,7 +270,7 @@ Section::Value* Section::get_next_val(char const* const name) {
 
 Section::Value& Section::create_val(char const* const name, char const* const value) {
 	for (Value& temp_value : values_) {
-		if (boost::iequals(temp_value.get_name(), name)) {
+		if (iequals(temp_value.get_name(), name)) {
 			temp_value.set_string(value);
 			return temp_value;
 		}
@@ -291,7 +289,7 @@ Section::Value& Section::create_val_duplicate(char const* const name, char const
  */
 int32_t Section::get_safe_int(char const* const name) {
 	Value* const v = get_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		throw wexception("[%s]: missing integer key '%s'", get_name(), name);
 	}
 	return v->get_int();
@@ -301,18 +299,16 @@ int32_t Section::get_safe_int(char const* const name) {
 uint32_t Section::get_safe_natural(char const* const name) {
 	if (Value* const v = get_val(name)) {
 		return v->get_natural();
-	} else {
-		throw wexception("[%s]: missing natural key '%s'", get_name(), name);
 	}
+	throw wexception("[%s]: missing natural key '%s'", get_name(), name);
 }
 
 /// Return the positive value of the given key or throw an exception.
 uint32_t Section::get_safe_positive(char const* const name) {
 	if (Value* const v = get_val(name)) {
 		return v->get_positive();
-	} else {
-		throw wexception("[%s]: missing positive key '%s'", get_name(), name);
 	}
+	throw wexception("[%s]: missing positive key '%s'", get_name(), name);
 }
 
 /**
@@ -321,7 +317,7 @@ uint32_t Section::get_safe_positive(char const* const name) {
  */
 bool Section::get_safe_bool(char const* const name) {
 	Value* const v = get_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		throw wexception("[%s]: missing boolean key '%s'", get_name(), name);
 	}
 	return v->get_bool();
@@ -333,7 +329,7 @@ bool Section::get_safe_bool(char const* const name) {
  */
 char const* Section::get_safe_string(char const* const name) {
 	Value* const v = get_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		throw wexception("[%s]: missing key '%s'", get_name(), name);
 	}
 	return v->get_string();
@@ -341,7 +337,7 @@ char const* Section::get_safe_string(char const* const name) {
 
 char const* Section::get_safe_untranslated_string(char const* const name) {
 	Value* const v = get_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		throw wexception("[%s]: missing key '%s'", get_name(), name);
 	}
 	return v->get_untranslated_string();
@@ -366,7 +362,7 @@ const char* Section::get_safe_string(const std::string& name) {
  */
 int32_t Section::get_int(char const* const name, int32_t const def) {
 	Value* const v = get_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		return def;
 	}
 
@@ -416,7 +412,7 @@ uint32_t Section::get_positive(char const* const name, uint32_t const def) {
  */
 bool Section::get_bool(char const* const name, bool const def) {
 	Value* const v = get_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		return def;
 	}
 
@@ -441,12 +437,12 @@ bool Section::get_bool(char const* const name, bool const def) {
  */
 char const* Section::get_string(char const* const name, char const* const def) {
 	Value const* const v = get_val(name);
-	return v ? v->get_string() : def;
+	return v != nullptr ? v->get_string() : def;
 }
 
 Vector2i Section::get_point(const char* const name, const Vector2i def) {
 	Value const* const v = get_val(name);
-	return v ? v->get_point() : def;
+	return v != nullptr ? v->get_point() : def;
 }
 
 /**
@@ -459,11 +455,11 @@ Vector2i Section::get_point(const char* const name, const Vector2i def) {
  */
 char const* Section::get_next_bool(char const* const name, bool* const value) {
 	Value* const v = get_next_val(name);
-	if (!v) {
+	if (v == nullptr) {
 		return nullptr;
 	}
 
-	if (value) {
+	if (value != nullptr) {
 		*value = v->get_bool();
 	}
 	return v->get_name();
@@ -585,7 +581,7 @@ void Profile::check_used() const {
  */
 Section* Profile::get_section(const std::string& name) {
 	for (Section& temp_section : sections_) {
-		if (boost::iequals(temp_section.get_name(), name.c_str())) {
+		if (iequals(temp_section.get_name(), name)) {
 			temp_section.mark_used();
 			return &temp_section;
 		}
@@ -600,9 +596,8 @@ Section* Profile::get_section(const std::string& name) {
 Section& Profile::get_safe_section(const std::string& name) {
 	if (Section* const s = get_section(name)) {
 		return *s;
-	} else {
-		throw wexception("in \"%s\" section [%s] not found", filename_.c_str(), name.c_str());
 	}
+	throw wexception("in \"%s\" section [%s] not found", filename_.c_str(), name.c_str());
 }
 
 /**
@@ -626,7 +621,7 @@ Section& Profile::pull_section(char const* const name) {
 Section* Profile::get_next_section(char const* const name) {
 	for (Section& section : sections_) {
 		if (!section.is_used()) {
-			if (!name || boost::iequals(section.get_name(), name)) {
+			if ((name == nullptr) || iequals(section.get_name(), name)) {
 				section.mark_used();
 				return &section;
 			}
@@ -637,7 +632,7 @@ Section* Profile::get_next_section(char const* const name) {
 
 Section& Profile::create_section(char const* const name) {
 	for (Section& section : sections_) {
-		if (boost::iequals(section.get_name(), name)) {
+		if (iequals(section.get_name(), name)) {
 			return section;
 		}
 	}
@@ -650,7 +645,7 @@ Section& Profile::create_section_duplicate(char const* const name) {
 }
 
 inline char* skipwhite(char* p) {
-	while (*p && isspace(*p)) {
+	while ((*p != 0) && (isspace(*p) != 0)) {
 		++p;
 	}
 	return p;
@@ -658,7 +653,7 @@ inline char* skipwhite(char* p) {
 
 inline void rtrim(char* const str) {
 	for (char* p = strchr(str, '\0'); str < p; --p) {
-		if (!isspace(p[-1])) {
+		if (isspace(p[-1]) == 0) {
 			*p = 0;
 			break;
 		}
@@ -666,7 +661,7 @@ inline void rtrim(char* const str) {
 }
 
 inline void killcomments(char* p) {
-	while (*p) {
+	while (*p != 0) {
 		if (p[0] == '#') {
 			p[0] = '\0';
 			break;
@@ -702,7 +697,7 @@ void Profile::read(char const* const filename, char const* const global_section,
 			}
 
 			p = skipwhite(p);
-			if (!p[0] || p[0] == '#') {
+			if ((p[0] == 0) || p[0] == '#') {
 				continue;
 			}
 
@@ -740,7 +735,7 @@ void Profile::read(char const* const filename, char const* const global_section,
 					tail = line;
 				} else {
 					tail = strchr(p, '=');
-					if (!tail) {
+					if (tail == nullptr) {
 						throw wexception("invalid syntax: %s", line);
 					}
 					*tail++ = '\0';
@@ -765,11 +760,11 @@ void Profile::read(char const* const filename, char const* const global_section,
 						++tail;
 					}
 				}
-				if (tail) {
+				if (tail != nullptr) {
 					char* const eot = tail + strlen(tail) - 1;
 					if (*eot == '\'' || *eot == '"') {
 						*eot = '\0';
-						if (*tail) {
+						if (*tail != 0) {
 							char* const eot2 = tail + strlen(tail) - 1;
 							if (*eot2 == '\'' || *eot2 == '"') {
 								reading_multiline = false;
@@ -779,8 +774,8 @@ void Profile::read(char const* const filename, char const* const global_section,
 					}
 
 					// ready to insert
-					if (!s) {
-						if (global_section) {
+					if (s == nullptr) {
+						if (global_section != nullptr) {
 							s = &create_section_duplicate(global_section);
 						} else {
 							throw wexception("key %s outside section", p);
@@ -808,7 +803,7 @@ void Profile::read(char const* const filename, char const* const global_section,
 	}
 
 	//  Make sure that the requested global section exists, even if it is empty.
-	if (global_section && !get_section(global_section)) {
+	if ((global_section != nullptr) && (get_section(global_section) == nullptr)) {
 		create_section_duplicate(global_section);
 	}
 }
@@ -824,10 +819,9 @@ void Profile::write(char const* const filename,
                     char const* const comment) {
 	FileWrite fw;
 
-	fw.print_f(
-	   "# Automatically created by Widelands %s (%s)\n", build_id().c_str(), build_type().c_str());
+	fw.print_f("# Automatically created by Widelands %s\n", build_ver_details().c_str());
 
-	if (comment) {
+	if (comment != nullptr) {
 		fw.print_f("# %s\n", comment);
 	}
 
@@ -845,7 +839,7 @@ void Profile::write(char const* const filename,
 
 			char const* const str = temp_value.get_untranslated_string();
 
-			if (*str) {
+			if (*str != 0) {
 				uint32_t spaces = strlen(temp_value.get_name());
 				bool multiline = false;
 
@@ -864,7 +858,7 @@ void Profile::write(char const* const filename,
 					tempstr += "\"";
 				}
 
-				for (char const* it = str; *it; ++it) {
+				for (char const* it = str; *it != 0; ++it) {
 					// No speach marks - they would break the format
 					switch (*it) {
 					case '"':
