@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -70,9 +69,10 @@ initialization
 */
 EditorGameBase::EditorGameBase(LuaInterface* lua_interface)
    : did_postload_addons_(false),
+     did_postload_tribes_(false),
      gametime_(0),
      // TODO(SirVer): this is sooo ugly, I can't say
-     lua_(lua_interface ? lua_interface : new LuaEditorInterface(this)),
+     lua_(lua_interface != nullptr ? lua_interface : new LuaEditorInterface(this)),
      player_manager_(new PlayersManager(*this)),
      ibase_(nullptr),
      loader_ui_(nullptr),
@@ -98,8 +98,8 @@ static inline bool addon_initially_enabled(AddOns::AddOnCategory c) {
 void EditorGameBase::init_addons(bool world_only) {
 	enabled_addons_.clear();
 	for (const auto& pair : AddOns::g_addons) {
-		if (pair.second && (world_only ? pair.first.category == AddOns::AddOnCategory::kWorld :
-		                                 addon_initially_enabled(pair.first.category))) {
+		if (pair.second && (world_only ? pair.first->category == AddOns::AddOnCategory::kWorld :
+                                       addon_initially_enabled(pair.first->category))) {
 			enabled_addons_.push_back(pair.first);
 		}
 	}
@@ -137,7 +137,7 @@ void EditorGameBase::delete_tempfile() {
  * throws an exception if something goes wrong
  */
 void EditorGameBase::create_tempfile_and_save_mapdata(FileSystem::Type const type) {
-	if (!map_.filesystem()) {
+	if (map_.filesystem() == nullptr) {
 		return;
 	}
 
@@ -221,7 +221,7 @@ Descriptions* EditorGameBase::mutable_descriptions() {
 		ScopedTimer timer("Registering the descriptions took %ums", true);
 		assert(lua_);
 		descriptions_.reset(new Descriptions(lua_.get(), enabled_addons_));
-		if (game_tips_.get()) {
+		if (game_tips_) {
 			game_tips_.reset(new GameTips(*loader_ui_, registered_game_tips_, all_tribes()));
 		}
 	}
@@ -232,7 +232,7 @@ void EditorGameBase::set_ibase(InteractiveBase* const b) {
 	ibase_.reset(b);
 }
 
-InteractiveGameBase* EditorGameBase::get_igbase() {
+InteractiveGameBase* EditorGameBase::get_igbase() const {
 	return dynamic_cast<InteractiveGameBase*>(get_ibase());
 }
 
@@ -249,7 +249,7 @@ Player* EditorGameBase::add_player(PlayerNumber const player_number,
                                    const std::string& name,
                                    TeamNumber team) {
 	Notifications::publish(UI::NoteLoadingMessage(
-	   (boost::format(_("Creating player %d…")) % static_cast<unsigned int>(player_number)).str()));
+	   format(_("Creating player %d…"), static_cast<unsigned int>(player_number))));
 	return player_manager_->add_player(player_number, initialization_index, pc, tribe, name, team);
 }
 
@@ -308,6 +308,9 @@ void EditorGameBase::allocate_player_maps() {
 void EditorGameBase::postload() {
 	create_tempfile_and_save_mapdata(FileSystem::ZIP);
 	assert(descriptions_);
+
+	postload_addons();
+	postload_tribes();
 }
 
 void EditorGameBase::postload_addons() {
@@ -319,25 +322,41 @@ void EditorGameBase::postload_addons() {
 	Notifications::publish(UI::NoteLoadingMessage(_("Postloading world and tribes…")));
 
 	assert(lua_);
-	assert(descriptions_);
 
-	for (const AddOns::AddOnInfo& info : enabled_addons_) {
-		if (info.category == AddOns::AddOnCategory::kWorld ||
-		    info.category == AddOns::AddOnCategory::kTribes) {
-			const std::string script(kAddOnDir + FileSystem::file_separator() + info.internal_name +
+	mutable_descriptions()->ensure_tribes_are_registered();
+	if (is_game()) {
+		FileSystem* map_fs = map().filesystem();
+		// In new random matches, map_fs may be nullptr
+		if (map_fs != nullptr && map_fs->file_exists("scripting/tribes")) {
+			verb_log_info("Game: Reading Scenario Tribes ... ");
+			mutable_descriptions()->register_scenario_tribes(map_fs);
+		}
+	}
+
+	for (const auto& info : enabled_addons_) {
+		if (info->category == AddOns::AddOnCategory::kWorld ||
+		    info->category == AddOns::AddOnCategory::kTribes) {
+			const std::string script(kAddOnDir + FileSystem::file_separator() + info->internal_name +
 			                         FileSystem::file_separator() + "postload.lua");
 			if (g_fs->file_exists(script)) {
-				verb_log_info("Running postload script for add-on %s", info.internal_name.c_str());
+				verb_log_info("Running postload script for add-on %s", info->internal_name.c_str());
 				lua_->run_script(script);
 			}
 		}
 	}
+}
 
-	// Postload all tribes. We can do this only now to ensure that any changes
-	// made by add-ons are taken into account when computing dependency chains.
+void EditorGameBase::postload_tribes() {
+	if (did_postload_tribes_) {
+		return;
+	}
+	did_postload_tribes_ = true;
+
+	verb_log_info("Postloading tribes...");
 	for (DescriptionIndex i = 0; i < descriptions_->nr_tribes(); ++i) {
 		descriptions_->get_mutable_tribe_descr(i)->finalize_loading(*descriptions_);
 	}
+	descriptions_->finalize_loading();
 }
 
 UI::ProgressWindow& EditorGameBase::create_loader_ui(const std::vector<std::string>& tipstexts,
@@ -350,8 +369,8 @@ UI::ProgressWindow& EditorGameBase::create_loader_ui(const std::vector<std::stri
 	registered_game_tips_ = tipstexts;
 	if (show_game_tips) {
 		game_tips_.reset(registered_game_tips_.empty() ?
-		                    nullptr :
-		                    new GameTips(*loader_ui_, registered_game_tips_, all_tribes()));
+                          nullptr :
+                          new GameTips(*loader_ui_, registered_game_tips_, all_tribes()));
 	}
 	return *loader_ui_;
 }
@@ -408,7 +427,7 @@ EditorGameBase::warp_constructionsite(const Coords& c,
 	const TribeDescr& tribe = plr->tribe();
 	ConstructionSite& b = dynamic_cast<ConstructionSite&>(
 	   tribe.get_building_descr(idx)->create(*this, plr, c, true, loading, former_buildings));
-	if (settings) {
+	if (settings != nullptr) {
 		b.apply_settings(*settings);
 	}
 	b.add_dropout_wares(preserved_wares);
@@ -558,6 +577,9 @@ Player* EditorGameBase::get_safe_player(PlayerNumber const n) {
  */
 void EditorGameBase::cleanup_for_load() {
 	Notifications::publish(UI::NoteLoadingMessage(_("Cleaning up for loading: Map objects (1/3)")));
+	if (InteractiveBase* i = get_ibase()) {
+		i->cleanup_for_load();
+	}
 	cleanup_objects();  /// Clean all the stuff up, so we can load.
 
 	Notifications::publish(UI::NoteLoadingMessage(_("Cleaning up for loading: Players (2/3)")));
@@ -569,6 +591,22 @@ void EditorGameBase::cleanup_for_load() {
 	delete_tempfile();
 }
 
+/** Cleanup *everything* so we can load a completely new savegame. */
+void EditorGameBase::full_cleanup() {
+	cleanup_for_load();
+	enabled_addons().clear();
+	did_postload_addons_ = false;
+	did_postload_tribes_ = false;
+	descriptions_.reset(nullptr);
+	gametime_ = Time(0);
+	// See the comment about `lua_` in the ctor
+	if (is_game()) {
+		lua_.reset(new LuaGameInterface(dynamic_cast<Game*>(this)));
+	} else {
+		lua_.reset(new LuaEditorInterface(this));
+	}
+}
+
 void EditorGameBase::set_road(const FCoords& f,
                               uint8_t const direction,
                               RoadSegment const roadtype) {
@@ -604,10 +642,9 @@ void EditorGameBase::set_road(const FCoords& f,
 	MapIndex const i = f.field - &first_field;
 	MapIndex const neighbour_i = neighbour.field - &first_field;
 	iterate_players_existing_const(plnum, kMaxPlayers, *this, p) {
-		Player::Field& first_player_field = *p->fields_.get();
-		Player::Field& player_field = (&first_player_field)[i];
+		Player::Field& player_field = p->fields_[i];
 		if (VisibleState::kVisible == player_field.vision ||
-		    VisibleState::kVisible == (&first_player_field)[neighbour_i].vision) {
+		    VisibleState::kVisible == p->fields_[neighbour_i].vision) {
 			switch (direction) {
 			case WALK_SE:
 				player_field.r_se = roadtype;
@@ -671,7 +708,7 @@ void EditorGameBase::change_field_owner(const FCoords& fc, PlayerNumber const ne
 		return;
 	}
 
-	if (old_owner) {
+	if (old_owner != 0u) {
 		Notifications::publish(
 		   NoteFieldPossession(fc, NoteFieldPossession::Ownership::LOST, get_player(old_owner)));
 	}
@@ -683,7 +720,7 @@ void EditorGameBase::change_field_owner(const FCoords& fc, PlayerNumber const ne
 	// longer owned.
 	inform_players_about_ownership(fc.field - &first_field, new_owner);
 
-	if (new_owner) {
+	if (new_owner != 0u) {
 		Notifications::publish(
 		   NoteFieldPossession(fc, NoteFieldPossession::Ownership::GAINED, get_player(new_owner)));
 	}
@@ -744,19 +781,20 @@ void EditorGameBase::do_conquer_area(PlayerArea<Area<FCoords>> player_area,
 			//  adds the influence
 			MilitaryInfluence new_influence_modified = conquering_player->military_influence(index) +=
 			   influence;
-			if (owner && !conquer_guarded_location_by_superior_influence) {
+			if ((owner != 0u) && !conquer_guarded_location_by_superior_influence) {
 				new_influence_modified = 1;
 			}
-			if (!owner || player(owner).military_influence(index) < new_influence_modified) {
+			if ((owner == 0u) || player(owner).military_influence(index) < new_influence_modified) {
 				change_field_owner(mr.location(), player_area.player_number);
 			}
-		} else if (!(conquering_player->military_influence(index) -= influence) &&
+		} else if (((conquering_player->military_influence(index) -= influence) == 0u) &&
 		           owner == player_area.player_number) {
 			//  The player completely lost influence over the location, which he
 			//  owned. Now we must see if some other player has influence and if
 			//  so, transfer the ownership to that player.
 			PlayerNumber best_player;
-			if (preferred_player && player(preferred_player).military_influence(index)) {
+			if ((preferred_player != 0u) &&
+			    (player(preferred_player).military_influence(index) != 0u)) {
 				best_player = preferred_player;
 			} else {
 				best_player = neutral_when_no_influence ? 0 : player_area.player_number;
@@ -823,7 +861,7 @@ void EditorGameBase::cleanup_playerimmovables_area(PlayerArea<Area<FCoords>> con
 				flag_building->set_defeating_player(area.player_number);
 			}
 		}
-		if (game) {
+		if (game != nullptr) {
 			temp_imm->schedule_destroy(*game);
 		} else {
 			temp_imm->remove(*this);
