@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -200,7 +199,7 @@ public:
 		std::function<std::string()> descname;
 	};
 	static const std::vector<OldWorldInfo> kOldWorldNames;
-	static const OldWorldInfo& get_old_world_info_by_new_name(const std::string& old_name);
+	static const OldWorldInfo& get_old_world_info_by_new_name(const std::string& new_name);
 	static const OldWorldInfo& get_old_world_info_by_old_name(const std::string& old_name);
 
 	using PortSpacesSet = std::set<Coords>;
@@ -235,6 +234,7 @@ public:
 	    const std::string& description = _("No description defined"));
 
 	void recalc_whole_map(const EditorGameBase&);
+	void recalc_whole_map_brightness();
 	void recalc_for_field_area(const EditorGameBase&, Area<FCoords>);
 
 	/**
@@ -626,8 +626,8 @@ protected:
 	void calculate_minimum_required_widelands_version(bool is_post_one_world);
 
 private:
-	void recalc_border(const FCoords&);
-	void recalc_brightness(const FCoords&);
+	void recalc_border(const FCoords&) const;
+	void recalc_brightness(const FCoords&) const;
 	void recalc_nodecaps_pass1(const EditorGameBase&, const FCoords&);
 	void recalc_nodecaps_pass2(const EditorGameBase&, const FCoords& f);
 	NodeCaps
@@ -636,7 +636,7 @@ private:
 	                             const FCoords&,
 	                             bool consider_mobs = true,
 	                             NodeCaps initcaps = CAPS_NONE) const;
-	void check_neighbour_heights(FCoords, uint32_t& radius);
+	void check_neighbour_heights(FCoords, uint32_t& area);
 	int calc_buildsize(const EditorGameBase&,
 	                   const FCoords& f,
 	                   bool avoidnature,
