@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2021 by the Widelands Development Team
+ * Copyright (C) 2010-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -41,7 +40,7 @@ struct Buildcost : std::map<DescriptionIndex, uint8_t> {
 	Quantity total() const;
 
 	void save(FileWrite& fw, const TribeDescr& tribe) const;
-	void load(FileRead& fw, const TribeDescr& tribe);
+	void load(FileRead& fr, const TribeDescr& tribe);
 };
 
 }  // namespace Widelands
