@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -54,8 +53,9 @@ namespace Widelands {
 
 uint32_t Descriptions::instances_ = 0;
 
-Descriptions::Descriptions(LuaInterface* lua, const std::vector<AddOns::AddOnInfo>& addons)
+Descriptions::Descriptions(LuaInterface* lua, const AddOns::AddOnsList& addons)
    : all_tribes_(get_all_tribeinfos(&addons)),
+     addons_(addons),
      critters_(new DescriptionMaintainer<CritterDescr>()),
      immovables_(new DescriptionMaintainer<ImmovableDescr>()),
      terrains_(new DescriptionMaintainer<TerrainDescription>()),
@@ -79,35 +79,39 @@ Descriptions::Descriptions(LuaInterface* lua, const std::vector<AddOns::AddOnInf
 	// very early than to risk crashes because it was done too late…
 
 	assert(lua_);
-	for (const AddOns::AddOnInfo& info : addons) {
-		if (info.category == AddOns::AddOnCategory::kWorld ||
-		    info.category == AddOns::AddOnCategory::kTribes) {
-			const std::string script(kAddOnDir + FileSystem::file_separator() + info.internal_name +
+	for (const auto& info : addons) {
+		if (info->category == AddOns::AddOnCategory::kWorld ||
+		    info->category == AddOns::AddOnCategory::kTribes) {
+			const std::string script(kAddOnDir + FileSystem::file_separator() + info->internal_name +
 			                         FileSystem::file_separator() + "preload.lua");
 			if (g_fs->file_exists(script)) {
-				log_info("Running preload script for add-on %s", info.internal_name.c_str());
+				log_info("Running preload script for add-on %s", info->internal_name.c_str());
 				lua_->run_script(script);
 			}
 		}
 	}
 
-	for (const AddOns::AddOnInfo& info : addons) {
-		if (info.category == AddOns::AddOnCategory::kWorld) {
+	for (const auto& info : addons) {
+		if (info->category == AddOns::AddOnCategory::kWorld) {
 			description_manager_->register_directory(
-			   kAddOnDir + FileSystem::file_separator() + info.internal_name, g_fs,
-			   DescriptionManager::RegistryCaller::kWorldAddon);
-		} else if (info.category == AddOns::AddOnCategory::kTribes) {
+			   kAddOnDir + FileSystem::file_separator() + info->internal_name, g_fs,
+			   DescriptionManager::RegistryCallerInfo(
+			      DescriptionManager::RegistryCallerType::kWorldAddon, info->internal_name));
+		} else if (info->category == AddOns::AddOnCategory::kTribes) {
 			description_manager_->register_directory(
-			   kAddOnDir + FileSystem::file_separator() + info.internal_name, g_fs,
-			   DescriptionManager::RegistryCaller::kTribeAddon);
+			   kAddOnDir + FileSystem::file_separator() + info->internal_name, g_fs,
+			   DescriptionManager::RegistryCallerInfo(
+			      DescriptionManager::RegistryCallerType::kTribeAddon, info->internal_name));
 		}
 	}
 
 	// Register tribe names. Tribes have no attributes.
 	std::vector<std::string> attributes;
 	for (const TribeBasicInfo& tribeinfo : all_tribes_) {
-		description_manager_->register_description(tribeinfo.name, tribeinfo.script, attributes,
-		                                           DescriptionManager::RegistryCaller::kDefault);
+		description_manager_->register_description(
+		   tribeinfo.name, tribeinfo.script, attributes,
+		   DescriptionManager::RegistryCallerInfo(
+		      DescriptionManager::RegistryCallerType::kDefault, std::string()));
 		if (!attributes.empty()) {
 			throw GameDataError("Tribes can't have attributes - please remove all attributes in "
 			                    "'register.lua' for tribe '%s'.",
@@ -117,7 +121,9 @@ Descriptions::Descriptions(LuaInterface* lua, const std::vector<AddOns::AddOnInf
 
 	// Walk world directory and register objects
 	description_manager_->register_directory(
-	   "world", g_fs, DescriptionManager::RegistryCaller::kDefault);
+	   "world", g_fs,
+	   DescriptionManager::RegistryCallerInfo(
+	      DescriptionManager::RegistryCallerType::kDefault, std::string()));
 
 	// We register tribes on demand in load_tribe for performance reasons
 }
@@ -391,13 +397,14 @@ TribeDescr* Descriptions::get_mutable_tribe_descr(DescriptionIndex index) const
 void Descriptions::register_scenario_tribes(FileSystem* filesystem) {
 	// If the map is a scenario with custom tribe entites, load them.
 	if (filesystem->file_exists("scripting/tribes")) {
-		scenario_tribes_.reset(nullptr);
 		description_manager_->clear_scenario_descriptions();
-		if (filesystem->file_exists("scripting/tribes/init.lua")) {
+		if (scenario_tribes_ == nullptr && filesystem->file_exists("scripting/tribes/init.lua")) {
 			scenario_tribes_ = lua_->run_script("map:scripting/tribes/init.lua");
 		}
 		description_manager_->register_directory(
-		   "scripting/tribes", filesystem, DescriptionManager::RegistryCaller::kScenario);
+		   "scripting/tribes", filesystem,
+		   DescriptionManager::RegistryCallerInfo(
+		      DescriptionManager::RegistryCallerType::kScenario, std::string()));
 	}
 }
 
@@ -423,7 +430,7 @@ void Descriptions::add_object_description(const LuaTable& table, MapObjectType t
 		resources_->add(new ResourceDescription(table));
 		break;
 	case MapObjectType::TERRAIN:
-		terrains_->add(new TerrainDescription(table, *this));
+		add_terrain_description(type_name, table);
 		break;
 	case MapObjectType::CARRIER:
 		workers_->add(new CarrierDescr(type_descname, table, *this));
@@ -476,6 +483,28 @@ void Descriptions::add_object_description(const LuaTable& table, MapObjectType t
 	description_manager_->mark_loading_done(type_name);
 }
 
+void Descriptions::add_terrain_description(const std::string& type_name, const LuaTable& table) {
+	const DescriptionManager::RegistryCallerInfo& c =
+	   description_manager_->get_registry_caller_info(type_name);
+	assert(c.second.empty() ^ (c.first == DescriptionManager::RegistryCallerType::kWorldAddon));
+	uint32_t index = 0;
+	if (!c.second.empty()) {
+		index = addons_.size();
+		for (uint32_t i = 0; i < addons_.size(); ++i) {
+			if (addons_.at(i)->internal_name == c.second) {
+				index = i;
+				break;
+			}
+		}
+		if (index >= addons_.size()) {
+			throw wexception("Terrain %s defined by add-on %s which is not enabled", type_name.c_str(),
+			                 c.second.c_str());
+		}
+		++index;  // To avoid conflicts between add-on number 0 and official units.
+	}
+	terrains_->add(new TerrainDescription(table, *this, index));
+}
+
 void Descriptions::add_tribe(const LuaTable& table) {
 	const std::string name = table.get_string("name");
 	// Register as in progress
@@ -499,15 +528,22 @@ void Descriptions::add_tribe(const LuaTable& table) {
 	description_manager_->mark_loading_done(name);
 }
 
+void Descriptions::ensure_tribes_are_registered() {
+	if (tribes_have_been_registered_) {
+		return;
+	}
+	description_manager_->register_directory(
+	   "tribes", g_fs,
+	   DescriptionManager::RegistryCallerInfo(
+	      DescriptionManager::RegistryCallerType::kDefault, std::string()));
+	tribes_have_been_registered_ = true;
+}
+
 DescriptionIndex Descriptions::load_tribe(const std::string& tribename) {
 	try {
 		// Register tribes on demand for better performance during mapselect, for the editor and for
 		// the website tools
-		if (!tribes_have_been_registered_) {
-			description_manager_->register_directory(
-			   "tribes", g_fs, DescriptionManager::RegistryCaller::kDefault);
-			tribes_have_been_registered_ = true;
-		}
+		ensure_tribes_are_registered();
 		description_manager_->load_description(tribename);
 	} catch (WException& e) {
 		throw GameDataError("Error while loading tribe '%s': %s", tribename.c_str(), e.what());
@@ -619,12 +655,17 @@ void Descriptions::set_old_world_name(const std::string& name) {
 	}
 }
 
+void Descriptions::finalize_loading() {
+	postload_immovable_relations();
+	scenario_tribes_.reset();
+}
+
 void Descriptions::add_immovable_relation(const std::string& a, const std::string& b) {
-	immovable_relations_.push_back(std::make_pair(a, b));
+	immovable_relations_.emplace_back(a, b);
 }
 void Descriptions::postload_immovable_relations() {
 	for (const auto& pair : immovable_relations_) {
-		get_mutable_immovable_descr(load_immovable(pair.second))->add_became_from(pair.first);
+		get_mutable_immovable_descr(load_immovable(pair.second))->add_became_from(*this, pair.first);
 	}
 	immovable_relations_.clear();
 }
@@ -636,13 +677,21 @@ void Descriptions::postload_immovable_relations() {
 	}
 
 void Descriptions::check(const DescriptionManager::NoteMapObjectDescriptionTypeCheck& note) const {
-	switch (note.caller) {
-	case DescriptionManager::RegistryCaller::kTribeAddon:
+	switch (note.caller.first) {
+	case DescriptionManager::RegistryCallerType::kTribeAddon:
+		if (note.caller.second.empty()) {
+			throw wexception("Unidentified registry caller for tribe add-on (unit: %s)",
+			                 note.description_name.c_str());
+		}
 		CHECK_FACTORY(Tribe, critter)
 		CHECK_FACTORY(Tribe, terrain)
 		CHECK_FACTORY(Tribe, resource)
 		break;
-	case DescriptionManager::RegistryCaller::kWorldAddon:
+	case DescriptionManager::RegistryCallerType::kWorldAddon:
+		if (note.caller.second.empty()) {
+			throw wexception("Unidentified registry caller for world add-on (unit: %s)",
+			                 note.description_name.c_str());
+		}
 		CHECK_FACTORY(World, tribe)
 		CHECK_FACTORY(World, ware)
 		CHECK_FACTORY(World, worker)
@@ -650,6 +699,10 @@ void Descriptions::check(const DescriptionManager::NoteMapObjectDescriptionTypeC
 		CHECK_FACTORY(World, ship)
 		break;
 	default:
+		if (!note.caller.second.empty()) {
+			throw wexception("Unexpected registry caller identifier '%s' for offical/scenario unit %s",
+			                 note.caller.second.c_str(), note.description_name.c_str());
+		}
 		// Scenarios and official scripts may load anything
 		break;
 	}
