@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -133,6 +132,10 @@ terrains_
 	{"schnee", "summer_snow"},
 	{"wasser", "summer_water"},
 	{"ashes", "ashes1"},
+	{"taiga", "winter_tundra"},
+	{"tundra", "winter_taiga1"},
+	{"tundra2", "winter_taiga2"},
+	{"tundra3", "winter_taiga3"},
 }
 {
 }
@@ -185,6 +188,10 @@ OneWorldLegacyLookupTable::OneWorldLegacyLookupTable(const std::string& old_worl
                           {"strand", "summer_beach"},
                           {"schnee", "summer_snow"},
                           {"wasser", "summer_water"},
+                          {"taiga", "winter_tundra"},
+                          {"tundra", "winter_taiga1"},
+                          {"tundra2", "winter_taiga2"},
+                          {"tundra3", "winter_taiga3"},
                        }),
         std::make_pair("blackland",
                        std::map<std::string, std::string>{
@@ -195,6 +202,10 @@ OneWorldLegacyLookupTable::OneWorldLegacyLookupTable(const std::string& old_worl
                           {"mountain4", "wasteland_mountain4"},
                           {"strand", "wasteland_beach"},
                           {"water", "wasteland_water"},
+                          {"taiga", "winter_tundra"},
+                          {"tundra", "winter_taiga1"},
+                          {"tundra2", "winter_taiga2"},
+                          {"tundra3", "winter_taiga3"},
                        }),
         std::make_pair("winterland",
                        std::map<std::string, std::string>{
@@ -207,6 +218,10 @@ OneWorldLegacyLookupTable::OneWorldLegacyLookupTable(const std::string& old_worl
                           {"mountain4", "winter_mountain4"},
                           {"strand", "winter_beach"},
                           {"water", "winter_water"},
+                          {"taiga", "winter_tundra"},
+                          {"tundra", "winter_taiga1"},
+                          {"tundra2", "winter_taiga2"},
+                          {"tundra3", "winter_taiga3"},
                        }),
         std::make_pair("desert",
                        std::map<std::string, std::string>{
@@ -214,6 +229,10 @@ OneWorldLegacyLookupTable::OneWorldLegacyLookupTable(const std::string& old_worl
                           {"beach", "desert_beach"},
                           {"steppe", "desert_steppe"},
                           {"wasser", "desert_water"},
+                          {"taiga", "winter_tundra"},
+                          {"tundra", "winter_taiga1"},
+                          {"tundra2", "winter_taiga2"},
+                          {"tundra3", "winter_taiga3"},
                        }),
      },
 
