@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -24,6 +23,7 @@
 #include <memory>
 
 #include "base/log.h"
+#include "base/multithreading.h"
 #include "base/wexception.h"
 #include "graphic/animation/animation_manager.h"
 #include "graphic/font_handler.h"
@@ -168,9 +168,11 @@ void ObjectManager::cleanup(EditorGameBase& egbase) {
 	// Destruction happens in correct order after this dirty quickie.
 	// Run at the end of game, algorithmic efficiency may be what it is.
 	const static std::vector<MapObjectType> killusfirst{
-	   MapObjectType::WATERWAY, MapObjectType::FERRY,      MapObjectType::FERRY_FLEET,
-	   MapObjectType::SHIP,     MapObjectType::SHIP_FLEET, MapObjectType::PORTDOCK,
-	   MapObjectType::WORKER};
+	   MapObjectType::WARE,        MapObjectType::WATERWAY, MapObjectType::FERRY,
+	   MapObjectType::FERRY_FLEET, MapObjectType::SHIP,     MapObjectType::SHIP_FLEET,
+	   MapObjectType::PORTDOCK,    MapObjectType::WORKER,   MapObjectType::CARRIER,
+	   MapObjectType::SOLDIER,
+	};
 	for (auto moi : killusfirst) {
 		while (!objects_.empty()) {
 			MapObjectMap::iterator it = objects_.begin();
@@ -225,11 +227,11 @@ std::vector<Serial> ObjectManager::all_object_serials_ordered() const {
 }
 
 MapObject* ObjectPointer::get(const EditorGameBase& egbase) {
-	if (!serial_) {
+	if (serial_ == 0u) {
 		return nullptr;
 	}
 	MapObject* const obj = egbase.objects().get_object(serial_);
-	if (!obj) {
+	if (obj == nullptr) {
 		serial_ = 0;
 	}
 	return obj;
@@ -240,7 +242,7 @@ MapObject* ObjectPointer::get(const EditorGameBase& egbase) {
 // that is pointed to.
 // That is, a 'const ObjectPointer' behaves like a 'ObjectPointer * const'.
 MapObject* ObjectPointer::get(const EditorGameBase& egbase) const {
-	return serial_ ? egbase.objects().get_object(serial_) : nullptr;
+	return serial_ != 0u ? egbase.objects().get_object(serial_) : nullptr;
 }
 
 /*
@@ -377,7 +379,7 @@ void MapObjectDescr::add_animations(const LuaTable& table,
 }
 
 void MapObjectDescr::assign_directional_animation(DirAnimations* anims,
-                                                  const std::string& basename) {
+                                                  const std::string& basename) const {
 	for (int32_t dir = 1; dir <= 6; ++dir) {
 		const std::string anim_name = basename + animation_direction_names[dir - 1];
 		try {
@@ -388,7 +390,8 @@ void MapObjectDescr::assign_directional_animation(DirAnimations* anims,
 	}
 }
 
-uint32_t MapObjectDescr::get_animation(const std::string& animname, const MapObject*) const {
+uint32_t MapObjectDescr::get_animation(const std::string& animname,
+                                       const MapObject* /* object */) const {
 	std::map<std::string, uint32_t>::const_iterator it = anims_.find(animname);
 	if (it == anims_.end()) {
 		throw GameDataError("Unknown animation: %s for %s", animname.c_str(), name().c_str());
@@ -426,7 +429,7 @@ const Image* MapObjectDescr::representative_image(const RGBColor* player_color)
 	return nullptr;
 }
 
-void MapObjectDescr::check_representative_image() {
+void MapObjectDescr::check_representative_image() const {
 	if (representative_image() == nullptr) {
 		throw Widelands::GameDataError(
 		   "The %s %s has no representative image. Does it have an \"idle\" animation?",
@@ -536,6 +539,8 @@ MapObject::MapObject(const MapObjectDescr* const the_descr)
  * any effects.
  */
 void MapObject::remove(EditorGameBase& egbase) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	removed(serial_);  // Signal call
 	cleanup(egbase);
 	delete this;
@@ -570,6 +575,7 @@ void MapObject::schedule_destroy(Game& game) {
  * \warning Make sure you call this from derived classes!
  */
 bool MapObject::init(EditorGameBase& egbase) {
+	MutexLock m(MutexLock::ID::kObjects);
 	egbase.objects().insert(this);
 	return true;
 }
@@ -587,7 +593,7 @@ void MapObject::do_draw_info(const InfoToDraw& info_to_draw,
                              const Vector2f& field_on_dst,
                              float scale,
                              RenderTarget* dst) const {
-	if (!(info_to_draw & (InfoToDraw::kCensus | InfoToDraw::kStatistics))) {
+	if ((info_to_draw & (InfoToDraw::kCensus | InfoToDraw::kStatistics)) == 0) {
 		return;
 	}
 
@@ -611,12 +617,12 @@ void MapObject::do_draw_info(const InfoToDraw& info_to_draw,
 	std::shared_ptr<const UI::RenderedText> rendered_census =
 	   UI::g_fh->render(as_richtext_paragraph(census, census_font, UI::Align::kCenter), 120 * scale);
 	Vector2i position = field_on_dst.cast<int>() - Vector2i(0, 48) * scale;
-	if (info_to_draw & InfoToDraw::kCensus) {
+	if ((info_to_draw & InfoToDraw::kCensus) != 0) {
 		rendered_census->draw(*dst, position, UI::Align::kCenter);
 	}
 
 	// Draw statistics if we want them, they are available and they fill fit
-	if (info_to_draw & InfoToDraw::kStatistics && !statictics.empty() && scale >= 0.5f) {
+	if (((info_to_draw & InfoToDraw::kStatistics) != 0) && !statictics.empty() && scale >= 0.5f) {
 		UI::FontStyleInfo statistics_font(
 		   g_style_manager->building_statistics_style().statistics_font());
 		statistics_font.set_size(scale * statistics_font.size());
@@ -629,13 +635,14 @@ void MapObject::do_draw_info(const InfoToDraw& info_to_draw,
 }
 
 const Image* MapObject::representative_image() const {
-	return descr().representative_image(get_owner() ? &get_owner()->get_playercolor() : nullptr);
+	return descr().representative_image(get_owner() != nullptr ? &get_owner()->get_playercolor() :
+                                                                nullptr);
 }
 
 /**
  * Default implementation
  */
-int32_t MapObject::get_training_attribute(TrainingAttribute) const {
+int32_t MapObject::get_training_attribute(TrainingAttribute /* attr */) const {
 	return -1;
 }
 
@@ -658,7 +665,7 @@ Time MapObject::schedule_act(Game& game, const Duration& tdelta, uint32_t const
 /**
  * Called when a CMD_ACT triggers.
  */
-void MapObject::act(Game&, uint32_t) {
+void MapObject::act(Game& /* game */, uint32_t /* data */) {
 }
 
 /**
@@ -668,7 +675,7 @@ void MapObject::set_logsink(LogSink* const sink) {
 	logsink_ = sink;
 }
 
-void MapObject::log_general_info(const EditorGameBase&) const {
+void MapObject::log_general_info(const EditorGameBase& /* egbase */) const {
 }
 
 const Player& MapObject::owner() const {
@@ -682,7 +689,7 @@ const Player& MapObject::owner() const {
  * Prints a log message prepended by the object's serial number.
  */
 void MapObject::molog(const Time& gametime, char const* fmt, ...) const {
-	if (!g_verbose && !logsink_) {
+	if (!g_verbose && (logsink_ == nullptr)) {
 		return;
 	}
 
@@ -693,7 +700,7 @@ void MapObject::molog(const Time& gametime, char const* fmt, ...) const {
 	vsnprintf(buffer, sizeof(buffer), fmt, va);
 	va_end(va);
 
-	if (logsink_) {
+	if (logsink_ != nullptr) {
 		logsink_->log(buffer);
 	}
 
@@ -741,7 +748,7 @@ void MapObject::Loader::load(FileRead& fr) {
 
 		if (packet_version >= 2) {
 			MapObject& obj = *get_object();
-			obj.reserved_by_worker_ = fr.unsigned_8();
+			obj.reserved_by_worker_ = (fr.unsigned_8() != 0u);
 		}
 	} catch (const WException& e) {
 		throw wexception("map object: %s", e.what());
@@ -779,12 +786,12 @@ void MapObject::Loader::load_finish() {
 /**
  * Save the MapObject to the given file.
  */
-void MapObject::save(EditorGameBase&, MapObjectSaver& mos, FileWrite& fw) {
+void MapObject::save(EditorGameBase& /* egbase */, MapObjectSaver& mos, FileWrite& fw) {
 	fw.unsigned_8(HeaderMapObject);
 	fw.unsigned_8(kCurrentPacketVersionMapObject);
 
 	fw.unsigned_32(mos.get_object_file_index(*this));
-	fw.unsigned_8(reserved_by_worker_);
+	fw.unsigned_8(static_cast<uint8_t>(reserved_by_worker_));
 }
 
 }  // namespace Widelands
