@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -21,10 +20,10 @@
 
 #include <memory>
 
-#include <boost/algorithm/string.hpp>
-
 #include "base/log.h"
 #include "base/macros.h"
+#include "base/multithreading.h"
+#include "base/string.h"
 #include "base/wexception.h"
 #include "economy/flag.h"
 #include "economy/input_queue.h"
@@ -78,16 +77,16 @@ BuildingDescr::BuildingDescr(const std::string& init_descname,
 	if (type() != MapObjectType::CONSTRUCTIONSITE && type() != MapObjectType::DISMANTLESITE) {
 		try {
 			const std::string size = table.get_string("size");
-			if (boost::iequals(size, "small")) {
+			if (iequals(size, "small")) {
 				size_ = BaseImmovable::SMALL;
-			} else if (boost::iequals(size, "medium")) {
+			} else if (iequals(size, "medium")) {
 				size_ = BaseImmovable::MEDIUM;
-			} else if (boost::iequals(size, "big")) {
+			} else if (iequals(size, "big")) {
 				size_ = BaseImmovable::BIG;
-			} else if (boost::iequals(size, "mine")) {
+			} else if (iequals(size, "mine")) {
 				size_ = BaseImmovable::SMALL;
 				mine_ = true;
-			} else if (boost::iequals(size, "port")) {
+			} else if (iequals(size, "port")) {
 				size_ = BaseImmovable::BIG;
 				port_ = true;
 			} else {
@@ -123,7 +122,7 @@ BuildingDescr::BuildingDescr(const std::string& init_descname,
 				//  Merge the enhancements workarea info into this building's
 				//  workarea info.
 				const BuildingDescr* tmp_enhancement = descriptions.get_building_descr(en_i);
-				for (auto area : tmp_enhancement->workarea_info_) {
+				for (const auto& area : tmp_enhancement->workarea_info_) {
 					std::set<std::string>& strs = workarea_info_[area.first];
 					for (const std::string& str : area.second) {
 						strs.insert(str);
@@ -207,7 +206,7 @@ void BuildingDescr::set_enhancement(Descriptions& descriptions, LuaTable& enhanc
 	   Buildcost(enhancement_table.get_table("enhancement_return_on_dismantle"), descriptions));
 
 	// Merge the enhancements workarea info into this building's workarea info
-	for (auto area : enhanced_building->workarea_info_) {
+	for (const auto& area : enhanced_building->workarea_info_) {
 		std::set<std::string>& strs = workarea_info_[area.first];
 		for (const std::string& str : area.second) {
 			strs.insert(str);
@@ -246,7 +245,7 @@ Building& BuildingDescr::create(EditorGameBase& egbase,
 		if (!immovable_previously_found) {
 			// Must be done first, because the immovable will be gone the moment the building is placed
 			const FCoords f = egbase.map().get_fcoords(pos);
-			if (f.field->get_immovable() &&
+			if ((f.field->get_immovable() != nullptr) &&
 			    f.field->get_immovable()->has_attribute(built_over_immovable_)) {
 				upcast(const ImmovableDescr, imm, &f.field->get_immovable()->descr());
 				assert(imm);
@@ -278,13 +277,14 @@ Building& BuildingDescr::create(EditorGameBase& egbase,
 	return b;
 }
 
-bool BuildingDescr::suitability(const Map&, const FCoords& fc) const {
-	return (mine_ ? fc.field->nodecaps() & Widelands::BUILDCAPS_MINE :
-	                size_ <= ((built_over_immovable_ == INVALID_INDEX ? fc.field->nodecaps() :
-	                                                                    fc.field->maxcaps()) &
-	                          Widelands::BUILDCAPS_SIZEMASK)) &&
+bool BuildingDescr::suitability(const Map& /* map */, const FCoords& fc) const {
+	return ((mine_ ? fc.field->nodecaps() & Widelands::BUILDCAPS_MINE :
+                    static_cast<int>(
+	                    size_ <= ((built_over_immovable_ == INVALID_INDEX ? fc.field->nodecaps() :
+                                                                           fc.field->maxcaps()) &
+	                              Widelands::BUILDCAPS_SIZEMASK))) != 0) &&
 	       (built_over_immovable_ == INVALID_INDEX ||
-	        (fc.field->get_immovable() &&
+	        ((fc.field->get_immovable() != nullptr) &&
 	         fc.field->get_immovable()->has_attribute(built_over_immovable_)));
 }
 
@@ -327,7 +327,7 @@ uint32_t BuildingDescr::get_conquers() const {
  * building.
  */
 uint32_t BuildingDescr::vision_range() const {
-	return vision_range_ ? vision_range_ : get_conquers() + 4;
+	return vision_range_ != 0u ? vision_range_ : get_conquers() + 4;
 }
 
 /*
@@ -391,7 +391,7 @@ void Building::load_finish(EditorGameBase& egbase) {
 		}
 
 		Bob::State const* const state = worker.get_state(Worker::taskLeavebuilding);
-		if (!state) {
+		if (state == nullptr) {
 			log_warn("worker %u is in the leave queue of building %u but "
 			         "does not have a leavebuilding task! Removing from queue.\n",
 			         worker.serial(), serial());
@@ -457,6 +457,8 @@ derived class' init.
 ===============
 */
 bool Building::init(EditorGameBase& egbase) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	PlayerImmovable::init(egbase);
 
 	// Set the building onto the map
@@ -481,7 +483,7 @@ bool Building::init(EditorGameBase& egbase) {
 	map.get_brn(position_, &neighb);
 	{
 		Flag* flag = dynamic_cast<Flag*>(map.get_immovable(neighb));
-		if (!flag) {
+		if (flag == nullptr) {
 			flag = new Flag(egbase, get_owner(), neighb);
 		}
 		flag_ = flag;
@@ -512,9 +514,9 @@ bool Building::init(EditorGameBase& egbase) {
 void Building::cleanup(EditorGameBase& egbase) {
 	set_seeing(false);
 
-	if (defeating_player_) {
+	if (defeating_player_ != 0u) {
 		Player* defeating_player = egbase.get_player(defeating_player_);
-		if (descr().get_conquers()) {
+		if (descr().get_conquers() != 0u) {
 			get_owner()->count_msite_lost();
 			defeating_player->count_msite_defeated();
 		} else {
@@ -621,7 +623,9 @@ std::string Building::info_string(const InfoStringFormat& format) {
 	return result;
 }
 
-InputQueue& Building::inputqueue(DescriptionIndex const wi, WareWorker const, const Request*) {
+InputQueue& Building::inputqueue(DescriptionIndex const wi,
+                                 WareWorker const /* type */,
+                                 const Request* /* req */) {
 	throw wexception("%s (%u) has no InputQueue for %u", descr().name().c_str(), serial(), wi);
 }
 
@@ -634,7 +638,7 @@ signal).
 Return false if there's nothing to be done.
 ===============
 */
-bool Building::get_building_work(Game&, Worker& worker, bool) {
+bool Building::get_building_work(Game& /* game */, Worker& worker, bool /* success */) {
 	throw wexception("MO(%u): get_building_work() for unknown worker %u", serial(), worker.serial());
 }
 
@@ -680,7 +684,7 @@ bool Building::leave_check_and_wait(Game& game, Worker& w) {
  *
  * \see Building::leave_check_and_wait()
  */
-void Building::leave_skip(Game&, Worker& w) {
+void Building::leave_skip(Game& /* game */, Worker& w) {
 	LeaveQueue::iterator const it = std::find(leave_queue_.begin(), leave_queue_.end(), &w);
 
 	if (it != leave_queue_.end()) {
@@ -705,7 +709,7 @@ void Building::act(Game& game, uint32_t const data) {
 
 			leave_queue_.erase(leave_queue_.begin());
 
-			if (worker) {
+			if (worker != nullptr) {
 				leave_allow_ = worker;
 
 				if (worker->wakeup_leave_building(game, *this)) {
@@ -751,8 +755,8 @@ void Building::draw(const Time& gametime,
                     RenderTarget* dst) {
 	const Time t((gametime - animstart_).get());
 
-	if (was_immovable_) {
-		if (info_to_draw & InfoToDraw::kShowBuildings) {
+	if (was_immovable_ != nullptr) {
+		if ((info_to_draw & InfoToDraw::kShowBuildings) != 0) {
 			dst->blit_animation(point_on_dst, coords, scale, was_immovable_->main_animation(), t,
 			                    &get_owner()->get_playercolor());
 		} else {
@@ -761,7 +765,7 @@ void Building::draw(const Time& gametime,
 		}
 	}
 
-	if (info_to_draw & InfoToDraw::kShowBuildings) {
+	if ((info_to_draw & InfoToDraw::kShowBuildings) != 0) {
 		dst->blit_animation(point_on_dst, coords, scale, anim_, t, &get_owner()->get_playercolor());
 	} else {
 		dst->blit_animation(
@@ -783,8 +787,9 @@ void Building::draw_info(const InfoToDraw info_to_draw,
                          const Vector2f& point_on_dst,
                          const float scale,
                          RenderTarget* dst) {
-	const std::string statistics_string =
-	   (info_to_draw & InfoToDraw::kStatistics) ? info_string(InfoStringFormat::kStatistics) : "";
+	const std::string statistics_string = (info_to_draw & InfoToDraw::kStatistics) != 0 ?
+                                            info_string(InfoStringFormat::kStatistics) :
+                                            "";
 	do_draw_info(info_to_draw, info_string(InfoStringFormat::kCensus), statistics_string,
 	             point_on_dst, scale, dst);
 }
@@ -809,7 +814,7 @@ void Building::set_priority(const WareWorker type,
                             const WarePriority& new_priority) {
 	if (type == wwWARE) {
 		// WarePriority is not default-constructible, so no [] access :(
-		if (ware_priorities_.count(ware_index)) {
+		if (ware_priorities_.count(ware_index) != 0u) {
 			ware_priorities_.at(ware_index) = new_priority;
 		} else {
 			ware_priorities_.emplace(ware_index, new_priority);
@@ -839,6 +844,8 @@ void Building::log_general_info(const EditorGameBase& egbase) const {
 }
 
 void Building::add_worker(Worker& worker) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	// Builders should make partially finished building see, but not finished buildings.
 	// So we prevent builders from seeing here and override this in PartiallyFinishedBuilding.
 	if (owner().tribe().safe_worker_index(worker.descr().name()) != owner().tribe().builder()) {
@@ -849,6 +856,8 @@ void Building::add_worker(Worker& worker) {
 }
 
 void Building::remove_worker(Worker& worker) {
+	MutexLock m(MutexLock::ID::kObjects);
+
 	PlayerImmovable::remove_worker(worker);
 	if (get_workers().empty() && descr().type() != MapObjectType::WAREHOUSE) {
 		set_seeing(false);
@@ -856,7 +865,7 @@ void Building::remove_worker(Worker& worker) {
 	Notifications::publish(NoteBuilding(serial(), NoteBuilding::Action::kWorkersChanged));
 }
 
-void Building::notify_worker_evicted(Game& game, Worker&) {
+void Building::notify_worker_evicted(Game& game, Worker& /* worker */) {
 	// If the building was working, we do not tell it to cancel – it'll notice by itself soon –
 	// but we already change the animation so it won't look strange
 	start_animation(game, descr().get_unoccupied_animation());
