@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -81,7 +80,7 @@ void Carrier::road_update(Game& game, State& state) {
 	}
 
 	if (promised_pickup_to_ != NOONE) {
-		if (state.ivar1) {
+		if (state.ivar1 != 0) {
 			state.ivar1 = 0;
 			return start_task_transport(game, promised_pickup_to_);
 		}
@@ -120,7 +119,7 @@ void Carrier::road_update(Game& game, State& state) {
  * a ware there, we have to make sure that they do not count on us anymore.
  */
 void Carrier::road_pop(Game& game, State& /* state */) {
-	if (promised_pickup_to_ != NOONE && get_location(game)) {
+	if (promised_pickup_to_ != NOONE && (get_location(game) != nullptr)) {
 		RoadBase& road = dynamic_cast<RoadBase&>(*get_location(game));
 		Flag& flag = road.get_flag(static_cast<RoadBase::FlagId>(promised_pickup_to_));
 		Flag& otherflag = road.get_flag(static_cast<RoadBase::FlagId>(promised_pickup_to_ ^ 1));
@@ -168,10 +167,15 @@ void Carrier::transport_update(Game& game, State& state) {
 		// If we don't carry something, walk to the flag
 		pickup_from_flag(game, state);
 	} else {
-		RoadBase& road = dynamic_cast<RoadBase&>(*get_location(game));
+		RoadBase* road = dynamic_cast<RoadBase*>(get_location(game));
+		if (road == nullptr) {
+			molog(game.get_gametime(), "[transport]: Road was deleted, cancel");
+			send_signal(game, "cancel");
+			return pop_task(game);
+		}
 		// If the ware should go to the building attached to our flag, walk
 		// directly into said building
-		Flag& flag = road.get_flag(static_cast<RoadBase::FlagId>(state.ivar1 ^ 1));
+		Flag& flag = road->get_flag(static_cast<RoadBase::FlagId>(state.ivar1 ^ 1));
 
 		WareInstance& ware = *get_carried_ware(game);
 		assert(ware.get_location(game) == this);
@@ -179,13 +183,13 @@ void Carrier::transport_update(Game& game, State& state) {
 		// A sanity check is necessary, in case the building has been destroyed
 		PlayerImmovable* const next = ware.get_next_move_step(game);
 
-		if (next && next != &flag && &next->base_flag() == &flag &&
-		    road.descr().type() == MapObjectType::ROAD) {
+		if ((next != nullptr) && next != &flag && &next->base_flag() == &flag &&
+		    road->descr().type() == MapObjectType::ROAD) {
 			// Pay some coins before entering the building,
 			// to compensate for the time to be spent in its street-segment.
 			// Ferries cannot enter buildings, so they lave the ware at the flag
 			// for the building's worker to fetch it.
-			if (upcast(Road, r, &road)) {
+			if (upcast(Road, r, road)) {
 				r->pay_for_building();
 			}
 			enter_building(game, state);
@@ -209,7 +213,7 @@ void Carrier::transport_update(Game& game, State& state) {
 void Carrier::deliver_to_building(Game& game, State& state) {
 	BaseImmovable* const pos = game.map()[get_position()].get_immovable();
 
-	if (pos && pos->descr().type() == Widelands::MapObjectType::FLAG) {
+	if ((pos != nullptr) && pos->descr().type() == Widelands::MapObjectType::FLAG) {
 		return pop_task(game);  //  we are done
 	}
 	if (upcast(Building, building, pos)) {
@@ -227,8 +231,9 @@ void Carrier::deliver_to_building(Game& game, State& state) {
 				molog(
 				   game.get_gametime(), "[Carrier]: Building switch from under us, return to road.\n");
 
-				state.ivar1 = &building->base_flag() == &dynamic_cast<RoadBase&>(*get_location(game))
-				                                            .get_flag(static_cast<RoadBase::FlagId>(0));
+				state.ivar1 = static_cast<int32_t>(&building->base_flag() ==
+				                                   &dynamic_cast<RoadBase&>(*get_location(game))
+				                                       .get_flag(static_cast<RoadBase::FlagId>(0)));
 				break;
 			}
 		}
@@ -236,11 +241,9 @@ void Carrier::deliver_to_building(Game& game, State& state) {
 		// No more deliverable wares. Walk out to the flag.
 		return start_task_move(
 		   game, WALK_SE, descr().get_right_walk_anims(does_carry_ware(), this), true);
-	} else {
-		//  tough luck, the building has disappeared
-		molog(game.get_gametime(), "[Carrier]: Building disappeared while in building.\n");
-		set_location(nullptr);
-	}
+	}  //  tough luck, the building has disappeared
+	molog(game.get_gametime(), "[Carrier]: Building disappeared while in building.\n");
+	set_location(nullptr);
 }
 
 /**
@@ -292,7 +295,7 @@ void Carrier::drop_ware(Game& game, State& state) {
 		// flooded
 		other = flag.fetch_pending_ware(game, otherflag);
 
-		if (!other && !flag.has_capacity()) {
+		if ((other == nullptr) && !flag.has_capacity()) {
 			molog(game.get_gametime(), "[Carrier]: strange: acked ware from busy flag no longer "
 			                           "present.\n");
 
@@ -309,7 +312,7 @@ void Carrier::drop_ware(Game& game, State& state) {
 	flag.add_ware(game, *fetch_carried_ware(game));
 
 	// Pick up new load, if any
-	if (other) {
+	if (other != nullptr) {
 		// pay before getting the ware, while checking for road promotion
 		if (upcast(Road, r, &road)) {
 			r->pay_for_road(game, flag.count_wares_in_queue(otherflag));
@@ -470,7 +473,8 @@ int32_t Carrier::find_closest_flag(const Game& game) {
 	CoordPath startpath(map, dynamic_cast<RoadBase&>(*get_location(game)).get_path());
 
 	CoordPath endpath;
-	int32_t startcost, endcost;
+	int32_t startcost;
+	int32_t endcost;
 	int32_t curidx = startpath.get_index(get_position());
 
 	// Apparently, we're in a building
@@ -501,7 +505,7 @@ int32_t Carrier::find_closest_flag(const Game& game) {
 	map.calc_cost(startpath, &startcost, nullptr);
 	map.calc_cost(endpath, &endcost, nullptr);
 
-	return endcost < startcost;
+	return static_cast<int32_t>(endcost < startcost);
 }
 
 /**
@@ -514,7 +518,7 @@ bool Carrier::start_task_walktoflag(Game& game, int32_t const flag, bool const o
 	const Path& path = dynamic_cast<RoadBase&>(*get_location(game)).get_path();
 	int32_t idx;
 
-	if (!flag) {
+	if (flag == 0) {
 		idx = 0;
 		if (offset) {
 			++idx;
