@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -83,7 +82,7 @@ this::
    programs = {
       main = {
          -- TRANSLATORS: Completed/Skipped/Did not start working because ...
-         descname = _"working",
+         descname = _("working"),
          actions = {
             <list of actions>
          }
@@ -109,7 +108,7 @@ A program can call another program, for example::
    programs = {
       main = {
          -- TRANSLATORS: Completed/Skipped/Did not start working because ...
-         descname = _"working",
+         descname = _("working"),
          actions = {
             "call=produce_ration",
             "call=produce_snack",
@@ -118,14 +117,14 @@ A program can call another program, for example::
       },
       produce_ration = {
          -- TRANSLATORS: Completed/Skipped/Did not start preparing a ration because ...
-         descname = _"preparing a ration",
+         descname = _("preparing a ration"),
          actions = {
             <list of actions>
          }
       },
       produce_snack = {
          -- TRANSLATORS: Completed/Skipped/Did not start preparing a snack because ...
-         descname = _"preparing a snack",
+         descname = _("preparing a snack"),
          actions = {
             <list of actions>
          }
@@ -137,7 +136,7 @@ A program consists of a sequence of actions. An action is written as
 
    produce_snack = {
       -- TRANSLATORS: Completed/Skipped/Did not start preparing a snack because ...
-      descname = _"preparing a snack",
+      descname = _("preparing a snack"),
       actions = {
          "return=skipped unless economy needs snack",
          "sleep=duration:2s500ms",
@@ -212,11 +211,15 @@ TrainingAttribute parse_training_attribute(const std::string& argument) {
 }
 }  // namespace
 
-bool ProductionProgram::Action::get_building_work(Game&, ProductionSite&, Worker&) const {
+bool ProductionProgram::Action::get_building_work(Game& /* game */,
+                                                  ProductionSite& /* site */,
+                                                  Worker& /* worker */) const {
 	return false;
 }
 
-void ProductionProgram::Action::building_work_failed(Game&, ProductionSite&, Worker&) const {
+void ProductionProgram::Action::building_work_failed(Game& /* game */,
+                                                     ProductionSite& /* site */,
+                                                     Worker& /* worker */) const {
 }
 
 ProductionProgram::Groups
@@ -289,8 +292,8 @@ BillOfMaterials ProductionProgram::parse_bill_of_materials(
 		const std::pair<std::string, std::string> produceme = read_key_value_pair(argument, ':', "1");
 
 		const DescriptionIndex index = ww == WareWorker::wwWARE ?
-		                                  descriptions.load_ware(produceme.first) :
-		                                  descriptions.load_worker(produceme.first);
+                                        descriptions.load_ware(produceme.first) :
+                                        descriptions.load_worker(produceme.first);
 
 		result.push_back(std::make_pair(index, read_positive(produceme.second)));
 	}
@@ -452,20 +455,18 @@ bool ProductionProgram::ActReturn::EconomyNeedsWare::evaluate(const ProductionSi
 }
 std::string ProductionProgram::ActReturn::EconomyNeedsWare::description(
    const Descriptions& descriptions) const {
-	/** TRANSLATORS: e.g. Completed/Skipped/Did not start ... because the economy needs the ware
-	 * '%s' */
-	std::string result = (boost::format(_("the economy needs the ware ‘%s’")) %
-	                      descriptions.get_ware_descr(ware_type)->descname())
-	                        .str();
+	std::string result = format(
+	   /** TRANSLATORS: e.g. Completed/Skipped/Did not start ... because the economy needs the ware
+	    * '%s' */
+	   _("the economy needs the ware ‘%s’"), descriptions.get_ware_descr(ware_type)->descname());
 	return result;
 }
 std::string ProductionProgram::ActReturn::EconomyNeedsWare::description_negation(
    const Descriptions& descriptions) const {
 	/** TRANSLATORS: e.g. Completed/Skipped/Did not start ... because the economy doesn't need the
 	 * ware '%s' */
-	std::string result = (boost::format(_("the economy doesn’t need the ware ‘%s’")) %
-	                      descriptions.get_ware_descr(ware_type)->descname())
-	                        .str();
+	std::string result = format(_("the economy doesn’t need the ware ‘%s’"),
+	                            descriptions.get_ware_descr(ware_type)->descname());
 	return result;
 }
 
@@ -476,9 +477,8 @@ std::string ProductionProgram::ActReturn::EconomyNeedsWorker::description(
    const Descriptions& descriptions) const {
 	/** TRANSLATORS: e.g. Completed/Skipped/Did not start ... because the economy needs the worker
 	 * '%s' */
-	std::string result = (boost::format(_("the economy needs the worker ‘%s’")) %
-	                      descriptions.get_worker_descr(worker_type)->descname())
-	                        .str();
+	std::string result = format(_("the economy needs the worker ‘%s’"),
+	                            descriptions.get_worker_descr(worker_type)->descname());
 	return result;
 }
 
@@ -486,9 +486,8 @@ std::string ProductionProgram::ActReturn::EconomyNeedsWorker::description_negati
    const Descriptions& descriptions) const {
 	/** TRANSLATORS: e.g. Completed/Skipped/Did not start ... */
 	/** TRANSLATORS:      ... because the economy doesn’t need the worker '%s' */
-	std::string result = (boost::format(_("the economy doesn’t need the worker ‘%s’")) %
-	                      descriptions.get_worker_descr(worker_type)->descname())
-	                        .str();
+	std::string result = format(_("the economy doesn’t need the worker ‘%s’"),
+	                            descriptions.get_worker_descr(worker_type)->descname());
 	return result;
 }
 
@@ -527,8 +526,8 @@ ProductionProgram::ActReturn::SiteHas::description(const Descriptions& descripti
 	std::vector<std::string> condition_list;
 	for (const auto& entry : group.first) {
 		condition_list.push_back(entry.second == wwWARE ?
-		                            descriptions.get_ware_descr(entry.first)->descname() :
-		                            descriptions.get_worker_descr(entry.first)->descname());
+                                  descriptions.get_ware_descr(entry.first)->descname() :
+                                  descriptions.get_worker_descr(entry.first)->descname());
 	}
 	std::string condition = i18n::localize_list(condition_list, i18n::ConcatenateWith::AND);
 	if (1 < group.second) {
@@ -536,13 +535,12 @@ ProductionProgram::ActReturn::SiteHas::description(const Descriptions& descripti
 		   /** TRANSLATORS: This is an item in a list of wares, e.g. "3x water": */
 		   /** TRANSLATORS:    %1$i = "3" */
 		   /** TRANSLATORS:    %2$s = "water" */
-		   (boost::format(_("%1$ix %2$s")) % static_cast<unsigned int>(group.second) % condition)
-		      .str();
+		   format(_("%1$ix %2$s"), static_cast<unsigned int>(group.second), condition);
 	}
 
 	std::string result =
 	   /** TRANSLATORS: %s is a list of wares*/
-	   (boost::format(_("the building has the following wares: %s")) % condition).str();
+	   format(_("the building has the following wares: %s"), condition);
 	return result;
 }
 
@@ -551,8 +549,8 @@ std::string ProductionProgram::ActReturn::SiteHas::description_negation(
 	std::vector<std::string> condition_list;
 	for (const auto& entry : group.first) {
 		condition_list.push_back(entry.second == wwWARE ?
-		                            descriptions.get_ware_descr(entry.first)->descname() :
-		                            descriptions.get_worker_descr(entry.first)->descname());
+                                  descriptions.get_ware_descr(entry.first)->descname() :
+                                  descriptions.get_worker_descr(entry.first)->descname());
 	}
 	std::string condition = i18n::localize_list(condition_list, i18n::ConcatenateWith::AND);
 	if (1 < group.second) {
@@ -560,33 +558,32 @@ std::string ProductionProgram::ActReturn::SiteHas::description_negation(
 		   /** TRANSLATORS: This is an item in a list of wares, e.g. "3x water": */
 		   /** TRANSLATORS:    %1$i = "3" */
 		   /** TRANSLATORS:    %2$s = "water" */
-		   (boost::format(_("%1$ix %2$s")) % static_cast<unsigned int>(group.second) % condition)
-		      .str();
+		   format(_("%1$ix %2$s"), static_cast<unsigned int>(group.second), condition);
 	}
 
 	std::string result =
 	   /** TRANSLATORS: %s is a list of wares*/
-	   (boost::format(_("the building doesn’t have the following wares: %s")) % condition).str();
+	   format(_("the building doesn’t have the following wares: %s"), condition);
 	return result;
 }
 
 bool ProductionProgram::ActReturn::WorkersNeedExperience::evaluate(const ProductionSite& ps) const {
-	ProductionSite::WorkingPosition const* const wp = ps.working_positions_;
-	for (uint32_t i = ps.descr().nr_working_positions(); i;) {
-		if (wp[--i].worker.get(ps.get_owner()->egbase())->needs_experience()) {
+	const std::vector<ProductionSite::WorkingPosition>& wp = ps.working_positions_;
+	for (uint32_t i = ps.descr().nr_working_positions(); i != 0u;) {
+		if (wp.at(--i).worker.get(ps.get_owner()->egbase())->needs_experience()) {
 			return true;
 		}
 	}
 	return false;
 }
-std::string
-ProductionProgram::ActReturn::WorkersNeedExperience::description(const Descriptions&) const {
+std::string ProductionProgram::ActReturn::WorkersNeedExperience::description(
+   const Descriptions& /* descriptions */) const {
 	/** TRANSLATORS: 'Completed/Skipped/Did not start ... because a worker needs experience'. */
 	return _("a worker needs experience");
 }
 
 std::string ProductionProgram::ActReturn::WorkersNeedExperience::description_negation(
-   const Descriptions&) const {
+   const Descriptions& /* descriptions */) const {
 	/** TRANSLATORS: 'Completed/Skipped/Did not start ... because the workers need no experience'. */
 	return _("the workers need no experience");
 }
@@ -646,9 +643,6 @@ ProductionProgram::ActReturn::ActReturn(const std::vector<std::string>& argument
 		result_ = ProgramResult::kFailed;
 	} else if (match_and_skip(arguments, begin, "completed")) {
 		result_ = ProgramResult::kCompleted;
-	} else if (match_and_skip(arguments, begin, "no_stats")) {
-		// TODO(GunChleoc): Savegame cpmpatibility - remove no_stats after Build 21
-		result_ = ProgramResult::kCompleted;
 	} else if (match_and_skip(arguments, begin, "skipped")) {
 		result_ = ProgramResult::kSkipped;
 	} else {
@@ -721,27 +715,23 @@ void ProductionProgram::ActReturn::execute(Game& game, ProductionSite& ps) const
 		switch (result_) {
 		case ProgramResult::kFailed: {
 			/** TRANSLATORS: "Did not start working because the economy needs the ware '%s'" */
-			result_string = (boost::format(_("Did not start %1$s because %2$s")) %
-			                 ps.top_state().program->descname() % condition_string)
-			                   .str();
+			result_string = format(_("Did not start %1$s because %2$s"),
+			                       ps.top_state().program->descname(), condition_string);
 		} break;
 		case ProgramResult::kCompleted: {
-			/** TRANSLATORS: "Completed working because the economy needs the ware '%s'" */
-			result_string = (boost::format(_("Completed %1$s because %2$s")) %
-			                 ps.top_state().program->descname() % condition_string)
-			                   .str();
+			result_string = format(
+			   /** TRANSLATORS: "Completed working because the economy needs the ware '%s'" */
+			   _("Completed %1$s because %2$s"), ps.top_state().program->descname(), condition_string);
 		} break;
 		case ProgramResult::kSkipped: {
-			/** TRANSLATORS: "Skipped working because the economy needs the ware '%s'" */
-			result_string = (boost::format(_("Skipped %1$s because %2$s")) %
-			                 ps.top_state().program->descname() % condition_string)
-			                   .str();
+			result_string = format(
+			   /** TRANSLATORS: "Skipped working because the economy needs the ware '%s'" */
+			   _("Skipped %1$s because %2$s"), ps.top_state().program->descname(), condition_string);
 		} break;
 		case ProgramResult::kNone: {
 			// TODO(GunChleoc): Same as skipped - is this on purpose?
-			result_string = (boost::format(_("Skipped %1$s because %2$s")) %
-			                 ps.top_state().program->descname() % condition_string)
-			                   .str();
+			result_string = format(
+			   _("Skipped %1$s because %2$s"), ps.top_state().program->descname(), condition_string);
 		}
 		}
 		if (ps.production_result() != ps.descr().out_of_resource_heading() ||
@@ -777,7 +767,7 @@ Calls another program of the same productionsite. Example:
       programs = {
          main = {
             -- TRANSLATORS: Completed/Skipped/Did not start working because ...
-            descname = _"working",
+            descname = _("working"),
             actions = {
                "call=mine_granite on failure fail",
                "call=mine_granite on failure fail",
@@ -786,7 +776,7 @@ Calls another program of the same productionsite. Example:
          },
          mine_granite = {
             -- TRANSLATORS: Completed/Skipped/Did not start quarrying granite because ...
-            descname = _"quarrying granite",
+            descname = _("quarrying granite"),
             actions = {
                "callworker=cut_granite",
                "sleep=duration:17s500ms"
@@ -794,7 +784,7 @@ Calls another program of the same productionsite. Example:
          },
          mine_marble = {
             -- TRANSLATORS: Completed/Skipped/Did not start quarrying marble because ...
-            descname = _"quarrying marble",
+            descname = _("quarrying marble"),
             actions = {
                "callworker=cut_marble",
                "sleep=duration:17s500ms"
@@ -893,11 +883,15 @@ void ProductionProgram::ActCall::execute(Game& game, ProductionSite& ps) const {
 /* RST
 callworker
 ----------
-.. function:: callworker=\<worker_program_name\>
+.. function:: callworker=\<worker_program_name\> \[on failure fail|complete|skip\]
 
    :arg string worker_program_name: The name of a :ref:`worker program <tribes_worker_programs>`
       defined in the productionsite's main :ref:`worker <lua_tribes_basic_workers>`.
 
+   :arg string on: Defines what to do if the worker program fails. The production program
+      is always terminated immediately when ``callworker`` fails; this parameter specifies
+      what result status the production program should report. Default is ``fail``.
+
 Calls a program of the productionsite's main worker. Example:
 
 .. code-block:: lua
@@ -925,13 +919,32 @@ Calls a program of the productionsite's main worker. Example:
 ProductionProgram::ActCallWorker::ActCallWorker(const std::vector<std::string>& arguments,
                                                 const std::string& production_program_name,
                                                 ProductionSiteDescr* descr,
-                                                const Descriptions& descriptions) {
-	if (arguments.size() != 1) {
-		throw GameDataError("Usage: callworker=<worker_program_name>");
+                                                const Descriptions& descriptions)
+   : on_failure_(ProgramResult::kFailed) {
+	const size_t nr_args = arguments.size();
+	if (nr_args != 1 && nr_args != 4) {
+		throw GameDataError(
+		   "Usage: callworker=<worker_program_name> [on failure fail|complete|skip]");
 	}
 
 	program_ = arguments.front();
 
+	if (nr_args > 1) {
+		if (arguments.at(1) != "on" || arguments.at(2) != "failure") {
+			throw GameDataError("Expected 'on failure' after worker program name");
+		}
+		if (arguments.at(3) == "fail") {
+			on_failure_ = ProgramResult::kFailed;
+		} else if (arguments.at(3) == "complete") {
+			on_failure_ = ProgramResult::kCompleted;
+		} else if (arguments.at(3) == "skip") {
+			on_failure_ = ProgramResult::kSkipped;
+		} else {
+			throw GameDataError("Expected fail|complete|skip in final position but found '%s'",
+			                    arguments.at(3).c_str());
+		}
+	}
+
 	//  Quote from "void ProductionSite::program_act(Game &)":
 	//  "Always main worker is doing stuff"
 	const WorkerDescr& main_worker_descr =
@@ -985,7 +998,7 @@ ProductionProgram::ActCallWorker::ActCallWorker(const std::vector<std::string>&
 
 void ProductionProgram::ActCallWorker::execute(Game& game, ProductionSite& ps) const {
 	// Always main worker is doing stuff
-	ps.working_positions_[ps.main_worker_].worker.get(game)->update_task_buildingwork(game);
+	ps.working_positions_.at(ps.main_worker_).worker.get(game)->update_task_buildingwork(game);
 }
 
 bool ProductionProgram::ActCallWorker::get_building_work(Game& game,
@@ -994,7 +1007,7 @@ bool ProductionProgram::ActCallWorker::get_building_work(Game& game,
 	ProductionSite::State& state = psite.top_state();
 	if (state.phase == ProgramResult::kNone) {
 		worker.start_task_program(game, program());
-		if (state.flags & ProductionSite::State::StateFlags::kStateFlagHasExtraData) {
+		if ((state.flags & ProductionSite::State::StateFlags::kStateFlagHasExtraData) != 0u) {
 			worker.top_state().objvar1 = state.objvar;
 		}
 		state.phase = ProgramResult::kFailed;
@@ -1006,8 +1019,8 @@ bool ProductionProgram::ActCallWorker::get_building_work(Game& game,
 
 void ProductionProgram::ActCallWorker::building_work_failed(Game& game,
                                                             ProductionSite& psite,
-                                                            Worker&) const {
-	psite.program_end(game, ProgramResult::kFailed);
+                                                            Worker& /* worker */) const {
+	psite.program_end(game, on_failure_);
 }
 
 /* RST
@@ -1051,7 +1064,8 @@ ProductionProgram::ActSleep::ActSleep(const std::vector<std::string>& arguments,
 }
 
 void ProductionProgram::ActSleep::execute(Game& game, ProductionSite& ps) const {
-	return ps.program_step(game, duration_.get() ? duration_ : Duration(0), ps.top_state().phase);
+	return ps.program_step(
+	   game, duration_.get() != 0u ? duration_ : Duration(0), ps.top_state().phase);
 }
 
 /* RST
@@ -1066,8 +1080,8 @@ ProductionProgram::ActAnimate::ActAnimate(const std::vector<std::string>& argume
 
 void ProductionProgram::ActAnimate::execute(Game& game, ProductionSite& ps) const {
 	ps.start_animation(game, parameters.animation);
-	return ps.program_step(
-	   game, parameters.duration.get() ? parameters.duration : Duration(0), ps.top_state().phase);
+	return ps.program_step(game, parameters.duration.get() != 0u ? parameters.duration : Duration(0),
+	                       ps.top_state().phase);
 }
 
 /* RST
@@ -1198,8 +1212,8 @@ void ProductionProgram::ActConsume::execute(Game& game, ProductionSite& ps) cons
 			std::vector<std::string> ware_list;
 			for (const auto& entry : group.first) {
 				ware_list.push_back(entry.second == wwWARE ?
-				                       tribe.get_ware_descr(entry.first)->descname() :
-				                       tribe.get_worker_descr(entry.first)->descname());
+                                   tribe.get_ware_descr(entry.first)->descname() :
+                                   tribe.get_worker_descr(entry.first)->descname());
 			}
 			std::string ware_string = i18n::localize_list(ware_list, i18n::ConcatenateWith::OR);
 
@@ -1211,8 +1225,7 @@ void ProductionProgram::ActConsume::execute(Game& game, ProductionSite& ps) cons
 				   /** TRANSLATORS: For this example, this is what's in the place holders: */
 				   /** TRANSLATORS:    %1$i = "3" */
 				   /** TRANSLATORS:    %2$s = "water" */
-				   (boost::format(_("%1$ix %2$s")) % static_cast<unsigned int>(count) % ware_string)
-				      .str();
+				   format(_("%1$ix %2$s"), static_cast<unsigned int>(count), ware_string);
 			}
 			group_list.push_back(ware_string);
 		}
@@ -1221,9 +1234,8 @@ void ProductionProgram::ActConsume::execute(Game& game, ProductionSite& ps) cons
 		   /** TRANSLATORS: e.g. 'Did not start working because 3x water and 3x wheat are missing' */
 		   /** TRANSLATORS: e.g. 'Did not start working because fish, meat or pitta bread is missing'
 		    */
-		   (boost::format(ngettext("%s is missing", "%s are missing", nr_missing_groups)) %
-		    i18n::localize_list(group_list, i18n::ConcatenateWith::AND))
-		      .str();
+		   format(ngettext("%s is missing", "%s are missing", nr_missing_groups),
+		          i18n::localize_list(group_list, i18n::ConcatenateWith::AND));
 
 		std::string result_string =
 		   /** TRANSLATORS: e.g. 'Did not start working because 3x water and 3x wheat are missing' */
@@ -1235,9 +1247,8 @@ void ProductionProgram::ActConsume::execute(Game& game, ProductionSite& ps) cons
 		   /** TRANSLATORS: on a development build if you can, and let us know if there are any issues
 		    */
 		   /** TRANSLATORS: we need to address for your language. */
-		   (boost::format(_("Did not start %1$s because %2$s")) % ps.top_state().program->descname() %
-		    is_missing_string)
-		      .str();
+		   format(_("Did not start %1$s because %2$s"), ps.top_state().program->descname(),
+		          is_missing_string);
 
 		if (ps.production_result() != ps.descr().out_of_resource_heading() ||
 		    ps.descr().out_of_resource_heading().empty()) {
@@ -1304,7 +1315,7 @@ ProductionProgram::ActProduce::ActProduce(const std::vector<std::string>& argume
 void ProductionProgram::ActProduce::execute(Game& game, ProductionSite& ps) const {
 	assert(ps.produced_wares_.empty());
 	ps.produced_wares_ = produced_wares_;
-	ps.working_positions_[ps.main_worker_].worker.get(game)->update_task_buildingwork(game);
+	ps.working_positions_.at(ps.main_worker_).worker.get(game)->update_task_buildingwork(game);
 
 	const TribeDescr& tribe = ps.owner().tribe();
 	assert(!produced_wares_.empty());
@@ -1315,12 +1326,11 @@ void ProductionProgram::ActProduce::execute(Game& game, ProductionSite& ps) cons
 		count += item_pair.second;
 		std::string ware_descname = tribe.get_ware_descr(item_pair.first)->descname();
 		if (1 < item_pair.second || 1 < produced_wares_.size()) {
-			/** TRANSLATORS: This is an item in a list of wares, e.g. "Produced 2x Coal": */
-			/** TRANSLATORS:    %%1$i = "2" */
-			/** TRANSLATORS:    %2$s = "Coal" */
-			ware_descname = (boost::format(_("%1$ix %2$s")) %
-			                 static_cast<unsigned int>(item_pair.second) % ware_descname)
-			                   .str();
+			ware_descname = format(
+			   /** TRANSLATORS: This is an item in a list of wares, e.g. "Produced 2x Coal": */
+			   /** TRANSLATORS:    %1$i = "2" */
+			   /** TRANSLATORS:    %2$s = "Coal" */
+			   _("%1$ix %2$s"), static_cast<unsigned int>(item_pair.second), ware_descname);
 		}
 		ware_descnames.push_back(ware_descname);
 	}
@@ -1329,7 +1339,7 @@ void ProductionProgram::ActProduce::execute(Game& game, ProductionSite& ps) cons
 	const std::string result_string =
 	   /** TRANSLATORS: %s is a list of wares. String is fetched according to total amount of
 	      wares. */
-	   (boost::format(ngettext("Produced %s", "Produced %s", count)) % ware_list).str();
+	   format(ngettext("Produced %s", "Produced %s", count), ware_list);
 	if (ps.production_result() != ps.descr().out_of_resource_heading() ||
 	    ps.descr().out_of_resource_heading().empty()) {
 		ps.set_production_result(result_string);
@@ -1388,7 +1398,7 @@ ProductionProgram::ActRecruit::ActRecruit(const std::vector<std::string>& argume
 void ProductionProgram::ActRecruit::execute(Game& game, ProductionSite& ps) const {
 	assert(ps.recruited_workers_.empty());
 	ps.recruited_workers_ = recruited_workers_;
-	ps.working_positions_[ps.main_worker_].worker.get(game)->update_task_buildingwork(game);
+	ps.working_positions_.at(ps.main_worker_).worker.get(game)->update_task_buildingwork(game);
 
 	const TribeDescr& tribe = ps.owner().tribe();
 	assert(!recruited_workers_.empty());
@@ -1398,12 +1408,11 @@ void ProductionProgram::ActRecruit::execute(Game& game, ProductionSite& ps) cons
 		count += item_pair.second;
 		std::string worker_descname = tribe.get_worker_descr(item_pair.first)->descname();
 		if (1 < item_pair.second || 1 < recruited_workers_.size()) {
-			/** TRANSLATORS: This is an item in a list of workers, e.g. "Recruited 2x Ox": */
-			/** TRANSLATORS:    %1$i = "2" */
-			/** TRANSLATORS:    %2$s = "Ox" */
-			worker_descname = (boost::format(_("%1$ix %2$s")) %
-			                   static_cast<unsigned int>(item_pair.second) % worker_descname)
-			                     .str();
+			worker_descname = format(
+			   /** TRANSLATORS: This is an item in a list of workers, e.g. "Recruited 2x Ox": */
+			   /** TRANSLATORS:    %1$i = "2" */
+			   /** TRANSLATORS:    %2$s = "Ox" */
+			   _("%1$ix %2$s"), static_cast<unsigned int>(item_pair.second), worker_descname);
 		}
 		worker_descnames.push_back(worker_descname);
 	}
@@ -1412,7 +1421,7 @@ void ProductionProgram::ActRecruit::execute(Game& game, ProductionSite& ps) cons
 	const std::string result_string =
 	   /** TRANSLATORS: %s is a list of workers. String is fetched according to total amount of
 	      workers. */
-	   (boost::format(ngettext("Recruited %s", "Recruited %s", count)) % unit_string).str();
+	   format(ngettext("Recruited %s", "Recruited %s", count), unit_string);
 	ps.set_production_result(result_string);
 }
 
@@ -1429,7 +1438,7 @@ mine
 ----
 
 .. function:: mine=\<resource_name\> radius:\<number\> yield:\<percent\> when_empty:\<percent\>
-     \[experience_on_fail:\<percent\>\]
+     \[experience_on_fail:\<percent\>\] [no_notify]
 
    :arg string resource_name: The name of the resource to mine, e.g. 'coal' or 'water'.
    :arg int radius: The workarea radius that is searched for resources. Must be ``>0``.
@@ -1440,6 +1449,7 @@ mine
    :arg percent experience_on_fail: The :ref:`map_object_programs_datatypes_percent` chance that the
       mine's workers will still gain some experience when mining fails after its resources have been
       depleted.
+   :arg empty no_notify: Do not send a message to the player if this step fails.
 
    Takes resources from the ground. A building that mines will deplete when the percentage of
    resources given in ``resources`` has been dug up, leaving a chance of ``depleted`` that it
@@ -1471,17 +1481,18 @@ mine
 ProductionProgram::ActMine::ActMine(const std::vector<std::string>& arguments,
                                     Descriptions& descriptions,
                                     const std::string& production_program_name,
-                                    ProductionSiteDescr* descr) {
-	if (arguments.size() != 5 && arguments.size() != 4) {
+                                    ProductionSiteDescr* descr)
+   : resource_(INVALID_INDEX), notify_on_failure_(true) {
+	if (arguments.size() > 6 || arguments.size() < 4) {
 		throw GameDataError("Usage: mine=<resource name> radius:<number> yield:<percent> "
-		                    "when_empty:<percent> [experience:<percent>]");
+		                    "when_empty:<percent> [experience:<percent>] [no_notify]");
 	}
 	experience_chance_ = 0U;
 
 	if (read_key_value_pair(arguments.at(2), ':').second.empty()) {
 		// TODO(GunChleoc): Savegame compatibility, remove after v1.0
 		log_warn("Using old syntax in %s. Please use 'mine=<resource name> radius:<number> "
-		         "yield:<percent> when_empty:<percent> [experience_on_fail:<percent>]'\n",
+		         "yield:<percent> when_empty:<percent> [experience_on_fail:<percent>] [no_notify]'",
 		         descr->name().c_str());
 		resource_ = descriptions.load_resource(arguments.front());
 		workarea_ = read_positive(arguments.at(1));
@@ -1494,7 +1505,12 @@ ProductionProgram::ActMine::ActMine(const std::vector<std::string>& arguments,
 		for (const std::string& argument : arguments) {
 			const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
 			if (item.second.empty()) {
-				resource_ = descriptions.load_resource(item.first);
+				// The safeguard is for the case that someone creates a resource called "no_notify"
+				if (item.first == "no_notify" && resource_ != INVALID_INDEX) {
+					notify_on_failure_ = false;
+				} else {
+					resource_ = descriptions.load_resource(item.first);
+				}
 			} else if (item.first == "radius") {
 				workarea_ = read_positive(item.second);
 			} else if (item.first == "yield") {
@@ -1561,10 +1577,10 @@ void ProductionProgram::ActMine::execute(Game& game, ProductionSite& ps) const {
 
 	//  how much is dug
 	unsigned dug_percentage = math::k100PercentAsInt;
-	if (totalstart) {
+	if (totalstart != 0u) {
 		dug_percentage = (totalstart - totalres) * math::k100PercentAsInt / totalstart;
 	}
-	if (!totalres) {
+	if (totalres == 0u) {
 		dug_percentage = math::k100PercentAsInt;
 	}
 
@@ -1609,7 +1625,7 @@ void ProductionProgram::ActMine::execute(Game& game, ProductionSite& ps) const {
 		//  there is a sufficiently high chance, that the mine
 		//  will still produce enough.
 		//  e.g. mines have chance=5, wells have 65
-		if (depleted_chance_ <= 20 * math::k100PercentAsInt / 100U) {
+		if (notify_on_failure_ && depleted_chance_ <= 20 * math::k100PercentAsInt / 100U) {
 			ps.notify_player(game, 60);
 			// and change the default animation
 			ps.set_default_anim("empty");
@@ -1660,7 +1676,7 @@ Example:
 
       upgrade_soldier_attack_3 = {
          -- TRANSLATORS: Completed/Skipped/Did not start upgrading ... because ...
-         descname = _"upgrading soldier attack from level 3 to level 4",
+         descname = _("upgrading soldier attack from level 3 to level 4"),
          actions = {
             -- Fails when there aren't any soldiers with attack level 3
             "checksoldier=soldier:attack level:3",
@@ -1872,7 +1888,7 @@ void ProductionProgram::ActTrain::execute(Game& game, ProductionSite& ps) const
 	ps.set_production_result(
 	   /** TRANSLATORS: Success message of a trainingsite '%s' stands for the description of the
 	    * training program, e.g. Completed upgrading soldier evade from level 0 to level 1 */
-	   (boost::format(_("Completed %s")) % ps.top_state().program->descname()).str());
+	   format(_("Completed %s"), ps.top_state().program->descname()));
 
 	ts.training_successful(training_.attribute, current_level);
 
@@ -1982,7 +1998,7 @@ const ImmovableDescr&
 ProductionProgram::ActConstruct::get_construction_descr(const Descriptions& descriptions) const {
 	const ImmovableDescr* descr =
 	   descriptions.get_immovable_descr(descriptions.immovable_index(objectname));
-	if (!descr) {
+	if (descr == nullptr) {
 		throw wexception("ActConstruct: immovable '%s' does not exist", objectname.c_str());
 	}
 
@@ -2014,10 +2030,13 @@ void ProductionProgram::ActConstruct::execute(Game& game, ProductionSite& psite)
 	std::vector<ImmovableFound> immovables;
 	CheckStepWalkOn cstep(MOVECAPS_WALK, true);
 	Area<FCoords> area(map.get_fcoords(psite.get_position()), radius);
-	if (map.find_reachable_immovables(game, area, &immovables, cstep, FindImmovableByDescr(descr))) {
+	if (map.find_reachable_immovables(game, area, &immovables, cstep, FindImmovableByDescr(descr)) !=
+	    0u) {
 		state.objvar = immovables[0].object;
 
-		psite.working_positions_[psite.main_worker_].worker.get(game)->update_task_buildingwork(game);
+		psite.working_positions_.at(psite.main_worker_)
+		   .worker.get(game)
+		   ->update_task_buildingwork(game);
 		return;
 	}
 
@@ -2027,7 +2046,7 @@ void ProductionProgram::ActConstruct::execute(Game& game, ProductionSite& psite)
 	// 10 is custom value to make sure the "water" is at least 10 nodes big
 	fna.add(FindNodeShore(10));
 	fna.add(FindNodeImmovableSize(FindNodeImmovableSize::sizeNone));
-	if (map.find_reachable_fields(game, area, &fields, cstep, fna)) {
+	if (map.find_reachable_fields(game, area, &fields, cstep, fna) != 0u) {
 		// Testing received fields to get one with less immovables nearby
 		Coords best_coords = fields.back();  // Just to initialize it
 		uint32_t best_score = std::numeric_limits<uint32_t>::max();
@@ -2053,7 +2072,9 @@ void ProductionProgram::ActConstruct::execute(Game& game, ProductionSite& psite)
 
 		state.coord = best_coords;
 
-		psite.working_positions_[psite.main_worker_].worker.get(game)->update_task_buildingwork(game);
+		psite.working_positions_.at(psite.main_worker_)
+		   .worker.get(game)
+		   ->update_task_buildingwork(game);
 		return;
 	}
 
@@ -2075,7 +2096,7 @@ bool ProductionProgram::ActConstruct::get_building_work(Game& game,
 	WaresQueue* wq = nullptr;
 
 	Immovable* construction = dynamic_cast<Immovable*>(state.objvar.get(game));
-	if (construction) {
+	if (construction != nullptr) {
 		if (!construction->construct_remaining_buildcost(game, &remaining)) {
 			psite.molog(game.get_gametime(), "construct: immovable %u not under construction",
 			            construction->serial());
@@ -2095,7 +2116,7 @@ bool ProductionProgram::ActConstruct::get_building_work(Game& game,
 		}
 	}
 
-	if (!wq) {
+	if (wq == nullptr) {
 		psite.program_end(game, ProgramResult::kFailed);
 		return false;
 	}
@@ -2118,7 +2139,7 @@ bool ProductionProgram::ActConstruct::get_building_work(Game& game,
 
 void ProductionProgram::ActConstruct::building_work_failed(Game& game,
                                                            ProductionSite& psite,
-                                                           Worker&) const {
+                                                           Worker& /* worker */) const {
 	psite.program_end(game, ProgramResult::kFailed);
 }
 
