@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -313,6 +312,7 @@ struct ProductionProgram : public MapObjectProgram {
 
 	private:
 		std::string program_;
+		ProgramResult on_failure_;
 	};
 
 	/// Does nothing.
@@ -467,6 +467,7 @@ struct ProductionProgram : public MapObjectProgram {
 		unsigned max_resources_;      // Can work up to this percent (of total mountain resources)
 		unsigned depleted_chance_;    // odds of finding resources from empty mine
 		unsigned experience_chance_;  // probability of training in _empty_ mines
+		bool notify_on_failure_;      // Send a message if no resources could be found.
 	};
 
 	struct ActCheckSoldier : public Action {
