@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -164,8 +163,8 @@ ProductionSiteDescr::ProductionSiteDescr(const std::string& init_descname,
 	// Get programs
 	items_table = table.get_table("programs");
 	for (std::string program_name : items_table->keys<std::string>()) {
-		std::transform(program_name.begin(), program_name.end(), program_name.begin(), tolower);
-		if (programs_.count(program_name)) {
+		program_name = to_lower(program_name);
+		if (programs_.count(program_name) != 0u) {
 			throw GameDataError("Program '%s' has already been declared for productionsite '%s'",
 			                    program_name.c_str(), name().c_str());
 		}
@@ -286,7 +285,7 @@ IMPLEMENTATION
 
 ProductionSite::ProductionSite(const ProductionSiteDescr& ps_descr)
    : Building(ps_descr),
-     working_positions_(new WorkingPosition[ps_descr.nr_working_positions()]),
+     working_positions_(ps_descr.nr_working_positions()),
      fetchfromflag_(0),
      program_timer_(false),
      program_time_(0),
@@ -300,11 +299,6 @@ ProductionSite::ProductionSite(const ProductionSiteDescr& ps_descr)
 	format_statistics_string();
 }
 
-ProductionSite::~ProductionSite() {
-	// TODO(sirver): Use std::vector<std::unique_ptr<>> to avoid naked delete.
-	delete[] working_positions_;
-}
-
 void ProductionSite::load_finish(EditorGameBase& egbase) {
 	Building::load_finish(egbase);
 	format_statistics_string();
@@ -317,15 +311,15 @@ void ProductionSite::update_statistics_string(std::string* s) {
 	uint32_t nr_requests = 0;
 	uint32_t nr_coming = 0;
 	for (uint32_t i = 0; i < descr().nr_working_positions(); ++i) {
-		const Widelands::Request* request = working_positions_[i].worker_request;
+		const Widelands::Request* request = working_positions_.at(i).worker_request;
 		// Check whether a request is being fulfilled or not
-		if (request) {
+		if (request != nullptr) {
 			if (request->is_open()) {
 				++nr_requests;
 			} else {
 				++nr_coming;
 			}
-		} else if (working_positions_[i].worker == nullptr) {
+		} else if (working_positions_.at(i).worker == nullptr) {
 			// We might have no request, but no worker either
 			++nr_requests;
 		}
@@ -333,25 +327,17 @@ void ProductionSite::update_statistics_string(std::string* s) {
 
 	if (nr_requests > 0) {
 		*s = StyleManager::color_tag(
-		   (nr_requests == 1 ?
-		       /** TRANSLATORS: Productivity label on a building if there is 1 worker missing */
-		       _("Worker missing") :
-		       /** TRANSLATORS: Productivity label on a building if there is more than 1 worker
-		          missing. If you need plural forms here, please let us know. */
-		       _("Workers missing")),
+		   (nr_requests == 1 ? owner().tribe().get_productionsite_worker_missing_string() :
+                             owner().tribe().get_productionsite_workers_missing_string()),
 		   g_style_manager->building_statistics_style().low_color());
 		return;
 	}
 
 	if (nr_coming > 0) {
 		*s = StyleManager::color_tag(
-		   (nr_coming == 1 ?
-		       /** TRANSLATORS: Productivity label on a building if there is 1 worker missing */
-		       _("Worker is coming") :
-		       /** TRANSLATORS: Productivity label on a building if there is more than 1 worker
-		          missing. If you need plural forms here, please let us know. */
-		       _("Workers are coming")),
-		   g_style_manager->building_statistics_style().low_color());
+		   (nr_coming == 1 ? owner().tribe().get_productionsite_worker_coming_string() :
+                           owner().tribe().get_productionsite_workers_coming_string()),
+		   g_style_manager->building_statistics_style().medium_color());
 		return;
 	}
 
@@ -372,7 +358,7 @@ bool ProductionSite::has_workers(DescriptionIndex targetSite, Game& game) {
 	// bld holds the description of the building we want to have
 	if (upcast(ProductionSiteDescr const, bld, owner().tribe().get_building_descr(targetSite))) {
 
-		if (bld->nr_working_positions()) {
+		if (bld->nr_working_positions() != 0u) {
 
 			// Iterating over workers positions in target building
 			for (const auto& wp : bld->working_positions()) {
@@ -387,8 +373,8 @@ bool ProductionSite::has_workers(DescriptionIndex targetSite, Game& game) {
 				const DescriptionIndex needed_worker = wp.first;
 				bool worker_available = false;
 				for (unsigned int i = 0; i < descr().nr_working_positions(); ++i) {
-					const Worker* cw = working_positions()[i].worker.get(game);
-					if (cw) {
+					const Worker* cw = working_positions()->at(i).worker.get(game);
+					if (cw != nullptr) {
 						DescriptionIndex current_worker = cw->descr().worker_index();
 						if (owner().tribe().get_worker_descr(current_worker)->can_act_as(needed_worker)) {
 							worker_available = true;  // We found a worker for the position
@@ -406,13 +392,13 @@ bool ProductionSite::has_workers(DescriptionIndex targetSite, Game& game) {
 			return true;
 		}
 		throw wexception("Building, index: %d, needs no workers!\n", targetSite);
-	} else {
-		throw wexception("No such building, index: %d\n", targetSite);
 	}
+	throw wexception("No such building, index: %d\n", targetSite);
 }
 
-InputQueue&
-ProductionSite::inputqueue(DescriptionIndex const wi, WareWorker const type, const Request*) {
+InputQueue& ProductionSite::inputqueue(DescriptionIndex const wi,
+                                       WareWorker const type,
+                                       const Request* /* req */) {
 	for (InputQueue* ip_queue : input_queues_) {
 		if (ip_queue->get_index() == wi && ip_queue->get_type() == type) {
 			return *ip_queue;
@@ -425,8 +411,8 @@ ProductionSite::inputqueue(DescriptionIndex const wi, WareWorker const type, con
 	throw wexception("%s (%u) has no InputQueue for %s %u: %s", descr().name().c_str(), serial(),
 	                 type == WareWorker::wwWARE ? "ware" : "worker", wi,
 	                 type == WareWorker::wwWARE ?
-	                    owner().tribe().get_ware_descr(wi)->name().c_str() :
-	                    owner().tribe().get_worker_descr(wi)->name().c_str());
+                       owner().tribe().get_ware_descr(wi)->name().c_str() :
+                       owner().tribe().get_worker_descr(wi)->name().c_str());
 }
 
 /**
@@ -437,14 +423,13 @@ void ProductionSite::format_statistics_string() {
 	// as a string and persists them into a string for reuse when the class is
 	// asked for the statistics string. However this string should only then be constructed.
 
-	// boost::format would treat uint8_t as char
+	// format would treat uint8_t as char
 	const unsigned int percent = std::min(get_actual_statistics() * 100 / 98, 100);
 	const std::string perc_str = StyleManager::color_tag(
-	   (boost::format(_("%i%%")) % percent).str(),
-	   (percent < 33) ?
-	      g_style_manager->building_statistics_style().low_color() :
-	      (percent < 66) ? g_style_manager->building_statistics_style().medium_color() :
-	                       g_style_manager->building_statistics_style().high_color());
+	   format(_("%i%%"), percent),
+	   (percent < 33) ? g_style_manager->building_statistics_style().low_color() :
+	   (percent < 66) ? g_style_manager->building_statistics_style().medium_color() :
+                       g_style_manager->building_statistics_style().high_color());
 
 	if (0 < percent && percent < 100) {
 		RGBColor color = g_style_manager->building_statistics_style().high_color();
@@ -465,7 +450,7 @@ void ProductionSite::format_statistics_string() {
 
 		// TODO(GunChleoc): We might need to reverse the order here for RTL languages
 		statistics_string_on_changed_statistics_ =
-		   (boost::format("%s\u2009%s") % perc_str % StyleManager::color_tag(trend, color)).str();
+		   format("%s\u2009%s", perc_str, StyleManager::color_tag(trend, color));
 	} else {
 		statistics_string_on_changed_statistics_ = perc_str;
 	}
@@ -491,10 +476,10 @@ bool ProductionSite::init(EditorGameBase& egbase) {
 	}
 
 	//  Request missing workers.
-	WorkingPosition* wp = working_positions_;
+	auto wp = working_positions_.begin();
 	for (const auto& temp_wp : descr().working_positions()) {
 		DescriptionIndex const worker_index = temp_wp.first;
-		for (uint32_t j = temp_wp.second; j; --j, ++wp) {
+		for (uint32_t j = temp_wp.second; j != 0u; --j, ++wp) {
 			if (Worker* const worker = wp->worker.get(egbase)) {
 				worker->set_location(this);
 			} else {
@@ -524,15 +509,15 @@ void ProductionSite::set_economy(Economy* const e, WareWorker type) {
 	}
 
 	Building::set_economy(e, type);
-	for (uint32_t i = descr().nr_working_positions(); i;) {
-		if (Request* const r = working_positions_[--i].worker_request) {
+	for (uint32_t i = descr().nr_working_positions(); i != 0u;) {
+		if (Request* const r = working_positions_.at(--i).worker_request) {
 			if (r->get_type() == type) {
 				r->set_economy(e);
 			}
 		}
 	}
 
-	if (e) {
+	if (e != nullptr) {
 		for (InputQueue* ip_queue : input_queues_) {
 			if (ip_queue->get_type() == type) {
 				ip_queue->add_to_economy(*e);
@@ -545,17 +530,17 @@ void ProductionSite::set_economy(Economy* const e, WareWorker type) {
  * Cleanup after a production site is removed
  */
 void ProductionSite::cleanup(EditorGameBase& egbase) {
-	for (uint32_t i = descr().nr_working_positions(); i;) {
+	for (uint32_t i = descr().nr_working_positions(); i != 0u;) {
 		--i;
-		delete working_positions_[i].worker_request;
-		working_positions_[i].worker_request = nullptr;
-		Worker* const w = working_positions_[i].worker.get(egbase);
+		delete working_positions_.at(i).worker_request;
+		working_positions_.at(i).worker_request = nullptr;
+		Worker* const w = working_positions_.at(i).worker.get(egbase);
 
 		//  Ensure we do not re-request the worker when remove_worker is called.
-		working_positions_[i].worker = nullptr;
+		working_positions_.at(i).worker = nullptr;
 
 		// Actually remove the worker
-		if (w) {
+		if (w != nullptr) {
 			w->set_location(nullptr);
 		}
 	}
@@ -576,11 +561,10 @@ void ProductionSite::cleanup(EditorGameBase& egbase) {
  * returns true on success and false if there is no room for this worker
  */
 bool ProductionSite::warp_worker(EditorGameBase& egbase, const WorkerDescr& wdes, int32_t slot) {
-	WorkingPosition* current = working_positions_;
-	for (WorkingPosition* const end = current + descr().nr_working_positions(); current < end;
-	     ++current) {
+	auto current = working_positions_.begin();
+	for (int i = descr().nr_working_positions(); i != 0; i--, ++current) {
 		if (slot < 0) {
-			if (current->worker.get(egbase)) {
+			if (current->worker.get(egbase) != nullptr) {
 				continue;
 			}
 			assert(current->worker_request);
@@ -614,37 +598,93 @@ bool ProductionSite::warp_worker(EditorGameBase& egbase, const WorkerDescr& wdes
 	return false;
 }
 
+/**
+ * Locate the worker slot at which the given worker is currently employed.
+ * @return Index in descr().working_positions() (slot requirement) and working_positions_ (current
+ *         slot), or (-1, -1) if the worker does not work here.
+ */
+std::pair<int32_t, int32_t> ProductionSite::find_worker(OPtr<Worker> w) {
+	auto wp = working_positions_.begin();
+	const BillOfMaterials& bom = descr().working_positions();
+	int32_t wp_index = 0;
+	int32_t bom_index = 0;
+	for (auto temp_wp = bom.begin(); temp_wp != bom.end(); ++temp_wp, ++bom_index) {
+		for (uint32_t j = temp_wp->second; j != 0u; --j, ++wp, ++wp_index) {
+			if (wp->worker == w) {
+				return std::pair<int32_t, int32_t>(bom_index, wp_index);
+			}
+		}
+	}
+	return std::pair<int32_t, int32_t>(-1, -1);
+}
+
+/**
+ * Tries to replace a new worker request with a worker already present on the site
+ * @param game: Game reference to retrieve description info
+ * @param worker_index: The required worker type
+ * @param wp: Pointer to the slot to be filled (contains the worker request)
+ */
+void ProductionSite::try_replace_worker(const Game* game,
+                                        DescriptionIndex worker_index,
+                                        WorkingPosition* wp) {
+	// Search replacement worker in the same building
+	for (auto& wp_repl : working_positions_) {
+		if (!wp_repl.worker.is_set()) {
+			continue;
+		}
+		std::pair<int32_t, int32_t> w_pair = find_worker(wp_repl.worker);
+		const DescriptionIndex worker_index_repl = descr().working_positions().at(w_pair.first).first;
+
+		// Only move workers into higher qualified jobs
+		const WorkerDescr& wd = game->descriptions().workers().get(worker_index);
+		bool needs_higher_qualification =
+		   worker_index_repl != worker_index && wd.can_act_as(worker_index_repl);
+
+		Worker* w_repl = wp_repl.worker.get(*game);
+		assert(w_repl != nullptr);
+		bool is_over_qualified = w_repl->descr().worker_index() != worker_index_repl;
+		if (is_over_qualified && needs_higher_qualification &&
+		    w_repl->descr().can_act_as(worker_index)) {
+			// Move worker to evicted slot
+			delete wp->worker_request;
+			*wp = wp_repl;
+			molog(owner().egbase().get_gametime(), "%s promoted\n", w_repl->descr().name().c_str());
+			// Request the now missing worker instead and loop again
+			wp_repl = WorkingPosition(&request_worker(worker_index_repl), nullptr);
+			try_replace_worker(game, worker_index_repl, &wp_repl);
+			return;
+		}
+	}
+}
+
 /**
  * Intercept remove_worker() calls to unassign our worker, if necessary.
  */
 void ProductionSite::remove_worker(Worker& w) {
 	molog(owner().egbase().get_gametime(), "%s leaving\n", w.descr().name().c_str());
-	WorkingPosition* wp = working_positions_;
-	int32_t wp_index = 0;
 
-	for (const auto& temp_wp : descr().working_positions()) {
-		DescriptionIndex const worker_index = temp_wp.first;
-		for (uint32_t j = temp_wp.second; j; --j, ++wp, ++wp_index) {
-			if (wp->worker == &w) {
-				// do not request the type of worker that is currently assigned – maybe a
-				// trained worker was evicted to make place for a level 0 worker.
-				// Therefore we again request the worker from the WorkingPosition of descr()
-				if (main_worker_ == wp_index) {
-					main_worker_ = -1;
-				}
-				*wp = WorkingPosition(&request_worker(worker_index), nullptr);
-				Building::remove_worker(w);
-				// If the main worker was evicted, perhaps another worker is
-				// still there to perform basic tasks
-				if (upcast(Game, game, &get_owner()->egbase())) {
-					try_start_working(*game);
-				}
-				return;
-			}
-		}
+	std::pair<int32_t, int32_t> wp_indexes = find_worker(OPtr<Worker>(&w));
+	if (wp_indexes.first < 0) {
+		Building::remove_worker(w);
+		return;
 	}
 
+	DescriptionIndex const worker_index = descr().working_positions().at(wp_indexes.first).first;
+	WorkingPosition* wp = &working_positions_.at(wp_indexes.second);
+	// do not request the type of worker that is currently assigned – maybe a
+	// trained worker was evicted to make place for a level 0 worker.
+	// Therefore we again request the worker from the WorkingPosition of descr()
+	if (main_worker_ == wp_indexes.second) {
+		main_worker_ = -1;
+	}
+	*wp = WorkingPosition(&request_worker(worker_index), nullptr);
 	Building::remove_worker(w);
+	// If the main worker was evicted, perhaps another worker is
+	// still there to perform basic tasks
+	if (upcast(Game, game, &get_owner()->egbase())) {
+		try_replace_worker(game, worker_index, wp);
+		try_start_working(*game);
+	}
 }
 
 /**
@@ -677,18 +717,18 @@ void ProductionSite::request_worker_callback(
 	// placed on the slot that originally requested the arrived worker.
 	bool worker_placed = false;
 	DescriptionIndex idx = w->descr().worker_index();
-	for (WorkingPosition* wp = psite.working_positions_;; ++wp) {
-		if (wp->worker_request == &rq) {
-			if (wp->worker_request->get_index() == idx) {
+	for (WorkingPosition& wp : psite.working_positions_) {
+		if (wp.worker_request == &rq) {
+			if (wp.worker_request->get_index() == idx) {
 				// Place worker
 				delete &rq;
-				*wp = WorkingPosition(nullptr, w);
+				wp = WorkingPosition(nullptr, w);
 				worker_placed = true;
 			} else {
 				// Set new request for this slot
-				DescriptionIndex workerid = wp->worker_request->get_index();
-				delete wp->worker_request;
-				wp->worker_request = &psite.request_worker(workerid);
+				DescriptionIndex workerid = wp.worker_request->get_index();
+				delete wp.worker_request;
+				wp.worker_request = &psite.request_worker(workerid);
 			}
 			break;
 		}
@@ -697,10 +737,10 @@ void ProductionSite::request_worker_callback(
 		{
 			uint8_t nwp = psite.descr().nr_working_positions();
 			uint8_t pos = 0;
-			WorkingPosition* wp = psite.working_positions_;
+			auto wp = psite.working_positions_.begin();
 			for (; pos < nwp; ++wp, ++pos) {
 				// Find a fitting slot
-				if (!wp->worker.get(game) && !worker_placed) {
+				if ((wp->worker.get(game) == nullptr) && !worker_placed) {
 					if (wp->worker_request->get_index() == idx) {
 						delete wp->worker_request;
 						*wp = WorkingPosition(nullptr, w);
@@ -756,7 +796,7 @@ void ProductionSite::act(Game& game, uint32_t const data) {
 		} else {
 			assert(main_worker_ >= 0);
 			if (stack_.empty()) {
-				working_positions_[main_worker_].worker.get(game)->update_task_buildingwork(game);
+				working_positions_.at(main_worker_).worker.get(game)->update_task_buildingwork(game);
 				return;
 			}
 
@@ -783,7 +823,7 @@ void ProductionSite::set_next_program_override(Game& game,
 
 bool ProductionSite::has_forced_state() const {
 	for (const State& s : stack_) {
-		if (s.flags & State::StateFlags::kStateFlagIgnoreStopped) {
+		if ((s.flags & State::StateFlags::kStateFlagIgnoreStopped) != 0u) {
 			return true;
 		}
 	}
@@ -807,7 +847,7 @@ void ProductionSite::program_act(Game& game) {
 	// new productions cycle. Otherwise it can lead to consumption
 	// of input wares without producing anything
 	if (is_stopped_ && state.ip == 0 &&
-	    !(state.flags & State::StateFlags::kStateFlagIgnoreStopped)) {
+	    ((state.flags & State::StateFlags::kStateFlagIgnoreStopped) == 0u)) {
 		program_end(game, ProgramResult::kFailed);
 		program_timer_ = true;
 		program_time_ = schedule_act(game, Duration(20000));
@@ -823,8 +863,8 @@ bool ProductionSite::fetch_from_flag(Game& game) {
 	++fetchfromflag_;
 
 	if (main_worker_ >= 0) {
-		assert(working_positions_[main_worker_].worker.get(game));
-		working_positions_[main_worker_].worker.get(game)->update_task_buildingwork(game);
+		assert(working_positions_.at(main_worker_).worker.get(game));
+		working_positions_.at(main_worker_).worker.get(game)->update_task_buildingwork(game);
 	}
 
 	return true;
@@ -833,7 +873,7 @@ bool ProductionSite::fetch_from_flag(Game& game) {
 void ProductionSite::log_general_info(const EditorGameBase& egbase) const {
 	Building::log_general_info(egbase);
 
-	molog(egbase.get_gametime(), "is_stopped: %u\n", is_stopped_);
+	molog(egbase.get_gametime(), "is_stopped: %u\n", static_cast<int>(is_stopped_));
 	molog(egbase.get_gametime(), "main_worker: %i\n", main_worker_);
 }
 
@@ -849,8 +889,8 @@ void ProductionSite::set_stopped(bool const stopped) {
  * all workers are present)
  */
 bool ProductionSite::can_start_working() const {
-	for (uint32_t i = descr().nr_working_positions(); i;) {
-		if (working_positions_[--i].worker_request) {
+	for (uint32_t i = descr().nr_working_positions(); i != 0u;) {
+		if (working_positions_.at(--i).worker_request != nullptr) {
 			return false;
 		}
 	}
@@ -861,7 +901,7 @@ void ProductionSite::try_start_working(Game& game) {
 	const size_t nr_workers = descr().nr_working_positions();
 	for (uint32_t i = 0; i < nr_workers; ++i) {
 		if (main_worker_ == static_cast<int>(i) || main_worker_ < 0) {
-			if (Worker* worker = working_positions_[i].worker.get(game)) {
+			if (Worker* worker = working_positions_.at(i).worker.get(game)) {
 				// We may start even if can_start_working() returns false, because basic actions
 				// like unloading extra wares should take place anyway
 				main_worker_ = i;
@@ -881,7 +921,7 @@ void ProductionSite::try_start_working(Game& game) {
 bool ProductionSite::get_building_work(Game& game, Worker& worker, bool const success) {
 	assert(!descr().working_positions().empty());
 	assert(main_worker_ >= 0);
-	assert(&worker == working_positions_[main_worker_].worker.get(game));
+	assert(&worker == working_positions_.at(main_worker_).worker.get(game));
 
 	// If unsuccessful: Check if we need to abort current program
 	if (!success) {
@@ -897,7 +937,7 @@ bool ProductionSite::get_building_work(Game& game, Worker& worker, bool const su
 		return true;
 	}
 
-	if (fetchfromflag_) {
+	if (fetchfromflag_ != 0) {
 		--fetchfromflag_;
 		worker.start_task_fetchfromflag(game);
 		return true;
@@ -971,7 +1011,7 @@ bool ProductionSite::get_building_work(Game& game, Worker& worker, bool const su
 
 	// Start program if we haven't already done so
 	State* state = get_state();
-	if (!state) {
+	if (state == nullptr) {
 		find_and_start_next_program(game);
 	} else if (state->ip < state->program->size()) {
 		const ProductionProgram::Action& action = (*state->program)[state->ip];
@@ -1009,7 +1049,7 @@ void ProductionSite::program_start(Game& game,
 	if (force) {
 		state.flags |= State::StateFlags::kStateFlagIgnoreStopped;
 	}
-	if (extra_data) {
+	if (extra_data != nullptr) {
 		state.flags |= State::StateFlags::kStateFlagHasExtraData;
 	}
 
@@ -1075,8 +1115,8 @@ void ProductionSite::program_end(Game& game, ProgramResult const result) {
 }
 
 void ProductionSite::train_workers(Game& game) {
-	for (uint32_t i = descr().nr_working_positions(); i;) {
-		working_positions_[--i].worker.get(game)->gain_experience(game);
+	for (uint32_t i = descr().nr_working_positions(); i != 0u;) {
+		working_positions_.at(--i).worker.get(game)->gain_experience(game);
 	}
 	Notifications::publish(NoteBuilding(serial(), NoteBuilding::Action::kWorkersChanged));
 }
@@ -1165,9 +1205,8 @@ void ProductionSite::set_default_anim(const std::string& anim) {
 	default_anim_ = anim;
 }
 
-constexpr Duration kStatsEntireDuration = Duration(5 * 60 * 1000);  // statistic evaluation base
-constexpr Duration kStatsDurationCap =
-   Duration(180 * 1000);  // This is highest allowed program duration
+constexpr Duration kStatsEntireDuration(5 * 60 * 1000);  // statistic evaluation base
+constexpr Duration kStatsDurationCap(180 * 1000);        // This is highest allowed program duration
 
 void ProductionSite::update_actual_statistics(Duration duration, const bool produced) {
 	// just for case something went very wrong...
@@ -1175,9 +1214,10 @@ void ProductionSite::update_actual_statistics(Duration duration, const bool prod
 		duration = kStatsDurationCap;
 	}
 	const Duration past_duration = kStatsEntireDuration - duration;
-	actual_percent_ = (actual_percent_ * past_duration.get() + produced * duration.get() * 1000) /
+	actual_percent_ = (actual_percent_ * past_duration.get() +
+	                   static_cast<unsigned int>(produced) * duration.get() * 1000) /
 	                  kStatsEntireDuration.get();
-	assert(actual_percent_ <= 1000);  // be sure we do not go above 100 %
+	assert(actual_percent_ <= 1000);  // be sure we do not go above 100%
 }
 
 }  // namespace Widelands
