@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -327,7 +326,6 @@ class ProductionSite : public Building {
 
 public:
 	explicit ProductionSite(const ProductionSiteDescr& descr);
-	~ProductionSite() override;
 
 	void log_general_info(const EditorGameBase&) const override;
 
@@ -344,8 +342,8 @@ public:
 		OPtr<Worker> worker;
 	};
 
-	WorkingPosition const* working_positions() const {
-		return working_positions_;
+	const std::vector<WorkingPosition>* working_positions() const {
+		return &working_positions_;
 	}
 
 	virtual bool has_workers(DescriptionIndex targetSite, Game& game);
@@ -378,6 +376,8 @@ public:
 	void cleanup(EditorGameBase&) override;
 	void act(Game&, uint32_t data) override;
 
+	std::pair<int32_t, int32_t> find_worker(OPtr<Worker>);
+	void try_replace_worker(const Game*, DescriptionIndex, WorkingPosition*);
 	void remove_worker(Worker&) override;
 	bool warp_worker(EditorGameBase&, const WorkerDescr& wd, int32_t slot = -1);
 
@@ -481,7 +481,7 @@ protected:
 	}
 
 protected:  // TrainingSite must have access to this stuff
-	WorkingPosition* working_positions_;
+	std::vector<WorkingPosition> working_positions_;
 
 	int32_t fetchfromflag_;  ///< Number of wares to fetch from flag
 
