@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -91,7 +90,7 @@ void walk_immovables(Widelands::DescriptionIndex index,
 
 namespace Widelands {
 
-void TribeDescr::load_helptexts(Widelands::MapObjectDescr* descr, const LuaTable& table) {
+void TribeDescr::load_helptexts(Widelands::MapObjectDescr* descr, const LuaTable& table) const {
 	std::map<std::string, std::string> helptexts;
 	if (table.has_key("helptexts")) {
 		std::unique_ptr<LuaTable> helptext_table = table.get_table("helptexts");
@@ -142,14 +141,9 @@ TribeDescr::TribeDescr(const Widelands::TribeBasicInfo& info,
                        const LuaTable* scenario_table)
    : name_(table.get_string("name")),
      descname_(info.descname),
-     military_capacity_script_(table.has_key<std::string>("military_capacity_script") ?
-                                  table.get_string("military_capacity_script") :
-                                  ""),
      descriptions_(descriptions),
      bridge_height_(table.get_int("bridge_height")),
      builder_(Widelands::INVALID_INDEX),
-     carrier_(Widelands::INVALID_INDEX),
-     carrier2_(Widelands::INVALID_INDEX),
      geologist_(Widelands::INVALID_INDEX),
      soldier_(Widelands::INVALID_INDEX),
      ship_(Widelands::INVALID_INDEX),
@@ -160,16 +154,10 @@ TribeDescr::TribeDescr(const Widelands::TribeBasicInfo& info,
 	verb_log_info("┏━ Loading %s", name_.c_str());
 	ScopedTimer timer("┗━ took %ums", true);
 
-	if (military_capacity_script_.empty() || !g_fs->file_exists(military_capacity_script_)) {
-		// TODO(GunChleoc): API compatibility - require after v 1.0
-		log_warn("File '%s' for military_capacity_script for tribe '%s' does not exist",
-		         military_capacity_script_.c_str(), name().c_str());
-	}
-
 	auto set_progress_message = [this](const std::string& str, int i) {
 		Notifications::publish(UI::NoteLoadingMessage(
 		   /** TRANSLATORS: Example: Loading Barbarians: Buildings (2/6) */
-		   (boost::format(_("Loading %1%: %2% (%3%/%4%)")) % descname() % str % i % 6).str()));
+		   format(_("Loading %1%: %2% (%3%/%4%)"), descname(), str, i, 6)));
 	};
 
 	try {
@@ -212,6 +200,71 @@ TribeDescr::TribeDescr(const Widelands::TribeBasicInfo& info,
 			toolbar_image_set_.reset(new ToolbarImageset(*table.get_table("toolbar")));
 		}
 
+		if (table.has_key("fastplace")) {
+			std::unique_ptr<LuaTable> fp = table.get_table("fastplace");
+			for (const std::string& key : fp->keys<std::string>()) {
+				const std::string& val = fp->get_string(key);
+				if (has_building(building_index(val))) {
+					fastplace_defaults_.emplace(key, val);
+				} else {
+					log_warn(
+					   "fastplace: tribe %s does not use building '%s'", name().c_str(), val.c_str());
+				}
+			}
+		}
+
+		// TODO(Nordfriese): Require these strings after v1.1
+		auto load_productionsite_string = [this, &table](std::string& target, const std::string& key,
+		                                                 const std::string& default_value) {
+			if (table.has_key(key)) {
+				target = table.get_string(key);
+			} else {
+				log_warn(
+				   "Tribe '%s' defines no productionsite string '%s'", name().c_str(), key.c_str());
+				target = default_value;
+			}
+		};
+		load_productionsite_string(
+		   productionsite_worker_missing_, "productionsite_worker_missing",
+		   /** TRANSLATORS: Productivity label on a building if there is 1 worker missing */
+		   _("Worker missing"));
+		load_productionsite_string(
+		   productionsite_workers_missing_, "productionsite_workers_missing",
+		   /** TRANSLATORS: Productivity label on a building if there is more than 1 worker missing.
+		      If you need plural forms here, please let us know. */
+		   _("Workers missing"));
+		load_productionsite_string(
+		   productionsite_worker_coming_, "productionsite_worker_coming",
+		   /** TRANSLATORS: Productivity label on a building if there is 1 worker coming */
+		   _("Worker is coming"));
+		load_productionsite_string(
+		   productionsite_workers_coming_, "productionsite_workers_coming",
+		   /** TRANSLATORS: Productivity label on a building if there is more than 1 worker coming. If
+		      you need plural forms here, please let us know. */
+		   _("Workers are coming"));
+
+		auto load_soldier_string = [this, &table](std::string& target, const std::string& key,
+		                                          const std::string& default_value) {
+			if (table.has_key(key)) {
+				target = table.get_string(key);
+			} else {
+				log_warn("Tribe '%s' defines no soldier string '%s'", name().c_str(), key.c_str());
+				target = default_value;
+			}
+		};
+
+		load_soldier_string(soldier_context_, "soldier_context", "empire_soldier");
+		load_soldier_string(soldier_capacity_strings_sg_[0], "soldier_0_sg", "%1% soldier (+%2%)");
+		load_soldier_string(soldier_capacity_strings_pl_[0], "soldier_0_pl", "%1% soldiers (+%2%)");
+		load_soldier_string(soldier_capacity_strings_sg_[1], "soldier_1_sg", "%1% soldier");
+		load_soldier_string(soldier_capacity_strings_pl_[1], "soldier_1_pl", "%1% soldiers");
+		load_soldier_string(
+		   soldier_capacity_strings_sg_[2], "soldier_2_sg", "%1%(+%2%) soldier (+%3%)");
+		load_soldier_string(
+		   soldier_capacity_strings_pl_[2], "soldier_2_pl", "%1%(+%2%) soldiers (+%3%)");
+		load_soldier_string(soldier_capacity_strings_sg_[3], "soldier_3_sg", "%1%(+%2%) soldier");
+		load_soldier_string(soldier_capacity_strings_pl_[3], "soldier_3_pl", "%1%(+%2%) soldiers");
+
 		std::unique_ptr<LuaTable> collectors_points_table =
 		   table.get_table("collectors_points_table");
 		for (int key : collectors_points_table->keys<int>()) {
@@ -220,7 +273,7 @@ TribeDescr::TribeDescr(const Widelands::TribeBasicInfo& info,
 			if (!has_ware(descriptions.safe_ware_index(ware))) {
 				throw GameDataError("Collectors: Tribe does not use ware '%s'", ware.c_str());
 			}
-			collectors_points_table_.push_back(std::make_pair(ware, t->get_int("points")));
+			collectors_points_table_.emplace_back(ware, t->get_int("points"));
 		}
 	} catch (const GameDataError& e) {
 		throw GameDataError("tribe %s: %s", name_.c_str(), e.what());
@@ -414,6 +467,11 @@ void TribeDescr::load_workers(const LuaTable& table, Descriptions& descriptions)
 			try {
 				DescriptionIndex workerindex = descriptions.load_worker(workername);
 				if (has_worker(workerindex)) {
+					if (workername == "frisians_diker") {
+						// TODO(Nordfriese): Ugly v1.0 savegame compatibility hack, remove after v1.1
+						continue;
+					}
+
 					throw GameDataError("Duplicate definition of worker");
 				}
 
@@ -451,15 +509,24 @@ void TribeDescr::load_workers(const LuaTable& table, Descriptions& descriptions)
 		}
 	}
 
+	if (table.has_key("carriers")) {
+		for (const std::string& name : table.get_table("carriers")->array_entries<std::string>()) {
+			carriers_.push_back(add_special_worker(name, descriptions));
+		}
+	} else {
+		log_warn("Tribe %s: Specifying `carrier`/`carrier2` instead of `carriers` is deprecated",
+		         name().c_str());
+		if (table.has_key("carrier")) {
+			carriers_.push_back(add_special_worker(table.get_string("carrier"), descriptions));
+		}
+		if (table.has_key("carrier2")) {
+			carriers_.push_back(add_special_worker(table.get_string("carrier2"), descriptions));
+		}
+	}
+
 	if (table.has_key("builder")) {
 		builder_ = add_special_worker(table.get_string("builder"), descriptions);
 	}
-	if (table.has_key("carrier")) {
-		carrier_ = add_special_worker(table.get_string("carrier"), descriptions);
-	}
-	if (table.has_key("carrier2")) {
-		carrier2_ = add_special_worker(table.get_string("carrier2"), descriptions);
-	}
 	if (table.has_key("geologist")) {
 		geologist_ = add_special_worker(table.get_string("geologist"), descriptions);
 	}
@@ -478,6 +545,11 @@ void TribeDescr::load_buildings(const LuaTable& table, Descriptions& description
 		try {
 			DescriptionIndex index = descriptions.load_building(buildingname);
 			if (has_building(index)) {
+				if (buildingname == "frisians_dikers_house") {
+					// TODO(Nordfriese): Ugly v1.0 savegame compatibility hack, remove after v1.1
+					continue;
+				}
+
 				throw GameDataError("Duplicate definition of building '%s'", buildingname.c_str());
 			}
 			buildings_.insert(index);
@@ -532,9 +604,6 @@ const std::string& TribeDescr::name() const {
 const std::string& TribeDescr::descname() const {
 	return descname_;
 }
-const std::string& TribeDescr::military_capacity_script() const {
-	return military_capacity_script_;
-}
 
 size_t TribeDescr::get_nrwares() const {
 	return wares_.size();
@@ -630,14 +699,6 @@ DescriptionIndex TribeDescr::builder() const {
 	assert(descriptions_.worker_exists(builder_));
 	return builder_;
 }
-DescriptionIndex TribeDescr::carrier() const {
-	assert(descriptions_.worker_exists(carrier_));
-	return carrier_;
-}
-DescriptionIndex TribeDescr::carrier2() const {
-	assert(descriptions_.worker_exists(carrier2_));
-	return carrier2_;
-}
 DescriptionIndex TribeDescr::geologist() const {
 	assert(descriptions_.worker_exists(geologist_));
 	return geologist_;
@@ -706,7 +767,7 @@ Find the best matching indicator for the given amount.
 */
 DescriptionIndex TribeDescr::get_resource_indicator(ResourceDescription const* const res,
                                                     const ResourceAmount amount) const {
-	if (!res || !amount) {
+	if ((res == nullptr) || (amount == 0u)) {
 		auto list = resource_indicators_.find("");
 		if (list == resource_indicators_.end() || list->second.empty()) {
 			throw GameDataError("Tribe '%s' has no indicator for no resources!", name_.c_str());
@@ -748,7 +809,7 @@ ToolbarImageset* TribeDescr::toolbar_image_set() const {
  */
 
 DescriptionIndex TribeDescr::add_special_worker(const std::string& workername,
-                                                Descriptions& descriptions) {
+                                                Descriptions& descriptions) const {
 	try {
 		DescriptionIndex worker = descriptions.load_worker(workername);
 		if (!has_worker(worker)) {
@@ -761,7 +822,7 @@ DescriptionIndex TribeDescr::add_special_worker(const std::string& workername,
 }
 
 DescriptionIndex TribeDescr::add_special_building(const std::string& buildingname,
-                                                  Descriptions& descriptions) {
+                                                  Descriptions& descriptions) const {
 	try {
 		DescriptionIndex building = descriptions.load_building(buildingname);
 		if (!has_building(building)) {
@@ -779,11 +840,9 @@ void TribeDescr::finalize_loading(Descriptions& descriptions) {
 	if (builder_ == Widelands::INVALID_INDEX) {
 		throw GameDataError("special worker 'builder' not defined");
 	}
-	if (carrier_ == Widelands::INVALID_INDEX) {
-		throw GameDataError("special worker 'carrier' not defined");
-	}
-	if (carrier2_ == Widelands::INVALID_INDEX) {
-		throw GameDataError("special worker 'carrier2' not defined");
+	if (carriers_.size() < 2) {
+		throw GameDataError(
+		   "only %d type(s) of carriers defined", static_cast<int>(carriers_.size()));
 	}
 	if (geologist_ == Widelands::INVALID_INDEX) {
 		throw GameDataError("special worker 'geologist' not defined");
@@ -809,7 +868,7 @@ void TribeDescr::finalize_loading(Descriptions& descriptions) {
 }
 
 // Set default trainingsites proportions for AI. Make sure that we get a sum of ca. 100
-void TribeDescr::calculate_trainingsites_proportions(const Descriptions& descriptions) {
+void TribeDescr::calculate_trainingsites_proportions(const Descriptions& descriptions) const {
 	unsigned int trainingsites_without_percent = 0;
 	int used_percent = 0;
 	std::vector<BuildingDescr*> traingsites_with_percent;
@@ -845,7 +904,7 @@ void TribeDescr::calculate_trainingsites_proportions(const Descriptions& descrip
 		}
 		if (percent_to_use < 1) {
 			throw GameDataError(
-			   "%s: Training sites without predefined proportions add up to < 1%% and "
+			   "%s: Training sites without predefined proportions add up to < 1%%, and "
 			   "will never be built: %d",
 			   name().c_str(), used_percent);
 		}
@@ -931,8 +990,6 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 
 	const DescriptionMaintainer<ImmovableDescr>& all_immovables = descriptions.immovables();
 
-	descriptions.postload_immovable_relations();
-
 	// Find all attributes that we need to collect from map
 	std::set<MapObjectDescr::AttributeIndex> needed_attributes;
 	for (ProductionSiteDescr* prod : productionsites) {
@@ -948,7 +1005,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 					const ImmovableDescr& immovable_descr = all_immovables.get(i);
 					if (immovable_descr.has_attribute(attribute_id)) {
 						prod->add_needed_immovable(immovable_descr.name());
-						if (prod->collected_attributes().count(attribinfo)) {
+						if (prod->collected_attributes().count(attribinfo) != 0u) {
 							prod->add_collected_immovable(immovable_descr.name());
 						}
 						const_cast<ImmovableDescr&>(immovable_descr)
@@ -1078,7 +1135,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 
 		// Sites that create a bob should not overlap sites that create the same bob
 		for (const std::string& item : prod->created_bobs()) {
-			if (creators.count(item)) {
+			if (creators.count(item) != 0u) {
 				for (ProductionSiteDescr* creator : creators.at(item)) {
 					prod->add_competing_productionsite(creator->name());
 					creator->add_competing_productionsite(prod->name());
@@ -1089,7 +1146,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 		for (const std::string& item : prod->needed_immovables()) {
 			// Sites that collect immovables and sites of other types that create immovables for them
 			// should overlap each other
-			if (creators.count(item)) {
+			if (creators.count(item) != 0u) {
 				for (ProductionSiteDescr* creator : creators.at(item)) {
 					if (creator != prod) {
 						prod->add_supported_by_productionsite(creator->name());
@@ -1098,7 +1155,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 				}
 			}
 			// Sites that collect immovables should not overlap sites that collect the same immovable
-			if (collectors.count(item)) {
+			if (collectors.count(item) != 0u) {
 				for (const ProductionSiteDescr* collector : collectors.at(item)) {
 					prod->add_competing_productionsite(collector->name());
 				}
@@ -1107,7 +1164,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 		for (const std::string& item : prod->collected_bobs()) {
 			// Sites that collect bobs and sites of other types that create bobs for them should
 			// overlap each other
-			if (creators.count(item)) {
+			if (creators.count(item) != 0u) {
 				for (ProductionSiteDescr* creator : creators.at(item)) {
 					if (creator != prod) {
 						prod->add_supported_by_productionsite(creator->name());
@@ -1116,7 +1173,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 				}
 			}
 			// Sites that collect bobs should not overlap sites that collect the same bob
-			if (collectors.count(item)) {
+			if (collectors.count(item) != 0u) {
 				for (const ProductionSiteDescr* collector : collectors.at(item)) {
 					prod->add_competing_productionsite(collector->name());
 				}
@@ -1125,7 +1182,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 		for (const auto& item : prod->collected_resources()) {
 			// Sites that collect resources and sites of other types that create resources for them
 			// should overlap each other
-			if (creators.count(item.first)) {
+			if (creators.count(item.first) != 0u) {
 				for (ProductionSiteDescr* creator : creators.at(item.first)) {
 					if (creator != prod) {
 						prod->add_supported_by_productionsite(creator->name());
@@ -1134,7 +1191,7 @@ void TribeDescr::process_productionsites(Descriptions& descriptions) {
 				}
 			}
 			// Sites that collect resources should not overlap sites that collect the same resource
-			if (collectors.count(item.first)) {
+			if (collectors.count(item.first) != 0u) {
 				for (const ProductionSiteDescr* collector : collectors.at(item.first)) {
 					prod->add_competing_productionsite(collector->name());
 				}
