@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -122,13 +121,13 @@ WorkerProgram::WorkerProgram(const std::string& init_name,
 			Worker::Action act;
 			uint32_t mapidx;
 
-			for (mapidx = 0; parsemap_[mapidx].name; ++mapidx) {
+			for (mapidx = 0; parsemap_[mapidx].name != nullptr; ++mapidx) {
 				if (parseinput.name == parsemap_[mapidx].name) {
 					break;
 				}
 			}
 
-			if (!parsemap_[mapidx].name) {
+			if (parsemap_[mapidx].name == nullptr) {
 				throw GameDataError(
 				   "Unknown command '%s' in line '%s'", parseinput.name.c_str(), line.c_str());
 			}
@@ -301,10 +300,12 @@ void WorkerProgram::parse_breed(Worker::Action* act, const std::vector<std::stri
 findobject
 ^^^^^^^^^^
 .. function:: findobject=radius:\<distance\> [type:\<map_object_type\>] [attrib:\<attribute\>]
+   [no_notify]
 
    :arg int radius: Search for an object within the given radius around the worker.
    :arg string type: The type of map object to search for. Defaults to ``immovable``.
    :arg string attrib: The attribute that the map object should possess.
+   :arg empty no_notify: Do not send a message to the player if this step fails.
 
    Find and select an object based on a number of predicates, which can be specified
    in arbitrary order. The object can then be used in other commands like ``walk``
@@ -334,16 +335,23 @@ findobject
 /**
  * iparam1 = radius predicate
  * iparam2 = attribute predicate (if >= 0)
+ * iparam3 = send message on failure (if != 0)
  * sparam1 = type
  */
 void WorkerProgram::parse_findobject(Worker::Action* act, const std::vector<std::string>& cmd) {
 	act->function = &Worker::run_findobject;
 	act->iparam1 = -1;
 	act->iparam2 = -1;
+	act->iparam3 = 1;
 	act->sparam1 = "immovable";
 
 	// Parse predicates
 	for (const std::string& argument : cmd) {
+		if (argument == "no_notify") {
+			act->iparam3 = 0;
+			continue;
+		}
+
 		const std::pair<std::string, std::string> item = read_key_value_pair(argument, ':');
 
 		if (item.first == "radius") {
@@ -372,7 +380,8 @@ void WorkerProgram::parse_findobject(Worker::Action* act, const std::vector<std:
 findspace
 ^^^^^^^^^
 .. function:: findspace=size:\<plot\> radius:\<distance\> [breed] [resource:\<name\>]
-   [avoid:\<immovable_attribute\>] [saplingsearches:\<number\>] [space] [terraform:\<category\>]
+   [avoid:\<immovable_attribute\>] [saplingsearches:\<number\>] [space] [ferry]
+   [terraform:\<category\>] [no_notify]
 
    :arg string size: The size or building plot type of the free space.
       The possible values are:
@@ -404,9 +413,13 @@ findspace
       neighbors are also walkable (an exception is made if one of the neighboring
       fields is owned by this worker's location).
 
+   :arg empty ferry: Find only fields reachable by a ferry.
+
    :arg string terraform: Find only nodes where at least one adjacent triangle has
       terrain that can be enhanced.
 
+   :arg empty no_notify: Do not send a message to the player if this step fails.
+
    Find a map field based on a number of predicates.
    The field can then be used in other commands like ``walk``. Examples::
 
@@ -443,13 +456,16 @@ findspace
 /**
  * iparam1 = radius
  * iparam2 = FindNodeSize::sizeXXX
- * iparam3 = whether the "space" flag is set
+ * iparam3 = Bitset of "space" (1st bit), "no_notify" (2nd bit), and "ferry" (3rd bit).
  * iparam4 = whether the "breed" flag is set
  * iparam5 = Immovable attribute id
  * iparam6 = Forester retries
  * sparam1 = Resource
  * sparamv = The terraform category (if any)
  */
+// TODO(Nordfriese): All boolean flags (space. breed, no_notify) should be placed in
+// just one iparam. Unfortunately there is no way to have saveloading versioning here.
+// Refactor this when we next break savegame compatibility completely.
 void WorkerProgram::parse_findspace(Worker::Action* act, const std::vector<std::string>& cmd) {
 	act->function = &Worker::run_findspace;
 	act->iparam1 = -1;
@@ -487,7 +503,11 @@ void WorkerProgram::parse_findspace(Worker::Action* act, const std::vector<std::
 			} else if (item.first == "resource") {
 				act->sparam1 = item.second;
 			} else if (item.first == "space") {
-				act->iparam3 = 1;
+				act->iparam3 |= 1 << 0;
+			} else if (item.first == "no_notify") {
+				act->iparam3 |= 1 << 1;
+			} else if (item.first == "ferry") {
+				act->iparam3 |= 1 << 2;
 			} else if (item.first == "avoid") {
 				act->iparam5 = MapObjectDescr::get_attribute_id(item.second);
 			} else if (item.first == "saplingsearches") {
@@ -853,7 +873,8 @@ removeobject
          "return"
       }
 */
-void WorkerProgram::parse_removeobject(Worker::Action* act, const std::vector<std::string>&) {
+void WorkerProgram::parse_removeobject(Worker::Action* act,
+                                       const std::vector<std::string>& /* cmd */) {
 	act->function = &Worker::run_removeobject;
 	if (!needed_attributes_.empty()) {
 		collected_attributes_.insert(needed_attributes_.begin(), needed_attributes_.end());
