@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -77,7 +76,7 @@ Remove this critter
 
 ==============================
 */
-bool Critter::run_remove(Game& game, State& state, const CritterAction&) {
+bool Critter::run_remove(Game& game, State& state, const CritterAction& /* action */) {
 	++state.ivar1;
 	// Bye, bye cruel world
 	schedule_destroy(game);
@@ -267,7 +266,7 @@ void Critter::roam_update(Game& game, State& state) {
 	uint32_t idle_time_min = 1000;
 	uint32_t idle_time_rnd = kCritterMaxIdleTime;
 
-	if (state.ivar1) {
+	if (state.ivar1 != 0) {
 		state.ivar1 = 0;
 		// lots of magic numbers for reasonable weighting of various nearby animals
 		uint32_t roaming_dist = 24;
@@ -327,7 +326,7 @@ void Critter::roam_update(Game& game, State& state) {
 
 	std::vector<Critter*> candidates_for_eating;
 	bool can_eat_immovable = false;
-	if (descr().is_herbivore() && get_position().field->get_immovable()) {
+	if (descr().is_herbivore() && (get_position().field->get_immovable() != nullptr)) {
 		for (uint32_t a : descr().food_plants()) {
 			if (get_position().field->get_immovable()->descr().has_attribute(a)) {
 				can_eat_immovable = true;
@@ -341,7 +340,7 @@ void Critter::roam_update(Game& game, State& state) {
 	bool foundme = false;
 	std::vector<Critter*> all_critters_on_field;
 	all_critters_on_field.push_back(this);  // not caught by the following loop
-	for (Bob* b = get_position().field->get_first_bob(); b; b = b->get_next_bob()) {
+	for (Bob* b = get_position().field->get_first_bob(); b != nullptr; b = b->get_next_bob()) {
 		assert(b);
 		if (b == this) {
 			assert(!foundme);
@@ -413,10 +412,10 @@ void Critter::roam_update(Game& game, State& state) {
 				                                                          // and only if S >= N
 				const double weighted_success_chance =
 				   S <= -N ? 0.0 :
-				             S >= N ? 1.0 :
-				                      -(std::log(S * S + 1) - 2 * S * std::atan(S) - kPi * S -
-				                        std::log(N * N + 1) + N * (2 * std::atan(N) - kPi)) /
-				                         (2 * N * kPi);
+				   S >= N  ? 1.0 :
+                         -(std::log(S * S + 1) - 2 * S * std::atan(S) - kPi * S -
+                          std::log(N * N + 1) + N * (2 * std::atan(N) - kPi)) /
+				               (2 * N * kPi);
 				molog(game.get_gametime(),
 				      "    *** [total strength %d] vs [prey %d] *** success chance %lf\n", S,
 				      defender_strength, weighted_success_chance);
