@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,22 +12,22 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "logic/player.h"
 
+#include <atomic>
 #include <cassert>
 #include <cstdlib>
 #include <memory>
-
-#include <boost/algorithm/string.hpp>
+#include <sstream>
 
 #include "base/i18n.h"
 #include "base/log.h"
 #include "base/macros.h"
+#include "base/string.h"
 #include "base/warning.h"
 #include "base/wexception.h"
 #include "economy/economy.h"
@@ -146,6 +146,7 @@ Player::Player(EditorGameBase& the_egbase,
      see_all_(false),
      player_number_(plnum),
      tribe_(tribe_descr),
+     random_tribe_(false),
      casualties_(0),
      kills_(0),
      msites_lost_(0),
@@ -280,12 +281,12 @@ bool Player::get_starting_position_suitability(const Coords& c) const {
 	}
 
 	const Widelands::Field& neighbour = map[map.br_n(c)];
-	if (neighbour.get_owned_by() != 0 || !(neighbour.nodecaps() & BUILDCAPS_FLAG)) {
+	if (neighbour.get_owned_by() != 0 || ((neighbour.nodecaps() & BUILDCAPS_FLAG) == 0)) {
 		return false;
 	}
 
 	bool is_starting_position = false;
-	for (unsigned p = map.get_nrplayers(); p; --p) {
+	for (unsigned p = map.get_nrplayers(); p != 0u; --p) {
 		if (map.get_starting_pos(p) == c) {
 			is_starting_position = true;
 			break;
@@ -297,10 +298,10 @@ bool Player::get_starting_position_suitability(const Coords& c) const {
 		unsigned obstacles_nearby = 0;
 		MapRegion<Area<FCoords>> mr(map, Area<FCoords>(f, 4));
 		do {
-			if (mr.location().field->get_owned_by()) {
+			if (mr.location().field->get_owned_by() != 0u) {
 				return false;
 			}
-			if (!(mr.location().field->nodecaps() & BUILDCAPS_FLAG)) {
+			if ((mr.location().field->nodecaps() & BUILDCAPS_FLAG) == 0) {
 				++obstacles_nearby;
 				if (obstacles_nearby > 3) {
 					return false;
@@ -312,7 +313,7 @@ bool Player::get_starting_position_suitability(const Coords& c) const {
 	// Check for enemy players nearby
 	MapRegion<Area<FCoords>> mr(map, Area<FCoords>(f, kMinSpaceAroundPlayers));
 	do {
-		if (mr.location().field->get_owned_by()) {
+		if (mr.location().field->get_owned_by() != 0u) {
 			return false;
 		}
 	} while (mr.advance(map));
@@ -335,7 +336,7 @@ void Player::set_team_number(TeamNumber team) {
  * each other.
  */
 bool Player::is_hostile(const Player& other) const {
-	return &other != this && (!team_number_ || team_number_ != other.team_number_) &&
+	return &other != this && ((team_number_ == 0u) || team_number_ != other.team_number_) &&
 	       !is_attack_forbidden(other.player_number());
 }
 
@@ -357,7 +358,7 @@ void Player::AiPersistentState::initialize() {
 	no_more_expeditions = false;
 	target_military_score = 100;
 	least_military_score = 0;
-	ai_productionsites_ratio = std::rand() % 5 + 7;  // NOLINT
+	ai_productionsites_ratio = RNG::static_rand(5) + 7;
 	ai_personality_mil_upper_limit = 100;
 
 	// all zeroes
@@ -387,7 +388,7 @@ void Player::update_team_players() {
 	team_players_ = {};
 
 	// …and tell our new allies to note us
-	if (team_number_) {
+	if (team_number_ != 0u) {
 		for (PlayerNumber p = 1; p <= kMaxPlayers; ++p) {
 			if (p != player_number()) {
 				if (Player* player = egbase().get_player(p)) {
@@ -416,7 +417,7 @@ void Player::show_watch_window(Game& game, Bob& b) {
  * Plays the corresponding sound when a message is received and if sound is
  * enabled.
  */
-void Player::play_message_sound(const Message* message) {
+void Player::play_message_sound(const Message* message) const {
 	if (g_sh->is_sound_enabled(SoundType::kMessage)) {
 		FxId fx;
 		switch (message->type()) {
@@ -441,7 +442,12 @@ MessageId Player::add_message(Game& game, std::unique_ptr<Message> new_message,
 	// MapObject connection
 	if (message->serial() > 0) {
 		MapObject* mo = egbase().objects().get_object(message->serial());
-		mo->removed.connect([this, id](unsigned) { message_object_removed(id); });
+		mo->removed.connect([this, id](unsigned /* serial */) { message_object_removed(id); });
+		if (mo->descr().type() >= MapObjectType::BUILDING) {
+			upcast(Building, site, mo);
+			assert(site != nullptr);
+			site->muted.connect([this, id](unsigned /* serial */) { message_object_removed(id); });
+		}
 	}
 
 	// Sound & popup
@@ -478,7 +484,7 @@ MessageId Player::add_message_with_timeout(Game& game,
 void Player::message_object_removed(MessageId message_id) const {
 	// Send delete command
 	upcast(Game, game, &egbase_);
-	if (!game) {
+	if (game == nullptr) {
 		return;
 	}
 
@@ -509,8 +515,8 @@ NodeCaps Player::get_buildcaps(const FCoords& fc) const {
 
 	if (!fc.field->is_interior(player_number_)) {
 		buildcaps = 0;
-	} else if (buildcaps & BUILDCAPS_BUILDINGMASK) {  // Check if a building's flag can't be build
-		                                               // due to ownership
+	} else if ((buildcaps & BUILDCAPS_BUILDINGMASK) != 0) {  // Check if a building's flag can't be
+		                                                      // build due to ownership
 		FCoords flagcoords;
 		map.get_brn(fc, &flagcoords);
 		if (!flagcoords.field->is_interior(player_number_)) {
@@ -536,7 +542,7 @@ NodeCaps Player::get_buildcaps(const FCoords& fc) const {
 Flag* Player::build_flag(const Coords& c) {
 	int32_t buildcaps = get_buildcaps(egbase().map().get_fcoords(c));
 
-	if (buildcaps & BUILDCAPS_FLAG) {
+	if ((buildcaps & BUILDCAPS_FLAG) != 0) {
 		return new Flag(egbase(), this, c);
 	}
 	return nullptr;
@@ -550,8 +556,9 @@ Flag& Player::force_flag(const FCoords& c) {
 			if (existing_flag->get_owner() == this) {
 				return *existing_flag;
 			}
-		} else if (!dynamic_cast<RoadBase const*>(immovable)) {  //  A road or waterway is OK
-			immovable->remove(egbase());                          //  Make room for the flag.
+		} else if (dynamic_cast<RoadBase const*>(immovable) ==
+		           nullptr) {            //  A road or waterway is OK
+			immovable->remove(egbase());  //  Make room for the flag.
 		}
 	}
 	MapRegion<Area<FCoords>> mr(map, Area<FCoords>(c, 1));
@@ -592,7 +599,7 @@ Road* Player::build_road(const Path& path) {
 						return nullptr;
 					}
 				}
-				if (!(get_buildcaps(fc) & MOVECAPS_WALK)) {
+				if ((get_buildcaps(fc) & MOVECAPS_WALK) == 0) {
 					log_warn_time(
 					   egbase().get_gametime(), "%i: building road, unwalkable\n", player_number());
 					return nullptr;
@@ -742,7 +749,10 @@ Building& Player::force_csite(Coords const location,
 }
 
 bool Player::check_can_build(const BuildingDescr& descr, const FCoords& fc) const {
-	if (!descr.is_buildable()) {
+	if (!descr.is_buildable() ||
+	    !is_building_type_allowed(egbase().descriptions().safe_building_index(descr.name())) ||
+	    !descr.is_useful_on_map(
+	       egbase().map().allows_seafaring(), egbase().map().get_waterway_max_length() >= 2)) {
 		return false;
 	}
 
@@ -760,29 +770,29 @@ bool Player::check_can_build(const BuildingDescr& descr, const FCoords& fc) cons
 	}
 
 	if (descr.get_built_over_immovable() != INVALID_INDEX &&
-	    !(fc.field->get_immovable() &&
+	    !((fc.field->get_immovable() != nullptr) &&
 	      fc.field->get_immovable()->has_attribute(descr.get_built_over_immovable()))) {
 		return false;
 	}
 
 	const NodeCaps buildcaps = descr.get_built_over_immovable() == INVALID_INDEX ?
-	                              get_buildcaps(fc) :
-	                              map.get_max_nodecaps(egbase(), fc);
+                                 get_buildcaps(fc) :
+                                 map.get_max_nodecaps(egbase(), fc);
 	if (descr.get_ismine()) {
-		if (!(buildcaps & BUILDCAPS_MINE)) {
+		if ((buildcaps & BUILDCAPS_MINE) == 0) {
 			return false;
 		}
 	} else {
 		if ((buildcaps & BUILDCAPS_SIZEMASK) < descr.get_size() - BaseImmovable::SMALL + 1) {
 			return false;
 		}
-		if (descr.get_isport() && !(buildcaps & BUILDCAPS_PORT)) {
+		if (descr.get_isport() && ((buildcaps & BUILDCAPS_PORT) == 0)) {
 			return false;
 		}
 	}
 
-	return (get_buildcaps(brn) & BUILDCAPS_FLAG) ||
-	       (brn.field->get_immovable() &&
+	return ((get_buildcaps(brn) & BUILDCAPS_FLAG) != 0) ||
+	       ((brn.field->get_immovable() != nullptr) &&
 	        brn.field->get_immovable()->descr().type() == MapObjectType::FLAG);
 }
 
@@ -820,12 +830,12 @@ Bulldoze the given road, waterway, flag or building.
 */
 void Player::bulldoze(PlayerImmovable& imm, bool const recurse) {
 	std::vector<OPtr<PlayerImmovable>> bulldozelist;
-	bulldozelist.push_back(&imm);
+	bulldozelist.emplace_back(&imm);
 
 	while (!bulldozelist.empty()) {
 		PlayerImmovable* immovable = bulldozelist.back().get(egbase());
 		bulldozelist.pop_back();
-		if (!immovable) {
+		if (immovable == nullptr) {
 			continue;
 		}
 
@@ -836,7 +846,7 @@ void Player::bulldoze(PlayerImmovable& imm, bool const recurse) {
 
 		// Destroy, after extended security check
 		if (upcast(Building, building, immovable)) {
-			if (!(building->get_playercaps() & Building::PCap_Bulldoze)) {
+			if ((building->get_playercaps() & Building::PCap_Bulldoze) == 0u) {
 				return;
 			}
 
@@ -845,11 +855,11 @@ void Player::bulldoze(PlayerImmovable& imm, bool const recurse) {
 			//  Now imm and building are dangling reference/pointer! Do not use!
 
 			if (recurse && flag.is_dead_end()) {
-				bulldozelist.push_back(&flag);
+				bulldozelist.emplace_back(&flag);
 			}
 		} else if (upcast(Flag, flag, immovable)) {
 			if (Building* const flagbuilding = flag->get_building()) {
-				if (!(flagbuilding->get_playercaps() & Building::PCap_Bulldoze)) {
+				if ((flagbuilding->get_playercaps() & Building::PCap_Bulldoze) == 0u) {
 					log_warn_time(egbase().get_gametime(),
 					              "Player trying to rip flag (%u) with undestroyable "
 					              "building (%u)\n",
@@ -860,17 +870,17 @@ void Player::bulldoze(PlayerImmovable& imm, bool const recurse) {
 
 			OPtr<Flag> flagcopy = flag;
 			if (recurse) {
-				for (uint8_t primary_road_id = 6; primary_road_id; --primary_road_id) {
+				for (uint8_t primary_road_id = 6; primary_road_id != 0u; --primary_road_id) {
 					// Recursive bulldoze calls may cause flag to disappear
-					if (!flagcopy.get(egbase())) {
+					if (flagcopy.get(egbase()) == nullptr) {
 						return;
 					}
 
 					if (RoadBase* const primary_road = flag->get_roadbase(primary_road_id)) {
 						Flag& primary_start = primary_road->get_flag(RoadBase::FlagStart);
 						Flag& primary_other = flag == &primary_start ?
-						                         primary_road->get_flag(RoadBase::FlagEnd) :
-						                         primary_start;
+                                           primary_road->get_flag(RoadBase::FlagEnd) :
+                                           primary_start;
 						primary_road->destroy(egbase());
 						verb_log_info_time(egbase().get_gametime(),
 						                   "destroying road/waterway from (%i, %i) going in dir %u\n",
@@ -879,14 +889,14 @@ void Player::bulldoze(PlayerImmovable& imm, bool const recurse) {
 						//  The primary road is gone. Now see if the flag at the other
 						//  end of it is a dead-end.
 						if (primary_other.is_dead_end()) {
-							bulldozelist.push_back(&primary_other);
+							bulldozelist.emplace_back(&primary_other);
 						}
 					}
 				}
 			}
 
 			// Recursive bulldoze calls may cause flag to disappear
-			if (flagcopy.get(egbase())) {
+			if (flagcopy.get(egbase()) != nullptr) {
 				flag->destroy(egbase());
 			}
 		} else if (upcast(RoadBase, road, immovable)) {
@@ -904,12 +914,12 @@ void Player::bulldoze(PlayerImmovable& imm, bool const recurse) {
 
 				OPtr<Flag> endcopy = &end;
 				if (start.is_dead_end()) {
-					bulldozelist.push_back(&start);
+					bulldozelist.emplace_back(&start);
 				}
 				// At this point, end may have become dangling
 				if (Flag* pend = endcopy.get(egbase())) {
 					if (pend->is_dead_end()) {
-						bulldozelist.push_back(&end);
+						bulldozelist.emplace_back(&end);
 					}
 				}
 			}
@@ -995,7 +1005,7 @@ void Player::enhance_or_dismantle(Building* building,
 		if (upcast(Warehouse, wh, building)) {
 			workers = wh->get_incorporated_workers();
 			if (keep_wares) {
-				for (DescriptionIndex di = wh->get_wares().get_nrwareids(); di; --di) {
+				for (DescriptionIndex di = wh->get_wares().get_nrwareids(); di != 0u; --di) {
 					wares[di - 1] = wh->get_wares().stock(di - 1);
 				}
 				if (PortDock* pd = wh->get_portdock()) {
@@ -1212,7 +1222,7 @@ uint32_t Player::find_attack_soldiers(const Flag& flag,
                                       uint32_t nr_wanted) {
 	uint32_t count = 0;
 
-	if (soldiers) {
+	if (soldiers != nullptr) {
 		soldiers->clear();
 	}
 
@@ -1236,12 +1246,12 @@ uint32_t Player::find_attack_soldiers(const Flag& flag,
 		uint32_t const nr_present = present.size();
 		if (nr_staying < nr_present) {
 			uint32_t const nr_taken = std::min(nr_wanted, nr_present - nr_staying);
-			if (soldiers) {
+			if (soldiers != nullptr) {
 				soldiers->insert(soldiers->end(), present.begin(), present.begin() + nr_taken);
 			}
 			count += nr_taken;
 			nr_wanted -= nr_taken;
-			if (!nr_wanted) {
+			if (nr_wanted == 0u) {
 				break;
 			}
 		}
@@ -1273,11 +1283,11 @@ void Player::enemyflagaction(const Flag& flag,
 						} else {
 							// The soldier may not be in a militarysite anymore if he was kicked out
 							// in the short delay between sending and executing a playercommand
-							verb_log_warn_time(
-							   egbase().get_gametime(),
-							   "Player(%u)::enemyflagaction: Not sending soldier %u because he left the "
-							   "building\n",
-							   player_number(), temp_attacker->serial());
+							verb_log_warn_time(egbase().get_gametime(),
+							                   "Player(%u)::enemyflagaction: Not sending soldier %u "
+							                   "because he left the "
+							                   "building\n",
+							                   player_number(), temp_attacker->serial());
 						}
 					}
 				}
@@ -1371,7 +1381,7 @@ void Player::rediscover_node(const Map& map, const FCoords& f) {
 		FCoords tr = map.tr_n(f);
 		Field& tr_field = fields_[tr.field - &first_map_field];
 		if (tr_field.vision != VisibleState::kVisible) {
-			tr_field.terrains.d = tr.field->terrain_d();
+			tr_field.terrains.store({tr.field->terrain_d(), tr_field.terrains.load().r});
 			tr_field.r_sw = tr.field->get_road(WALK_SW);
 			tr_field.owner = tr.field->get_owned_by();
 		}
@@ -1389,7 +1399,7 @@ void Player::rediscover_node(const Map& map, const FCoords& f) {
 		FCoords l = map.l_n(f);
 		Field& l_field = fields_[l.field - &first_map_field];
 		if (l_field.vision != VisibleState::kVisible) {
-			l_field.terrains.r = l.field->terrain_r();
+			l_field.terrains.store({l_field.terrains.load().d, l.field->terrain_r()});
 			l_field.r_e = l.field->get_road(WALK_E);
 			l_field.owner = l.field->get_owned_by();
 		}
@@ -1652,7 +1662,8 @@ void Player::sample_statistics() {
 	assert(worker_stocks_.size() == tribe().workers().size());
 
 	// Calculate stocks
-	std::map<DescriptionIndex, Quantity> ware_stocks, worker_stocks;
+	std::map<DescriptionIndex, Quantity> ware_stocks;
+	std::map<DescriptionIndex, Quantity> worker_stocks;
 	for (DescriptionIndex idx : tribe().wares()) {
 		ware_stocks.insert(std::make_pair(idx, 0U));
 	}
@@ -1773,7 +1784,7 @@ Player::BuildingStatsVector* Player::get_mutable_building_statistics(const Descr
 void Player::update_building_statistics(Building& building, NoteImmovable::Ownership ownership) {
 	upcast(ConstructionSite const, constructionsite, &building);
 	const std::string& building_name =
-	   constructionsite ? constructionsite->building().name() : building.descr().name();
+	   constructionsite != nullptr ? constructionsite->building().name() : building.descr().name();
 
 	const size_t nr_buildings = egbase().descriptions().nr_buildings();
 
@@ -1787,7 +1798,7 @@ void Player::update_building_statistics(Building& building, NoteImmovable::Owner
 
 	if (ownership == NoteImmovable::Ownership::GAINED) {
 		BuildingStats new_building;
-		new_building.is_constructionsite = constructionsite;
+		new_building.is_constructionsite = (constructionsite != nullptr);
 		new_building.pos = building.get_position();
 		stat.push_back(new_building);
 	} else {
@@ -1818,6 +1829,16 @@ const std::string& Player::get_ai() const {
 
 void Player::set_muted(DescriptionIndex di, bool mute) {
 	if (mute) {
+		// trigger muted signal for message deleting
+		const std::vector<Widelands::Player::BuildingStats>& stats_vector =
+		   get_building_statistics(di);
+		for (const Widelands::Player::BuildingStats& stats : stats_vector) {
+			if (!stats.is_constructionsite) {
+				Widelands::BaseImmovable& immovable = *egbase().map()[stats.pos].get_immovable();
+				Widelands::Building& current_building = dynamic_cast<Widelands::Building&>(immovable);
+				current_building.muted(current_building.serial());
+			}
+		}
 		muted_building_types_.insert(di);
 	} else {
 		auto it = muted_building_types_.find(di);
@@ -1914,7 +1935,7 @@ const std::string Player::pick_shipname() {
 	++ship_name_counter_;
 
 	if (remaining_shipnames_.empty()) {
-		return (boost::format(pgettext("shipname", "Ship %d")) % ship_name_counter_).str();
+		return format(pgettext("shipname", "Ship %d"), ship_name_counter_);
 	}
 
 	Game& game = dynamic_cast<Game&>(egbase());
@@ -1977,15 +1998,14 @@ void Player::read_statistics(FileRead& fr, const uint16_t packet_version) {
 	                                       const std::string& description) {
 		if (!stats_string.empty()) {
 			std::vector<std::string> stats_vector;
-			boost::split(stats_vector, stats_string, boost::is_any_of("|"));
+			split(stats_vector, stats_string, {'|'});
 			if (stats_vector.size() != nr_entries) {
 				throw GameDataError("wrong number of %s statistics - expected %" PRIuS
 				                    " but got %" PRIuS,
 				                    description.c_str(), nr_entries, stats_vector.size());
 			}
 			for (size_t j = 0; j < nr_entries; ++j) {
-				stats->at(ware_index)[j] =
-				   boost::lexical_cast<unsigned int>(stats_vector.at(j).c_str());
+				stats->at(ware_index)[j] = stoul(stats_vector.at(j));
 			}
 		} else if (nr_entries > 0) {
 			throw GameDataError("wrong number of %s statistics - expected %" PRIuS " but got 0",
