@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,14 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #ifndef WL_LOGIC_PLAYER_H
 #define WL_LOGIC_PLAYER_H
 
+#include <atomic>
 #include <memory>
 
 #include "base/macros.h"
@@ -99,7 +99,7 @@ public:
 
 	/// Indicates that the object linked to the message has been removed
 	/// from the game. This implementation deletes the message.
-	void message_object_removed(MessageId mid) const;
+	void message_object_removed(MessageId message_id) const;
 
 	void set_message_status(const MessageId& id, Message::Status const status) {
 		get_messages()->set_message_status(id, status);
@@ -173,7 +173,7 @@ public:
 		// Seafaring constants for controlling expeditions
 		static constexpr uint32_t kColonyScanStartArea = 35;
 		static constexpr uint32_t kColonyScanMinArea = 12;
-		static constexpr Time kNoExpedition = Time(0);
+		static constexpr Time kNoExpedition{0};
 
 		AiPersistentState()
 		   : initialized(false),
@@ -239,7 +239,7 @@ public:
 			//  Must be initialized because the rendering code is accessing it
 			//  even for triangles that the player does not see (it is the
 			//  darkening that actually hides the ground from the user).
-			terrains.d = terrains.r = 0;
+			terrains.store(Widelands::Field::Terrains{0, 0});
 
 			time_triangle_last_surveyed[0] = Time();
 			time_triangle_last_surveyed[1] = Time();
@@ -298,7 +298,7 @@ public:
 		 * Each value is only valid when one of the corner nodes of the triangle
 		 * has been seen.
 		 */
-		Widelands::Field::Terrains terrains;
+		std::atomic<Widelands::Field::Terrains> terrains;
 
 		/**
 		 * The road types of the 3 edges, as far as this player knows.
@@ -533,7 +533,7 @@ public:
 
 	uint32_t find_attack_soldiers(const Flag&,
 	                              std::vector<Soldier*>* soldiers = nullptr,
-	                              uint32_t max = std::numeric_limits<uint32_t>::max());
+	                              uint32_t nr_wanted = std::numeric_limits<uint32_t>::max());
 	void enemyflagaction(const Flag&,
 	                     PlayerNumber attacker,
 	                     const std::vector<Widelands::Soldier*>&,
@@ -600,6 +600,13 @@ public:
 	void set_ai(const std::string&);
 	const std::string& get_ai() const;
 
+	void set_random_tribe(bool random) {
+		random_tribe_ = random;
+	}
+	bool has_random_tribe() const {
+		return random_tribe_;
+	}
+
 	// used in shared kingdom mode
 	void add_further_starting_position(uint8_t plr, uint8_t init) {
 		further_shared_in_player_.push_back(plr);
@@ -655,7 +662,7 @@ private:
 	BuildingStatsVector* get_mutable_building_statistics(const DescriptionIndex& i);
 	void update_building_statistics(Building&, NoteImmovable::Ownership ownership);
 	void update_team_players();
-	void play_message_sound(const Message* message);
+	void play_message_sound(const Message* message) const;
 	void enhance_or_dismantle(Building*, DescriptionIndex index_of_new_building, bool keep_wares);
 
 	/// Called when a node becomes seen, stops being seen or has changed. Discovers the node and
@@ -678,6 +685,7 @@ private:
 	bool see_all_;
 	const PlayerNumber player_number_;
 	const TribeDescr& tribe_;  // buildings, wares, workers, sciences
+	bool random_tribe_;
 	uint32_t casualties_, kills_;
 	uint32_t msites_lost_, msites_defeated_;
 	uint32_t civil_blds_lost_, civil_blds_defeated_;
