@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2021 by the Widelands Development Team
+ * Copyright (C) 2004-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,13 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "logic/playercommand.h"
 
+#include <memory>
+
 #include "base/log.h"
 #include "base/macros.h"
 #include "base/wexception.h"
@@ -29,10 +30,12 @@
 #include "io/filewrite.h"
 #include "io/streamwrite.h"
 #include "logic/game.h"
+#include "logic/game_controller.h"
 #include "logic/map_objects/tribes/market.h"
 #include "logic/map_objects/tribes/soldier.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/player.h"
+#include "logic/playersmanager.h"
 #include "logic/widelands_geometry_io.h"
 #include "map_io/map_object_loader.h"
 #include "map_io/map_object_saver.h"
@@ -158,6 +161,8 @@ PlayerCommand* PlayerCommand::deserialize(StreamRead& des) {
 		return new CmdPickCustomStartingPosition(des);
 	case QueueCommandTypes::kMarkMapObjectForRemoval:
 		return new CmdMarkMapObjectForRemoval(des);
+	case QueueCommandTypes::kDiplomacy:
+		return new CmdDiplomacy(des);
 
 	default:
 		throw wexception("PlayerCommand::deserialize(): Encountered invalid command id: %d",
@@ -186,7 +191,7 @@ void PlayerCommand::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader&
 		if (packet_version == kCurrentPacketVersionPlayerCommand) {
 			GameLogicCommand::read(fr, egbase, mol);
 			sender_ = fr.unsigned_8();
-			if (!egbase.get_player(sender_)) {
+			if (egbase.get_player(sender_) == nullptr) {
 				throw GameDataError("player %u does not exist", sender_);
 			}
 			cmdserial_ = fr.unsigned_32();
@@ -204,7 +209,7 @@ void PlayerCommand::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader&
 CmdBulldoze::CmdBulldoze(StreamRead& des)
    : PlayerCommand(Time(0), des.unsigned_8()),
      serial(des.unsigned_32()),
-     recurse(des.unsigned_8()) {
+     recurse(des.unsigned_8() != 0u) {
 }
 
 void CmdBulldoze::execute(Game& game) {
@@ -216,7 +221,7 @@ void CmdBulldoze::execute(Game& game) {
 void CmdBulldoze::serialize(StreamWrite& ser) {
 	write_id_and_sender(ser);
 	ser.unsigned_32(serial);
-	ser.unsigned_8(recurse);
+	ser.unsigned_8(static_cast<uint8_t>(recurse));
 }
 
 constexpr uint16_t kCurrentPacketVersionCmdBulldoze = 2;
@@ -227,7 +232,7 @@ void CmdBulldoze::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mo
 		if (packet_version == kCurrentPacketVersionCmdBulldoze) {
 			PlayerCommand::read(fr, egbase, mol);
 			serial = get_object_serial_or_zero<PlayerImmovable>(fr.unsigned_32(), mol);
-			recurse = 2 <= packet_version ? fr.unsigned_8() : false;
+			recurse = (fr.unsigned_8() != 0);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdBulldoze", packet_version, kCurrentPacketVersionCmdBulldoze);
@@ -243,7 +248,7 @@ void CmdBulldoze::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& m
 	PlayerCommand::write(fw, egbase, mos);
 	// Now serial
 	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(serial)));
-	fw.unsigned_8(recurse);
+	fw.unsigned_8(static_cast<uint8_t>(recurse));
 }
 
 /*** class Cmd_Build ***/
@@ -525,7 +530,7 @@ void CmdFlagAction::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader&
 			serial_ = get_object_serial_or_zero<Flag>(fr.unsigned_32(), mol);
 			// TODO(Nordfriese): Savegame compatibility
 			type_ = packet_version < 3 ? FlagJob::Type::kGeologist :
-			                             static_cast<FlagJob::Type>(fr.unsigned_8());
+                                      static_cast<FlagJob::Type>(fr.unsigned_8());
 		} else {
 			throw UnhandledVersionError(
 			   "CmdFlagAction", packet_version, kCurrentPacketVersionCmdFlagAction);
@@ -706,7 +711,7 @@ CmdExpeditionConfig::CmdExpeditionConfig(StreamRead& des)
 	serial = des.unsigned_32();
 	type = des.unsigned_8() == 0 ? wwWARE : wwWORKER;
 	index = des.unsigned_32();
-	add = des.unsigned_8();
+	add = (des.unsigned_8() != 0u);
 }
 
 void CmdExpeditionConfig::execute(Game& game) {
@@ -735,7 +740,7 @@ void CmdExpeditionConfig::read(FileRead& fr, EditorGameBase& egbase, MapObjectLo
 			serial = get_object_serial_or_zero<PortDock>(fr.unsigned_32(), mol);
 			type = fr.unsigned_8() == 0 ? wwWARE : wwWORKER;
 			index = fr.unsigned_32();
-			add = fr.unsigned_8();
+			add = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdExpeditionConfig", packet_version, kCurrentPacketVersionCmdExpeditionConfig);
@@ -759,7 +764,7 @@ void CmdExpeditionConfig::write(FileWrite& fw, EditorGameBase& egbase, MapObject
 CmdEnhanceBuilding::CmdEnhanceBuilding(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
 	serial_ = des.unsigned_32();
 	bi_ = des.unsigned_16();
-	keep_wares_ = des.unsigned_8();
+	keep_wares_ = (des.unsigned_8() != 0u);
 }
 
 void CmdEnhanceBuilding::execute(Game& game) {
@@ -789,7 +794,7 @@ void CmdEnhanceBuilding::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoa
 			PlayerCommand::read(fr, egbase, mol);
 			serial_ = get_object_serial_or_zero<Building>(fr.unsigned_32(), mol);
 			bi_ = fr.unsigned_16();
-			keep_wares_ = fr.unsigned_8();
+			keep_wares_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdEnhanceBuilding", packet_version, kCurrentPacketVersionCmdEnhanceBuilding);
@@ -810,7 +815,7 @@ void CmdEnhanceBuilding::write(FileWrite& fw, EditorGameBase& egbase, MapObjectS
 CmdDismantleBuilding::CmdDismantleBuilding(StreamRead& des)
    : PlayerCommand(Time(0), des.unsigned_8()) {
 	serial_ = des.unsigned_32();
-	keep_wares_ = des.unsigned_8();
+	keep_wares_ = (des.unsigned_8() != 0u);
 }
 
 void CmdDismantleBuilding::execute(Game& game) {
@@ -833,7 +838,7 @@ void CmdDismantleBuilding::read(FileRead& fr, EditorGameBase& egbase, MapObjectL
 		if (packet_version == kCurrentPacketVersionDismantleBuilding) {
 			PlayerCommand::read(fr, egbase, mol);
 			serial_ = get_object_serial_or_zero<Building>(fr.unsigned_32(), mol);
-			keep_wares_ = fr.unsigned_8();
+			keep_wares_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdDismantleBuilding", packet_version, kCurrentPacketVersionDismantleBuilding);
@@ -860,7 +865,7 @@ CmdEvictWorker::CmdEvictWorker(StreamRead& des) : PlayerCommand(Time(0), des.uns
 
 void CmdEvictWorker::execute(Game& game) {
 	upcast(Worker, worker, game.objects().get_object(serial));
-	if (worker && worker->owner().player_number() == sender()) {
+	if ((worker != nullptr) && worker->owner().player_number() == sender()) {
 		worker->evict(game);
 	}
 }
@@ -905,7 +910,7 @@ CmdShipScoutDirection::CmdShipScoutDirection(StreamRead& des)
 
 void CmdShipScoutDirection::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if (ship && ship->get_owner()->player_number() == sender()) {
+	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
 		if (!(ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionWaiting ||
 		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionPortspaceFound ||
 		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionScouting)) {
@@ -968,7 +973,7 @@ CmdShipConstructPort::CmdShipConstructPort(StreamRead& des)
 
 void CmdShipConstructPort::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if (ship && ship->get_owner()->player_number() == sender()) {
+	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
 		if (ship->get_ship_state() != Widelands::Ship::ShipStates::kExpeditionPortspaceFound) {
 			log_warn_time(game.get_gametime(),
 			              " %1d:ship on %3dx%3d received build port command but "
@@ -1027,7 +1032,7 @@ CmdShipExploreIsland::CmdShipExploreIsland(StreamRead& des)
 
 void CmdShipExploreIsland::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if (ship && ship->get_owner()->player_number() == sender()) {
+	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
 		if (!(ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionWaiting ||
 		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionPortspaceFound ||
 		      ship->get_ship_state() == Widelands::Ship::ShipStates::kExpeditionScouting)) {
@@ -1087,7 +1092,7 @@ CmdShipSink::CmdShipSink(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_
 
 void CmdShipSink::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if (ship && ship->get_owner()->player_number() == sender()) {
+	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
 		ship->sink_ship(game);
 	}
 }
@@ -1131,7 +1136,7 @@ CmdShipCancelExpedition::CmdShipCancelExpedition(StreamRead& des)
 
 void CmdShipCancelExpedition::execute(Game& game) {
 	upcast(Ship, ship, game.objects().get_object(serial));
-	if (ship && ship->get_owner()->player_number() == sender()) {
+	if ((ship != nullptr) && ship->get_owner()->player_number() == sender()) {
 		ship->exp_cancel(game);
 	}
 }
@@ -1227,7 +1232,7 @@ void CmdSetWarePriority::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoa
 			type_ = WareWorker(fr.unsigned_8());
 			index_ = fr.signed_32();
 			priority_ = WarePriority(fr);
-			is_constructionsite_setting_ = fr.unsigned_8();
+			is_constructionsite_setting_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdSetWarePriority", packet_version, kCurrentPacketVersionCmdSetWarePriority);
@@ -1244,7 +1249,7 @@ CmdSetWarePriority::CmdSetWarePriority(StreamRead& des)
      type_(WareWorker(des.unsigned_8())),
      index_(des.signed_32()),
      priority_(des),
-     is_constructionsite_setting_(des.unsigned_8()) {
+     is_constructionsite_setting_(des.unsigned_8() != 0u) {
 }
 
 void CmdSetWarePriority::serialize(StreamWrite& ser) {
@@ -1346,7 +1351,7 @@ void CmdSetInputMaxFill::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoa
 				type_ = wwWORKER;
 			}
 			max_fill_ = fr.unsigned_32();
-			is_constructionsite_setting_ = fr.unsigned_8();
+			is_constructionsite_setting_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdSetInputMaxFill", packet_version, kCurrentPacketVersionCmdSetInputMaxFill);
@@ -1365,7 +1370,7 @@ CmdSetInputMaxFill::CmdSetInputMaxFill(StreamRead& des) : PlayerCommand(Time(0),
 		type_ = wwWORKER;
 	}
 	max_fill_ = des.unsigned_32();
-	is_constructionsite_setting_ = des.unsigned_8();
+	is_constructionsite_setting_ = (des.unsigned_8() != 0u);
 }
 
 void CmdSetInputMaxFill::serialize(StreamWrite& ser) {
@@ -1634,14 +1639,12 @@ CmdChangeSoldierCapacity::CmdChangeSoldierCapacity(StreamRead& des)
 void CmdChangeSoldierCapacity::execute(Game& game) {
 	MapObject* mo = game.objects().get_object(serial);
 	if (upcast(ConstructionSite, cs, mo)) {
-		assert(val >= 0);
-		uint32_t capacity = static_cast<uint32_t>(val);
+		// Clamp the capacity between the minimum and maximum value because the player may
+		// have sent multiple decrease/increase commands at the same time (bug #5006).
 		if (upcast(MilitarysiteSettings, ms, cs->get_settings())) {
-			assert(ms->max_capacity >= capacity);
-			ms->desired_capacity = capacity;
+			ms->desired_capacity = std::max(1, std::min<int32_t>(ms->max_capacity, val));
 		} else if (upcast(TrainingsiteSettings, ts, cs->get_settings())) {
-			assert(ts->max_capacity >= capacity);
-			ts->desired_capacity = capacity;
+			ts->desired_capacity = std::max(0, std::min<int32_t>(ts->max_capacity, val));
 		}
 	} else if (upcast(Building, building, mo)) {
 		if (building->get_owner() == game.get_player(sender()) &&
@@ -1706,7 +1709,7 @@ CmdEnemyFlagAction::CmdEnemyFlagAction(StreamRead& des) : PlayerCommand(Time(0),
 	for (uint32_t i = 0; i < number; ++i) {
 		soldiers_.push_back(des.unsigned_32());
 	}
-	allow_conquer_ = des.unsigned_8();
+	allow_conquer_ = (des.unsigned_8() != 0u);
 }
 
 void CmdEnemyFlagAction::execute(Game& game) {
@@ -1783,7 +1786,7 @@ void CmdEnemyFlagAction::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoa
 					soldiers_.push_back(s->serial());
 				}
 			}
-			allow_conquer_ = packet_version < 5 || fr.unsigned_8();
+			allow_conquer_ = packet_version < 5 || (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdEnemyFlagAction", packet_version, kCurrentPacketVersionCmdEnemyFlagAction);
@@ -1881,8 +1884,7 @@ CmdSetStockPolicy::CmdSetStockPolicy(const Time& time,
 	policy_ = policy;
 }
 
-CmdSetStockPolicy::CmdSetStockPolicy()
-   : PlayerCommand(), warehouse_(0), isworker_(false), ware_(0), policy_() {
+CmdSetStockPolicy::CmdSetStockPolicy() : warehouse_(0), isworker_(false), ware_(0), policy_() {
 }
 
 void CmdSetStockPolicy::execute(Game& game) {
@@ -1928,7 +1930,7 @@ void CmdSetStockPolicy::execute(Game& game) {
 
 CmdSetStockPolicy::CmdSetStockPolicy(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
 	warehouse_ = des.unsigned_32();
-	isworker_ = des.unsigned_8();
+	isworker_ = (des.unsigned_8() != 0u);
 	ware_ = DescriptionIndex(des.unsigned_8());
 	policy_ = static_cast<StockPolicy>(des.unsigned_8());
 }
@@ -1936,7 +1938,7 @@ CmdSetStockPolicy::CmdSetStockPolicy(StreamRead& des) : PlayerCommand(Time(0), d
 void CmdSetStockPolicy::serialize(StreamWrite& ser) {
 	write_id_and_sender(ser);
 	ser.unsigned_32(warehouse_);
-	ser.unsigned_8(isworker_);
+	ser.unsigned_8(static_cast<uint8_t>(isworker_));
 	ser.unsigned_8(ware_);
 	ser.unsigned_8(static_cast<uint8_t>(policy_));
 }
@@ -1949,7 +1951,7 @@ void CmdSetStockPolicy::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoad
 		if (packet_version == kCurrentPacketVersionCmdSetStockPolicy) {
 			PlayerCommand::read(fr, egbase, mol);
 			warehouse_ = fr.unsigned_32();
-			isworker_ = fr.unsigned_8();
+			isworker_ = (fr.unsigned_8() != 0u);
 			ware_ = DescriptionIndex(fr.unsigned_8());
 			policy_ = static_cast<StockPolicy>(fr.unsigned_8());
 		} else {
@@ -1965,7 +1967,7 @@ void CmdSetStockPolicy::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSa
 	fw.unsigned_8(kCurrentPacketVersionCmdSetStockPolicy);
 	PlayerCommand::write(fw, egbase, mos);
 	fw.unsigned_32(warehouse_);
-	fw.unsigned_8(isworker_);
+	fw.unsigned_8(static_cast<uint8_t>(isworker_));
 	fw.unsigned_8(ware_);
 	fw.unsigned_8(static_cast<uint8_t>(policy_));
 }
@@ -1974,8 +1976,7 @@ CmdProposeTrade::CmdProposeTrade(const Time& time, PlayerNumber pn, const Trade&
    : PlayerCommand(time, pn), trade_(trade) {
 }
 
-CmdProposeTrade::CmdProposeTrade() : PlayerCommand() {
-}
+CmdProposeTrade::CmdProposeTrade() = default;
 
 void CmdProposeTrade::execute(Game& game) {
 	Player* plr = game.get_player(sender());
@@ -2057,7 +2058,7 @@ void CmdToggleMuteMessages::execute(Game& game) {
 CmdToggleMuteMessages::CmdToggleMuteMessages(StreamRead& des)
    : PlayerCommand(Time(0), des.unsigned_8()) {
 	building_ = des.unsigned_32();
-	all_ = des.unsigned_8();
+	all_ = (des.unsigned_8() != 0u);
 }
 
 void CmdToggleMuteMessages::serialize(StreamWrite& ser) {
@@ -2074,7 +2075,7 @@ void CmdToggleMuteMessages::read(FileRead& fr, EditorGameBase& egbase, MapObject
 		if (packet_version == kCurrentPacketVersionCmdToggleMuteMessages) {
 			PlayerCommand::read(fr, egbase, mol);
 			building_ = fr.unsigned_32();
-			all_ = fr.unsigned_8();
+			all_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError(
 			   "CmdToggleMuteMessages", packet_version, kCurrentPacketVersionCmdToggleMuteMessages);
@@ -2088,7 +2089,7 @@ void CmdToggleMuteMessages::write(FileWrite& fw, EditorGameBase& egbase, MapObje
 	fw.unsigned_8(kCurrentPacketVersionCmdToggleMuteMessages);
 	PlayerCommand::write(fw, egbase, mos);
 	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(building_)));
-	fw.unsigned_8(all_);
+	fw.unsigned_8(static_cast<uint8_t>(all_));
 }
 
 // CmdMarkMapObjectForRemoval
@@ -2101,7 +2102,7 @@ void CmdMarkMapObjectForRemoval::execute(Game& game) {
 CmdMarkMapObjectForRemoval::CmdMarkMapObjectForRemoval(StreamRead& des)
    : PlayerCommand(Time(0), des.unsigned_8()) {
 	object_ = des.unsigned_32();
-	mark_ = des.unsigned_8();
+	mark_ = (des.unsigned_8() != 0u);
 }
 
 void CmdMarkMapObjectForRemoval::serialize(StreamWrite& ser) {
@@ -2118,7 +2119,7 @@ void CmdMarkMapObjectForRemoval::read(FileRead& fr, EditorGameBase& egbase, MapO
 		if (packet_version == kCurrentPacketVersionCmdMarkMapObjectForRemoval) {
 			PlayerCommand::read(fr, egbase, mol);
 			object_ = fr.unsigned_32();
-			mark_ = fr.unsigned_8();
+			mark_ = (fr.unsigned_8() != 0u);
 		} else {
 			throw UnhandledVersionError("CmdMarkMapObjectForRemoval", packet_version,
 			                            kCurrentPacketVersionCmdMarkMapObjectForRemoval);
@@ -2132,7 +2133,158 @@ void CmdMarkMapObjectForRemoval::write(FileWrite& fw, EditorGameBase& egbase, Ma
 	fw.unsigned_8(kCurrentPacketVersionCmdMarkMapObjectForRemoval);
 	PlayerCommand::write(fw, egbase, mos);
 	fw.unsigned_32(mos.get_object_file_index_or_zero(egbase.objects().get_object(object_)));
-	fw.unsigned_8(mark_);
+	fw.unsigned_8(static_cast<uint8_t>(mark_));
+}
+
+// CmdDiplomacy
+void CmdDiplomacy::execute(Game& game) {
+	if (!game.diplomacy_allowed()) {
+		return;
+	}
+	Player& sending_player = *game.get_safe_player(sender());
+	auto broadcast_message = [&game](const std::string& heading, const std::string& text) {
+		iterate_players_existing(p, game.map().get_nrplayers(), game, player) {
+			player->add_message(game, std::unique_ptr<Message>(new Message(
+			                             Message::Type::kScenario, game.get_gametime(), _("Diplomacy"),
+			                             "images/players/team.png", heading, text)));
+		}
+	};
+
+	switch (action_) {
+	case DiplomacyAction::kResign:
+		for (const auto& status : game.player_manager()->get_players_end_status()) {
+			if (status.player == sender()) {
+				// Ignore if the player lost a moment ago
+				return;
+			}
+		}
+		broadcast_message(_("Player Resigned"), format(_("%s has resigned and is now a spectator."),
+		                                               sending_player.get_name()));
+		game.game_controller()->report_result(sender(), PlayerEndResult::kResigned, "");
+		// TODO(Nordfriese): Making the player a spectator is currently left to the win condition.
+		// In Endless Game and scenarios this makes it effectively impossible to resign currently.
+		break;
+
+	case DiplomacyAction::kLeaveTeam:
+		if (sending_player.team_number() == 0) {
+			break;
+		}
+		broadcast_message(
+		   _("Player Leaves Team"),
+		   format(_("%1$s has left team %2$u and is now teamless."), sending_player.get_name(),
+		          static_cast<unsigned>(sending_player.team_number())));
+		sending_player.set_team_number(0);
+		break;
+
+	case DiplomacyAction::kJoin:
+	case DiplomacyAction::kInvite:
+		broadcast_message(
+		   action_ == DiplomacyAction::kJoin ? _("Team Joining Request") :
+                                             _("Team Joining Invitation"),
+		   format(action_ == DiplomacyAction::kJoin ?
+                   _("%1$s has requested to join the team of %2$s.") :
+                   _("%1$s has invited %2$s to join their team."),
+		          sending_player.get_name(), game.get_safe_player(other_player_)->get_name()));
+		game.pending_diplomacy_actions().emplace_back(sender(), action_, other_player_);
+		// If other_player_ is the interactive player, the IBase
+		// will open a confirmation window on next think()
+		break;
+
+	case DiplomacyAction::kAcceptJoin:
+	case DiplomacyAction::kRefuseJoin:
+	case DiplomacyAction::kAcceptInvite:
+	case DiplomacyAction::kRefuseInvite: {
+		assert(other_player_ != sender());
+		const DiplomacyAction original_action =
+		   (action_ == DiplomacyAction::kAcceptJoin || action_ == DiplomacyAction::kRefuseJoin) ?
+            DiplomacyAction::kJoin :
+            DiplomacyAction::kInvite;
+		for (auto it = game.pending_diplomacy_actions().begin();
+		     it != game.pending_diplomacy_actions().end(); ++it) {
+			// Note that in the response the numbers of the two players
+			// are swapped compared to the original message.
+			if (it->action == original_action && it->sender == other_player_ &&
+			    it->other == sender()) {
+				const bool accept =
+				   action_ == DiplomacyAction::kAcceptJoin || action_ == DiplomacyAction::kAcceptInvite;
+				broadcast_message(
+				   accept ? _("Team Change Accepted") : _("Team Change Rejected"),
+				   format(accept ? original_action == DiplomacyAction::kJoin ?
+                               _("%1$s has accepted %2$s into their team.") :
+                               _("%1$s has accepted the invitation to join the team of %2$s.") :
+				          original_action == DiplomacyAction::kJoin ?
+                               _("%1$s has denied %2$s membership in their team.") :
+                               _("%1$s has rejected the invitation to join the team of %2$s."),
+				          sending_player.get_name(), game.get_safe_player(other_player_)->get_name()));
+
+				if (accept) {
+					Player* joiner = game.get_safe_player(
+					   original_action == DiplomacyAction::kJoin ? other_player_ : sender());
+					Player* other = game.get_safe_player(
+					   original_action != DiplomacyAction::kJoin ? other_player_ : sender());
+					if (other->team_number() == 0) {
+						// Assign both players to a previously unused team slot
+						std::set<TeamNumber> teams;
+						iterate_players_existing_const(p, game.map().get_nrplayers(), game, player) {
+							teams.insert(player->team_number());
+						}
+						for (TeamNumber t = 1;; ++t) {
+							if (teams.count(t) > 0) {
+								continue;
+							}
+							other->set_team_number(t);
+							joiner->set_team_number(t);
+							break;
+						}
+					} else {
+						joiner->set_team_number(other->team_number());
+					}
+				}
+
+				game.pending_diplomacy_actions().erase(it);
+				break;
+			}
+		}
+		// If we found nothing, perhaps the command had been sent twice. Ignore.
+		break;
+	}
+	}
+}
+
+CmdDiplomacy::CmdDiplomacy(StreamRead& des) : PlayerCommand(Time(0), des.unsigned_8()) {
+	action_ = static_cast<DiplomacyAction>(des.unsigned_8());
+	other_player_ = des.unsigned_8();
+}
+
+void CmdDiplomacy::serialize(StreamWrite& ser) {
+	write_id_and_sender(ser);
+	ser.unsigned_8(static_cast<uint8_t>(action_));
+	ser.unsigned_8(other_player_);
+}
+
+constexpr uint8_t kCurrentPacketVersionCmdDiplomacy = 1;
+
+void CmdDiplomacy::read(FileRead& fr, EditorGameBase& egbase, MapObjectLoader& mol) {
+	try {
+		uint8_t packet_version = fr.unsigned_8();
+		if (packet_version == kCurrentPacketVersionCmdDiplomacy) {
+			PlayerCommand::read(fr, egbase, mol);
+			action_ = static_cast<DiplomacyAction>(fr.unsigned_8());
+			other_player_ = fr.unsigned_8();
+		} else {
+			throw UnhandledVersionError(
+			   "CmdDiplomacy", packet_version, kCurrentPacketVersionCmdDiplomacy);
+		}
+	} catch (const std::exception& e) {
+		throw GameDataError("Cmd_Diplomacy: %s", e.what());
+	}
+}
+
+void CmdDiplomacy::write(FileWrite& fw, EditorGameBase& egbase, MapObjectSaver& mos) {
+	fw.unsigned_8(kCurrentPacketVersionCmdDiplomacy);
+	PlayerCommand::write(fw, egbase, mos);
+	fw.unsigned_8(static_cast<uint8_t>(action_));
+	fw.unsigned_8(other_player_);
 }
 
 // CmdPickCustomStartingPosition
