@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2021 by the Widelands Development Team
+ * Copyright (C) 2004-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -524,7 +523,6 @@ struct CmdSetWarePriority : public PlayerCommand {
 	   : PlayerCommand(),
 	     serial_(0),
 	     type_(wwWARE),
-	     index_(),
 	     priority_(WarePriority::kNormal),
 	     is_constructionsite_setting_(false) {
 	}
@@ -561,7 +559,6 @@ struct CmdSetInputMaxFill : public PlayerCommand {
 	CmdSetInputMaxFill()
 	   : PlayerCommand(),
 	     serial_(0),
-	     index_(),
 	     type_(wwWARE),
 	     max_fill_(0),
 	     is_constructionsite_setting_(false) {
@@ -571,7 +568,7 @@ struct CmdSetInputMaxFill : public PlayerCommand {
 	                   PlayerImmovable&,
 	                   DescriptionIndex,
 	                   WareWorker,
-	                   uint32_t maxfill,
+	                   uint32_t max_fill,
 	                   bool cs);
 
 	// Write these commands to a file (for savegames)
@@ -601,7 +598,7 @@ struct CmdChangeTargetQuantity : public PlayerCommand {
 	CmdChangeTargetQuantity(const Time& duetime,
 	                        PlayerNumber sender,
 	                        uint32_t economy,
-	                        DescriptionIndex index);
+	                        DescriptionIndex init_ware_type);
 
 	//  Write/Read these commands to/from a file (for savegames).
 	void write(FileWrite&, EditorGameBase&, MapObjectSaver&) override;
@@ -630,7 +627,7 @@ struct CmdSetWareTargetQuantity : public CmdChangeTargetQuantity {
 	CmdSetWareTargetQuantity(const Time& duetime,
 	                         PlayerNumber sender,
 	                         uint32_t economy,
-	                         DescriptionIndex index,
+	                         DescriptionIndex init_ware_type,
 	                         uint32_t permanent);
 
 	//  Write/Read these commands to/from a file (for savegames).
@@ -656,7 +653,7 @@ struct CmdSetWorkerTargetQuantity : public CmdChangeTargetQuantity {
 	CmdSetWorkerTargetQuantity(const Time& duetime,
 	                           PlayerNumber sender,
 	                           uint32_t economy,
-	                           DescriptionIndex index,
+	                           DescriptionIndex init_ware_type,
 	                           uint32_t permanent);
 
 	//  Write/Read these commands to/from a file (for savegames).
@@ -943,6 +940,30 @@ private:
 	bool mark_;
 };
 
+struct CmdDiplomacy : PlayerCommand {
+	CmdDiplomacy(const Time& t, PlayerNumber p, DiplomacyAction a, PlayerNumber o)
+	   : PlayerCommand(t, p), action_(a), other_player_(o) {
+	}
+
+	QueueCommandTypes id() const override {
+		return QueueCommandTypes::kDiplomacy;
+	}
+
+	void execute(Game& game) override;
+
+	explicit CmdDiplomacy(StreamRead& des);
+	void serialize(StreamWrite& ser) override;
+
+	CmdDiplomacy() : PlayerCommand(), action_(DiplomacyAction::kResign), other_player_(0) {
+	}
+	void write(FileWrite&, EditorGameBase&, MapObjectSaver&) override;
+	void read(FileRead&, EditorGameBase&, MapObjectLoader&) override;
+
+private:
+	DiplomacyAction action_;
+	PlayerNumber other_player_;  // Ignored for kLeaveTeam and kResign
+};
+
 struct CmdPickCustomStartingPosition : PlayerCommand {
 	CmdPickCustomStartingPosition(const Time& t, PlayerNumber p, const Coords& c)
 	   : PlayerCommand(t, p), coords_(c) {
