@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2021 by the Widelands Development Team
+ * Copyright (C) 2007-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,16 +12,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "map_io/map_players_view_packet.h"
 
-#include <boost/algorithm/string.hpp>
+#include <atomic>
+#include <sstream>
 
 #include "base/log.h"
+#include "base/string.h"
 #include "base/wexception.h"
 #include "economy/flag.h"
 #include "economy/road.h"
@@ -56,8 +57,10 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 	FileRead fr;
 	if (!fr.try_open(fs, "binary/view")) {
 		// TODO(Nordfriese): Savegame compatibility – require this packet after v1.0
-		log_warn("New-style view packet not found. There may be strange effects regarding unseen "
-		         "areas.\n");
+		// and suppress this warning when starting a scenario (where it's a false-positive).
+		verb_log_warn(
+		   "New-style view packet not found. There may be strange effects regarding unseen "
+		   "areas.\n");
 		return;
 	}
 
@@ -112,7 +115,7 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// TODO(Niektory): Savegame compatibility
 				if (packet_version <= 4) {
-					boost::split(field_vector, parseme, boost::is_any_of("|"));
+					split(field_vector, parseme, {'|'});
 					assert(field_vector.size() == no_of_fields);
 				}
 
@@ -127,7 +130,7 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 						    saved_vision == VisibleState::kPreviouslySeen) {
 							f.vision = Vision(VisibleState::kPreviouslySeen);
 						}
-						if (revealed_fields.count(m)) {
+						if (revealed_fields.count(m) != 0u) {
 							f.vision.set_revealed(true);
 							assert(f.vision.is_revealed());
 						}
@@ -164,7 +167,7 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 					no_of_seen_fields += additionally_seen;
 					if (additionally_seen > 0) {
 						parseme = fr.c_string();
-						boost::split(field_vector, parseme, boost::is_any_of("|"));
+						split(field_vector, parseme, {'|'});
 						assert(field_vector.size() == additionally_seen);
 						for (size_t i = 0; i < additionally_seen; ++i) {
 							Player::Field& f = player->fields_[stoi(field_vector[i])];
@@ -182,11 +185,11 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Owner: playernumber|playernumber|playernumber ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == seen_fields.size());
 
 				size_t counter = 0;
-				for (auto& field : seen_fields) {
+				for (const auto& field : seen_fields) {
 					field->owner = stoi(field_vector[counter]);
 					++counter;
 				}
@@ -194,11 +197,11 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Last Unseen: time|time|time ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == no_of_seen_fields);
 
 				counter = 0;
-				for (auto& field : seen_fields) {
+				for (const auto& field : seen_fields) {
 					field->time_node_last_unseen = Time(stoll(field_vector[counter]));
 					++counter;
 				}
@@ -206,12 +209,12 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Last Surveyed: time|time|time ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == no_of_seen_fields);
 
 				counter = 0;
-				for (auto& field : seen_fields) {
-					boost::split(data_vector, field_vector[counter], boost::is_any_of("*"));
+				for (const auto& field : seen_fields) {
+					split(data_vector, field_vector[counter], {'*'});
 					assert(data_vector.size() == 2);
 
 					field->time_triangle_last_surveyed[0] = Time(stoll(data_vector[0]));
@@ -222,12 +225,12 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Resource Amounts: down*right|down*right|down*right| ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == no_of_seen_fields);
 
 				counter = 0;
-				for (auto& field : seen_fields) {
-					boost::split(data_vector, field_vector[counter], boost::is_any_of("*"));
+				for (const auto& field : seen_fields) {
+					split(data_vector, field_vector[counter], {'*'});
 					assert(data_vector.size() == 2);
 
 					field->resource_amounts.d = stoi(data_vector[0]);
@@ -238,28 +241,28 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Terrains: down*right|down*right|down*right| ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == no_of_seen_fields);
 
 				counter = 0;
-				for (auto& field : seen_fields) {
-					boost::split(data_vector, field_vector[counter], boost::is_any_of("*"));
+				for (const auto& field : seen_fields) {
+					split(data_vector, field_vector[counter], {'*'});
 					assert(data_vector.size() == 2);
 
-					field->terrains.d = stoi(data_vector[0]);
-					field->terrains.r = stoi(data_vector[1]);
+					field->terrains.store({static_cast<DescriptionIndex>(stoi(data_vector[0])),
+					                       static_cast<DescriptionIndex>(stoi(data_vector[1]))});
 					++counter;
 				}
 				assert(counter == no_of_seen_fields);
 
 				// Roads east*southeast*southwest|east*southeast*southwest| ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == no_of_seen_fields);
 
 				counter = 0;
-				for (auto& field : seen_fields) {
-					boost::split(data_vector, field_vector[counter], boost::is_any_of("*"));
+				for (const auto& field : seen_fields) {
+					split(data_vector, field_vector[counter], {'*'});
 					assert(data_vector.size() == 3);
 
 					field->r_e = static_cast<Widelands::RoadSegment>(stoi(data_vector[0]));
@@ -271,12 +274,12 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Borders: here*right*bottom_right*bottom_left|here*right*bottom_right*bottom_left| ...
 				parseme = fr.c_string();
-				boost::split(field_vector, parseme, boost::is_any_of("|"));
+				split(field_vector, parseme, {'|'});
 				assert(field_vector.size() == no_of_seen_fields);
 
 				counter = 0;
-				for (auto& field : seen_fields) {
-					boost::split(data_vector, field_vector[counter], boost::is_any_of("*"));
+				for (const auto& field : seen_fields) {
+					split(data_vector, field_vector[counter], {'*'});
 					assert(data_vector.size() == 4);
 
 					field->border = from_unsigned(stoi(data_vector[0]));
@@ -289,7 +292,7 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 				// Map objects
 				const Descriptions& descriptions = egbase.descriptions();
-				for (auto& field : seen_fields) {
+				for (const auto& field : seen_fields) {
 					std::string descr = fr.string();
 					if (descr.empty()) {
 						field->map_object_descr = nullptr;
@@ -321,10 +324,10 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 							descr = fr.string();
 							field->constructionsite->was =
 							   descr.empty() ?
-							      nullptr :
-							      descriptions.get_building_descr(descriptions.safe_building_index(descr));
+                           nullptr :
+                           descriptions.get_building_descr(descriptions.safe_building_index(descr));
 
-							for (uint32_t j = fr.unsigned_32(); j; --j) {
+							for (uint32_t j = fr.unsigned_32(); j != 0u; --j) {
 								field->constructionsite->intermediates.push_back(
 								   descriptions.get_building_descr(
 								      descriptions.safe_building_index(fr.string())));
@@ -351,15 +354,15 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 		} else if (packet_version >= 1 && packet_version <= 2) {
 			// TODO(Nordfriese): Savegame compatibility, remove after v1.0
-			for (uint8_t i = fr.unsigned_8(); i; --i) {
+			for (uint8_t i = fr.unsigned_8(); i != 0u; --i) {
 				Player& player = *egbase.get_player(fr.unsigned_8());
 
 				std::set<MapIndex> revealed_fields = {};
-				for (uint32_t j = fr.unsigned_32(); j; --j) {
+				for (uint32_t j = fr.unsigned_32(); j != 0u; --j) {
 					revealed_fields.insert(fr.unsigned_32());
 				}
 
-				for (MapIndex m = map.max_index(); m; --m) {
+				for (MapIndex m = map.max_index(); m != 0u; --m) {
 					Player::Field& f = player.fields_[m - 1];
 
 					f.owner = fr.unsigned_8();
@@ -369,7 +372,7 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 					    saved_vision == VisibleState::kPreviouslySeen) {
 						f.vision = Vision(VisibleState::kPreviouslySeen);
 					}
-					if (revealed_fields.count(m - 1)) {
+					if (revealed_fields.count(m - 1) != 0u) {
 						f.vision.set_revealed(true);
 					}
 
@@ -384,17 +387,18 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 					f.resource_amounts.d = fr.unsigned_8();
 					f.resource_amounts.r = fr.unsigned_8();
 
-					f.terrains.d = fr.unsigned_32();
-					f.terrains.r = fr.unsigned_32();
+					DescriptionIndex terrains_d = static_cast<DescriptionIndex>(fr.unsigned_32());
+					DescriptionIndex terrains_r = static_cast<DescriptionIndex>(fr.unsigned_32());
+					f.terrains.store({terrains_d, terrains_r});
 
 					f.r_e = static_cast<RoadSegment>(fr.unsigned_8());
 					f.r_se = static_cast<RoadSegment>(fr.unsigned_8());
 					f.r_sw = static_cast<RoadSegment>(fr.unsigned_8());
 
-					f.border = fr.unsigned_8();
-					f.border_r = fr.unsigned_8();
-					f.border_br = fr.unsigned_8();
-					f.border_bl = fr.unsigned_8();
+					f.border = (fr.unsigned_8() != 0u);
+					f.border_r = (fr.unsigned_8() != 0u);
+					f.border_br = (fr.unsigned_8() != 0u);
+					f.border_bl = (fr.unsigned_8() != 0u);
 
 					std::string descr = fr.string();
 					if (descr.empty()) {
@@ -428,11 +432,11 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 								   descriptions.safe_building_index(fr.string()));
 								descr = fr.string();
 								f.constructionsite->was = descr.empty() ?
-								                             nullptr :
-								                             descriptions.get_building_descr(
+                                                     nullptr :
+                                                     descriptions.get_building_descr(
 								                                descriptions.safe_building_index(descr));
 
-								for (uint32_t j = fr.unsigned_32(); j; --j) {
+								for (uint32_t j = fr.unsigned_32(); j != 0u; --j) {
 									f.constructionsite->intermediates.push_back(
 									   descriptions.get_building_descr(
 									      descriptions.safe_building_index(fr.string())));
@@ -454,11 +458,11 @@ void MapPlayersViewPacket::read(FileSystem& fs, EditorGameBase& egbase) {
 
 								descr = fr.string();
 								f.constructionsite->was = descr.empty() ?
-								                             nullptr :
-								                             descriptions.get_building_descr(
+                                                     nullptr :
+                                                     descriptions.get_building_descr(
 								                                descriptions.safe_building_index(descr));
 
-								for (uint32_t j = fr.unsigned_32(); j; --j) {
+								for (uint32_t j = fr.unsigned_32(); j != 0u; --j) {
 									f.constructionsite->intermediates.push_back(
 									   descriptions.get_building_descr(
 									      descriptions.safe_building_index(fr.string())));
@@ -505,13 +509,11 @@ void MapPlayersViewPacket::write(FileSystem& fs, EditorGameBase& egbase) {
 			const MapIndex upper_bound = map.max_index() - 1;
 			for (MapIndex m = 0; m < upper_bound; ++m) {
 				const Player::Field& f = player->fields_[m];
-				oss << static_cast<char>(f.vision.is_revealed() ?
-				                            SavedVisionState::kRevealed :
-				                            f.vision.is_hidden() ?
-				                            SavedVisionState::kHidden :
-				                            f.vision == VisibleState::kPreviouslySeen ?
-				                            SavedVisionState::kPreviouslySeen :
-				                            SavedVisionState::kNone);
+				oss << static_cast<char>(f.vision.is_revealed() ? SavedVisionState::kRevealed :
+				                         f.vision.is_hidden()   ? SavedVisionState::kHidden :
+				                         f.vision == VisibleState::kPreviouslySeen ?
+                                                            SavedVisionState::kPreviouslySeen :
+                                                            SavedVisionState::kNone);
 				if (f.vision == VisibleState::kPreviouslySeen) {
 					seen_fields.insert(&f);
 					// The data for some of the terrains and edges between PreviouslySeen
@@ -527,13 +529,11 @@ void MapPlayersViewPacket::write(FileSystem& fs, EditorGameBase& egbase) {
 				}
 			}
 			const Player::Field& f = player->fields_[upper_bound];
-			oss << static_cast<char>(f.vision.is_revealed() ?
-			                            SavedVisionState::kRevealed :
-			                            f.vision.is_hidden() ?
-			                            SavedVisionState::kHidden :
-			                            f.vision == VisibleState::kPreviouslySeen ?
-			                            SavedVisionState::kPreviouslySeen :
-			                            SavedVisionState::kNone);
+			oss << static_cast<char>(f.vision.is_revealed() ? SavedVisionState::kRevealed :
+			                         f.vision.is_hidden()   ? SavedVisionState::kHidden :
+			                         f.vision == VisibleState::kPreviouslySeen ?
+                                                         SavedVisionState::kPreviouslySeen :
+                                                         SavedVisionState::kNone);
 			if (f.vision == VisibleState::kPreviouslySeen) {
 				seen_fields.insert(&f);
 			}
@@ -618,7 +618,7 @@ void MapPlayersViewPacket::write(FileSystem& fs, EditorGameBase& egbase) {
 			// Terrains
 			std::ostringstream oss("");
 			for (auto it = seen_fields.begin(); it != seen_fields.end();) {
-				oss << (*it)->terrains.d << "*" << (*it)->terrains.r;
+				oss << (*it)->terrains.load().d << "*" << (*it)->terrains.load().r;
 				++it;
 				if (it != seen_fields.end()) {
 					oss << "|";
@@ -655,18 +655,21 @@ void MapPlayersViewPacket::write(FileSystem& fs, EditorGameBase& egbase) {
 
 		// Map objects
 		for (const auto& field : seen_fields) {
-			if (field->map_object_descr) {
+			if (field->map_object_descr != nullptr) {
 				fw.string(field->map_object_descr->name());
 
 				if (field->map_object_descr->type() == MapObjectType::DISMANTLESITE) {
 					// `building` can only be nullptr in compatibility cases.
 					// Remove the non-null check after v1.0
-					fw.string(field->dismantlesite.building ? field->dismantlesite.building->name() :
-					                                          "dismantlesite");
+					fw.string(field->dismantlesite.building != nullptr ?
+                            field->dismantlesite.building->name() :
+                            "dismantlesite");
 					fw.unsigned_32(field->dismantlesite.progress);
 				} else if (field->map_object_descr->type() == MapObjectType::CONSTRUCTIONSITE) {
 					fw.string(field->constructionsite->becomes->name());
-					fw.string(field->constructionsite->was ? field->constructionsite->was->name() : "");
+					fw.string(field->constructionsite->was != nullptr ?
+                            field->constructionsite->was->name() :
+                            "");
 
 					fw.unsigned_32(field->constructionsite->intermediates.size());
 					for (const BuildingDescr* d : field->constructionsite->intermediates) {
