@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -63,7 +62,7 @@ bool is_valid_header(const S2MapDescrHeader& header) {
 	if (strncmp(header.magic, "WORLD_V1.0", 10) != 0) {
 		return false;
 	}
-	if (header.name[19]) {
+	if (header.name[19] != 0) {
 		return false;
 	}
 	if (header.w <= 0 || header.h <= 0) {
@@ -75,7 +74,7 @@ bool is_valid_header(const S2MapDescrHeader& header) {
 	if (header.nplayers < 0 || header.nplayers > 7) {
 		return false;
 	}
-	if (header.author[19]) {
+	if (header.author[19] != 0) {
 		return false;
 	}
 	return true;
@@ -228,7 +227,8 @@ load_s2mf_section(FileRead& fr, int32_t const width, int32_t const height) {
 		fr.data(dw - width);  // skip the alignment junk
 	}
 	while (y < dh) {
-		fr.data(dw);  // more alignment junk++ y;
+		fr.data(dw);  // more alignment junk
+		++y;
 	}
 
 	return section;
@@ -370,14 +370,14 @@ S2MapLoader::S2MapLoader(const std::string& filename, Widelands::Map& M)
 
 /// Load the header. The map will then return valid information when
 /// get_width(), get_nrplayers(), get_author() and so on are called.
-int32_t S2MapLoader::preload_map(bool const scenario, std::vector<AddOns::AddOnInfo>* addons) {
+int32_t S2MapLoader::preload_map(bool const scenario, AddOns::AddOnsList* addons) {
 	assert(get_state() != State::kLoaded);
 
 	// s2 maps don't have world add-ons
 	// (UNTESTED because I don't have an s2 map to test this with)
-	if (addons) {
+	if (addons != nullptr) {
 		for (auto it = addons->begin(); it != addons->end();) {
-			if (it->category == AddOns::AddOnCategory::kWorld) {
+			if ((*it)->category == AddOns::AddOnCategory::kWorld) {
 				it = addons->erase(it);
 			} else {
 				++it;
@@ -419,7 +419,8 @@ int32_t S2MapLoader::preload_map(bool const scenario, std::vector<AddOns::AddOnI
  * Completely loads the map, loads the graphics and places all the objects.
  * From now on the Map* can't be set to another one.
  */
-int32_t S2MapLoader::load_map_complete(Widelands::EditorGameBase& egbase, MapLoader::LoadType) {
+int32_t S2MapLoader::load_map_complete(Widelands::EditorGameBase& egbase,
+                                       MapLoader::LoadType /* type */) {
 	std::string timer_message = "S2MapLoader::load_map_complete() for '";
 	timer_message += map_.get_name();
 	timer_message += "' took %ums";
@@ -521,7 +522,7 @@ void S2MapLoader::load_s2mf(Widelands::EditorGameBase& egbase) {
 		for (int16_t x = 0; x < mapwidth; ++x, ++f, ++pc) {
 			uint8_t c = *pc;
 			// Harbour buildspace & textures - Information taken from:
-			if (c & 0x40) {
+			if ((c & 0x40) != 0) {
 				port_spaces_to_set_.insert(Widelands::Coords(x, y));
 			}
 			f->set_terrain_d(terrain_converter.lookup(worldtype_, c & 0x1f));
@@ -541,7 +542,7 @@ void S2MapLoader::load_s2mf(Widelands::EditorGameBase& egbase) {
 			uint8_t c = *pc;
 			// Harbour buildspace & textures - Information taken from:
 			// https://settlers2.net/documentation/world-map-file-format-wldswd/
-			if (c & 0x40) {
+			if ((c & 0x40) != 0) {
 				port_spaces_to_set_.insert(Widelands::Coords(x, y));
 			}
 			f->set_terrain_r(terrain_converter.lookup(worldtype_, c & 0x1f));
@@ -739,7 +740,7 @@ void S2MapLoader::load_s2mf(Widelands::EditorGameBase& egbase) {
 			}
 
 			Widelands::DescriptionIndex nres = 0;
-			if (*res) {
+			if (*res != 0) {
 				try {
 					nres = descriptions->load_resource(res);
 				} catch (const WException&) {
@@ -1079,15 +1080,15 @@ void S2MapLoader::load_s2mf(Widelands::EditorGameBase& egbase) {
 		}
 		Widelands::FCoords fpos = map_.get_fcoords(starting_pos);
 
-		if (!(map_.get_max_nodecaps(egbase, fpos) & Widelands::BUILDCAPS_BIG)) {
+		if ((map_.get_max_nodecaps(egbase, fpos) & Widelands::BUILDCAPS_BIG) == 0) {
 			log_warn("wrong size - trying to fix it: ");
 			bool fixed = false;
 
 			Widelands::MapRegion<Widelands::Area<Widelands::FCoords>> mr(
 			   map_, Widelands::Area<Widelands::FCoords>(fpos, 3));
 			do {
-				if (map_.get_max_nodecaps(egbase, const_cast<Widelands::FCoords&>(mr.location())) &
-				    Widelands::BUILDCAPS_BIG) {
+				if ((map_.get_max_nodecaps(egbase, const_cast<Widelands::FCoords&>(mr.location())) &
+				     Widelands::BUILDCAPS_BIG) != 0) {
 					map_.set_starting_pos(p, mr.location());
 					fixed = true;
 					break;
