@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2021 by the Widelands Development Team
+ * Copyright (C) 2008-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -74,6 +73,7 @@ struct GameClientImpl {
 
 	/// Currently active modal panel. Receives an end_modal on disconnect
 	UI::Panel* modal;
+	UI::Panel* panel_whose_mutex_needs_resetting_on_game_start;
 
 	/// Current game. Only non-null if a game is actually running.
 	Widelands::Game* game;
@@ -108,15 +108,15 @@ struct GameClientImpl {
 	// Whether disconnect() has been called previously.
 	bool disconnect_called_;
 
-	void send_hello();
-	void send_player_command(Widelands::PlayerCommand*);
+	void send_hello() const;
+	void send_player_command(Widelands::PlayerCommand* /*pc*/) const;
 
 	void run_game(InteractiveGameBase* igb);
 
-	InteractiveGameBase* init_game(GameClient* parent, UI::ProgressWindow&);
+	InteractiveGameBase* init_game(GameClient* parent, UI::ProgressWindow& /*loader*/);
 };
 
-void GameClientImpl::send_hello() {
+void GameClientImpl::send_hello() const {
 	SendPacket s;
 	s.unsigned_8(NETCMD_HELLO);
 	s.unsigned_8(NETWORK_PROTOCOL_VERSION);
@@ -125,7 +125,7 @@ void GameClientImpl::send_hello() {
 	net->send(s);
 }
 
-void GameClientImpl::send_player_command(Widelands::PlayerCommand* pc) {
+void GameClientImpl::send_player_command(Widelands::PlayerCommand* pc) const {
 	SendPacket s;
 	s.unsigned_8(NETCMD_PLAYERCOMMAND);
 	s.unsigned_32(game->get_gametime().get());
@@ -141,10 +141,10 @@ InteractiveGameBase* GameClientImpl::init_game(GameClient* parent, UI::ProgressW
 
 	Notifications::publish(UI::NoteLoadingMessage(_("Preparing game…")));
 
-	game->set_game_controller(parent);
+	game->set_game_controller(parent->get_pointer());
 	uint8_t const pn = settings.playernum + 1;
 	game->save_handler().set_autosave_filename(
-	   (boost::format("%s_netclient%u") % kAutosavePrefix % static_cast<unsigned int>(pn)).str());
+	   format("%s_netclient%u", kAutosavePrefix, static_cast<unsigned int>(pn)));
 	InteractiveGameBase* igb;
 	if (pn > 0) {
 		igb = new InteractivePlayer(*game, get_config_section(), pn, true, parent);
@@ -171,11 +171,10 @@ void GameClientImpl::run_game(InteractiveGameBase* igb) {
 
 	modal = igb;
 
-	game->run(settings.savegame ?
-	             Widelands::Game::StartGameType::kSaveGame :
-	             settings.scenario ? Widelands::Game::StartGameType::kMultiPlayerScenario :
-	                                 Widelands::Game::StartGameType::kMap,
-	          "", false, (boost::format("netclient_%d") % static_cast<int>(settings.usernum)).str());
+	game->run(settings.savegame ? Widelands::Game::StartGameType::kSaveGame :
+	          settings.scenario ? Widelands::Game::StartGameType::kMultiPlayerScenario :
+                                 Widelands::Game::StartGameType::kMap,
+	          "", false, format("netclient_%d", static_cast<int>(settings.usernum)));
 
 	// if this is an internet game, tell the metaserver that the game is done.
 	if (internet_) {
@@ -186,12 +185,12 @@ void GameClientImpl::run_game(InteractiveGameBase* igb) {
 }
 
 GameClient::GameClient(FsMenu::MenuCapsule& c,
-                       std::unique_ptr<GameController>& ptr,
+                       std::shared_ptr<GameController>& ptr,
                        const std::pair<NetAddress, NetAddress>& host,
                        const std::string& playername,
                        bool internet,
                        const std::string& gamename)
-   : d(new GameClientImpl), capsule_(c) {
+   : d(new GameClientImpl), capsule_(c), pointer_(ptr) {
 
 	d->internet_ = internet;
 
@@ -220,6 +219,7 @@ GameClient::GameClient(FsMenu::MenuCapsule& c,
 	d->settings.usernum = -2;
 	d->localplayername = playername;
 	d->modal = nullptr;
+	d->panel_whose_mutex_needs_resetting_on_game_start = nullptr;
 	d->game = nullptr;
 	d->realspeed = 0;
 	d->desiredspeed = 1000;
@@ -232,7 +232,7 @@ GameClient::GameClient(FsMenu::MenuCapsule& c,
 	d->participants.reset(new ParticipantList(&(d->settings), d->game, d->localplayername));
 	participants_ = d->participants.get();
 
-	run(ptr);
+	run();
 }
 
 GameClient::~GameClient() {
@@ -244,14 +244,23 @@ GameClient::~GameClient() {
 	delete d;
 }
 
-void GameClient::run(std::unique_ptr<GameController>& ptr) {
+void GameClient::run() {
 	d->send_hello();
 	d->settings.multiplayer = true;
 
 	// Fill the list of possible system messages
 	NetworkGamingMessages::fill_map();
 
-	d->modal = new FsMenu::LaunchMPG(capsule_, *this, *this, *this, *d->game, ptr, d->internet_);
+	d->modal = new FsMenu::LaunchMPG(capsule_, *this, *this, *this, *d->game, d->internet_);
+	// The main menu's think() loop creates a mutex lock that would permanently block the Game
+	// from locking this mutex. So when the game starts we'll need to break the lock by force.
+	for (UI::Panel* p = d->modal; p != nullptr; p = p->get_parent()) {
+		if (p->is_modal()) {
+			d->panel_whose_mutex_needs_resetting_on_game_start = p;
+			return;
+		}
+	}
+	NEVER_HERE();
 }
 
 void GameClient::do_run(RecvPacket& packet) {
@@ -261,11 +270,11 @@ void GameClient::do_run(RecvPacket& packet) {
 	game.set_write_syncstream(get_config_bool("write_syncstreams", true));
 
 	game.enabled_addons().clear();
-	for (size_t i = packet.unsigned_32(); i; --i) {
+	for (size_t i = packet.unsigned_32(); i != 0u; --i) {
 		const std::string name = packet.string();
 		bool found = false;
 		for (const auto& pair : AddOns::g_addons) {
-			if (pair.first.internal_name == name) {
+			if (pair.first->internal_name == name) {
 				game.enabled_addons().push_back(pair.first);
 				found = true;
 				break;
@@ -287,6 +296,10 @@ void GameClient::do_run(RecvPacket& packet) {
 
 		d->game = &game;
 		InteractiveGameBase* igb = d->init_game(this, loader_ui);
+		if (d->panel_whose_mutex_needs_resetting_on_game_start != nullptr) {
+			d->panel_whose_mutex_needs_resetting_on_game_start->clear_current_think_mutex();
+			d->panel_whose_mutex_needs_resetting_on_game_start = nullptr;
+		}
 		d->run_game(igb);
 
 	} catch (const WLWarning& e) {
@@ -308,9 +321,15 @@ void GameClient::do_run(RecvPacket& packet) {
 }
 
 void GameClient::think() {
-	handle_network();
+	NoteThreadSafeFunction::instantiate([this]() { handle_network(); }, true, false);
 
-	if (d->game) {
+	while (!pending_player_commands_.empty()) {
+		MutexLock m(MutexLock::ID::kCommands);
+		do_send_player_command(pending_player_commands_.front());
+		pending_player_commands_.pop_front();
+	}
+
+	if (d->game != nullptr) {
 		// TODO(Klaus Halfmann): what kind of time tricks are done here?
 		if (d->realspeed == 0 || d->server_is_waiting) {
 			d->time.fastforward();
@@ -336,12 +355,16 @@ void GameClient::think() {
  * @param pc will always be deleted in the end.
  */
 void GameClient::send_player_command(Widelands::PlayerCommand* pc) {
+	pending_player_commands_.push_back(pc);
+}
+
+void GameClient::do_send_player_command(Widelands::PlayerCommand* pc) {
 	assert(d->game);
 
 	// TODO(Klaus Halfmann): should this be an assert?
 	if (pc->sender() == d->settings.playernum + 1)  //  allow command for current player only
 	{
-		verb_log_info("[Client]: send playercommand at time %i", d->game->get_gametime().get());
+		verb_log_info("[Client]: enqueue playercommand at time %i\n", d->game->get_gametime().get());
 
 		d->send_player_command(pc);
 
@@ -380,14 +403,14 @@ const GameSettings& GameClient::settings() {
 	return d->settings;
 }
 
-void GameClient::set_scenario(bool) {
+void GameClient::set_scenario(bool /*set*/) {
 }
 
 bool GameClient::can_change_map() {
 	return false;
 }
 
-bool GameClient::can_change_player_state(uint8_t const) {
+bool GameClient::can_change_player_state(uint8_t const /*number*/) {
 	return false;
 }
 
@@ -399,7 +422,7 @@ bool GameClient::can_change_player_team(uint8_t number) {
 	return (number == d->settings.playernum) && !d->settings.scenario && !d->settings.savegame;
 }
 
-bool GameClient::can_change_player_init(uint8_t) {
+bool GameClient::can_change_player_init(uint8_t /*number*/) {
 	return false;
 }
 
@@ -407,20 +430,26 @@ bool GameClient::can_launch() {
 	return false;
 }
 
-void GameClient::set_player_state(uint8_t, PlayerSettings::State) {
+void GameClient::set_player_state(uint8_t /*number*/, PlayerSettings::State /* state */) {
 	// client is not allowed to do this
 }
 
-void GameClient::set_player_ai(uint8_t, const std::string&, bool const /* random_ai */) {
+void GameClient::set_player_ai(uint8_t /*number*/,
+                               const std::string& /* ai */,
+                               bool const /* random_ai */) {
 	// client is not allowed to do this
 }
 
-void GameClient::next_player_state(uint8_t) {
+void GameClient::next_player_state(uint8_t /* state */) {
 	// client is not allowed to do this
 }
 
-void GameClient::set_map(
-   const std::string&, const std::string&, const std::string&, const std::string&, uint32_t, bool) {
+void GameClient::set_map(const std::string& /*mapname*/,
+                         const std::string& /*mapfilename*/,
+                         const std::string& /*map_theme*/,
+                         const std::string& /*map_bg*/,
+                         uint32_t /*maxplayers*/,
+                         bool /*savegame*/) {
 	// client is not allowed to do this
 }
 
@@ -475,7 +504,7 @@ void GameClient::set_player_team(uint8_t number, Widelands::TeamNumber team) {
 	d->net->send(s);
 }
 
-void GameClient::set_player_closeable(uint8_t, bool) {
+void GameClient::set_player_closeable(uint8_t /*number*/, bool /*closeable*/) {
 	//  client is not allowed to do this
 }
 
@@ -503,12 +532,12 @@ void GameClient::set_player_init(uint8_t number, uint8_t initialization_index) {
 	d->net->send(s);
 }
 
-void GameClient::set_player_name(uint8_t, const std::string&) {
+void GameClient::set_player_name(uint8_t /*number*/, const std::string& /* name */) {
 	// until now the name is set before joining - if you allow a change in
 	// launchgame-menu, here properly should be a set_name function
 }
 
-void GameClient::set_player(uint8_t, const PlayerSettings&) {
+void GameClient::set_player(uint8_t /*number*/, const PlayerSettings& /* settings */) {
 	// do nothing here - the request for a positionchange is send in
 	// set_player_number(uint8_t) to the host.
 }
@@ -533,7 +562,7 @@ std::string GameClient::get_win_condition_script() {
 	return d->settings.win_condition_script;
 }
 
-void GameClient::set_win_condition_script(const std::string&) {
+void GameClient::set_win_condition_script(const std::string& /*wc*/) {
 	// Clients are not allowed to change this
 	NEVER_HERE();
 }
@@ -598,10 +627,10 @@ void GameClient::receive_one_player(uint8_t const number, StreamRead& packet) {
 	player.state = static_cast<PlayerSettings::State>(packet.unsigned_8());
 	player.name = packet.string();
 	player.tribe = packet.string();
-	player.random_tribe = packet.unsigned_8();
+	player.random_tribe = (packet.unsigned_8() != 0u);
 	player.initialization_index = packet.unsigned_8();
 	player.ai = packet.string();
-	player.random_ai = packet.unsigned_8();
+	player.random_ai = (packet.unsigned_8() != 0u);
 	player.team = packet.unsigned_8();
 	player.shared_in = packet.unsigned_8();
 	player.color.r = packet.unsigned_8();
@@ -617,12 +646,12 @@ void GameClient::receive_one_user(uint32_t const number, StreamRead& packet) {
 
 	// This might happen, if a users connects after the game starts.
 	if (number == d->settings.users.size()) {
-		d->settings.users.push_back(UserSettings());
+		d->settings.users.emplace_back();
 	}
 
 	d->settings.users.at(number).name = packet.string();
 	d->settings.users.at(number).position = packet.signed_32();
-	d->settings.users.at(number).ready = packet.unsigned_8();
+	d->settings.users.at(number).ready = (packet.unsigned_8() != 0u);
 
 	if (static_cast<int32_t>(number) == d->settings.usernum) {
 		d->localplayername = d->settings.users.at(number).name;
@@ -695,22 +724,22 @@ void GameClient::handle_hello(RecvPacket& packet) {
 
 	d->addons_guard_.reset();
 	std::vector<AddOns::AddOnState> new_g_addons;
-	std::map<std::string, const AddOns::AddOnInfo*> disabled_installed_addons;
+	std::map<std::string, std::shared_ptr<AddOns::AddOnInfo>> disabled_installed_addons;
 	for (const auto& pair : AddOns::g_addons) {
-		disabled_installed_addons[pair.first.internal_name] = &pair.first;
+		disabled_installed_addons[pair.first->internal_name] = pair.first;
 	}
 	std::set<std::string> missing_addons;
 	std::map<std::string, std::pair<std::string /* installed */, std::string /* host */>>
 	   wrong_version_addons;
-	for (size_t i = packet.unsigned_32(); i; --i) {
+	for (size_t i = packet.unsigned_32(); i != 0u; --i) {
 		const std::string name = packet.string();
 		disabled_installed_addons.erase(name);
 		const AddOns::AddOnVersion v = AddOns::string_to_version(packet.string());
 		AddOns::AddOnVersion found;
 		for (const auto& pair : AddOns::g_addons) {
-			if (pair.first.internal_name == name) {
-				found = pair.first.version;
-				new_g_addons.push_back(std::make_pair(pair.first, true));
+			if (pair.first->internal_name == name) {
+				found = pair.first->version;
+				new_g_addons.emplace_back(pair.first, true);
 				break;
 			}
 		}
@@ -723,24 +752,19 @@ void GameClient::handle_hello(RecvPacket& packet) {
 	}
 	if (!missing_addons.empty() || !wrong_version_addons.empty()) {
 		const size_t nr = missing_addons.size() + wrong_version_addons.size();
-		std::string message = (boost::format(ngettext("%u add-on mismatch detected:\n",
-		                                              "%u add-on mismatches detected:\n", nr)) %
-		                       nr)
-		                         .str();
+		std::string message = format(
+		   ngettext("%u add-on mismatch detected:\n", "%u add-on mismatches detected:\n", nr), nr);
 		for (const std::string& name : missing_addons) {
-			message =
-			   (boost::format(_("%1$s\n· ‘%2$s’ required by host not found")) % message % name).str();
+			message = format(_("%1$s\n· ‘%2$s’ required by host not found"), message, name);
 		}
 		for (const auto& pair : wrong_version_addons) {
-			message = (boost::format(
-			              _("%1$s\n· ‘%2$s’ installed at version %3$s but host uses version %4$s")) %
-			           message % pair.first % pair.second.first % pair.second.second)
-			             .str();
+			message = format(_("%1$s\n· ‘%2$s’ installed at version %3$s but host uses version %4$s"),
+			                 message, pair.first, pair.second.first, pair.second.second);
 		}
 		throw AddOnsMismatchException(message);
 	}
 	for (const auto& pair : disabled_installed_addons) {
-		new_g_addons.push_back(std::make_pair(*pair.second, false));
+		new_g_addons.emplace_back(pair.second, false);
 	}
 	AddOns::g_addons = new_g_addons;
 }
@@ -748,7 +772,7 @@ void GameClient::handle_hello(RecvPacket& packet) {
 /**
  * Give a pong for a ping
  */
-void GameClient::handle_ping(RecvPacket&) {
+void GameClient::handle_ping(RecvPacket& /* packet */) {
 	SendPacket s;
 	s.unsigned_8(NETCMD_PONG);
 	d->net->send(s);
@@ -764,8 +788,8 @@ void GameClient::handle_setting_map(RecvPacket& packet) {
 	d->settings.mapfilename = g_fs->FileSystem::fix_cross_file(packet.string());
 	d->settings.map_theme = packet.string();
 	d->settings.map_background = packet.string();
-	d->settings.savegame = packet.unsigned_8();
-	d->settings.scenario = packet.unsigned_8();
+	d->settings.savegame = (packet.unsigned_8() != 0u);
+	d->settings.scenario = (packet.unsigned_8() != 0u);
 	verb_log_info("[Client] SETTING_MAP '%s' '%s'", d->settings.mapname.c_str(),
 	              d->settings.mapfilename.c_str());
 
@@ -962,7 +986,7 @@ void GameClient::handle_file_part(RecvPacket& packet) {
 void GameClient::handle_setting_tribes(RecvPacket& packet) {
 	d->settings.tribes.clear();
 	Widelands::AllTribes all_tribes = Widelands::get_all_tribeinfos(nullptr);
-	for (uint8_t i = packet.unsigned_8(); i; --i) {
+	for (uint8_t i = packet.unsigned_8(); i != 0u; --i) {
 		Widelands::TribeBasicInfo info = Widelands::get_tribeinfo(packet.string(), all_tribes);
 
 		// Get initializations (we have to do this locally, for translations)
@@ -987,11 +1011,10 @@ void GameClient::handle_setting_tribes(RecvPacket& packet) {
 					incompatible_wc.insert(w->get_string(key));
 				}
 			}
-			info.initializations.push_back(Widelands::TribeBasicInfo::Initialization(
+			info.initializations.emplace_back(
 			   initialization_script, t->get_string("descname"), t->get_string("tooltip"), tags,
 			   incompatible_wc,
-			   !t->has_key("uses_map_starting_position") ||
-			      t->get_bool("uses_map_starting_position")));
+			   !t->has_key("uses_map_starting_position") || t->get_bool("uses_map_starting_position"));
 		}
 		d->settings.tribes.push_back(info);
 	}
@@ -1007,7 +1030,7 @@ void GameClient::handle_setting_allplayers(RecvPacket& packet) {
 	}
 	// Map changes are finished here
 	Notifications::publish(NoteGameSettings(NoteGameSettings::Action::kMap));
-	if (participants_) {
+	if (participants_ != nullptr) {
 		participants_->participants_updated();
 	}
 }
@@ -1016,7 +1039,7 @@ void GameClient::handle_setting_allplayers(RecvPacket& packet) {
  *
  */
 void GameClient::handle_playercommand(RecvPacket& packet) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		throw DisconnectException("PLAYERCMD_WO_GAME");
 	}
 
@@ -1031,7 +1054,7 @@ void GameClient::handle_playercommand(RecvPacket& packet) {
  *
  */
 void GameClient::handle_syncrequest(RecvPacket& packet) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		throw DisconnectException("SYNCREQUEST_WO_GAME");
 	}
 	const Time time(packet.unsigned_32());
@@ -1073,10 +1096,10 @@ void GameClient::handle_system_message(RecvPacket& packet) {
 /**
  *
  */
-void GameClient::handle_desync(RecvPacket&) {
+void GameClient::handle_desync(RecvPacket& /* packet */) {
 	log_err("[Client] received NETCMD_INFO_DESYNC. Trying to salvage some "
 	        "information for debugging.\n");
-	if (d->game) {
+	if (d->game != nullptr) {
 		d->game->save_syncstream(true);
 		// We don't know our playernumber, so report as -1
 		d->game->report_desync(-1);
@@ -1111,7 +1134,7 @@ void GameClient::handle_packet(RecvPacket& packet) {
 	case NETCMD_SETTING_PLAYER: {
 		uint8_t player = packet.unsigned_8();
 		receive_one_player(player, packet);
-		if (participants_) {
+		if (participants_ != nullptr) {
 			participants_->participants_updated();
 		}
 	} break;
@@ -1120,14 +1143,14 @@ void GameClient::handle_packet(RecvPacket& packet) {
 		for (uint32_t i = 0; i < d->settings.users.size(); ++i) {
 			receive_one_user(i, packet);
 		}
-		if (participants_) {
+		if (participants_ != nullptr) {
 			participants_->participants_updated();
 		}
 	} break;
 	case NETCMD_SETTING_USER: {
 		uint32_t user = packet.unsigned_32();
 		receive_one_user(user, packet);
-		if (participants_) {
+		if (participants_ != nullptr) {
 			participants_->participants_updated();
 		}
 	} break;
@@ -1135,7 +1158,7 @@ void GameClient::handle_packet(RecvPacket& packet) {
 		int32_t number = packet.signed_32();
 		d->settings.playernum = number;
 		d->settings.users.at(d->settings.usernum).position = number;
-		if (participants_) {
+		if (participants_ != nullptr) {
 			participants_->participants_updated();
 		}
 	} break;
@@ -1143,13 +1166,13 @@ void GameClient::handle_packet(RecvPacket& packet) {
 		d->settings.win_condition_script = g_fs->FileSystem::fix_cross_file(packet.string());
 		break;
 	case NETCMD_PEACEFUL_MODE:
-		d->settings.peaceful = packet.unsigned_8();
+		d->settings.peaceful = (packet.unsigned_8() != 0u);
 		break;
 	case NETCMD_CUSTOM_STARTING_POSITIONS:
-		d->settings.custom_starting_positions = packet.unsigned_8();
+		d->settings.custom_starting_positions = (packet.unsigned_8() != 0u);
 		break;
 	case NETCMD_LAUNCH:
-		if (d->game || (d->modal && d->modal->is_modal())) {
+		if ((d->game != nullptr) || ((d->modal != nullptr) && d->modal->is_modal())) {
 			throw DisconnectException("UNEXPECTED_LAUNCH");
 		}
 		do_run(packet);
@@ -1244,29 +1267,26 @@ void GameClient::disconnect(const std::string& reason,
 	}
 
 	if (showmsg) {
-		if (d->game) {
+		if (d->game != nullptr) {
 			if (reason == "KICKED" || reason == "SERVER_LEFT" || reason == "SERVER_CRASHED") {
 				throw WLWarning("", "%s",
 				                arg.empty() ? NetworkGamingMessages::get_message(reason).c_str() :
-				                              NetworkGamingMessages::get_message(reason, arg).c_str());
+                                          NetworkGamingMessages::get_message(reason, arg).c_str());
 			}
 			throw wexception("%s", arg.empty() ?
-			                          NetworkGamingMessages::get_message(reason).c_str() :
-			                          NetworkGamingMessages::get_message(reason, arg).c_str());
-		} else {
-			capsule_.menu().show_messagebox(
-			   _("Disconnected"),
-			   (boost::format(
-			       _("The connection with the host was lost for the following reason:\n%s")) %
-			    (arg.empty() ? NetworkGamingMessages::get_message(reason) :
-			                   NetworkGamingMessages::get_message(reason, arg)))
-			      .str());
+                                   NetworkGamingMessages::get_message(reason).c_str() :
+                                   NetworkGamingMessages::get_message(reason, arg).c_str());
 		}
+		capsule_.menu().show_messagebox(
+		   _("Disconnected"),
+		   format(_("The connection with the host was lost for the following reason:\n%s"),
+		          (arg.empty() ? NetworkGamingMessages::get_message(reason) :
+                               NetworkGamingMessages::get_message(reason, arg))));
 	}
 
 	// TODO(Klaus Halfmann): Some of the modal windows are now handled by unique_ptr resulting in a
 	// double free.
-	if (d->modal) {
+	if (d->modal != nullptr) {
 		d->modal->die();
 	}
 	d->modal = nullptr;
