@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2021 by the Widelands Development Team
+ * Copyright (C) 2008-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -48,6 +47,7 @@
 #include "logic/playercommand.h"
 #include "logic/playersmanager.h"
 #include "network/constants.h"
+#include "network/host_game_settings_provider.h"
 #include "network/internet_gaming.h"
 #include "network/nethost.h"
 #include "network/nethostproxy.h"
@@ -64,181 +64,6 @@
 #include "wui/interactive_player.h"
 #include "wui/interactive_spectator.h"
 
-struct HostGameSettingsProvider : public GameSettingsProvider {
-	explicit HostGameSettingsProvider(GameHost* const init_host) : host_(init_host) {
-	}
-	~HostGameSettingsProvider() override = default;
-
-	void set_scenario(bool is_scenario) override {
-		host_->set_scenario(is_scenario);
-	}
-
-	const GameSettings& settings() override {
-		return host_->settings();
-	}
-
-	bool can_change_map() override {
-		return true;
-	}
-	bool can_change_player_state(uint8_t const number) override {
-		if (number >= settings().players.size()) {
-			return false;
-		}
-		if (settings().savegame) {
-			return settings().players.at(number).state != PlayerSettings::State::kClosed;
-		} else if (settings().scenario) {
-			return settings().players.at(number).state != PlayerSettings::State::kComputer;
-		}
-		return true;
-	}
-	bool can_change_player_tribe(uint8_t const number) override {
-		return can_change_player_team(number);
-	}
-	bool can_change_player_init(uint8_t const number) override {
-		if (settings().scenario || settings().savegame) {
-			return false;
-		}
-		return number < settings().players.size();
-	}
-	bool can_change_player_team(uint8_t number) override {
-		if (settings().scenario || settings().savegame) {
-			return false;
-		}
-		if (number >= settings().players.size()) {
-			return false;
-		}
-		if (number == settings().playernum) {
-			return true;
-		}
-		return settings().players.at(number).state == PlayerSettings::State::kComputer;
-	}
-
-	bool can_launch() override {
-		return host_->can_launch();
-	}
-
-	void set_map(const std::string& mapname,
-	             const std::string& mapfilename,
-	             const std::string& theme,
-	             const std::string& bg,
-	             uint32_t const maxplayers,
-	             bool const savegame = false) override {
-		host_->set_map(mapname, mapfilename, theme, bg, maxplayers, savegame);
-	}
-	void set_player_state(uint8_t number, PlayerSettings::State const state) override {
-		if (number >= settings().players.size()) {
-			return;
-		}
-		host_->set_player_state(number, state);
-	}
-
-	void
-	set_player_tribe(uint8_t number, const std::string& tribe, bool const random_tribe) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		if (number == settings().playernum ||
-		    settings().players.at(number).state == PlayerSettings::State::kComputer ||
-		    settings().players.at(number).state == PlayerSettings::State::kShared ||
-		    settings().players.at(number).state ==
-		       PlayerSettings::State::kOpen) {  // For savegame loading
-			host_->set_player_tribe(number, tribe, random_tribe);
-		}
-	}
-
-	void set_player_team(uint8_t number, Widelands::TeamNumber team) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		if (number == settings().playernum ||
-		    settings().players.at(number).state == PlayerSettings::State::kComputer) {
-			host_->set_player_team(number, team);
-		}
-	}
-
-	void set_player_color(const uint8_t number, const RGBColor& col) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		if (number == settings().playernum ||
-		    settings().players.at(number).state == PlayerSettings::State::kComputer) {
-			host_->set_player_color(number, col);
-		}
-	}
-
-	void set_player_closeable(uint8_t number, bool closeable) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		host_->set_player_closeable(number, closeable);
-	}
-
-	void set_player_shared(PlayerSlot number, Widelands::PlayerNumber shared) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		host_->set_player_shared(number, shared);
-	}
-
-	void set_player_init(uint8_t const number, uint8_t const index) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		host_->set_player_init(number, index);
-	}
-
-	void
-	set_player_ai(uint8_t number, const std::string& name, bool const random_ai = false) override {
-		host_->set_player_ai(number, name, random_ai);
-	}
-
-	void set_player_name(uint8_t const number, const std::string& name) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		host_->set_player_name(number, name);
-	}
-
-	void set_player(uint8_t const number, const PlayerSettings& ps) override {
-		if (number >= host_->settings().players.size()) {
-			return;
-		}
-		host_->set_player(number, ps);
-	}
-
-	void set_player_number(uint8_t const number) override {
-		if (number == UserSettings::none() || number < host_->settings().players.size()) {
-			host_->set_player_number(number);
-		}
-	}
-
-	std::string get_win_condition_script() override {
-		return host_->settings().win_condition_script;
-	}
-
-	void set_win_condition_script(const std::string& wc) override {
-		host_->set_win_condition_script(wc);
-	}
-
-	void set_peaceful_mode(bool peace) override {
-		host_->set_peaceful_mode(peace);
-	}
-	bool is_peaceful_mode() override {
-		return host_->settings().peaceful;
-	}
-
-	void set_custom_starting_positions(bool c) override {
-		host_->set_custom_starting_positions(c);
-	}
-	bool get_custom_starting_positions() override {
-		return host_->settings().custom_starting_positions;
-	}
-
-private:
-	GameHost* host_;
-	std::vector<std::string> wincondition_scripts_;
-};
-
 struct HostChatProvider : public ChatProvider {
 	explicit HostChatProvider(GameHost* const init_host) : h(init_host), kickClient(0) {
 	}
@@ -269,7 +94,9 @@ struct HostChatProvider : public ChatProvider {
 		else if (c.msg.size() > 1 && *c.msg.begin() == '/') {
 
 			// Split up in "cmd" "arg1" "arg2"
-			std::string cmd, arg1, arg2;
+			std::string cmd;
+			std::string arg1;
+			std::string arg2;
 			std::string temp = c.msg.substr(1);  // cut off '/'
 			h->split_command_array(temp, cmd, arg1, arg2);
 			verb_log_info("%s + \"%s\" + \"%s\"\n", cmd.c_str(), arg1.c_str(), arg2.c_str());
@@ -287,21 +114,20 @@ struct HostChatProvider : public ChatProvider {
 
 			// Help
 			if (cmd == "help") {
-				c.msg = (boost::format("<br>%s<br>%s<br>%s<br>%s<br>%s<br>%s<br>%s") %
-				         _("Available host commands are:") %
-				         /** TRANSLATORS: Available host command */
-				         _("/help  -  Shows this help") %
-				         /** TRANSLATORS: Available host command */
-				         _("/announce <msg>  -  Send a chatmessage as announcement (system chat)") %
-				         /** TRANSLATORS: Available host command */
-				         _("/warn <name> <reason>  -  Warn the user <name> because of <reason>") %
-				         /** TRANSLATORS: Available host command */
-				         _("/kick <name> <reason>  -  Kick the user <name> because of <reason>") %
-				         /** TRANSLATORS: Available host command */
-				         _("/forcePause            -  Force the game to pause.") %
-				         /** TRANSLATORS: Available host command */
-				         _("/endForcedPause        -  Return game to normal speed."))
-				           .str();
+				c.msg = format(
+				   "<br>%s<br>%s<br>%s<br>%s<br>%s<br>%s<br>%s", _("Available host commands are:"),
+				   /** TRANSLATORS: Available host command */
+				   _("/help  -  Shows this help"),
+				   /** TRANSLATORS: Available host command */
+				   _("/announce <msg>  -  Send a chatmessage as announcement (system chat)"),
+				   /** TRANSLATORS: Available host command */
+				   _("/warn <name> <reason>  -  Warn the user <name> because of <reason>"),
+				   /** TRANSLATORS: Available host command */
+				   _("/kick <name> <reason>  -  Kick the user <name> because of <reason>"),
+				   /** TRANSLATORS: Available host command */
+				   _("/forcePause            -  Force the game to pause."),
+				   /** TRANSLATORS: Available host command */
+				   _("/endForcedPause        -  Return game to normal speed."));
 			}
 
 			// Announce
@@ -323,9 +149,9 @@ struct HostChatProvider : public ChatProvider {
 				} else {
 					int32_t num = h->check_client(arg1);
 					if (num == -1) {
-						c.msg = (boost::format(_("The client %s could not be found.")) % arg1).str();
+						c.msg = format(_("The client %s could not be found."), arg1);
 					} else {
-						c.msg = (boost::format("HOST WARNING FOR %s: ") % arg1).str();
+						c.msg = format("HOST WARNING FOR %s: ", arg1);
 						c.msg += arg2;
 					}
 				}
@@ -347,14 +173,12 @@ struct HostChatProvider : public ChatProvider {
 					if (num == -2) {
 						c.msg = _("You can not kick yourself!");
 					} else if (num == -1) {
-						c.msg = (boost::format(_("The client %s could not be found.")) % arg1).str();
+						c.msg = format(_("The client %s could not be found."), arg1);
 					} else {
 						kickClient = num;
-						c.msg =
-						   (boost::format(_("Are you sure you want to kick %s?")) % arg1).str() + "<br>";
-						c.msg +=
-						   (boost::format(_("The stated reason was: %s")) % kickReason).str() + "<br>";
-						c.msg += (boost::format(_("If yes, type: /ack_kick %s")) % arg1).str();
+						c.msg = format(_("Are you sure you want to kick %s?"), arg1) + "<br>";
+						c.msg += format(_("The stated reason was: %s"), kickReason) + "<br>";
+						c.msg += format(_("If yes, type: /ack_kick %s"), arg1);
 					}
 				}
 			}
@@ -369,9 +193,8 @@ struct HostChatProvider : public ChatProvider {
 					if (arg1 == kickUser) {
 						h->kick_user(kickClient, kickReason);
 						return;
-					} else {
-						c.msg = _("Kick acknowledgement cancelled: Wrong name given!");
 					}
+					c.msg = _("Kick acknowledgement cancelled: Wrong name given!");
 				}
 				kickUser = "";
 				kickReason = "";
@@ -503,8 +326,7 @@ struct GameHostImpl {
 	     chat(h),
 	     hp(h),
 	     npsb(&hp),
-	     promoter(),
-	     net(),
+
 	     game(nullptr),
 	     pseudo_networktime(0),
 	     last_heartbeat(0),
@@ -526,12 +348,16 @@ struct GameHostImpl {
 	}
 };
 
-GameHost::GameHost(FsMenu::MenuCapsule& c,
-                   std::unique_ptr<GameController>& ptr,
+GameHost::GameHost(FsMenu::MenuCapsule* c,
+                   std::shared_ptr<GameController>& ptr,
                    const std::string& playername,
                    std::vector<Widelands::TribeBasicInfo> tribeinfos,
                    bool internet)
-   : capsule_(c), d(new GameHostImpl(this)), internet_(internet), forced_pause_(false) {
+   : capsule_(c),
+     pointer_(ptr),
+     d(new GameHostImpl(this)),
+     internet_(internet),
+     forced_pause_(false) {
 	verb_log_info("[Host]: starting up.");
 
 	d->localplayername = playername;
@@ -582,7 +408,9 @@ GameHost::GameHost(FsMenu::MenuCapsule& c,
 
 	d->set_participant_list(new ParticipantList(&(d->settings), d->game, d->localplayername));
 
-	run(ptr);
+	if (capsule_ != nullptr) {
+		run();
+	}
 }
 
 GameHost::~GameHost() {
@@ -657,12 +485,19 @@ void GameHost::init_computer_players() {
 	}
 }
 
-void GameHost::run(std::unique_ptr<GameController>& ptr) {
+void GameHost::run() {
 	game_.reset(new Widelands::Game());
 	// Fill the list of possible system messages
 	NetworkGamingMessages::fill_map();
 	new FsMenu::LaunchMPG(
-	   capsule_, d->hp, *this, d->chat, *game_, ptr, internet_, [this]() { run_callback(); });
+	   *capsule_, d->hp, *this, d->chat, *game_, internet_, [this]() { run_callback(); });
+}
+
+void GameHost::run_direct() {
+	game_.reset(new Widelands::Game());
+	// Fill the list of possible system messages
+	NetworkGamingMessages::fill_map();
+	run_callback();
 }
 
 // TODO(k.halfmann): refactor into smaller functions
@@ -685,8 +520,8 @@ void GameHost::run_callback() {
 	SendPacket packet;
 	packet.unsigned_8(NETCMD_LAUNCH);
 	packet.unsigned_32(game_->enabled_addons().size());
-	for (const AddOns::AddOnInfo& a : game_->enabled_addons()) {
-		packet.string(a.internal_name);
+	for (const auto& a : game_->enabled_addons()) {
+		packet.string(a->internal_name);
 	}
 	broadcast(packet);
 
@@ -694,7 +529,9 @@ void GameHost::run_callback() {
 	game_->set_auto_speed(get_config_bool("auto_speed", false));
 	game_->set_write_syncstream(get_config_bool("write_syncstreams", true));
 
-	capsule_.set_visible(false);
+	if (capsule_ != nullptr) {
+		capsule_->set_visible(false);
+	}
 	uint8_t player_number = 1;
 	try {
 		std::vector<std::string> tipstexts{"general_game", "multiplayer"};
@@ -705,11 +542,10 @@ void GameHost::run_callback() {
 		Notifications::publish(UI::NoteLoadingMessage(_("Preparing game…")));
 
 		d->game = game_.get();
-		game_->set_game_controller(this);
+		game_->set_game_controller(pointer_);
 		InteractiveGameBase* igb;
 		player_number = d->settings.playernum + 1;
-		game_->save_handler().set_autosave_filename(
-		   (boost::format("%s_nethost") % kAutosavePrefix).str());
+		game_->save_handler().set_autosave_filename(format("%s_nethost", kAutosavePrefix));
 
 		if (d->settings.savegame) {
 			// Read and broadcast original win condition
@@ -749,11 +585,10 @@ void GameHost::run_callback() {
 		// wait mode when there are no clients
 		check_hung_clients();
 		init_computer_players();
-		game_->run(d->settings.savegame ?
-		              Widelands::Game::StartGameType::kSaveGame :
-		              d->settings.scenario ? Widelands::Game::StartGameType::kMultiPlayerScenario :
-		                                     Widelands::Game::StartGameType::kMap,
-		           "", false, "nethost");
+		game_->run(d->settings.savegame ? Widelands::Game::StartGameType::kSaveGame :
+		           d->settings.scenario ? Widelands::Game::StartGameType::kMultiPlayerScenario :
+                                        Widelands::Game::StartGameType::kMap,
+		           script_to_run_, false, "nethost");
 
 		// if this is an internet game, tell the metaserver that the game is done.
 		if (internet_) {
@@ -761,8 +596,9 @@ void GameHost::run_callback() {
 		}
 		clear_computer_players();
 	} catch (const std::exception& e) {
-		FsMenu::MainMenu& parent = capsule_.menu();  // make includes script happy
-		WLApplication::emergency_save(&parent, *game_, e.what(), player_number);
+		FsMenu::MainMenu* parent =
+		   capsule_ != nullptr ? &capsule_->menu() : nullptr;  // make includes script happy
+		WLApplication::emergency_save(parent, *game_, e.what(), player_number);
 		clear_computer_players();
 
 		while (!d->clients.empty()) {
@@ -779,9 +615,15 @@ void GameHost::run_callback() {
 }
 
 void GameHost::think() {
-	handle_network();
+	NoteThreadSafeFunction::instantiate([this]() { handle_network(); }, true);
 
-	if (d->game) {
+	while (!pending_player_commands_.empty()) {
+		MutexLock m(MutexLock::ID::kCommands);
+		do_send_player_command(pending_player_commands_.front());
+		pending_player_commands_.pop_front();
+	}
+
+	if (d->game != nullptr) {
 		uint32_t curtime = SDL_GetTicks();
 		int32_t delta = curtime - d->lastframe;
 		d->lastframe = curtime;
@@ -817,6 +659,10 @@ void GameHost::think() {
 }
 
 void GameHost::send_player_command(Widelands::PlayerCommand* pc) {
+	pending_player_commands_.push_back(pc);
+}
+
+void GameHost::do_send_player_command(Widelands::PlayerCommand* pc) {
 	pc->set_duetime(d->committed_networktime + Duration(1));
 
 	SendPacket packet;
@@ -1018,9 +864,8 @@ int32_t GameHost::check_client(const std::string& name) {
 			throw wexception("WARNING: user was found but no client is connected to it!\n");
 		}
 		return client;  // client found
-	} else {
-		return -1;  // no client found
 	}
+	return -1;  // no client found
 }
 
 /**
@@ -1078,7 +923,7 @@ void GameHost::send_system_message_code(const std::string& code,
 	ChatMessage msg(NetworkGamingMessages::get_message(code, a, b, c));
 	msg.playern = UserSettings::none();  //  == System message
 	// c.sender remains empty to indicate a system message
-	d->chat.receive(msg);
+	NoteThreadSafeFunction::instantiate([this, &msg]() { d->chat.receive(msg); }, true);
 }
 
 Duration GameHost::get_frametime() {
@@ -1100,7 +945,7 @@ bool GameHost::can_launch() {
 	if (d->settings.players.empty()) {
 		return false;
 	}
-	if (d->game) {
+	if (d->game != nullptr) {
 		return false;
 	}
 
@@ -1205,6 +1050,14 @@ void GameHost::set_map(const std::string& mapname,
 		d->promoter->set_map(mapname.c_str());
 	}
 
+	if (d->settings.savegame) {
+		// Free all slots slots for now, so that the scenario data can be loaded properly
+		set_player_number(UserSettings::none());
+		for (const Client& client : d->clients) {
+			switch_to_player(client.usernum, UserSettings::none());
+		}
+	}
+
 	// Broadcast new player settings
 	packet.reset();
 	packet.unsigned_8(NETCMD_SETTING_ALLPLAYERS);
@@ -1213,6 +1066,30 @@ void GameHost::set_map(const std::string& mapname,
 	// Map changes are finished here
 	Notifications::publish(NoteGameSettings(NoteGameSettings::Action::kMap));
 
+	// Assign players from savegame
+	if (d->settings.savegame) {
+		for (uint8_t i = 0; i < d->settings.players.size(); ++i) {
+			const PlayerSettings& p = d->settings.players.at(i);
+			if (p.tribe.empty()) {
+				// Closed slot
+				continue;
+			}
+
+			std::stringstream playerstream(p.name);
+			std::string user_part;
+			while (getline(playerstream, user_part, ' ')) {
+				for (uint8_t usernum = 0; usernum < d->settings.users.size(); ++usernum) {
+					const std::string& username = d->settings.users.at(usernum).name;
+
+					if (user_part == username) {
+						switch_to_player(usernum, i);
+						break;
+					}
+				}
+			}
+		}
+	}
+
 	// If possible, offer the map / saved game as transfer
 	// TODO(unknown): not yet able to handle directory type maps / savegames, would involve zipping
 	// in place or such ...
@@ -1353,9 +1230,9 @@ void GameHost::set_player_tribe(uint8_t const number,
 	std::string actual_tribe = tribe;
 	player.random_tribe = random_tribe;
 
-	while (random_tribe) {
+	while (!d->settings.savegame && random_tribe) {
 		uint8_t num_tribes = d->settings.tribes.size();
-		uint8_t random = (std::rand() % num_tribes);  // NOLINT
+		uint8_t random = RNG::static_rand(num_tribes);
 		actual_tribe = d->settings.tribes.at(random).name;
 		if (player.state != PlayerSettings::State::kComputer ||
 		    d->settings.get_tribeinfo(actual_tribe).suited_for_ai) {
@@ -1398,11 +1275,11 @@ void GameHost::set_player_init(uint8_t const number, uint8_t const index) {
 				//  broadcast changes
 				broadcast_setting_player(number);
 				return;
-			} else {
-				log_warn("Attempted to change to out-of-range initialization index %u "
-				         "for player %u.\n",
-				         index, number);
 			}
+			log_warn("Attempted to change to out-of-range initialization index %u "
+			         "for player %u.\n",
+			         index, number);
+
 			return;
 		}
 	}
@@ -1422,6 +1299,24 @@ void GameHost::set_player_ai(uint8_t number, const std::string& name, bool const
 	broadcast_setting_player(number);
 }
 
+/**
+ * Removes one player from a combined slot, where the name consists
+ * of all players in this slot separated by spaces, i.e. "Player1 Player2"
+ * \return true if the name has no players left
+ */
+bool GameHost::remove_player_name(uint8_t const number, const std::string& name) {
+	PlayerSettings& p = d->settings.players.at(number);
+	std::string temp(p.name);
+	temp.erase(p.name.find(name), name.size());
+	if (temp.back() == ' ') {
+		temp.erase(temp.end() - 1);
+	} else if (temp.front() == ' ') {
+		temp.erase(temp.begin());
+	}
+	set_player_name(number, temp);
+	return temp.empty();
+}
+
 void GameHost::set_player_name(uint8_t const number, const std::string& name) {
 	if (number >= d->settings.players.size()) {
 		return;
@@ -1535,14 +1430,7 @@ void GameHost::switch_to_player(uint32_t user, uint8_t number) {
 	std::string name = d->settings.users.at(user).name;
 	// Remove clients name from old player slot
 	if (old < d->settings.players.size()) {
-		PlayerSettings& op = d->settings.players.at(old);
-		std::string temp(" ");
-		temp += name;
-		temp += " ";
-		std::string temp2(op.name);
-		temp2 = temp2.erase(op.name.find(temp), temp.size());
-		set_player_name(old, temp2);
-		if (temp2.empty()) {
+		if (remove_player_name(old, name)) {
 			set_player_state(old, PlayerSettings::State::kOpen);
 		}
 	}
@@ -1552,9 +1440,9 @@ void GameHost::switch_to_player(uint32_t user, uint8_t number) {
 		PlayerSettings& op = d->settings.players.at(number);
 		if (op.state == PlayerSettings::State::kOpen) {
 			set_player_state(number, PlayerSettings::State::kHuman);
-			set_player_name(number, " " + name + " ");
+			set_player_name(number, name);
 		} else {
-			set_player_name(number, op.name + " " + name + " ");
+			set_player_name(number, op.name + " " + name);
 		}
 	}
 	d->settings.users.at(user).position = number;
@@ -1670,7 +1558,7 @@ void GameHost::broadcast_setting_player(uint8_t const number) {
 	packet.unsigned_8(number);
 	write_setting_player(packet, number);
 	broadcast(packet);
-	if (d && d->chat.participants_) {
+	if ((d != nullptr) && (d->chat.participants_ != nullptr)) {
 		d->chat.participants_->participants_updated();
 	}
 }
@@ -1696,7 +1584,7 @@ void GameHost::broadcast_setting_user(uint32_t const number) {
 	packet.unsigned_32(number);
 	write_setting_user(packet, number);
 	broadcast(packet);
-	if (d && d->chat.participants_) {
+	if ((d != nullptr) && (d->chat.participants_ != nullptr)) {
 		d->chat.participants_->participants_updated();
 	}
 }
@@ -1740,7 +1628,7 @@ std::string GameHost::get_computer_player_name(uint8_t const playernum) {
 	std::string name;
 	uint32_t suffix = playernum;
 	do {
-		name = (boost::format(_("Computer %u")) % static_cast<unsigned int>(++suffix)).str();
+		name = format(_("Computer %u"), static_cast<unsigned int>(++suffix));
 	} while (has_user_name(name, playernum));
 	return name;
 }
@@ -1764,6 +1652,10 @@ bool GameHost::has_user_name(const std::string& name, uint8_t ignoreplayer) {
 		ignoreplayer = d->settings.users.at(ignoreplayer).position;
 	}
 	for (uint32_t i = 0; i < d->settings.players.size(); ++i) {
+		if (d->settings.savegame && d->settings.players.at(i).state == PlayerSettings::State::kOpen) {
+			// This might be this users slot from the configured savegame
+			continue;
+		}
 		if (i != ignoreplayer && d->settings.players.at(i).name == name) {
 			return true;
 		}
@@ -1784,7 +1676,8 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	// The client gets its own initial data set.
 	client.playernum = UserSettings::none();
 
-	if (!d->game) {  // just in case we allow connection of spectators/players after game start
+	if (d->game ==
+	    nullptr) {  // just in case we allow connection of spectators/players after game start
 		for (uint32_t i = 0; i < d->settings.users.size(); ++i) {
 			if (d->settings.users[i].position == UserSettings::not_connected()) {
 				client.usernum = i;
@@ -1811,7 +1704,7 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	if (has_user_name(effective_name, client.usernum)) {
 		uint32_t i = 1;
 		do {
-			effective_name = (boost::format("%s%u") % playername % i++).str();
+			effective_name = format("%s%u", playername, i++);
 		} while (has_user_name(effective_name, client.usernum));
 	}
 
@@ -1827,8 +1720,8 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	{
 		std::vector<const AddOns::AddOnInfo*> enabled_addons;
 		for (const auto& pair : AddOns::g_addons) {
-			if (pair.second && pair.first.category != AddOns::AddOnCategory::kTheme) {
-				enabled_addons.push_back(&pair.first);
+			if (pair.second && pair.first->category != AddOns::AddOnCategory::kTheme) {
+				enabled_addons.push_back(pair.first.get());
 			}
 		}
 		packet.unsigned_32(enabled_addons.size());
@@ -1900,8 +1793,11 @@ void GameHost::welcome_client(uint32_t const number, std::string& playername) {
 	broadcast_setting_user(client.usernum);
 
 	// Check if there is an unoccupied player left and if, assign.
+	// Assign the slot with the same username for savegames
 	for (uint8_t i = 0; i < d->settings.players.size(); ++i) {
-		if (d->settings.players.at(i).state == PlayerSettings::State::kOpen) {
+		const PlayerSettings& p = d->settings.players.at(i);
+		if (p.state == PlayerSettings::State::kOpen &&
+		    (!d->settings.savegame || p.name == effective_name)) {
 			switch_to_player(client.usernum, i);
 			break;
 		}
@@ -1975,13 +1871,13 @@ void GameHost::check_hung_clients() {
 				if (d->clients.at(i).hung_since == 0) {
 					d->clients.at(i).hung_since = time(nullptr);
 					d->clients.at(i).lastdelta = 0;
-				} else if (time_t deltanow = time(nullptr) - d->clients.at(i).hung_since > 60) {
+				} else if (time_t deltanow =
+				              static_cast<time_t>(time(nullptr) - d->clients.at(i).hung_since > 60)) {
 
 					// inform the other clients about the problem regulary
 					if (deltanow - d->clients.at(i).lastdelta > 30) {
 						std::string seconds =
-						   (boost::format(ngettext("%li second", "%li seconds", deltanow)) % deltanow)
-						      .str();
+						   format(ngettext("%li second", "%li seconds", deltanow), deltanow);
 						send_system_message_code(
 						   "CLIENT_HUNG", d->settings.users.at(d->clients.at(i).usernum).name, seconds);
 						d->clients.at(i).lastdelta = deltanow;
@@ -1992,7 +1888,7 @@ void GameHost::check_hung_clients() {
 	}
 
 	if (!d->waiting) {
-		if (nrhung) {
+		if (nrhung != 0) {
 			verb_log_info("[Host]: %i clients hung. Entering wait mode", nrhung);
 
 			// Brake and wait
@@ -2074,9 +1970,9 @@ void GameHost::update_network_speed() {
 			}
 		}
 
-		d->networkspeed = (speeds.size() % 2) ?
-		                     speeds.at(speeds.size() / 2) :
-		                     (speeds.at(speeds.size() / 2) + speeds.at((speeds.size() / 2) - 1)) / 2;
+		d->networkspeed = (speeds.size() % 2) != 0u ?
+                           speeds.at(speeds.size() / 2) :
+                           (speeds.at(speeds.size() / 2) + speeds.at((speeds.size() / 2) - 1)) / 2;
 
 		if (d->networkspeed > std::numeric_limits<uint16_t>::max()) {
 			d->networkspeed = std::numeric_limits<uint16_t>::max();
@@ -2093,6 +1989,9 @@ void GameHost::update_network_speed() {
  */
 void GameHost::request_sync_reports() {
 	assert(!d->syncreport_pending);
+	if (!d->game->is_loaded()) {
+		return;
+	}
 
 	d->syncreport_pending = true;
 	d->syncreport_arrived = false;
@@ -2275,7 +2174,7 @@ void GameHost::handle_hello(uint32_t const client_num,
                             RecvPacket& r) {
 	// Now we wait for the client to say Hi in the right language,
 	// unless the game has already started
-	if (d->game) {
+	if (d->game != nullptr) {
 		throw DisconnectException("GAME_ALREADY_STARTED");
 	}
 	if (cmd != NETCMD_HELLO) {
@@ -2295,13 +2194,13 @@ void GameHost::handle_hello(uint32_t const client_num,
 void GameHost::handle_changetribe(const Client& client, RecvPacket& r) {
 	//  Do not be harsh about packets of this type arriving out of order -
 	//  the client might just have had bad luck with the timing.
-	if (!d->game) {
+	if (d->game == nullptr) {
 		uint8_t num = r.unsigned_8();
 		if (num != client.playernum) {
 			throw DisconnectException("NO_ACCESS_TO_PLAYER");
 		}
 		std::string tribe = r.string();
-		bool random_tribe = r.unsigned_8();
+		bool random_tribe = r.unsigned_8() != 0u;
 		set_player_tribe(num, tribe, random_tribe);
 	}
 }
@@ -2310,7 +2209,7 @@ void GameHost::handle_changetribe(const Client& client, RecvPacket& r) {
 void GameHost::handle_changeshared(const Client& client, RecvPacket& r) {
 	//  Do not be harsh about packets of this type arriving out of order -
 	//  the client might just have had bad luck with the timing.
-	if (!d->game) {
+	if (d->game == nullptr) {
 		uint8_t num = r.unsigned_8();
 		if (num != client.playernum) {
 			throw DisconnectException("NO_ACCESS_TO_PLAYER");
@@ -2320,7 +2219,7 @@ void GameHost::handle_changeshared(const Client& client, RecvPacket& r) {
 }
 
 void GameHost::handle_changeteam(const Client& client, RecvPacket& r) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		uint8_t num = r.unsigned_8();
 		if (num != client.playernum) {
 			throw DisconnectException("NO_ACCESS_TO_PLAYER");
@@ -2330,7 +2229,7 @@ void GameHost::handle_changeteam(const Client& client, RecvPacket& r) {
 }
 
 void GameHost::handle_changecolor(const Client& client, RecvPacket& packet) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		const uint8_t num = packet.unsigned_8();
 		if (num != client.playernum) {
 			throw DisconnectException("NO_ACCESS_TO_PLAYER");
@@ -2343,7 +2242,7 @@ void GameHost::handle_changecolor(const Client& client, RecvPacket& packet) {
 }
 
 void GameHost::handle_changeinit(const Client& client, RecvPacket& r) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		// TODO(GunChleoc): For some nebulous reason, we don't receive the num that the client is
 		// sending when a player changes slot. So, keeping the access to the client off for now.
 		// Would be nice to have though.
@@ -2356,37 +2255,36 @@ void GameHost::handle_changeinit(const Client& client, RecvPacket& r) {
 }
 
 void GameHost::handle_changeposition(const Client& client, RecvPacket& r) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		uint8_t const pos = r.unsigned_8();
 		switch_to_player(client.usernum, pos);
 	}
 }
 
 void GameHost::handle_nettime(uint32_t const client_num, RecvPacket& r) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		throw DisconnectException("TIME_SENT_NOT_READY");
 	}
 	receive_client_time(client_num, Time(r.unsigned_32()));
 }
 
 void GameHost::handle_playercommmand(uint32_t const client_num, Client& client, RecvPacket& r) {
-	if (!d->game) {
+	if (d->game == nullptr) {
 		throw DisconnectException("PLAYERCMD_WO_GAME");
 	}
 	Time time(r.unsigned_32());
 	Widelands::PlayerCommand* plcmd = Widelands::PlayerCommand::deserialize(r);
-	verb_log_info("[Host]: Client %u (%u) sent player command %u for %u, time = %u", client_num,
+	verb_log_info("[Host]: Client %u (%u) sent player command %u for %u, time = %u\n", client_num,
 	              client.playernum, static_cast<unsigned int>(plcmd->id()), plcmd->sender(),
 	              time.get());
-	receive_client_time(client_num, time);
 	if (plcmd->sender() != client.playernum + 1) {
 		throw DisconnectException("PLAYERCMD_FOR_OTHER");
 	}
-	send_player_command(plcmd);
+	do_send_player_command(plcmd);
 }
 
 void GameHost::handle_syncreport(uint32_t const client_num, Client& client, RecvPacket& r) {
-	if (!d->game || !d->syncreport_pending || client.syncreport_arrived) {
+	if ((d->game == nullptr) || !d->syncreport_pending || client.syncreport_arrived) {
 		throw DisconnectException("UNEXPECTED_SYNC_REP");
 	}
 	Time time(r.unsigned_32());
@@ -2498,7 +2396,8 @@ void GameHost::handle_packet(uint32_t const client_num, RecvPacket& r) {
 	case NETCMD_PEACEFUL_MODE:
 	case NETCMD_CUSTOM_STARTING_POSITIONS:
 	case NETCMD_LAUNCH:
-		if (!d->game) {  // not expected while game is in progress -> something is wrong here
+		if (d->game ==
+		    nullptr) {  // not expected while game is in progress -> something is wrong here
 			log_err("[Host]: Unexpected command %u while in game\n", cmd);
 			throw DisconnectException(
 			   "NO_ACCESS_TO_SERVER");  // TODO(k.halfmann): better use "UNEXPECTED_COMMAND" ?
@@ -2570,15 +2469,8 @@ void GameHost::disconnect_player_controller(uint8_t const number, const std::str
 
 	for (const UserSettings& setting : d->settings.users) {
 		if (setting.position == number) {
-			if (!d->game) {
-				// Remove player name
-				PlayerSettings& p = d->settings.players.at(number);
-				std::string temp(" ");
-				temp += name;
-				temp += " ";
-				std::string temp2(p.name);
-				temp2 = temp2.erase(p.name.find(temp), temp.size());
-				set_player_name(number, temp2);
+			if (d->game == nullptr) {
+				remove_player_name(number, name);
 			}
 			return;
 		}
@@ -2662,7 +2554,7 @@ void GameHost::disconnect_client(uint32_t const client_number,
 		client.sock_id = 0;
 	}
 
-	if (d->game) {
+	if (d->game != nullptr) {
 		check_hung_clients();
 	}
 }
