@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2021 by the Widelands Development Team
+ * Copyright (C) 2004-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -22,19 +21,17 @@
 #include <algorithm>
 #include <memory>
 
-#include <boost/algorithm/string.hpp>
-#include <boost/format.hpp>
-
 #include "base/i18n.h"
 #include "base/log.h"
 #include "base/random.h"
+#include "base/string.h"
 #include "base/warning.h"
 #include "build_info.h"
 #include "io/fileread.h"
 #include "io/filesystem/layered_filesystem.h"
-#include "network/crypto.h"
 #include "network/internet_gaming_messages.h"
 #include "network/internet_gaming_protocol.h"
+#include "third_party/sha1/sha1.h"
 
 /// Max length of formatted time string
 const uint8_t kTimeFormatLength = 32;
@@ -48,7 +45,6 @@ InternetGaming::InternetGaming()
      reg_(false),
      port_(kInternetGamingPort),
      clientrights_(INTERNET_CLIENT_UNREGISTERED),
-     gameips_(),
      clientupdateonmetaserver_(true),
      gameupdateonmetaserver_(true),
      clientupdate_(false),
@@ -96,7 +92,7 @@ static InternetGaming* ig = nullptr;
 
 /// \returns the one and only InternetGaming instance.
 InternetGaming& InternetGaming::ref() {
-	if (!ig) {
+	if (ig == nullptr) {
 		ig = new InternetGaming();
 	}
 	return *ig;
@@ -159,7 +155,7 @@ bool InternetGaming::do_login(bool should_relogin) {
 	verb_log_info("InternetGaming: Sending login request.");
 	SendPacket s;
 	s.string(IGPCMD_LOGIN);
-	s.string(boost::lexical_cast<std::string>(kInternetGamingProtocolVersion));
+	s.string(as_string(kInternetGamingProtocolVersion));
 	s.string(clientname_);
 	s.string(build_id());
 	s.string(bool2str(reg_));
@@ -181,7 +177,8 @@ bool InternetGaming::do_login(bool should_relogin) {
 				}
 
 				return true;
-			} else if (error()) {
+			}
+			if (error()) {
 				return false;
 			}
 		}
@@ -255,7 +252,7 @@ bool InternetGaming::check_password(const std::string& nick,
 	{
 		SendPacket s;
 		s.string(IGPCMD_CHECK_PWD);
-		s.string(boost::lexical_cast<std::string>(kInternetGamingProtocolVersion));
+		s.string(as_string(kInternetGamingProtocolVersion));
 		s.string(nick);
 		s.string(build_id());
 		net->send(s);
@@ -274,7 +271,8 @@ bool InternetGaming::check_password(const std::string& nick,
 				net->send(s);
 				reset();
 				return true;
-			} else if (error()) {
+			}
+			if (error()) {
 				reset();
 				return false;
 			}
@@ -336,7 +334,7 @@ void InternetGaming::handle_metaserver_communication(bool relogin_on_error) {
 			}
 		}
 	} catch (const std::exception& e) {
-		logout((boost::format(_("Something went wrong: %s")) % e.what()).str());
+		logout(format(_("Something went wrong: %s"), e.what()));
 		set_error();
 	}
 
@@ -405,7 +403,8 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 			}
 		}
 		return;
-	} else if (cmd == IGPCMD_PING) {
+	}
+	if (cmd == IGPCMD_PING) {
 		// Client received a PING and should immediately PONG as requested
 		SendPacket s;
 		s.string(IGPCMD_PONG);
@@ -424,17 +423,16 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 			s.string(crypto::sha1(nonce + authenticator_));
 			net->send(s);
 			return;
-
-		} else if (cmd == IGPCMD_LOGIN) {
+		}
+		if (cmd == IGPCMD_LOGIN) {
 			// Clients request to login was granted
 			format_and_add_chat("", "", true, _("Welcome to the Widelands Metaserver!"));
 			const std::string assigned_name = packet.string();
 			if (clientname_ != assigned_name) {
 				format_and_add_chat("", "", true,
-				                    (boost::format(_("You have been logged in as ‘%s’ since your "
-				                                     "requested name is already in use or reserved.")) %
-				                     assigned_name)
-				                       .str());
+				                    format(_("You have been logged in as ‘%s’ since your "
+				                             "requested name is already in use or reserved."),
+				                           assigned_name));
 			}
 			clientname_ = assigned_name;
 			clientrights_ = packet.string();
@@ -456,15 +454,15 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 			verb_log_info(
 			   "InternetGaming: Client %s logged in at UTC %s", clientname_.c_str(), time_str);
 			return;
-
-		} else if (cmd == IGPCMD_PWD_OK) {
+		}
+		if (cmd == IGPCMD_PWD_OK) {
 			char time_str[kTimeFormatLength];
 			format_time(time_str, kTimeFormatLength);
 			verb_log_info("InternetGaming: Password check successful at UTC %s", time_str);
 			state_ = LOBBY;
 			return;
-
-		} else if (cmd == IGPCMD_ERROR) {
+		}
+		if (cmd == IGPCMD_ERROR) {
 			std::string errortype = packet.string();
 			if (errortype != IGPCMD_LOGIN && errortype != IGPCMD_PWD_CHALLENGE) {
 				log_err("InternetGaming: Strange ERROR in connecting state: %s\n", errortype.c_str());
@@ -475,43 +473,36 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 			logout(packet.string());
 			set_error();
 			return;
-
-		} else {
-			logout();
-			set_error();
-			log_err(
-			   "InternetGaming: Expected a LOGIN, PWD_CHALLENGE or ERROR packet from server, but "
-			   "received command %s. Maybe the metaserver is using a different protocol version?\n",
-			   cmd.c_str());
-			throw WLWarning(
-			   _("Unexpected packet"),
-			   _("Received an unexpected network packet from the metaserver. The metaserver could be "
-			     "using a different protocol version. If the error persists, try updating your "
-			     "game."));
 		}
+
+		logout();
+		set_error();
+		log_err("InternetGaming: Expected a LOGIN, PWD_CHALLENGE or ERROR packet from server, but "
+		        "received command %s. Maybe the metaserver is using a different protocol version?\n",
+		        cmd.c_str());
+		throw WLWarning(
+		   _("Unexpected packet"),
+		   _("Received an unexpected network packet from the metaserver. The metaserver could be "
+		     "using a different protocol version. If the error persists, try updating your "
+		     "game."));
 	}
 	try {
 		if (cmd == IGPCMD_LOGIN) {
 			// Login specific commands but not in CONNECTING state...
 			log_err("InternetGaming: Received %s cmd although client is not in CONNECTING state.\n",
 			        cmd.c_str());
-			std::string temp =
-			   (boost::format(
-			       _("WARNING: Received a %s command although we are not in CONNECTING state.")) %
-			    cmd)
-			      .str();
+			std::string temp = format(
+			   _("WARNING: Received a %s command although we are not in CONNECTING state."), cmd);
 			format_and_add_chat("", "", true, temp);
 		}
 
 		else if (cmd == IGPCMD_TIME) {
 			// Client received the server time
-			time_offset_ = boost::lexical_cast<int>(packet.string()) - time(nullptr);
+			time_offset_ = stoi(packet.string()) - time(nullptr);
 			verb_log_info("InternetGaming: Server time offset is %d second(s).", time_offset_);
-			std::string temp =
-			   (boost::format(ngettext("Server time offset is %d second.",
-			                           "Server time offset is %d seconds.", time_offset_)) %
-			    time_offset_)
-			      .str();
+			std::string temp = format(ngettext("Server time offset is %d second.",
+			                                   "Server time offset is %d seconds.", time_offset_),
+			                          time_offset_);
 			format_and_add_chat("", "", true, temp);
 		}
 
@@ -540,7 +531,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 
 		else if (cmd == IGPCMD_GAMES) {
 			// Client received the new list of games
-			uint8_t number = boost::lexical_cast<int>(packet.string()) & 0xff;
+			uint8_t number = stoi(packet.string()) & 0xff;
 			std::vector<InternetGame> old = gamelist_;
 			gamelist_.clear();
 			verb_log_info("InternetGaming: Received a game list update with %u items.", number);
@@ -563,8 +554,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 				    (ing->build_id == build_id() || (ing->build_id.compare(0, 6, "build-") != 0 &&
 				                                     build_id().compare(0, 6, "build-") != 0))) {
 					format_and_add_chat(
-					   "", "", true,
-					   (boost::format(_("The game %s is now available")) % ing->name).str());
+					   "", "", true, format(_("The game %s is now available"), ing->name));
 				}
 
 				delete ing;
@@ -574,8 +564,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 			for (InternetGame& old_game : old) {
 				if (!old_game.name.empty()) {
 					format_and_add_chat(
-					   "", "", true,
-					   (boost::format(_("The game %s has been closed")) % old_game.name).str());
+					   "", "", true, format(_("The game %s has been closed"), old_game.name));
 				}
 			}
 
@@ -590,7 +579,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 
 		else if (cmd == IGPCMD_CLIENTS) {
 			// Client received the new list of clients
-			uint8_t number = boost::lexical_cast<int>(packet.string()) & 0xff;
+			uint8_t number = stoi(packet.string()) & 0xff;
 			std::vector<InternetClient> old = clientlist_;
 			// Push admins/registred/IRC users to a temporary list and add them back later
 			clientlist_.clear();
@@ -614,8 +603,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 					}
 				}
 				if (!found) {
-					format_and_add_chat(
-					   "", "", true, (boost::format(_("%s joined the lobby")) % inc.name).str());
+					format_and_add_chat("", "", true, format(_("%s joined the lobby"), inc.name));
 				}
 			}
 
@@ -626,8 +614,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 
 			for (InternetClient& client : old) {
 				if (!client.name.empty()) {
-					format_and_add_chat(
-					   "", "", true, (boost::format(_("%s left the lobby")) % client.name).str());
+					format_and_add_chat("", "", true, format(_("%s left the lobby"), client.name));
 				}
 			}
 			clientupdate_ = true;
@@ -686,18 +673,14 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 				// Something went wrong with the chat message the user sent.
 				message += _("Chat message could not be sent.");
 				if (reason == "NO_SUCH_USER") {
-					message =
-					   (boost::format("%s %s") % message % InternetGamingMessages::get_message(reason))
-					      .str();
+					message = format("%s %s", message, InternetGamingMessages::get_message(reason));
 				}
 			}
 
 			else if (subcmd == IGPCMD_CMD) {
 				// Something went wrong with the command
 				message += _("Command could not be executed.");
-				message =
-				   (boost::format("%s %s") % message % InternetGamingMessages::get_message(reason))
-				      .str();
+				message = format("%s %s", message, InternetGamingMessages::get_message(reason));
 			}
 
 			else if (subcmd == IGPCMD_GAME_OPEN) {
@@ -714,12 +697,11 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 				waitcmd_ = "";
 			}
 			if (!message.empty()) {
-				message = (boost::format(_("ERROR: %s")) % message).str();
+				message = format(_("ERROR: %s"), message);
 			} else {
-				message = (boost::format(_(
-				              "An unexpected error message has been received about command %1%: %2%")) %
-				           subcmd % reason)
-				             .str();
+				message =
+				   format(_("An unexpected error message has been received about command %1%: %2%"),
+				          subcmd, reason);
 			}
 
 			// Finally send the error message as system chat to the client.
@@ -729,8 +711,7 @@ void InternetGaming::handle_packet(RecvPacket& packet, bool relogin_on_error) {
 		else {
 			// Inform the client about the unknown command
 			format_and_add_chat(
-			   "", "", true,
-			   (boost::format(_("Received an unknown command from the metaserver: %s")) % cmd).str());
+			   "", "", true, format(_("Received an unknown command from the metaserver: %s"), cmd));
 		}
 
 	} catch (WLWarning& e) {
@@ -855,7 +836,7 @@ bool InternetGaming::update_for_games() {
 	return temp;
 }
 
-/// \returns the tables in the room, if no error occured, or nullptr in case of error
+/// \returns the tables in the room, if no error occurred, or nullptr in case of error
 const std::vector<InternetGame>* InternetGaming::games() {
 	return error() ? nullptr : &gamelist_;
 }
@@ -868,7 +849,7 @@ bool InternetGaming::update_for_clients() {
 	return temp;
 }
 
-/// \returns the players in the room, if no error occured, or nullptr in case of error
+/// \returns the players in the room, if no error occurred, or nullptr in case of error
 const std::vector<InternetClient>* InternetGaming::clients() {
 	return error() ? nullptr : &clientlist_;
 }
@@ -887,7 +868,8 @@ void InternetGaming::send(const std::string& msg) {
 		return;
 	}
 
-	std::string trimmed = boost::algorithm::trim_copy(msg);
+	std::string trimmed = msg;
+	trim(trimmed);
 	if (trimmed.empty()) {
 		// Message is empty or only space characters. We don't want it either way
 		return;
@@ -905,7 +887,8 @@ void InternetGaming::send(const std::string& msg) {
 			   _("Message could not be sent: Was this supposed to be a private message?"));
 			return;
 		}
-		trimmed = boost::algorithm::trim_copy(msg.substr(space + 1));
+		trimmed = msg.substr(space + 1);
+		trim(trimmed);
 		if (trimmed.empty()) {
 			format_and_add_chat(
 			   "", "", true,
@@ -939,7 +922,8 @@ void InternetGaming::send(const std::string& msg) {
 		}
 
 		// Split up in "cmd" "arg"
-		std::string cmd, arg;
+		std::string cmd;
+		std::string arg;
 		std::string temp = msg.substr(1);  // cut off '/'
 		std::string::size_type const space = temp.find(' ');
 		if (space > temp.size()) {
@@ -949,7 +933,8 @@ void InternetGaming::send(const std::string& msg) {
 
 		// get the cmd and the arg
 		cmd = temp.substr(0, space);
-		arg = boost::algorithm::trim_copy(temp.substr(space + 1));
+		arg = temp.substr(space + 1);
+		trim(arg);
 
 		if (!arg.empty() && cmd == "motd") {
 			SendPacket m;
@@ -974,14 +959,16 @@ void InternetGaming::send(const std::string& msg) {
 			m.string(arg);
 			net->send(m);
 			return;
-		} else if (!arg.empty() && cmd == "announce") {
+		}
+		if (!arg.empty() && cmd == "announce") {
 			// send the request to make an announcement
 			SendPacket m;
 			m.string(IGPCMD_ANNOUNCEMENT);
 			m.string(arg);
 			net->send(m);
 			return;
-		} else if (!arg.empty() && (cmd == "warn" || cmd == "kick" || cmd == "ban")) {
+		}
+		if (!arg.empty() && (cmd == "warn" || cmd == "kick" || cmd == "ban")) {
 			// warn a user by sending a private system message or
 			// kick a user (for 5 minutes) or a game from the metaserver or
 			// ban a user for 24 hours
@@ -991,10 +978,9 @@ void InternetGaming::send(const std::string& msg) {
 			m.string(arg);
 			net->send(m);
 			return;
-		} else {
-			// let everything else pass
-			goto normal;
 		}
+		// let everything else pass
+		goto normal;
 	} else {
 	normal:
 		s.string(msg);
@@ -1037,8 +1023,7 @@ void InternetGaming::format_and_add_chat(const std::string& from,
                                          const std::string& msg) {
 	ChatMessage c(msg);
 	if (!system && from.empty()) {
-		std::string unkown_string =
-		   (boost::format("<%s>") % pgettext("chat_sender", "Unknown")).str();
+		std::string unkown_string = format("<%s>", pgettext("chat_sender", "Unknown"));
 		c.sender = unkown_string;
 	} else {
 		c.sender = from;
@@ -1065,9 +1050,5 @@ bool InternetGaming::valid_username(const std::string& username) {
 		return false;
 	}
 	// Check whether the username is not "team" without regarding upper/lower case
-	// Note: The memory for the lowercase version must be allocated before calling transform()
-	std::string lowercase = username;
-	std::transform(lowercase.begin(), lowercase.end(), lowercase.begin(),
-	               [](unsigned char c) { return std::tolower(c); });
-	return lowercase != "team";
+	return to_lower(username) != "team";
 }
