@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -46,6 +45,8 @@ enum class NoteId : uint32_t {
 	MapObjectDescription,
 	MapObjectDescriptionTypeCheck,
 	LoadingMessage,
+	ThreadSafeFunction,
+	ThreadSafeFunctionHandled,
 	Panel,
 	PlayerDetailsEvent
 };
