@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,7 +30,7 @@ lua_State* luaL_checkthread(lua_State* L, int n) {
 
 bool luaL_checkboolean(lua_State* L, int n) {
 	if (lua_isboolean(L, n)) {
-		return lua_toboolean(L, n);
+		return lua_toboolean(L, n) != 0;
 	}
-	return luaL_checkinteger(L, n);
+	return luaL_checkinteger(L, n) != 0;
 }
