@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -37,7 +36,7 @@ public:
 	void interpret_string(const std::string&);
 
 	// Runs 'script' and returns the table it returned.
-	virtual std::unique_ptr<LuaTable> run_script(const std::string& script);
+	virtual std::unique_ptr<LuaTable> run_script(const std::string& path);
 
 protected:
 	lua_State* lua_state_;
