@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -75,8 +74,10 @@ public:
 		report_error(L, "Cannot instantiate a 'Map' directly!");
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -126,14 +127,17 @@ public:
 		report_error(L, "Cannot instantiate a 'LuaTribeDescription' directly!");
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
 	 */
 	int get_buildings(lua_State*);
 	int get_builder(lua_State* L);
+	int get_carriers(lua_State*);
 	int get_carrier(lua_State*);
 	int get_carrier2(lua_State*);
 	int get_ferry(lua_State*);
@@ -190,8 +194,10 @@ public:
 		report_error(L, "Cannot instantiate a 'MapObjectDescription' directly!");
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -243,8 +249,10 @@ public:
 	explicit LuaImmovableDescription(lua_State* L) : LuaMapObjectDescription(L) {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -284,8 +292,10 @@ public:
 	explicit LuaBuildingDescription(lua_State* L) : LuaMapObjectDescription(L) {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -567,8 +577,10 @@ public:
 	explicit LuaWareDescription(lua_State* L) : LuaMapObjectDescription(L) {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -604,8 +616,10 @@ public:
 	explicit LuaWorkerDescription(lua_State* L) : LuaMapObjectDescription(L) {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -687,8 +701,10 @@ public:
 	explicit LuaShipDescription(lua_State* L) : LuaMapObjectDescription(L) {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -724,8 +740,10 @@ public:
 		report_error(L, "Cannot instantiate a 'LuaResourceDescription' directly!");
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -775,8 +793,10 @@ public:
 		report_error(L, "Cannot instantiate a 'LuaTerrainDescription' directly!");
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -828,8 +848,10 @@ public:
 		report_error(L, "Cannot instantiate a 'LuaEconomy' directly!");
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
@@ -881,20 +903,26 @@ public:
 		ptr_ = nullptr;
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * attributes
 	 */
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	int get___hash(lua_State*);
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 	int get_descr(lua_State*);
 	int get_serial(lua_State*);
 
 	/*
 	 * Lua Methods
 	 */
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	int __eq(lua_State* L);
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 	int remove(lua_State* L);
 	int destroy(lua_State* L);
 	int has_attribute(lua_State* L);
@@ -1016,6 +1044,7 @@ public:
 	 * Lua Methods
 	 */
 	int dismantle(lua_State* L);
+	int enhance(lua_State* L);
 
 	/*
 	 * C Methods
@@ -1497,13 +1526,17 @@ public:
 	~LuaField() override {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
 	 */
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	int get___hash(lua_State*);
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 	int get_x(lua_State* L);
 	int get_y(lua_State* L);
 	int get_viewpoint_x(lua_State* L);
@@ -1537,8 +1570,10 @@ public:
 	/*
 	 * Lua methods
 	 */
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	int __tostring(lua_State* L);
 	int __eq(lua_State* L);
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 	int region(lua_State* L);
 	int has_caps(lua_State*);
 	int has_max_caps(lua_State*);
@@ -1575,8 +1610,10 @@ public:
 	~LuaPlayerSlot() override {
 	}
 
+	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
 	void __persist(lua_State* L) override;
 	void __unpersist(lua_State* L) override;
+	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
 	/*
 	 * Properties
