@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2021 by the Widelands Development Team
+ * Copyright (C) 2006-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -154,11 +153,11 @@ int LuaPanel::get_dropdowns(lua_State* L) {
       (RO) An :class:`array` of all visible tabs inside this Panel.
 */
 static void put_all_tabs_into_table(lua_State* L, UI::Panel* g) {
-	if (!g) {
+	if (g == nullptr) {
 		return;
 	}
 
-	for (UI::Panel* f = g->get_first_child(); f; f = f->get_next_sibling()) {
+	for (UI::Panel* f = g->get_first_child(); f != nullptr; f = f->get_next_sibling()) {
 		put_all_tabs_into_table(L, f);
 
 		if (upcast(UI::TabPanel, t, f)) {
@@ -274,12 +273,12 @@ int LuaPanel::get_descendant_position(lua_State* L) {
 	UI::Panel* cur = (*get_base_user_class<LuaPanel>(L, 2))->panel_;
 
 	Vector2i cp = Vector2i::zero();
-	while (cur && cur != panel_) {
+	while (cur != nullptr && cur != panel_) {
 		cp += cur->to_parent(Vector2i::zero());
 		cur = cur->get_parent();
 	}
 
-	if (!cur) {
+	if (cur == nullptr) {
 		report_error(L, "Widget is not a descendant!");
 	}
 
@@ -337,8 +336,6 @@ Button
 
 .. class:: Button
 
-   Child of: :class:`Panel`
-
    This represents a simple push button.
 */
 const char LuaButton::className[] = "Button";
@@ -401,8 +398,6 @@ Dropdown
 
 .. class:: Dropdown
 
-   Child of: :class:`Panel`
-
    This represents a dropdown menu.
 */
 const char LuaDropdown::className[] = "Dropdown";
@@ -438,7 +433,7 @@ int LuaDropdown::get_name(lua_State* L) {
       (RO) True if the dropdown's list is currently expanded.
 */
 int LuaDropdown::get_expanded(lua_State* L) {
-	lua_pushboolean(L, get()->is_expanded());
+	lua_pushboolean(L, static_cast<int>(get()->is_expanded()));
 	return 1;
 }
 
@@ -584,8 +579,6 @@ Tab
 
 .. class:: Tab
 
-   Child of: :class:`Panel`
-
    A tab button.
 */
 const char LuaTab::className[] = "Tab";
@@ -615,7 +608,7 @@ int LuaTab::get_name(lua_State* L) {
       (RO) Is this the currently active tab in this window?
 */
 int LuaTab::get_active(lua_State* L) {
-	lua_pushboolean(L, get()->active());
+	lua_pushboolean(L, static_cast<int>(get()->active()));
 	return 1;
 }
 
@@ -643,8 +636,6 @@ Window
 
 .. class:: Window
 
-   Child of: :class:`Panel`
-
    This represents a Window.
 */
 const char LuaWindow::className[] = "Window";
@@ -679,7 +670,7 @@ int LuaWindow::get_name(lua_State* L) {
 */
 int LuaWindow::close(lua_State* /* L */) {
 	log_info("Closing window '%s'\n", get()->get_name().c_str());
-	delete panel_;
+	panel_->die();
 	panel_ = nullptr;
 	return 0;
 }
@@ -694,8 +685,6 @@ MapView
 
 .. class:: MapView
 
-   Child of :class:`Panel`
-
    The map view is the main widget and the root of all panels. It is the big
    view of the map that is visible at all times while playing.
 */
@@ -770,7 +759,7 @@ int LuaMapView::get_center_map_pixel(lua_State* L) {
       (RW) True if the buildhelp is show, false otherwise.
 */
 int LuaMapView::get_buildhelp(lua_State* L) {
-	lua_pushboolean(L, get()->buildhelp());
+	lua_pushboolean(L, static_cast<int>(get()->buildhelp()));
 	return 1;
 }
 int LuaMapView::set_buildhelp(lua_State* L) {
@@ -784,7 +773,7 @@ int LuaMapView::set_buildhelp(lua_State* L) {
       (RW) True if the census strings are shown on buildings, false otherwise
 */
 int LuaMapView::get_census(lua_State* L) {
-	lua_pushboolean(L, get()->get_display_flag(InteractiveBase::dfShowCensus));
+	lua_pushboolean(L, static_cast<int>(get()->get_display_flag(InteractiveBase::dfShowCensus)));
 	return 1;
 }
 int LuaMapView::set_census(lua_State* L) {
@@ -799,7 +788,7 @@ int LuaMapView::set_census(lua_State* L) {
       otherwise
 */
 int LuaMapView::get_statistics(lua_State* L) {
-	lua_pushboolean(L, get()->get_display_flag(InteractiveBase::dfShowStatistics));
+	lua_pushboolean(L, static_cast<int>(get()->get_display_flag(InteractiveBase::dfShowStatistics)));
 	return 1;
 }
 int LuaMapView::set_statistics(lua_State* L) {
@@ -813,7 +802,7 @@ int LuaMapView::set_statistics(lua_State* L) {
       (RO) Is the player currently in road/waterway building mode?
 */
 int LuaMapView::get_is_building_road(lua_State* L) {
-	lua_pushboolean(L, get()->in_road_building_mode());
+	lua_pushboolean(L, static_cast<int>(get()->in_road_building_mode()));
 	return 1;
 }
 
@@ -825,9 +814,9 @@ int LuaMapView::get_is_building_road(lua_State* L) {
 int LuaMapView::get_auto_roadbuilding_mode(lua_State* L) {
 	InteractivePlayer* ipl = get_game(L).get_ipl();
 	if (ipl == nullptr) {
-		lua_pushboolean(L, false);
+		lua_pushboolean(L, 0);
 	} else {
-		lua_pushboolean(L, ipl->auto_roadbuild_mode());
+		lua_pushboolean(L, static_cast<int>(ipl->auto_roadbuild_mode()));
 	}
 	return 1;
 }
@@ -838,7 +827,7 @@ int LuaMapView::get_auto_roadbuilding_mode(lua_State* L) {
       (RO) True if this MapView is currently panning or zooming.
 */
 int LuaMapView::get_is_animating(lua_State* L) {
-	lua_pushboolean(L, get()->map_view()->is_animating());
+	lua_pushboolean(L, static_cast<int>(get()->map_view()->is_animating()));
 	return 1;
 }
 
@@ -856,7 +845,7 @@ int LuaMapView::get_is_animating(lua_State* L) {
       :type field: :class:`wl.map.Field`
 */
 int LuaMapView::click(lua_State* L) {
-	const auto field = *get_user_class<LuaMaps::LuaField>(L, 2);
+	auto* const field = *get_user_class<LuaMaps::LuaField>(L, 2);
 	get()->map_view()->mouse_to_field(field->coords(), MapView::Transition::Jump);
 
 	// We fake the triangle here, since we only support clicking on Nodes from
@@ -891,7 +880,7 @@ int LuaMapView::start_road_building(lua_State* L) {
 	me->map_view()->mouse_to_field(starting_field, MapView::Transition::Jump);
 	me->start_build_road(starting_field, me->get_player()->player_number(),
 	                     lua_gettop(L) > 2 && luaL_checkboolean(L, 3) ? RoadBuildingType::kWaterway :
-	                                                                    RoadBuildingType::kRoad);
+                                                                       RoadBuildingType::kRoad);
 
 	return 0;
 }
@@ -973,8 +962,8 @@ int LuaMapView::scroll_to_field(lua_State* L) {
       :type field: :class:`wl.map.Field`
 */
 int LuaMapView::is_visible(lua_State* L) {
-	lua_pushboolean(L, get()->map_view()->view_area().contains(
-	                      (*get_user_class<LuaMaps::LuaField>(L, 2))->coords()));
+	lua_pushboolean(L, static_cast<int>(get()->map_view()->view_area().contains(
+	                      (*get_user_class<LuaMaps::LuaField>(L, 2))->coords())));
 	return 1;
 }
 
@@ -1045,7 +1034,7 @@ static int L_set_user_input_allowed(lua_State* L) {
    :rtype: :class:`boolean`
 */
 static int L_get_user_input_allowed(lua_State* L) {
-	lua_pushboolean(L, UI::Panel::allow_user_input());
+	lua_pushboolean(L, static_cast<int>(UI::Panel::allow_user_input()));
 	return 1;
 }
 
@@ -1060,7 +1049,7 @@ static int L_get_user_input_allowed(lua_State* L) {
 static int L_get_shortcut(lua_State* L) {
 	const std::string name = luaL_checkstring(L, -1);
 	try {
-		lua_pushstring(L, shortcut_string_for(shortcut_from_string(name), false).c_str());
+		lua_pushstring(L, shortcut_string_for(shortcut_from_string(name), true).c_str());
 	} catch (const WException& e) {
 		report_error(L, "Unable to query shortcut for '%s': %s", name.c_str(), e.what());
 	}
