@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2021 by the Widelands Development Team
+ * Copyright (C) 2005-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -145,7 +144,9 @@ SoundHandler::~SoundHandler() {
 	songs_.clear();
 	fxs_.clear();
 
-	int numtimesopened, frequency, channels;
+	int numtimesopened;
+	int frequency;
+	int channels;
 	uint16_t format;
 	numtimesopened = Mix_QuerySpec(&frequency, &format, &channels);
 	log_info("SoundHandler: Closing %i time%s, %i Hz, format %i, %i channel%s\n", numtimesopened,
@@ -170,7 +171,7 @@ SoundHandler::~SoundHandler() {
 		Mix_CloseAudio();
 	}
 
-	if (fx_lock_) {
+	if (fx_lock_ != nullptr) {
 		SDL_DestroyMutex(fx_lock_);
 		fx_lock_ = nullptr;
 	}
@@ -193,35 +194,13 @@ void SoundHandler::initialization_error(const char* const msg, bool quit_sdl) {
  * the constructor.
  */
 void SoundHandler::read_config() {
-	// TODO(GunChleoc): Compatibility code to avoid getting bug reports about unread sections. Remove
-	// after Build 21.
-	if (get_config_section_ptr("sound") == nullptr) {
-		for (auto& option : sound_options_) {
-			switch (option.first) {
-			case SoundType::kMusic:
-				option.second.volume = get_config_int("music_volume", option.second.volume);
-				option.second.enabled = !get_config_bool("disable_music", !option.second.enabled);
-				break;
-			case SoundType::kChat:
-				option.second.volume = get_config_int("fx_volume", option.second.volume);
-				option.second.enabled = get_config_bool("sound_at_message", option.second.enabled);
-				break;
-			default:
-				option.second.volume = get_config_int("fx_volume", option.second.volume);
-				option.second.enabled = !get_config_bool("disable_fx", !option.second.enabled);
-				break;
-			}
-		}
-		save_config();
-	}
-
-	// This is the code that we want to keep
 	for (auto& option : sound_options_) {
 		option.second.volume =
 		   get_config_int("sound", "volume_" + option.second.name, option.second.volume);
 		option.second.enabled =
 		   get_config_bool("sound", "enable_" + option.second.name, option.second.enabled);
 	}
+	use_custom_songset_instead_ingame_ = get_config_bool("sound", "custom_ingame_music", false);
 }
 
 /// Save the current sound options to config cache
@@ -237,6 +216,7 @@ void SoundHandler::save_config() {
 		const std::string volume_name = "volume_" + name;
 		set_config_int("sound", volume_name, volume);
 	}
+	set_config_bool("sound", "custom_ingame_music", use_custom_songset_instead_ingame_);
 }
 
 /// Read the sound options from the cache and apply them
@@ -276,9 +256,8 @@ FxId SoundHandler::do_register_fx(SoundType type, const std::string& fx_path) {
 		fxs_[type].insert(
 		   std::make_pair(new_id, std::unique_ptr<FXset>(new FXset(fx_path, rng_.rand()))));
 		return new_id;
-	} else {
-		return fx_ids_[type].at(fx_path);
 	}
+	return fx_ids_[type].at(fx_path);
 }
 
 /**
@@ -449,7 +428,7 @@ void SoundHandler::start_music(const std::string& songset_name) {
 		return;
 	}
 
-	if (Mix_PlayingMusic()) {
+	if (Mix_PlayingMusic() != 0) {
 		change_music(songset_name, kMinimumMusicFade);
 	}
 
@@ -476,7 +455,7 @@ void SoundHandler::stop_music(int fadeout_ms) {
 		return;
 	}
 
-	if (Mix_PlayingMusic()) {
+	if (Mix_PlayingMusic() != 0) {
 		Mix_FadeOutMusic(std::max(fadeout_ms, kMinimumMusicFade));
 	}
 }
@@ -496,15 +475,31 @@ void SoundHandler::change_music(const std::string& songset_name, int const fadeo
 	}
 
 	if (!songset_name.empty()) {
-		current_songset_ = songset_name;
+		if (songset_name == Songset::kIngame && use_custom_songset_instead_ingame_) {
+			current_songset_ = Songset::kCustom;
+		} else {
+			current_songset_ = songset_name;
+		}
 	}
 
-	if (Mix_PlayingMusic()) {
+	if (Mix_PlayingMusic() != 0) {
 		stop_music(fadeout_ms);
 	} else {
 		start_music(current_songset_);
 	}
 }
+bool SoundHandler::use_custom_songset() const {
+	return use_custom_songset_instead_ingame_;
+}
+
+void SoundHandler::use_custom_songset(bool on) {
+	use_custom_songset_instead_ingame_ = on;
+	if (current_songset_ == Songset::kIngame && on) {
+		change_music(Songset::kCustom);
+	} else if (current_songset_ == Songset::kCustom && !on) {
+		change_music(Songset::kIngame);
+	}
+}
 
 /// Returns the currently playing songset
 const std::string SoundHandler::current_songset() const {
@@ -540,7 +535,7 @@ void SoundHandler::set_enable_sound(SoundType type, bool const enable) {
 	switch (type) {
 	case SoundType::kMusic:
 		if (enable) {
-			if (!Mix_PlayingMusic()) {
+			if (Mix_PlayingMusic() == 0) {
 				start_music(current_songset_);
 			}
 		} else {
@@ -617,14 +612,14 @@ void SoundHandler::fx_finished_callback(int32_t const channel) {
 /// Lock the SDL mutex. Access to 'active_fx_' is protected by mutex because it can be accessed both
 /// from callbacks or from the main thread.
 void SoundHandler::lock_fx() {
-	if (fx_lock_) {
+	if (fx_lock_ != nullptr) {
 		SDL_LockMutex(fx_lock_);
 	}
 }
 
 /// Release the SDL mutex
 void SoundHandler::release_fx_lock() {
-	if (fx_lock_) {
+	if (fx_lock_ != nullptr) {
 		SDL_UnlockMutex(fx_lock_);
 	}
 }
