@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2021 by the Widelands Development Team
+ * Copyright (C) 2003-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -41,6 +40,7 @@
 namespace {
 constexpr int kMarginX = 4;
 constexpr int kLineMargin = 1;
+constexpr int CARET_BLINKING_DELAY = 1000;
 }  // namespace
 
 namespace UI {
@@ -116,7 +116,10 @@ EditBox::EditBox(Panel* const parent, int32_t x, int32_t y, uint32_t w, UI::Pane
      history_active_(false),
      history_position_(-1),
      password_(false),
-     warning_(false) {
+     warning_(false),
+     caret_timer_("", true) {
+	caret_ms = 0;
+	caret_timer_.ms_since_last_query();
 	set_thinks(false);
 
 	// yes, use *signed* max as maximum length; just a small safe-guard.
@@ -187,7 +190,7 @@ void EditBox::set_font_scale(float scale) {
 /**
  * The mouse was clicked on this editbox
  */
-bool EditBox::handle_mousepress(const uint8_t btn, int32_t x, int32_t) {
+bool EditBox::handle_mousepress(const uint8_t btn, int32_t x, int32_t /*y*/) {
 	if (btn == SDL_BUTTON_LEFT && get_can_focus()) {
 		reset_selection();
 		set_caret_to_cursor_pos(x);
@@ -201,7 +204,7 @@ bool EditBox::handle_mousepress(const uint8_t btn, int32_t x, int32_t) {
 
 bool EditBox::handle_mousemove(uint8_t state, int32_t x, int32_t y, int32_t xdiff, int32_t ydiff) {
 	// state != 0 -> mouse button is pressed
-	if (state && get_can_focus()) {
+	if ((state != 0u) && get_can_focus()) {
 		select_until(m_->caret);
 		set_caret_to_cursor_pos(x);
 		select_until(m_->caret);
@@ -267,7 +270,8 @@ int EditBox::calculate_text_width(int pos) const {
 // real unicode.
 bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 	if (down) {
-		if (matches_shortcut(KeyboardShortcut::kCommonTextPaste, code) && SDL_HasClipboardText()) {
+		if (matches_shortcut(KeyboardShortcut::kCommonTextPaste, code) &&
+		    (SDL_HasClipboardText() != 0u)) {
 			if (m_->mode == EditBoxImpl::Mode::kSelection) {
 				delete_selected_text();
 			}
@@ -301,8 +305,10 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 			// Let the panel handle the tab key
 			return get_parent()->handle_key(true, code);
 
-		case SDLK_KP_ENTER:
 		case SDLK_RETURN:
+			if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+				return false;
+			}
 			// Save history if active and text is not empty
 			if (history_active_) {
 				if (!m_->text.empty()) {
@@ -316,11 +322,6 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 			ok();
 			return true;
 
-		case SDLK_KP_PERIOD:
-			if (code.mod & KMOD_NUM) {
-				break;
-			}
-			FALLS_THROUGH;
 		case SDLK_DELETE:
 			if (m_->mode == EditBoxImpl::Mode::kSelection) {
 				delete_selected_text();
@@ -358,19 +359,19 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 		case SDLK_LEFT:
 			if (m_->caret > 0) {
 
-				if (code.mod & (KMOD_LCTRL | KMOD_RCTRL)) {
+				if ((code.mod & (KMOD_LCTRL | KMOD_RCTRL)) != 0) {
 					uint32_t newpos = prev_char(m_->caret);
-					while (newpos > 0 && isspace(m_->text[newpos])) {
+					while (newpos > 0 && (isspace(m_->text[newpos]) != 0)) {
 						newpos = prev_char(newpos);
 					}
 					while (newpos > 0) {
 						uint32_t prev = prev_char(newpos);
-						if (isspace(m_->text[prev])) {
+						if (isspace(m_->text[prev]) != 0) {
 							break;
 						}
 						newpos = prev;
 					}
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(newpos);
 					} else {
 						reset_selection();
@@ -378,7 +379,7 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 					m_->caret = newpos;
 
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(prev_char(m_->caret));
 					} else {
 						reset_selection();
@@ -392,15 +393,15 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 		case SDLK_RIGHT:
 			if (m_->caret < m_->text.size()) {
 
-				if (code.mod & (KMOD_LCTRL | KMOD_RCTRL)) {
+				if ((code.mod & (KMOD_LCTRL | KMOD_RCTRL)) != 0) {
 					uint32_t newpos = next_char(m_->caret);
-					while (newpos < m_->text.size() && isspace(m_->text[newpos])) {
+					while (newpos < m_->text.size() && (isspace(m_->text[newpos]) != 0)) {
 						newpos = next_char(newpos);
 					}
-					while (newpos < m_->text.size() && !isspace(m_->text[newpos])) {
+					while (newpos < m_->text.size() && (isspace(m_->text[newpos]) == 0)) {
 						newpos = next_char(newpos);
 					}
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(newpos);
 					} else {
 						reset_selection();
@@ -408,7 +409,7 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 					m_->caret = newpos;
 
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(next_char(m_->caret));
 					} else {
 						reset_selection();
@@ -421,7 +422,7 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 
 		case SDLK_HOME:
 			if (m_->caret > 0) {
-				if (SDL_GetModState() & KMOD_SHIFT) {
+				if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 					select_until(0);
 				} else {
 					reset_selection();
@@ -433,7 +434,7 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 
 		case SDLK_END:
 			if (m_->caret != m_->text.size()) {
-				if (SDL_GetModState() & KMOD_SHIFT) {
+				if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 					select_until(m_->text.size());
 				} else {
 					reset_selection();
@@ -481,7 +482,8 @@ bool EditBox::handle_key(bool const down, SDL_Keysym const code) {
 	return false;
 }
 void EditBox::copy_selected_text() {
-	uint32_t start, end;
+	uint32_t start;
+	uint32_t end;
 	calculate_selection_boundaries(start, end);
 
 	auto nr_characters = end - start;
@@ -501,7 +503,8 @@ bool EditBox::handle_textinput(const std::string& input_text) {
 }
 
 void EditBox::delete_selected_text() {
-	uint32_t start, end;
+	uint32_t start;
+	uint32_t end;
 	calculate_selection_boundaries(start, end);
 	uint32_t nbytes = end - start;
 	m_->text.erase(start, nbytes);
@@ -510,6 +513,12 @@ void EditBox::delete_selected_text() {
 	changed();
 }
 
+void EditBox::focus(bool topcaller) {
+	Panel::focus(topcaller);
+	caret_ms = CARET_BLINKING_DELAY;
+	caret_timer_.ms_since_last_query();
+}
+
 void EditBox::draw(RenderTarget& dst) {
 	draw_background(dst, m_->background_style());
 
@@ -592,33 +601,40 @@ void EditBox::draw(RenderTarget& dst) {
 	}
 
 	if (has_focus()) {
-		// Draw the caret
-		std::string line_to_caret;
-
-		if (password_) {
-			line_to_caret = text_to_asterisk().substr(0, m_->caret);
-		} else {
-			line_to_caret = m_->text.substr(0, m_->caret);
-		}
-
-		// TODO(GunChleoc): Arabic: Fix cursor position for BIDI text.
-		int caret_x = text_width(line_to_caret, m_->font_style(), m_->font_scale);
-
 		const uint16_t fontheight = text_height(m_->font_style(), m_->font_scale);
-
-		const Image* caret_image =
-		   g_image_cache->get(panel_style_ == PanelStyle::kWui ? "images/ui_basic/caret_wui.png" :
-		                                                         "images/ui_basic/caret_fs.png");
-		Vector2i caretpt = Vector2i::zero();
-		caretpt.x = point.x + m_->scrolloffset + caret_x - caret_image->width() + kLineMargin;
-		caretpt.y = point.y + (fontheight - caret_image->height()) / 2;
-		dst.blit(caretpt, caret_image);
+		draw_caret(dst, point, fontheight);
 
 		if (m_->mode == EditBoxImpl::Mode::kSelection) {
 			highlight_selection(dst, point, fontheight);
 		}
 	}
 }
+void EditBox::draw_caret(RenderTarget& dst, const Vector2i& point, const uint16_t fontheight) {
+	std::string line_to_caret;
+
+	if (password_) {
+		line_to_caret = text_to_asterisk().substr(0, m_->caret);
+	} else {
+		line_to_caret = m_->text.substr(0, m_->caret);
+	}
+
+	// TODO(GunChleoc): Arabic: Fix cursor position for BIDI text.
+	int caret_x = text_width(line_to_caret, m_->font_style(), m_->font_scale);
+
+	const Image* caret_image =
+	   g_image_cache->get(panel_style_ == PanelStyle::kWui ? "images/ui_basic/caret_wui.png" :
+                                                            "images/ui_basic/caret_fs.png");
+	Vector2i caretpt = Vector2i::zero();
+	caretpt.x = point.x + m_->scrolloffset + caret_x - caret_image->width() + kLineMargin;
+	caretpt.y = point.y + (fontheight - caret_image->height()) / 2;
+	if (caret_ms > CARET_BLINKING_DELAY) {
+		dst.blit(caretpt, caret_image);
+	}
+	if (caret_ms > 2 * CARET_BLINKING_DELAY) {
+		caret_ms = 0;
+	}
+	caret_ms += caret_timer_.ms_since_last_query();
+}
 
 void EditBox::set_caret_pos(const size_t pos) {
 	m_->caret = std::min(pos, m_->text.size());
@@ -633,7 +649,8 @@ void EditBox::highlight_selection(RenderTarget& dst,
                                   const Vector2i& point,
                                   const uint16_t fontheight) {
 
-	uint32_t start, end;
+	uint32_t start;
+	uint32_t end;
 	calculate_selection_boundaries(start, end);
 	auto nr_characters = end - start;
 
