@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,13 +12,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "ui_basic/listselect.h"
 
+#include <memory>
+
 #include <SDL_mouse.h>
 #include <SDL_timer.h>
 
@@ -30,6 +31,7 @@
 #include "graphic/text_layout.h"
 #include "ui_basic/dropdown.h"
 #include "ui_basic/mouse_constants.h"
+#include "wlapplication_options.h"
 
 constexpr int kMargin = 2;
 constexpr int kHotkeyGap = 16;
@@ -57,6 +59,14 @@ BaseListselect::EntryRecord::EntryRecord(const std::string& init_name,
 	   UI::g_fh->render(as_richtext_paragraph(richtext_escape(hotkey_text), style.hotkey()));
 }
 
+inline BaseListselect::EntryRecord::~EntryRecord() {
+	// This ensures that the last instance of this smart pointer does
+	// not go out of scope in a thread that is not the main thread.
+	std::shared_ptr<const UI::RenderedText> r1 = rendered_name;
+	std::shared_ptr<const UI::RenderedText> r2 = rendered_hotkey;
+	NoteThreadSafeFunction::instantiate([r1, r2]() {}, false);
+}
+
 /**
  * Initialize a list select panel
  *
@@ -76,14 +86,14 @@ BaseListselect::BaseListselect(Panel* const parent,
    : Panel(parent, style, x, y, w, h),
      widest_text_(0),
      widest_hotkey_(0),
-     scrollbar_(this, get_w() - Scrollbar::kSize, 0, Scrollbar::kSize, h, style),
+     scrollbar_(this, get_w() - Scrollbar::kSize, 0, 0, h, style),
      scrollpos_(0),
      selection_(no_selection_index()),
      last_click_time_(-10000),
      last_selection_(no_selection_index()),
      selection_mode_(selection_mode),
      lineheight_(text_height(table_style().enabled()) + kMargin),
-     notify_on_delete_(nullptr) {
+     linked_dropdown(nullptr) {
 	set_thinks(false);
 
 	scrollbar_.moved.connect([this](int32_t a) { set_scrollpos(a); });
@@ -107,8 +117,8 @@ BaseListselect::BaseListselect(Panel* const parent,
  */
 BaseListselect::~BaseListselect() {
 	clear();
-	if (notify_on_delete_) {
-		notify_on_delete_->notify_list_deleted();
+	if (linked_dropdown != nullptr) {
+		linked_dropdown->notify_list_deleted();
 	}
 }
 
@@ -117,8 +127,8 @@ inline const UI::TableStyleInfo& BaseListselect::table_style() const {
 }
 inline const UI::PanelStyleInfo* BaseListselect::background_style() const {
 	return selection_mode_ == ListselectLayout::kDropdown ?
-	          g_style_manager->dropdown_style(panel_style_) :
-	          nullptr;
+             g_style_manager->dropdown_style(panel_style_) :
+             nullptr;
 }
 
 /**
@@ -154,7 +164,7 @@ void BaseListselect::add(const std::string& name,
 	EntryRecord* er = new EntryRecord(name, entry, pic, tooltip_text, hotkey, indent, table_style());
 
 	int entry_height = lineheight_;
-	if (pic) {
+	if (pic != nullptr) {
 		int w = pic->width();
 		int h = pic->height();
 		entry_height = (h >= entry_height) ? h : entry_height;
@@ -252,7 +262,7 @@ bool BaseListselect::has_selection() const {
  */
 uint32_t BaseListselect::get_selected() const {
 	return selection_ < entry_records_.size() ? entry_records_[selection_]->entry_ :
-	                                            no_selection_index();
+                                               no_selection_index();
 }
 
 /**
@@ -314,20 +324,25 @@ int BaseListselect::calculate_desired_width() {
 		txt_width += widest_hotkey_;
 	}
 
-	const int picw = max_pic_width_ ? max_pic_width_ + 10 : 0;
+	const int picw = max_pic_width_ != 0 ? max_pic_width_ + 10 : 0;
 	const int old_width = get_w();
 	return txt_width + picw + 8 + old_width - get_eff_w();
 }
 
 void BaseListselect::layout() {
-	scrollbar_.set_size(scrollbar_.get_w(), get_h());
-	scrollbar_.set_pos(Vector2i(get_w() - Scrollbar::kSize, 0));
-	scrollbar_.set_pagesize(get_h() - 2 * get_lineheight());
-	scrollbar_.set_singlestepsize(get_lineheight());
 	const int steps = entry_records_.size() * get_lineheight() - get_h();
 	scrollbar_.set_steps(steps);
-	if (scrollbar_.is_enabled() && selection_mode_ == ListselectLayout::kDropdown) {
-		scrollbar_.set_steps(steps + kMargin);
+	if (scrollbar_.is_enabled()) {
+		scrollbar_.set_size(Scrollbar::kSize, get_h());
+		scrollbar_.set_pos(Vector2i(get_w() - Scrollbar::kSize, 0));
+		scrollbar_.set_pagesize(get_h() - 2 * get_lineheight());
+		scrollbar_.set_singlestepsize(get_lineheight());
+		if (selection_mode_ == ListselectLayout::kDropdown) {
+			scrollbar_.set_steps(steps + kMargin);
+		}
+	} else {
+		// Prevent invisible child
+		scrollbar_.set_size(0, get_h());
 	}
 	// For dropdowns, autoincrease width
 	if (selection_mode_ == ListselectLayout::kDropdown) {
@@ -405,13 +420,13 @@ void BaseListselect::draw(RenderTarget& dst) {
 			}
 		}
 
-		int picw = max_pic_width_ ? max_pic_width_ + 10 : 0;
+		int picw = max_pic_width_ != 0 ? max_pic_width_ + 10 : 0;
 
 		// Now draw pictures
-		if (er.pic) {
+		if (er.pic != nullptr) {
 			dst.blit(Vector2i(UI::g_fh->fontset()->is_rtl() ?
-			                     get_eff_w() - er.pic->width() - 1 - kIndentStrength * er.indent :
-			                     kIndentStrength * er.indent + 1,
+                              get_eff_w() - er.pic->width() - 1 - kIndentStrength * er.indent :
+                              kIndentStrength * er.indent + 1,
 			                  y + (lineheight_ - er.pic->height()) / 2),
 			         er.pic);
 		}
@@ -459,6 +474,13 @@ void BaseListselect::draw(RenderTarget& dst) {
 			er.rendered_hotkey->draw(dst, hotkey_point, Recti(0, 0, maxw - widest_text_, lineheight),
 			                         UI::Align::kLeft, RenderedText::CropMode::kSelf);
 		}
+
+		// Highlight text of current filter
+		Recti highlight_rect = get_highlight_rect(er.name, text_point.x, y);
+		if (highlight_rect.w != 0) {
+			dst.brighten_rect(highlight_rect, -ms_darken_value * 2);
+		}
+
 		y += get_lineheight();
 		++idx;
 	}
@@ -467,22 +489,31 @@ void BaseListselect::draw(RenderTarget& dst) {
 /**
  * Handle mouse wheel events
  */
-bool BaseListselect::handle_mousewheel(uint32_t which, int32_t x, int32_t y) {
+bool BaseListselect::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
 	const uint32_t selected_idx = selection_index();
-	const uint32_t max = size() - 1;
-	if (y > 0 && selected_idx > 0) {
-		select(selected_idx - 1);
-	} else if (y < 0 && selected_idx < max) {
-		select(selected_idx + 1);
+	uint32_t max = size();
+	if (max > 0) {
+		--max;
+	} else {
+		return false;
 	}
-
-	return scrollbar_.handle_mousewheel(which, x, y);
+	if (y != 0 && matches_keymod(modstate, KMOD_NONE)) {
+		if (selected_idx > max) {
+			select(y < 0 ? 0 : max);
+		} else if (y > 0 && selected_idx > 0) {
+			select(selected_idx - 1);
+		} else if (y < 0 && selected_idx < max) {
+			select(selected_idx + 1);
+		}
+		return scrollbar_.handle_mousewheel(x, y, modstate);
+	}
+	return false;
 }
 
 /**
  * Handle mouse presses: select the appropriate entry
  */
-bool BaseListselect::handle_mousepress(const uint8_t btn, int32_t, int32_t y) {
+bool BaseListselect::handle_mousepress(const uint8_t btn, int32_t /*x*/, int32_t y) {
 	switch (btn) {
 
 	case SDL_BUTTON_LEFT: {
@@ -499,6 +530,7 @@ bool BaseListselect::handle_mousepress(const uint8_t btn, int32_t, int32_t y) {
 		if (y < 0 || static_cast<int32_t>(entry_records_.size()) <= y) {
 			if (selection_mode_ == ListselectLayout::kDropdown) {
 				set_visible(false);
+				linked_dropdown->disable_textinput();
 				return true;
 			}
 			return false;
@@ -520,7 +552,8 @@ bool BaseListselect::handle_mousepress(const uint8_t btn, int32_t, int32_t y) {
 	}
 }
 
-bool BaseListselect::handle_mousemove(uint8_t, int32_t, int32_t y, int32_t, int32_t) {
+bool BaseListselect::handle_mousemove(
+   uint8_t /*state*/, int32_t /*x*/, int32_t y, int32_t /*xdiff*/, int32_t /*ydiff*/) {
 	y = (y + scrollpos_) / get_lineheight();
 	if (y < 0 || static_cast<int32_t>(entry_records_.size()) <= y) {
 		set_tooltip("");
@@ -534,68 +567,59 @@ bool BaseListselect::handle_mousemove(uint8_t, int32_t, int32_t y, int32_t, int3
 }
 
 bool BaseListselect::handle_key(bool const down, SDL_Keysym const code) {
-	if (down && size() > 1) {
+	if (down) {
+		switch (code.sym) {
+		case SDLK_BACKSPACE:
+			if (linked_dropdown != nullptr) {
+				linked_dropdown->delete_last_of_filter();
+				return true;
+			}
+			return UI::Panel::handle_key(down, code);
+		case SDLK_ESCAPE:
+		case SDLK_RETURN:
+			if (linked_dropdown != nullptr && (SDL_GetModState() & KMOD_CTRL) == 0) {
+				return linked_dropdown->handle_key(down, code);
+			}
+			return UI::Panel::handle_key(down, code);
+		default:
+			break;
+		}
+
 		bool handle = true;
 		uint32_t selected_idx = selection_index();
-		const uint32_t max = size() - 1;
+		const uint32_t max = empty() ? 0 : size() - 1;
 		const uint32_t pagesize = std::max(1, get_h() / get_lineheight());
-
-		if ((code.sym >= SDLK_1 && code.sym <= SDLK_9) ||
-		    (code.sym >= SDLK_KP_1 && code.sym <= SDLK_KP_9 && (code.mod & KMOD_NUM))) {
-			// Keys 1-9 directly address the 1st through 9th item in lists with less than 10 entries
-			if (max < 9) {
-				if (code.sym >= SDLK_1 && code.sym <= static_cast<int>(SDLK_1 + max)) {
-					selected_idx = code.sym - SDLK_1;
-				} else if (code.sym >= SDLK_KP_1 && code.sym <= static_cast<int>(SDLK_KP_1 + max)) {
-					selected_idx = code.sym - SDLK_KP_1;
-				} else {
-					// don't handle the '9' when there are less than 9 items
-					handle = false;
-				}
-			} else {
-				// 10 or more items – ignore number keys
-				handle = false;
-			}
-		} else {
-			// Up, Down, PageUp, PageDown, Home, End
-			switch (code.sym) {
-			case SDLK_KP_2:
-			case SDLK_DOWN:
-				if (!has_selection()) {
-					selected_idx = 0;
-				} else if (selected_idx < max) {
-					++selected_idx;
-				}
-				break;
-			case SDLK_KP_8:
-			case SDLK_UP:
-				if (!has_selection()) {
-					selected_idx = max;
-				} else if (selected_idx > 0) {
-					--selected_idx;
-				}
-				break;
-			case SDLK_KP_7:
-			case SDLK_HOME:
+		switch (code.sym) {
+		case SDLK_DOWN:
+			if (!has_selection()) {
 				selected_idx = 0;
-				break;
-			case SDLK_KP_1:
-			case SDLK_END:
-				selected_idx = max;
-				break;
-			case SDLK_KP_3:
-			case SDLK_PAGEDOWN:
-				selected_idx = has_selection() ? std::min(max, selected_idx + pagesize) : 0;
-				break;
-			case SDLK_KP_9:
-			case SDLK_PAGEUP:
-				selected_idx =
-				   has_selection() ? selected_idx > pagesize ? selected_idx - pagesize : 0 : max;
-				break;
-			default:
-				handle = false;
-				break;  // not handled
+			} else if (selected_idx < max) {
+				++selected_idx;
 			}
+			break;
+		case SDLK_UP:
+			if (!has_selection()) {
+				selected_idx = max;
+			} else if (selected_idx > 0) {
+				--selected_idx;
+			}
+			break;
+		case SDLK_HOME:
+			selected_idx = 0;
+			break;
+		case SDLK_END:
+			selected_idx = max;
+			break;
+		case SDLK_PAGEDOWN:
+			selected_idx = has_selection() ? std::min(max, selected_idx + pagesize) : 0;
+			break;
+		case SDLK_PAGEUP:
+			selected_idx =
+			   has_selection() ? selected_idx > pagesize ? selected_idx - pagesize : 0 : max;
+			break;
+		default:
+			handle = false;
+			break;  // not handled
 		}
 		assert((selected_idx <= max) ^ (selected_idx == no_selection_index()));
 		if (handle) {
@@ -611,7 +635,6 @@ bool BaseListselect::handle_key(bool const down, SDL_Keysym const code) {
 			return true;
 		}
 	}
-
 	return UI::Panel::handle_key(down, code);
 }
 
@@ -637,10 +660,41 @@ void BaseListselect::remove(const uint32_t i) {
  */
 void BaseListselect::remove(const char* const str) {
 	for (uint32_t i = 0; i < entry_records_.size(); ++i) {
-		if (!strcmp(entry_records_[i]->name.c_str(), str)) {
+		if (strcmp(entry_records_[i]->name.c_str(), str) == 0) {
 			remove(i);
 			return;
 		}
 	}
 }
+
+Recti BaseListselect::get_highlight_rect(const std::string& text, int x, int y) {
+
+	if (linked_dropdown == nullptr) {
+		return Recti();
+	}
+
+	std::string text2 = std::string(text);
+	// UI::g_fh::render ignores spaces at the beginning and end of a string.
+	replace_all(text2, " ", "_");
+
+	std::string filter = linked_dropdown->get_filter_text();
+	replace_all(filter, " ", "_");
+
+	std::size_t start = to_lower(text2).find(filter);
+	if (start == std::string::npos) {
+		return Recti();
+	}
+
+	std::shared_ptr<const UI::RenderedText> rendered_start = UI::g_fh->render(
+	   as_richtext_paragraph(richtext_escape(text2.substr(0, start)), table_style().enabled()));
+
+	std::shared_ptr<const UI::RenderedText> rendered_substring =
+	   UI::g_fh->render(as_richtext_paragraph(
+	      richtext_escape(text2.substr(start, filter.length())), table_style().enabled()));
+
+	Recti highlight_rect(x + rendered_start->width(), y, rendered_substring->width(), lineheight_);
+
+	return highlight_rect;
+}
+
 }  // namespace UI
