@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -71,13 +70,13 @@ struct MultilineEditbox::Data {
 	WordWrap ww;
 	/*@}*/
 
-	explicit Data(MultilineEditbox&);
+	explicit Data(MultilineEditbox& init_owner);
 	void refresh_ww();
 
 	void update();
 
 	void scroll_cursor_into_view();
-	void set_cursor_pos(uint32_t cursor_pos);
+	void set_cursor_pos(uint32_t newpos);
 
 	uint32_t prev_char(uint32_t cursor);
 	uint32_t next_char(uint32_t cursor);
@@ -108,12 +107,18 @@ MultilineEditbox::MultilineEditbox(
 	set_handle_textinput();
 }
 
-MultilineEditbox::Data::Data(MultilineEditbox& o)
-   : scrollbar(&o, o.get_w() - Scrollbar::kSize, 0, Scrollbar::kSize, o.get_h(), o.panel_style_),
-     style(o.panel_style_),
+MultilineEditbox::Data::Data(MultilineEditbox& init_owner)
+   : scrollbar(&init_owner,
+               init_owner.get_w() - Scrollbar::kSize,
+               0,
+               Scrollbar::kSize,
+               init_owner.get_h(),
+               init_owner.panel_style_),
+     style(init_owner.panel_style_),
      cursor_pos(0),
-     caret_image_path(o.panel_style_ == PanelStyle::kWui ? "images/ui_basic/caret_wui.png" :
-                                                           "images/ui_basic/caret_fs.png"),
+     caret_image_path(init_owner.panel_style_ == PanelStyle::kWui ?
+                         "images/ui_basic/caret_wui.png" :
+                         "images/ui_basic/caret_fs.png"),
      lineheight(text_height(get_style().font())),
      maxbytes(std::min(g_gr->max_texture_size_for_font_rendering() *
                           g_gr->max_texture_size_for_font_rendering() /
@@ -123,9 +128,9 @@ MultilineEditbox::Data::Data(MultilineEditbox& o)
      selection_start(0),
      selection_end(0),
      ww_valid(false),
-     ww(get_style().font().size(), get_style().font().color(), o.get_w()),
-     owner(o) {
-	scrollbar.moved.connect([&o](int32_t a) { o.scrollpos_changed(a); });
+     ww(get_style().font().size(), get_style().font().color(), init_owner.get_w()),
+     owner(init_owner) {
+	scrollbar.moved.connect([&init_owner](int32_t a) { init_owner.scrollpos_changed(a); });
 
 	scrollbar.set_pagesize(owner.get_h() - 2 * lineheight);
 	scrollbar.set_singlestepsize(lineheight);
@@ -153,7 +158,8 @@ void MultilineEditbox::Data::calculate_selection_boundaries(uint32_t& start, uin
 }
 
 void MultilineEditbox::Data::draw(RenderTarget& dst, bool with_caret) {
-	uint32_t start, end;
+	uint32_t start;
+	uint32_t end;
 	calculate_selection_boundaries(start, end);
 	ww.draw(dst, Vector2i(0, -int32_t(scrollbar.get_scrollpos())), UI::Align::kLeft,
 	        with_caret ? cursor_pos : std::numeric_limits<uint32_t>::max(),
@@ -180,6 +186,7 @@ void MultilineEditbox::set_text(const std::string& text) {
 		return;
 	}
 
+	d_->reset_selection();
 	d_->text = text;
 	while (d_->text.size() > d_->maxbytes) {
 		d_->erase_bytes(d_->prev_char(d_->text.size()), d_->text.size());
@@ -284,7 +291,7 @@ bool MultilineEditbox::handle_mousepress(const uint8_t btn, int32_t x, int32_t y
 bool MultilineEditbox::handle_mousemove(
    uint8_t state, int32_t x, int32_t y, int32_t xdiff, int32_t ydiff) {
 	// state != 0 -> mouse button is pressed
-	if (state && get_can_focus()) {
+	if ((state != 0u) && get_can_focus()) {
 		select_until(d_->cursor_pos);
 		set_caret_to_cursor_pos(x, y);
 		select_until(d_->cursor_pos);
@@ -335,12 +342,46 @@ int MultilineEditbox::calculate_text_width(std::string& text, int pos) const {
 	return d_->ww.text_width_of(prefix);
 }
 
+bool MultilineEditbox::has_selection() const {
+	return (d_->mode == Data::Mode::kSelection);
+}
+
+std::string MultilineEditbox::get_selected_text() {
+	assert(d_->mode == Data::Mode::kSelection);
+	uint32_t start;
+	uint32_t end;
+	d_->calculate_selection_boundaries(start, end);
+	return d_->text.substr(start, end - start);
+}
+
+void MultilineEditbox::replace_selected_text(const std::string& text) {
+	assert(d_->mode == Data::Mode::kSelection);
+	uint32_t start;
+	uint32_t end;
+	d_->calculate_selection_boundaries(start, end);
+	std::string str = d_->text.substr(0, start);
+	str += text;
+	str += d_->text.substr(end);
+	set_text(str);
+	set_caret_pos(start);
+	select_until(get_caret_pos() + text.size());
+}
+
+size_t MultilineEditbox::get_caret_pos() const {
+	return d_->cursor_pos;
+}
+
+void MultilineEditbox::set_caret_pos(const size_t caret) const {
+	d_->set_cursor_pos(d_->snap_to_char(caret));
+}
+
 /**
  * This is called by the UI code whenever a key press or release arrives
  */
 bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 	if (down) {
-		if (matches_shortcut(KeyboardShortcut::kCommonTextPaste, code) && SDL_HasClipboardText()) {
+		if (matches_shortcut(KeyboardShortcut::kCommonTextPaste, code) &&
+		    (SDL_HasClipboardText() != 0u)) {
 			if (d_->mode == Data::Mode::kSelection) {
 				delete_selected_text();
 			}
@@ -369,11 +410,6 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 		case SDLK_TAB:
 			// Let the panel handle the tab key
 			return get_parent()->handle_key(true, code);
-		case SDLK_KP_PERIOD:
-			if (code.mod & KMOD_NUM) {
-				break;
-			}
-			FALLS_THROUGH;
 		case SDLK_DELETE:
 			if (d_->mode == Data::Mode::kSelection) {
 				delete_selected_text();
@@ -394,26 +430,26 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 
 		case SDLK_LEFT: {
 			if (d_->cursor_pos > 0) {
-				if (code.mod & (KMOD_LCTRL | KMOD_RCTRL)) {
+				if ((code.mod & (KMOD_LCTRL | KMOD_RCTRL)) != 0) {
 					uint32_t newpos = d_->prev_char(d_->cursor_pos);
-					while (newpos > 0 && isspace(d_->text[newpos])) {
+					while (newpos > 0 && (isspace(d_->text[newpos]) != 0)) {
 						newpos = d_->prev_char(newpos);
 					}
 					while (newpos > 0) {
 						uint32_t prev = d_->prev_char(newpos);
-						if (isspace(d_->text[prev])) {
+						if (isspace(d_->text[prev]) != 0) {
 							break;
 						}
 						newpos = prev;
 					}
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(newpos);
 					} else {
 						d_->reset_selection();
 					}
 					d_->set_cursor_pos(newpos);
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(d_->prev_char(d_->cursor_pos));
 					} else {
 						d_->reset_selection();
@@ -426,22 +462,22 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 
 		case SDLK_RIGHT:
 			if (d_->cursor_pos < d_->text.size()) {
-				if (code.mod & (KMOD_LCTRL | KMOD_RCTRL)) {
+				if ((code.mod & (KMOD_LCTRL | KMOD_RCTRL)) != 0) {
 					uint32_t newpos = d_->next_char(d_->cursor_pos);
-					while (newpos < d_->text.size() && isspace(d_->text[newpos])) {
+					while (newpos < d_->text.size() && (isspace(d_->text[newpos]) != 0)) {
 						newpos = d_->next_char(newpos);
 					}
-					while (newpos < d_->text.size() && !isspace(d_->text[newpos])) {
+					while (newpos < d_->text.size() && (isspace(d_->text[newpos]) == 0)) {
 						newpos = d_->next_char(newpos);
 					}
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(newpos);
 					} else {
 						d_->reset_selection();
 					}
 					d_->set_cursor_pos(newpos);
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(d_->next_char(d_->cursor_pos));
 					} else {
 						d_->reset_selection();
@@ -455,7 +491,8 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 			if (d_->cursor_pos < d_->text.size()) {
 				d_->refresh_ww();
 
-				uint32_t cursorline, cursorpos = 0;
+				uint32_t cursorline;
+				uint32_t cursorpos = 0;
 				d_->ww.calc_wrapped_pos(d_->cursor_pos, cursorline, cursorpos);
 
 				if (cursorline + 1 < d_->ww.nrlines()) {
@@ -470,14 +507,14 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 					} else {
 						newpos = d_->snap_to_char(newpos);
 					}
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(newpos);
 					} else {
 						d_->reset_selection();
 					}
 					d_->set_cursor_pos(newpos);
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(d_->text.size());
 					} else {
 						d_->reset_selection();
@@ -491,7 +528,8 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 			if (d_->cursor_pos > 0) {
 				d_->refresh_ww();
 
-				uint32_t cursorline, cursorpos = 0;
+				uint32_t cursorline;
+				uint32_t cursorpos = 0;
 				d_->ww.calc_wrapped_pos(d_->cursor_pos, cursorline, cursorpos);
 
 				if (cursorline > 0) {
@@ -503,14 +541,14 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 					} else {
 						newpos = d_->snap_to_char(newpos);
 					}
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(newpos);
 					} else {
 						d_->reset_selection();
 					}
 					d_->set_cursor_pos(newpos);
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(0);
 					} else {
 						d_->reset_selection();
@@ -521,8 +559,8 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 			break;
 
 		case SDLK_HOME:
-			if (code.mod & (KMOD_LCTRL | KMOD_RCTRL)) {
-				if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((code.mod & (KMOD_LCTRL | KMOD_RCTRL)) != 0) {
+				if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 					select_until(0);
 				} else {
 					d_->reset_selection();
@@ -530,10 +568,11 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 				d_->set_cursor_pos(0);
 			} else {
 				d_->refresh_ww();
-				uint32_t cursorline, cursorpos = 0;
+				uint32_t cursorline;
+				uint32_t cursorpos = 0;
 				d_->ww.calc_wrapped_pos(d_->cursor_pos, cursorline, cursorpos);
 
-				if (SDL_GetModState() & KMOD_SHIFT) {
+				if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 					select_until(d_->ww.line_offset(cursorline));
 				} else {
 					d_->reset_selection();
@@ -543,8 +582,8 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 			break;
 
 		case SDLK_END:
-			if (code.mod & (KMOD_LCTRL | KMOD_RCTRL)) {
-				if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((code.mod & (KMOD_LCTRL | KMOD_RCTRL)) != 0) {
+				if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 					select_until(d_->text.size());
 				} else {
 					d_->reset_selection();
@@ -553,18 +592,19 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 			} else {
 				d_->refresh_ww();
 
-				uint32_t cursorline, cursorpos = 0;
+				uint32_t cursorline;
+				uint32_t cursorpos = 0;
 				d_->ww.calc_wrapped_pos(d_->cursor_pos, cursorline, cursorpos);
 
 				if (cursorline + 1 < d_->ww.nrlines()) {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(d_->prev_char(d_->ww.line_offset(cursorline + 1)));
 					} else {
 						d_->reset_selection();
 					}
 					d_->set_cursor_pos(d_->prev_char(d_->ww.line_offset(cursorline + 1)));
 				} else {
-					if (SDL_GetModState() & KMOD_SHIFT) {
+					if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 						select_until(d_->text.size());
 					} else {
 						d_->reset_selection();
@@ -574,8 +614,10 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 			}
 			break;
 
-		case SDLK_KP_ENTER:
 		case SDLK_RETURN:
+			if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+				return false;
+			}
 			d_->insert(d_->cursor_pos, "\n");
 			d_->update();
 			d_->reset_selection();
@@ -592,7 +634,8 @@ bool MultilineEditbox::handle_key(bool const down, SDL_Keysym const code) {
 }
 
 void MultilineEditbox::copy_selected_text() const {
-	uint32_t start, end;
+	uint32_t start;
+	uint32_t end;
 	d_->calculate_selection_boundaries(start, end);
 
 	auto nr_characters = end - start;
@@ -611,7 +654,8 @@ void MultilineEditbox::select_until(uint32_t end) const {
 	d_->selection_end = end;
 }
 void MultilineEditbox::delete_selected_text() const {
-	uint32_t start, end;
+	uint32_t start;
+	uint32_t end;
 	d_->calculate_selection_boundaries(start, end);
 	d_->erase_bytes(start, end);
 	changed();
@@ -633,6 +677,7 @@ bool MultilineEditbox::handle_textinput(const std::string& input_text) {
  */
 void MultilineEditbox::focus(bool topcaller) {
 	Panel::focus(topcaller);
+	d_->ww.focus();
 }
 
 /**
@@ -703,7 +748,8 @@ void MultilineEditbox::Data::set_cursor_pos(uint32_t newpos) {
 void MultilineEditbox::Data::scroll_cursor_into_view() {
 	refresh_ww();
 
-	uint32_t cursorline, cursorpos = 0;
+	uint32_t cursorline;
+	uint32_t cursorpos = 0;
 	ww.calc_wrapped_pos(cursor_pos, cursorline, cursorpos);
 
 	int32_t top = cursorline * lineheight;
@@ -717,7 +763,7 @@ void MultilineEditbox::Data::scroll_cursor_into_view() {
 /**
  * Callback function called by the scrollbar.
  */
-void MultilineEditbox::scrollpos_changed(int32_t) {
+void MultilineEditbox::scrollpos_changed(int32_t /* pos */) {
 }
 
 /**
