@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,19 +12,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "ui_basic/panel.h"
 
+#include <atomic>
 #include <memory>
 
 #include <SDL_timer.h>
 
+#include "base/i18n.h"
 #include "graphic/font_handler.h"
 #include "graphic/graphic.h"
+#include "graphic/graphic_functions.h"
 #include "graphic/mouse_cursor.h"
 #include "graphic/rendertarget.h"
 #include "graphic/style_manager.h"
@@ -36,8 +38,8 @@
 
 namespace UI {
 
-Panel* Panel::modal_ = nullptr;
-Panel* Panel::mousegrab_ = nullptr;
+std::atomic<Panel*> Panel::modal_(nullptr);
+std::atomic<Panel*> Panel::mousegrab_(nullptr);
 Panel* Panel::mousein_ = nullptr;
 Panel* Panel::tooltip_panel_ = nullptr;
 Vector2i Panel::tooltip_fixed_pos_ = Vector2i::invalid();
@@ -64,6 +66,7 @@ Panel::Panel(Panel* const nparent,
              const int nh,
              const std::string& tooltip_text)
    : panel_style_(s),
+     initialized_(false),
      parent_(nparent),
      first_child_(nullptr),
      last_child_(nullptr),
@@ -83,12 +86,13 @@ Panel::Panel(Panel* const nparent,
      desired_w_(nw),
      desired_h_(nh),
      running_(false),
-     tooltip_(tooltip_text) {
+     tooltip_(tooltip_text),
+     logic_thread_locked_(LogicThreadState::kEndingConfirmed) {
 	assert(nparent != this);
-	if (parent_) {
+	if (parent_ != nullptr) {
 		next_ = parent_->first_child_;
 		prev_ = nullptr;
-		if (next_) {
+		if (next_ != nullptr) {
 			next_->prev_ = this;
 		} else {
 			parent_->last_child_ = this;
@@ -103,6 +107,8 @@ Panel::Panel(Panel* const nparent,
  * Unlink the panel from the parent's queue
  */
 Panel::~Panel() {
+	initialized_ = false;
+
 	// Release pointers to this object
 	if (mousegrab_ == this) {
 		mousegrab_ = nullptr;
@@ -118,7 +124,7 @@ Panel::~Panel() {
 	free_children();
 
 	// Unlink
-	if (parent_) {
+	if (parent_ != nullptr) {
 		if (parent_->mousein_child_ == this) {
 			parent_->mousein_child_ = nullptr;
 		}
@@ -126,12 +132,12 @@ Panel::~Panel() {
 			parent_->focus_ = nullptr;
 		}
 
-		if (prev_) {
+		if (prev_ != nullptr) {
 			prev_->next_ = next_;
 		} else {
 			parent_->first_child_ = next_;
 		}
-		if (next_) {
+		if (next_ != nullptr) {
 			next_->prev_ = prev_;
 		} else {
 			parent_->last_child_ = prev_;
@@ -139,6 +145,13 @@ Panel::~Panel() {
 	}
 }
 
+void Panel::initialization_complete() {
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
+		child->initialization_complete();
+	}
+	initialized_ = true;
+}
+
 /**
  * Free all of the panel's children.
  */
@@ -147,7 +160,7 @@ void Panel::free_children() {
 	// This is a false positive.
 	// The reason is that the variable will be reassigned in the destructor of the deleted child.
 	// This is very uncommon behavior and bad style, but will be non trivial to fix.
-	while (first_child_) {
+	while (first_child_ != nullptr) {
 		Panel* next_child = first_child_->next_;
 		delete first_child_;
 		first_child_ = next_child;
@@ -155,22 +168,69 @@ void Panel::free_children() {
 	first_child_ = nullptr;
 }
 
-struct ModalGuard {
-	explicit ModalGuard(Panel& p) : bottom_panel_(Panel::modal_), top_panel_(p) {
-		Panel::modal_ = &top_panel_;
+Panel::ModalGuard::ModalGuard(Panel& p) : bottom_panel_(Panel::modal_), top_panel_(p) {
+	if (Panel::modal_ != nullptr) {
+		/* Clean up stale notes first. */
+		Panel::modal_.load()->handle_notes();
 	}
-	~ModalGuard() {
-		Panel::modal_ = bottom_panel_;
-		if (bottom_panel_) {
-			bottom_panel_->become_modal_again(top_panel_);
-		}
+	Panel::modal_ = &top_panel_;
+}
+Panel::ModalGuard::~ModalGuard() {
+	Panel::modal_ = bottom_panel_;
+	if (bottom_panel_ != nullptr) {
+		bottom_panel_->become_modal_again(top_panel_);
 	}
+}
 
-private:
-	Panel* bottom_panel_;
-	Panel& top_panel_;
-	DISALLOW_COPY_AND_ASSIGN(ModalGuard);
-};
+std::atomic_bool Panel::logic_thread_running_(false);
+
+constexpr uint32_t kGameLogicDelay = 50;
+
+// static
+void Panel::logic_thread() {
+	set_logic_thread();
+	logic_thread_running_ = true;
+	WLApplication* const app = WLApplication::get();
+
+	while (!app->should_die()) {
+		Panel* m =
+		   modal_;  // copy this because another panel may become modal during a lengthy logic frame
+
+		if ((m != nullptr) && ((m->flags_ & pf_logic_think) != 0u)) {
+			LogicThreadState lock_if_free = LogicThreadState::kFree;
+			if (m->logic_thread_locked_.compare_exchange_strong(
+			       lock_if_free, LogicThreadState::kLocked)) {
+				MutexLock lock(MutexLock::ID::kLogicFrame);
+
+				m->game_logic_think();  // actual game logic
+				LogicThreadState free_if_locked = LogicThreadState::kLocked;
+				m->logic_thread_locked_.compare_exchange_strong(
+				   free_if_locked, LogicThreadState::kFree);
+			}
+			LogicThreadState end_if_end_requested = LogicThreadState::kEndingRequested;
+			m->logic_thread_locked_.compare_exchange_strong(
+			   end_if_end_requested, LogicThreadState::kEndingConfirmed);
+		}
+
+		// Always sleep a bit because another thread might want to lock our mutex
+		SDL_Delay(kGameLogicDelay);
+	}
+	logic_thread_running_ = false;
+}
+
+void Panel::handle_notes() {
+	while (!notes_.empty()) {
+		if (handled_notes_.count(notes_.front().id) == 0) {
+			// If there are multiple modal panels, ensure each note is handled only once
+			Notifications::publish(NoteThreadSafeFunctionHandled(notes_.front().id));
+
+			notes_.front().run();
+		} else {
+			handled_notes_.erase(notes_.front().id);
+		}
+		notes_.pop_front();
+	}
+}
 
 Panel& Panel::get_topmost_forefather() {
 	Panel* forefather = this;
@@ -180,27 +240,94 @@ Panel& Panel::get_topmost_forefather() {
 	return *forefather;
 }
 
-void Panel::do_redraw_now() {
+void Panel::do_redraw_now(const bool handle_input, const std::string& message) {
+	assert(is_initializer_thread());
+
+	WLApplication* const app = WLApplication::get();
+	static InputCallback input_callback = {Panel::ui_mousepress, Panel::ui_mouserelease,
+	                                       Panel::ui_mousemove,  Panel::ui_key,
+	                                       Panel::ui_textinput,  Panel::ui_mousewheel};
+	if (handle_input && message.empty()) {
+		app->handle_input(&input_callback);
+	}
+
 	Panel& ff = get_topmost_forefather();
 	RenderTarget& rt = *g_gr->get_render_target();
 
-	ff.do_draw(rt);
+	{
+		MutexLock m(MutexLock::ID::kObjects, [this]() { handle_notes(); });
+		ff.do_draw(rt);
+	}
+
+	if (!message.empty()) {
+		// After the user clicked on Quit, it may sometimes take many seconds
+		// until the logic frame has ended. During this time, we no longer
+		// handle input, and we gray out the user interface to indicate this.
+
+		rt.tile(Recti(0, 0, g_gr->get_xres(), g_gr->get_yres()),
+		        &load_safe_template_image("loadscreens/ending.png"), Vector2i(0, 0));
+
+		draw_game_tip(rt, Recti(0, 0, g_gr->get_xres(), g_gr->get_yres()), message, 2);
+	}
 
 	if (g_mouse_cursor->is_visible()) {
-		WLApplication* const app = WLApplication::get();
 		g_mouse_cursor->change_cursor(app->is_mouse_pressed());
 		g_mouse_cursor->draw(rt, app->get_mouse_position());
 
-		if (is_modal()) {
-			do_tooltip();
-		} else {
-			ff.do_tooltip();
+		// Tooltip magic. Some panels never want to show a tooltip, and if the display an
+		// overlay message we ignore user input and don't draw tooltips any more either.
+		// When deciding which panel gets to draw a tooltip (if allowed), modal panels and
+		// their children take precedence over a (potentially non-modal) toplevel panel.
+		if (message.empty() && (flags_ & pf_hide_all_overlays) == 0) {
+			if (modal_ != nullptr) {
+				modal_.load()->do_tooltip();
+			} else if (is_modal()) {
+				do_tooltip();
+			} else {
+				ff.do_tooltip();
+			}
 		}
 	}
 
 	g_gr->refresh();
 }
 
+void Panel::stay_responsive() {
+	assert(modal_);
+	if (modal_ != this) {
+		return modal_.load()->stay_responsive();
+	}
+
+	handle_notes();
+	do_redraw_now(true, _("Please wait…"));
+}
+
+void Panel::wait_for_current_logic_frame() {
+	if (!is_initializer_thread()) {
+		// This is the logic thread, so there would be little
+		// point in waiting for ourself to do something
+		return;
+	}
+
+	assert(modal_);
+	if (modal_ != this) {
+		assert(get_parent());
+		get_parent()->wait_for_current_logic_frame();
+		return;
+	}
+
+	Panel& wait = *modal_;
+	while (wait.logic_thread_locked_ == LogicThreadState::kLocked) {
+		stay_responsive();
+		SDL_Delay(2);
+	}
+}
+
+void Panel::clear_current_think_mutex() {
+	assert(current_think_mutex_.get() != nullptr);
+	current_think_mutex_.reset();
+}
+
 /**
  * Enters the event loop; all events will be handled by this panel.
  *
@@ -209,21 +336,17 @@ void Panel::do_redraw_now() {
  * clicked the window's close button or similar).
  */
 int Panel::do_run() {
+	assert(initialized_);
+
+	logic_thread_locked_ =
+	   LogicThreadState::kEndingConfirmed;  // don't start the logic thread ere we're ready
+
 	// TODO(sirver): the main loop should not be in UI, but in WLApplication.
 	WLApplication* const app = WLApplication::get();
 	ModalGuard prevmodal(*this);
 	mousegrab_ = nullptr;        // good ol' paranoia
 	app->set_mouse_lock(false);  // more paranoia :-)
 
-	// Loop
-	running_ = true;
-
-	// Panel-specific startup code. This might call end_modal()!
-	start();
-
-	// think() is called at most 15 times per second, that is roughly ever 66ms.
-	const uint32_t kGameLogicDelay = 1000 / 15;
-
 	// With the default of 30FPS, the game will be drawn every 33ms.
 	const uint32_t draw_delay = 1000 / std::max(5, get_config_int("maxfps", 30));
 
@@ -231,38 +354,98 @@ int Panel::do_run() {
 	                                       Panel::ui_mousemove,  Panel::ui_key,
 	                                       Panel::ui_textinput,  Panel::ui_mousewheel};
 
-	const uint32_t initial_ticks = SDL_GetTicks();
-	uint32_t next_think_time = initial_ticks + kGameLogicDelay;
-	uint32_t next_draw_time = initial_ticks + draw_delay;
+	const bool is_initializer = is_initializer_thread();
+
+	notes_.clear();
+	handled_notes_.clear();
+	subscriber1_ = is_initializer ?
+                     Notifications::subscribe<NoteThreadSafeFunction>(
+	                     [this](const NoteThreadSafeFunction& note) { notes_.push_back(note); }) :
+                     nullptr;
+	subscriber2_ = is_initializer ? Notifications::subscribe<NoteThreadSafeFunctionHandled>(
+	                                   [this](const NoteThreadSafeFunctionHandled& note) {
+		                                   assert(!handled_notes_.count(note.id));
+		                                   handled_notes_.insert(note.id);
+	                                   }) :
+                                   nullptr;
+
+	// Loop
+	running_ = true;
+
+	// Panel-specific startup code. This might call end_modal()!
+	start();
+
+	logic_thread_locked_ = LogicThreadState::kFree;  // tell the logic thread we're ready
+
+	uint32_t next_time = SDL_GetTicks();
 	while (running_) {
 		const uint32_t start_time = SDL_GetTicks();
 
-		app->handle_input(&input_callback);
+		if (modal_ == this) {
+			handle_notes();
+		}
 
-		if (start_time >= next_think_time) {
+		if (is_initializer) {
+			app->handle_input(&input_callback);
+		}
+
+		if (start_time >= next_time) {
 			if (app->should_die()) {
 				end_modal<Returncodes>(Returncodes::kBack);
+				assert(!running_);
 			}
 
-			do_think();
-
-			if (flags_ & pf_child_die) {
-				check_child_death();
+			{
+				current_think_mutex_.reset(
+				   new MutexLock(MutexLock::ID::kObjects, [this]() { handle_notes(); }));
+				do_think();
+				current_think_mutex_.reset();
 			}
 
-			next_think_time = start_time + kGameLogicDelay;
+			check_child_death();
+
+			if (is_initializer) {
+				do_redraw_now();
+			}
+
+			next_time = start_time + draw_delay;
 		}
 
-		if (start_time >= next_draw_time) {
-			do_redraw_now();
-			next_draw_time = start_time + draw_delay;
-		}
-
-		int32_t delay = std::min<int32_t>(next_draw_time, next_think_time) - SDL_GetTicks();
-		if (delay > 0) {
+		const int32_t delay = next_time - SDL_GetTicks();
+		if (running_ && delay > 0) {
 			SDL_Delay(delay);
 		}
 	}
+
+	// Wait until the current logic frame ends or there may be segfaults.
+	// This may take quite a while if the game was running at low LOGIC-FPS,
+	// so we continue refreshing the graphics while we wait.
+	if (logic_thread_locked_ != LogicThreadState::kEndingConfirmed && logic_thread_running_) {
+		logic_thread_locked_ = LogicThreadState::kEndingRequested;
+		while (((flags_ & pf_logic_think) != 0u) && logic_thread_running_ &&
+		       logic_thread_locked_ != LogicThreadState::kEndingConfirmed) {
+			const uint32_t start_time = SDL_GetTicks();
+
+			handle_notes();
+
+			if (is_initializer) {
+				do_redraw_now(true, _("Game ending – please wait…"));
+			}
+
+			next_time = start_time + draw_delay;
+			const int32_t delay = next_time - SDL_GetTicks();
+			if (delay > 0) {
+				SDL_Delay(delay);
+			}
+		}
+	}
+
+	// Unsubscribe from notes, and eliminate old minimap rendering requests and other garbarge
+	subscriber2_.reset();
+	subscriber1_.reset();
+	handle_notes();
+
+	// Panel-specific post-running code
 	end();
 
 	return return_code_;
@@ -271,7 +454,7 @@ int Panel::do_run() {
 /**
  * \return \c true if this is the currently modal panel
  */
-bool Panel::is_modal() {
+bool Panel::is_modal() const {
 	return running_;
 }
 
@@ -291,7 +474,7 @@ void Panel::end() {
  * Called when another panel (passed as argument) ends being
  * modal and returns the modal attribute to this panel
  */
-void Panel::become_modal_again(Panel&) {
+void Panel::become_modal_again(Panel& /* prevmodal */) {
 }
 
 /**
@@ -309,7 +492,7 @@ void Panel::set_size(const int nw, const int nh) {
 	w_ = std::max(0, nw);
 	h_ = std::max(0, nh);
 
-	if (parent_) {
+	if (parent_ != nullptr) {
 		move_inside_parent();
 	}
 
@@ -352,7 +535,7 @@ void Panel::set_desired_size(int w, int h) {
 	// Make sure that we never get negative width/height.
 	desired_w_ = std::max(0, w);
 	desired_h_ = std::max(0, h);
-	if (!get_layout_toplevel() && parent_) {
+	if (!get_layout_toplevel() && (parent_ != nullptr)) {
 		parent_->update_desired_size();
 	} else {
 		set_size(desired_w_, desired_h_);
@@ -380,7 +563,7 @@ void Panel::set_layout_toplevel(bool ltl) {
 }
 
 bool Panel::get_layout_toplevel() const {
-	return flags_ & pf_layout_toplevel;
+	return (flags_ & pf_layout_toplevel) != 0u;
 }
 
 /**
@@ -389,7 +572,7 @@ bool Panel::get_layout_toplevel() const {
  * and return the result.
  */
 Vector2i Panel::to_parent(const Vector2i& pt) const {
-	if (!parent_) {
+	if (parent_ == nullptr) {
 		return pt;
 	}
 
@@ -449,17 +632,17 @@ int Panel::get_inner_h() const {
  * Make this panel the top-most panel in the parent's Z-order.
  */
 void Panel::move_to_top() {
-	if (!parent_) {
+	if (parent_ == nullptr) {
 		return;
 	}
 
 	// unlink
-	if (prev_) {
+	if (prev_ != nullptr) {
 		prev_->next_ = next_;
 	} else {
 		parent_->first_child_ = next_;
 	}
-	if (next_) {
+	if (next_ != nullptr) {
 		next_->prev_ = prev_;
 	} else {
 		parent_->last_child_ = prev_;
@@ -469,7 +652,7 @@ void Panel::move_to_top() {
 	prev_ = nullptr;
 	next_ = parent_->first_child_;
 	parent_->first_child_ = this;
-	if (next_) {
+	if (next_ != nullptr) {
 		next_->prev_ = this;
 	} else {
 		parent_->last_child_ = this;
@@ -487,10 +670,10 @@ void Panel::set_visible(bool const on) {
 	flags_ &= ~pf_visible;
 	if (on) {
 		flags_ |= pf_visible;
-	} else if (parent_ && parent_->focus_ == this) {
+	} else if ((parent_ != nullptr) && parent_->focus_ == this) {
 		parent_->focus_ = nullptr;
 	}
-	if (parent_) {
+	if (parent_ != nullptr) {
 		parent_->on_visibility_changed();
 	}
 }
@@ -506,17 +689,17 @@ void Panel::on_visibility_changed() {
  * Redraw the panel. Note that all drawing coordinates are relative to the
  * inner area: you cannot overwrite the panel border in this function.
  */
-void Panel::draw(RenderTarget&) {
+void Panel::draw(RenderTarget& /* rt */) {
 }
 
 /**
  * Redraw the panel border.
  */
-void Panel::draw_border(RenderTarget&) {
+void Panel::draw_border(RenderTarget& /* rt */) {
 }
 
 std::vector<Recti>
-Panel::focus_overlay_rects(const int off_x, const int off_y, const int strength_diff) {
+Panel::focus_overlay_rects(const int off_x, const int off_y, const int strength_diff) const {
 	const int f = g_style_manager->focus_border_thickness() + strength_diff;
 	const int16_t w = get_w();
 	const int16_t h = get_h();
@@ -538,7 +721,7 @@ std::vector<Recti> Panel::focus_overlay_rects() {
  */
 void Panel::draw_overlay(RenderTarget& dst) {
 	if (has_focus()) {
-		for (Panel* p = this; p->parent_; p = p->parent_) {
+		for (Panel* p = this; p->parent_ != nullptr; p = p->parent_) {
 			if (p->parent_->focus_ != p) {
 				// doesn't have toplevel focus
 				return;
@@ -548,9 +731,10 @@ void Panel::draw_overlay(RenderTarget& dst) {
 			}
 		}
 		for (const Recti& r : focus_overlay_rects()) {
-			dst.fill_rect(
-			   r, focus_ ? g_style_manager->semi_focused_color() : g_style_manager->focused_color(),
-			   BlendMode::Default);
+			dst.fill_rect(r,
+			              focus_ != nullptr ? g_style_manager->semi_focused_color() :
+                                           g_style_manager->focused_color(),
+			              BlendMode::Default);
 		}
 	}
 }
@@ -561,7 +745,7 @@ void Panel::draw_overlay(RenderTarget& dst) {
 void Panel::draw_background(RenderTarget& dst, const UI::PanelStyleInfo& info) {
 	draw_background(dst, Recti(0, 0, get_w(), get_h()), info);
 }
-void Panel::draw_background(RenderTarget& dst, Recti rect, const UI::PanelStyleInfo& info) {
+void Panel::draw_background(RenderTarget& dst, Recti rect, const UI::PanelStyleInfo& info) const {
 	if (info.image() != nullptr) {
 		dst.fill_rect(rect, RGBAColor(0, 0, 0, 255));
 		dst.tile(rect, info.image(), Vector2i(get_x(), get_y()));
@@ -573,8 +757,8 @@ void Panel::draw_background(RenderTarget& dst, Recti rect, const UI::PanelStyleI
 
 void Panel::template_directory_changed() {
 	update_template();
-	for (Panel* child = first_child_; child; child = child->next_) {
-		child->update_template();
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
+		child->template_directory_changed();
 	}
 }
 
@@ -591,7 +775,7 @@ void Panel::think() {
  */
 void Panel::do_think() {
 	// No longer think when we are about to die
-	if (is_dying()) {
+	if (is_dying() || !initialized_) {
 		return;
 	}
 
@@ -606,7 +790,12 @@ void Panel::do_think() {
 		}
 	}
 
-	for (Panel* child = first_child_; child; child = child->next_) {
+	// think() may have called die()
+	if ((flags_ & pf_die) != 0u) {
+		return;
+	}
+
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
 		child->do_think();
 	}
 }
@@ -615,7 +804,8 @@ void Panel::do_think() {
  * Get mouse position relative to this panel
  */
 Vector2i Panel::get_mouse_position() const {
-	return (parent_ ? parent_->get_mouse_position() : WLApplication::get()->get_mouse_position()) -
+	return (parent_ != nullptr ? parent_->get_mouse_position() :
+                                WLApplication::get()->get_mouse_position()) -
 	       Vector2i(get_x() + get_lborder(), get_y() + get_tborder());
 }
 
@@ -624,7 +814,7 @@ Vector2i Panel::get_mouse_position() const {
  */
 void Panel::set_mouse_pos(const Vector2i p) {
 	const Vector2i relative_p = p + Vector2i(get_x() + get_lborder(), get_y() + get_tborder());
-	if (parent_) {
+	if (parent_ != nullptr) {
 		parent_->set_mouse_pos(relative_p);
 	} else {
 		WLApplication::get()->warp_mouse(relative_p);
@@ -644,7 +834,7 @@ void Panel::center_mouse() {
  * position received in handle_mousemove may be negative while the mouse is
  * still inside the panel as far as handle_mousein is concerned.
  */
-void Panel::handle_mousein(bool) {
+void Panel::handle_mousein(bool /* inside */) {
 }
 
 /**
@@ -654,7 +844,7 @@ void Panel::handle_mousein(bool) {
  *
  * \return true if the mouseclick was processed, false otherwise
  */
-bool Panel::handle_mousepress(const uint8_t btn, int32_t, int32_t) {
+bool Panel::handle_mousepress(const uint8_t btn, int32_t /* x */, int32_t /* y */) {
 	if (btn == SDL_BUTTON_LEFT && get_can_focus()) {
 		focus();
 		clicked();
@@ -669,7 +859,7 @@ bool Panel::handle_mousepress(const uint8_t btn, int32_t, int32_t) {
  *
  * \return true if the mouseclick was processed, false otherwise
  */
-bool Panel::handle_mouserelease(const uint8_t, int32_t, int32_t) {
+bool Panel::handle_mouserelease(const uint8_t /* btn */, int32_t /* x */, int32_t /* y */) {
 	return false;
 }
 
@@ -680,29 +870,33 @@ bool Panel::handle_mouserelease(const uint8_t, int32_t, int32_t) {
  *
  * \return true if the mouseclick was processed, false otherwise
  */
-bool Panel::handle_mousewheel(uint32_t, int32_t, int32_t) {
+bool Panel::handle_mousewheel(int32_t /* x */, int32_t /* y */, uint16_t /* modstate */) {
 	return false;
 }
 
 /**
  * Called when the mouse is moved while inside the panel
  */
-bool Panel::handle_mousemove(const uint8_t, int32_t, int32_t, int32_t, int32_t) {
+bool Panel::handle_mousemove(const uint8_t /* state */,
+                             int32_t /* x */,
+                             int32_t /* y */,
+                             int32_t /* xdiff */,
+                             int32_t /* ydiff */) {
 	return !tooltip_.empty();
 }
 
 bool Panel::handle_key(bool down, SDL_Keysym code) {
 	if (down) {
-		if (tooltip_panel_ &&
+		if ((tooltip_panel_ != nullptr) &&
 		    matches_shortcut(KeyboardShortcut::kCommonTooltipAccessibilityMode, code)) {
 			tooltip_fixed_pos_ = Vector2i::invalid();
 			return true;
 		}
 		switch (code.sym) {
 		case SDLK_TAB:
-			return handle_tab_pressed(SDL_GetModState() & KMOD_SHIFT);
+			return handle_tab_pressed((SDL_GetModState() & KMOD_SHIFT) != 0);
 		case SDLK_ESCAPE:
-			if (parent_ && parent_->focus_ == this && get_can_focus()) {
+			if ((parent_ != nullptr) && parent_->focus_ == this && get_can_focus()) {
 				parent_->focus_ = nullptr;
 				return true;
 			}
@@ -729,7 +923,7 @@ bool Panel::handle_tooltip() {
 
 // Whether TAB events should be handled by this panel's parent (`false`) or by `this` (`true`)
 bool Panel::is_focus_toplevel() const {
-	return !parent_ || this == modal_;
+	return (parent_ == nullptr) || this == modal_;
 }
 
 // Let the toplevel panel transfer the focus to the next/prev focusable child
@@ -752,7 +946,7 @@ bool Panel::handle_tab_pressed(const bool reverse) {
 	}
 
 	Panel* currently_focused = focus_;
-	while (currently_focused->focus_ && currently_focused->focus_->is_visible()) {
+	while ((currently_focused->focus_ != nullptr) && currently_focused->focus_->is_visible()) {
 		currently_focused = currently_focused->focus_;
 	}
 	// tell the next/prev panel to focus
@@ -773,7 +967,7 @@ std::deque<Panel*> Panel::gather_focusable_children() {
 		return {this};
 	}
 	std::deque<Panel*> list;
-	for (Panel* child = first_child_; child; child = child->next_) {
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
 		if (child->is_visible()) {
 			for (Panel* p : child->gather_focusable_children()) {
 				list.push_back(p);
@@ -828,12 +1022,17 @@ void Panel::set_can_focus(bool const yes) {
 	} else {
 		flags_ &= ~pf_can_focus;
 
-		if (parent_ && parent_->focus_ == this) {
+		if ((parent_ != nullptr) && parent_->focus_ == this) {
 			parent_->focus_ = nullptr;
 		}
 	}
 }
 
+void Panel::disable_sdl_textinput() {
+	if (SDL_IsTextInputActive() != 0u) {
+		SDL_StopTextInput();
+	}
+}
 /**
  * Grabs the keyboard focus, if it can,
  * topcaller identifies widget at the beginning of the recursion
@@ -841,18 +1040,16 @@ void Panel::set_can_focus(bool const yes) {
 void Panel::focus(const bool topcaller) {
 	if (topcaller) {
 		if (handles_textinput()) {
-			if (!SDL_IsTextInputActive()) {
+			if (SDL_IsTextInputActive() == 0u) {
 				SDL_StartTextInput();
 			}
 		} else {
-			if (SDL_IsTextInputActive()) {
-				SDL_StopTextInput();
-			}
+			disable_sdl_textinput();
 		}
 		focus_ = nullptr;
 	}
 
-	if (!parent_ || this == modal_) {
+	if ((parent_ == nullptr) || this == modal_) {
 		return;
 	}
 
@@ -881,9 +1078,12 @@ void Panel::set_thinks(bool const yes) {
  * Do NOT use this to delete a hierarchy of panels that have been modal.
  */
 void Panel::die() {
+	initialized_ = false;
+
+	flags_ &= ~pf_visible;
 	flags_ |= pf_die;
 
-	for (Panel* p = parent_; p; p = p->parent_) {
+	for (Panel* p = parent_; p != nullptr; p = p->parent_) {
 		p->flags_ |= pf_child_die;
 		if (p == modal_) {
 			break;
@@ -894,7 +1094,7 @@ void Panel::die() {
  * Called on a child's parent just before child is deleted.
  * Overridden in UI::Box
  */
-void Panel::on_death(Panel*) {
+void Panel::on_death(Panel* /* p */) {
 }
 
 /**
@@ -906,16 +1106,16 @@ void Panel::play_click() {
 }
 
 /**
- * This needs to be called once after g_soundhandler has been instantiated and before play_click()
- * is called. We do it this way so that we don't have to register the same sound every time we
- * create a new panel.
+ * This needs to be called once after g_soundhandler has been instantiated and before
+ * play_click() is called. We do it this way so that we don't have to register the same sound
+ * every time we create a new panel.
  */
 void Panel::register_click() {
 	click_fx_ = SoundHandler::register_fx(SoundType::kUI, "sound/click");
 }
 
 void Panel::do_delete() {
-	if (parent_) {
+	if (parent_ != nullptr) {
 		parent_->on_death(this);
 	}
 	delete this;
@@ -927,14 +1127,14 @@ void Panel::do_delete() {
  */
 void Panel::check_child_death() {
 	Panel* next = first_child_;
-	while (next) {
+	while (next != nullptr) {
 		Panel* p = next;
 		next = p->next_;
 
-		if (p->flags_ & pf_die) {
+		if ((p->flags_ & pf_die) != 0u) {
 			p->do_delete();
 			p = nullptr;
-		} else if (p->flags_ & pf_child_die) {
+		} else if ((p->flags_ & pf_child_die) != 0u) {
 			p->check_child_death();
 		}
 	}
@@ -952,7 +1152,7 @@ void Panel::do_draw_inner(RenderTarget& dst) {
 	draw(dst);
 
 	// draw back to front
-	for (Panel* child = last_child_; child; child = child->prev_) {
+	for (Panel* child = last_child_; child != nullptr; child = child->prev_) {
 		child->do_draw(dst);
 	}
 
@@ -968,14 +1168,21 @@ void Panel::do_draw_inner(RenderTarget& dst) {
  * \param dst RenderTarget for the parent Panel
  */
 void Panel::do_draw(RenderTarget& dst) {
+#ifdef MUTEX_LOCK_DEBUG
+	if (!initialized_) {
+		dst.fill_rect(Recti(x_, y_, w_, h_), RGBAColor(100, 100, 200, 100), BlendMode::Default);
+		return;
+	}
+#endif
+
+	if (!initialized_ || !is_visible()) {
+		return;
+	}
+
 	// Make sure the panel's size is sane. If it's bigger than 10000 it's likely a bug.
 	assert(desired_w_ <= std::max(10000, g_gr->get_xres()));
 	assert(desired_h_ <= std::max(10000, g_gr->get_yres()));
 
-	if (!is_visible()) {
-		return;
-	}
-
 	Recti outerrc;
 	Vector2i outerofs = Vector2i::zero();
 
@@ -1016,7 +1223,7 @@ void Panel::set_tooltip(const std::string& text) {
 void Panel::find_all_children_at(const int16_t x,
                                  const int16_t y,
                                  std::vector<Panel*>& result) const {
-	for (Panel* child = first_child_; child; child = child->next_) {
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
 		if (child->get_x() <= x && child->get_y() <= y && child->get_x() + child->get_w() > x &&
 		    child->get_y() + child->get_h() > y) {
 			result.push_back(child);
@@ -1032,7 +1239,7 @@ void Panel::find_all_children_at(const int16_t x,
  */
 inline Panel* Panel::child_at_mouse_cursor(int32_t const x, int32_t const y, Panel* child) {
 
-	for (; child; child = child->next_) {
+	for (; child != nullptr; child = child->next_) {
 		if (!child->handles_mouse() || !child->is_visible()) {
 			continue;
 		}
@@ -1042,11 +1249,11 @@ inline Panel* Panel::child_at_mouse_cursor(int32_t const x, int32_t const y, Pan
 		}
 	}
 
-	if (mousein_child_ && mousein_child_ != child) {
+	if ((mousein_child_ != nullptr) && mousein_child_ != child) {
 		mousein_child_->do_mousein(false);
 	}
 	mousein_child_ = child;
-	if (child) {
+	if (child != nullptr) {
 		child->do_mousein(true);
 	}
 
@@ -1058,14 +1265,19 @@ inline Panel* Panel::child_at_mouse_cursor(int32_t const x, int32_t const y, Pan
  * window)
  */
 void Panel::do_mousein(bool const inside) {
-	if (!inside && mousein_child_) {
+	if (!initialized_) {
+		return;
+	}
+
+	if (!inside && (mousein_child_ != nullptr)) {
 		mousein_child_->do_mousein(false);
 		mousein_child_ = nullptr;
 	}
 
 	if (tooltip_accessibility_mode()) {
 		if (inside && tooltip_panel_ != this &&
-		    (!tooltip_panel_ || tooltip_fixed_pos_ == Vector2i::invalid()) && !tooltip().empty()) {
+		    ((tooltip_panel_ == nullptr) || tooltip_fixed_pos_ == Vector2i::invalid()) &&
+		    !tooltip().empty()) {
 			tooltip_panel_ = this;
 			tooltip_fixed_pos_ = WLApplication::get()->get_mouse_position();
 		} else if (!inside && tooltip_panel_ == this && tooltip_fixed_pos_ == Vector2i::invalid()) {
@@ -1082,17 +1294,21 @@ void Panel::do_mousein(bool const inside) {
  * Returns whether the event was processed.
  */
 bool Panel::do_mousepress(const uint8_t btn, int32_t x, int32_t y) {
+	if (!initialized_) {
+		return false;
+	}
+
 	if (get_can_focus()) {
 		focus();
 	}
 	x -= lborder_;
 	y -= tborder_;
-	if (flags_ & pf_top_on_click) {
+	if ((flags_ & pf_top_on_click) != 0u) {
 		move_to_top();
 	}
 
 	if (mousegrab_ != this) {
-		for (Panel* child = first_child_; (child = child_at_mouse_cursor(x, y, child));
+		for (Panel* child = first_child_; (child = child_at_mouse_cursor(x, y, child)) != nullptr;
 		     child = child->next_) {
 			if (child->do_mousepress(btn, x - child->x_, y - child->y_)) {
 				return true;
@@ -1102,9 +1318,13 @@ bool Panel::do_mousepress(const uint8_t btn, int32_t x, int32_t y) {
 	return handle_mousepress(btn, x, y);
 }
 
-bool Panel::do_mousewheel(uint32_t which, int32_t x, int32_t y, Vector2i rel_mouse_pos) {
+bool Panel::do_mousewheel(int32_t x, int32_t y, uint16_t modstate, Vector2i rel_mouse_pos) {
+	if (!initialized_) {
+		return false;
+	}
+
 	// Check if a child-panel is beneath the mouse and processes the event
-	for (Panel* child = first_child_; child; child = child->next_) {
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
 		if (!child->handles_mouse() || !child->is_visible()) {
 			continue;
 		}
@@ -1115,20 +1335,24 @@ bool Panel::do_mousewheel(uint32_t which, int32_t x, int32_t y, Vector2i rel_mou
 			continue;
 		}
 		// Found a child at the position
-		if (child->do_mousewheel(which, x, y,
+		if (child->do_mousewheel(x, y, modstate,
 		                         rel_mouse_pos - Vector2i(child->get_x() + child->get_lborder(),
 		                                                  child->get_y() + child->get_tborder()))) {
 			return true;
 		}
 	}
-	return handle_mousewheel(which, x, y);
+	return handle_mousewheel(x, y, modstate);
 }
 
 bool Panel::do_mouserelease(const uint8_t btn, int32_t x, int32_t y) {
+	if (!initialized_) {
+		return false;
+	}
+
 	x -= lborder_;
 	y -= tborder_;
 	if (mousegrab_ != this) {
-		for (Panel* child = first_child_; (child = child_at_mouse_cursor(x, y, child));
+		for (Panel* child = first_child_; (child = child_at_mouse_cursor(x, y, child)) != nullptr;
 		     child = child->next_) {
 			if (child->do_mouserelease(btn, x - child->x_, y - child->y_)) {
 				return true;
@@ -1140,10 +1364,14 @@ bool Panel::do_mouserelease(const uint8_t btn, int32_t x, int32_t y) {
 
 bool Panel::do_mousemove(
    uint8_t const state, int32_t x, int32_t y, int32_t const xdiff, int32_t const ydiff) {
+	if (!initialized_) {
+		return false;
+	}
+
 	x -= lborder_;
 	y -= tborder_;
 	if (mousegrab_ != this) {
-		for (Panel* child = first_child_; (child = child_at_mouse_cursor(x, y, child));
+		for (Panel* child = first_child_; (child = child_at_mouse_cursor(x, y, child)) != nullptr;
 		     child = child->next_) {
 			if (child->do_mousemove(state, x - child->x_, y - child->y_, xdiff, ydiff)) {
 				return true;
@@ -1158,7 +1386,11 @@ bool Panel::do_mousemove(
  * If it doesn't process the key, we'll see if we can use the event.
  */
 bool Panel::do_key(bool const down, SDL_Keysym const code) {
-	if (focus_ && focus_->do_key(down, code)) {
+	if (!initialized_) {
+		return false;
+	}
+
+	if ((focus_ != nullptr) && focus_->do_key(down, code)) {
 		return true;
 	}
 
@@ -1196,14 +1428,18 @@ bool Panel::do_key(bool const down, SDL_Keysym const code) {
 		case SDLK_LALT:
 			return false;
 		}
-		return !(code.mod & KMOD_CTRL || (code.sym >= SDLK_F1 && code.sym <= SDLK_F12));
+		return !(((code.mod & KMOD_CTRL) != 0) || (code.sym >= SDLK_F1 && code.sym <= SDLK_F12));
 	}
 
 	return false;
 }
 
 bool Panel::do_textinput(const std::string& text) {
-	if (focus_ && focus_->do_textinput(text)) {
+	if (!initialized_) {
+		return false;
+	}
+
+	if ((focus_ != nullptr) && focus_->do_textinput(text)) {
 		return true;
 	}
 
@@ -1215,12 +1451,16 @@ bool Panel::do_textinput(const std::string& text) {
 }
 
 bool Panel::do_tooltip() {
-	if (mousein_child_ && mousein_child_->do_tooltip()) {
+	if (!initialized_) {
+		return false;
+	}
+
+	if ((mousein_child_ != nullptr) && mousein_child_->do_tooltip()) {
 		return true;
 	}
 
 	if (tooltip_accessibility_mode()) {
-		if (tooltip_panel_ && tooltip_fixed_pos_ != Vector2i::invalid()) {
+		if ((tooltip_panel_ != nullptr) && tooltip_fixed_pos_ != Vector2i::invalid()) {
 			draw_tooltip(tooltip_panel_->tooltip(), tooltip_panel_->panel_style_, tooltip_fixed_pos_);
 		}
 		return true;
@@ -1237,7 +1477,7 @@ bool Panel::get_key_state(const SDL_Scancode key) const {
 }
 
 UI::Panel* Panel::get_open_dropdown() {
-	for (Panel* child = first_child_; child; child = child->next_) {
+	for (Panel* child = first_child_; child != nullptr; child = child->next_) {
 		if (UI::Panel* dd = child->get_open_dropdown()) {
 			return dd;
 		}
@@ -1254,14 +1494,17 @@ Panel* Panel::ui_trackmouse(int32_t& x, int32_t& y) {
 	Panel* mousein;
 	Panel* rcv = nullptr;
 
-	if (mousegrab_) {
+	if (mousegrab_ != nullptr) {
 		mousein = rcv = mousegrab_;
 	} else {
 		mousein = modal_;
 	}
+	if (mousein == nullptr) {
+		return nullptr;
+	}
 
 	// ugly hack to handle dropdowns in modal windows correctly
-	if (mousein->get_parent()) {
+	if (mousein->get_parent() != nullptr) {
 		if (UI::Panel* dd = mousein->get_open_dropdown()) {
 			mousein = rcv = dd;
 		}
@@ -1269,7 +1512,7 @@ Panel* Panel::ui_trackmouse(int32_t& x, int32_t& y) {
 
 	x -= mousein->x_;
 	y -= mousein->y_;
-	for (Panel* p = mousein->parent_; p; p = p->parent_) {
+	for (Panel* p = mousein->parent_; p != nullptr; p = p->parent_) {
 		x -= p->lborder_ + p->x_;
 		y -= p->tborder_ + p->y_;
 	}
@@ -1282,11 +1525,11 @@ Panel* Panel::ui_trackmouse(int32_t& x, int32_t& y) {
 	}
 
 	if (mousein != mousein_) {
-		if (mousein_) {
+		if (mousein_ != nullptr) {
 			mousein_->do_mousein(false);
 		}
 		mousein_ = mousein;
-		if (mousein_) {
+		if (mousein_ != nullptr) {
 			mousein_->do_mousein(true);
 		}
 	}
@@ -1303,7 +1546,7 @@ bool Panel::ui_mousepress(const uint8_t button, int32_t x, int32_t y) {
 		return true;
 	}
 
-	if (tooltip_panel_ && tooltip_fixed_pos_ != Vector2i::invalid()) {
+	if ((tooltip_panel_ != nullptr) && tooltip_fixed_pos_ != Vector2i::invalid()) {
 		const bool inside = tooltip_fixed_rect_.x < x && tooltip_fixed_rect_.y < y &&
 		                    tooltip_fixed_rect_.x + tooltip_fixed_rect_.w > x &&
 		                    tooltip_fixed_rect_.y + tooltip_fixed_rect_.h > y;
@@ -1336,13 +1579,12 @@ bool Panel::ui_mouserelease(const uint8_t button, int32_t x, int32_t y) {
  * Input callback function. Pass the mousemove event to the currently modal
  * panel.
  */
-bool Panel::ui_mousemove(
-   uint8_t const state, int32_t x, int32_t y, int32_t const xdiff, int32_t const ydiff) {
+bool Panel::ui_mousemove(uint8_t const state, int32_t x, int32_t y, int32_t xdiff, int32_t ydiff) {
 	if (!allow_user_input_) {
 		return true;
 	}
 
-	if (!xdiff && !ydiff) {
+	if ((xdiff == 0) && (ydiff == 0)) {
 		return true;
 	}
 
@@ -1351,30 +1593,39 @@ bool Panel::ui_mousemove(
 		return false;
 	}
 
-	return p->do_mousemove(state, x, y, xdiff, ydiff);
+	int factor = 1;
+	if (WLApplication::get()->is_mouse_locked()) {
+		if (matches_keymod(SDL_GetModState(), KMOD_CTRL)) {
+			factor = 4;
+		} else if (!matches_keymod(SDL_GetModState(), KMOD_SHIFT)) {
+			factor = 2;
+		}
+	}
+
+	return p->do_mousemove(state, x, y, xdiff * factor, ydiff * factor);
 }
 
 /**
  * Input callback function. Pass the mousewheel event to the currently modal
  * panel.
  */
-bool Panel::ui_mousewheel(uint32_t which, int32_t x, int32_t y) {
+bool Panel::ui_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
 	if (!allow_user_input_) {
 		return true;
 	}
-	if (!x && !y) {
+	if ((x == 0) && (y == 0)) {
 		return true;
 	}
 	Panel* p = nullptr;
-	if (mousein_) {
+	if (mousein_ != nullptr) {
 		p = mousein_;
 	} else {
-		p = mousegrab_ ? mousegrab_ : modal_;
+		p = mousegrab_ != nullptr ? mousegrab_ : modal_;
 	}
-	if (!p) {
+	if (p == nullptr) {
 		return false;
 	}
-	return p->do_mousewheel(which, x, y, p->get_mouse_position());
+	return p->do_mousewheel(x, y, modstate, p->get_mouse_position());
 }
 
 /**
@@ -1385,7 +1636,10 @@ bool Panel::ui_key(bool const down, SDL_Keysym const code) {
 		return true;
 	}
 	Panel* p = modal_;
-	if (p->get_parent()) {
+	if (p == nullptr) {
+		return false;
+	}
+	if (p->get_parent() != nullptr) {
 		if (UI::Panel* dd = p->get_open_dropdown()) {
 			p = dd;
 		}
@@ -1400,7 +1654,10 @@ bool Panel::ui_textinput(const std::string& text) {
 	if (!allow_user_input_) {
 		return true;
 	}
-	return modal_->do_textinput(text);
+	if (modal_ == nullptr) {
+		return false;
+	}
+	return modal_.load()->do_textinput(text);
 }
 
 /**
@@ -1416,8 +1673,8 @@ bool Panel::draw_tooltip(const std::string& text, const PanelStyle style, Vector
 	std::string text_to_render = text;
 	if (!is_richtext(text_to_render)) {
 		text_to_render = as_richtext_paragraph(text_to_render, style == PanelStyle::kWui ?
-		                                                          UI::FontStyle::kWuiTooltip :
-		                                                          UI::FontStyle::kFsTooltip);
+                                                                UI::FontStyle::kWuiTooltip :
+                                                                UI::FontStyle::kFsTooltip);
 	}
 
 	constexpr int kTipWidthMax = 360;
