@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2021 by the Widelands Development Team
+ * Copyright (C) 2004-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,6 +31,7 @@
 #include "network/internet_gaming_protocol.h"
 #include "sound/sound_handler.h"
 #include "ui_basic/messagebox.h"
+#include "ui_fsmenu/main.h"
 #include "wlapplication_options.h"
 
 namespace {
@@ -153,16 +153,15 @@ InternetLobby::InternetLobby(MenuCapsule& fsmm,
 	});
 
 	// Prepare the lists
-	const std::string t_tip =
-	   (boost::format("<rt padding=2><p align=center spacing=3>%s</p>"
-	                  "<p valign=bottom><img src=images/wui/overlays/road_building_green.png> %s"
-	                  "<br><img src=images/wui/overlays/road_building_yellow.png> %s"
-	                  "<br><img src=images/wui/overlays/road_building_red.png> %s</p></rt>") %
-	    g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader).as_font_tag(_("User Status")) %
-	    g_style_manager->font_style(UI::FontStyle::kFsTooltip).as_font_tag(_("Administrator")) %
-	    g_style_manager->font_style(UI::FontStyle::kFsTooltip).as_font_tag(_("Registered")) %
-	    g_style_manager->font_style(UI::FontStyle::kFsTooltip).as_font_tag(_("Unregistered")))
-	      .str();
+	const std::string t_tip = format(
+	   "<rt padding=2><p align=center spacing=3>%s</p>"
+	   "<p valign=bottom><img src=images/wui/overlays/road_building_green.png> %s"
+	   "<br><img src=images/wui/overlays/road_building_yellow.png> %s"
+	   "<br><img src=images/wui/overlays/road_building_red.png> %s</p></rt>",
+	   g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader).as_font_tag(_("User Status")),
+	   g_style_manager->font_style(UI::FontStyle::kFsTooltip).as_font_tag(_("Administrator")),
+	   g_style_manager->font_style(UI::FontStyle::kFsTooltip).as_font_tag(_("Registered")),
+	   g_style_manager->font_style(UI::FontStyle::kFsTooltip).as_font_tag(_("Unregistered")));
 	clientsonline_table_.add_column(22, "*", t_tip);
 	/** TRANSLATORS: Player Name */
 	clientsonline_table_.add_column(180, pgettext("player", "Name"));
@@ -174,8 +173,8 @@ InternetLobby::InternetLobby(MenuCapsule& fsmm,
 	   0, [this](uint32_t a, uint32_t b) { return compare_clienttype(a, b); });
 	clientsonline_table_.double_clicked.connect(
 	   [this](uint32_t a) { return client_doubleclicked(a); });
-	opengames_list_.selected.connect([this](uint32_t) { server_selected(); });
-	opengames_list_.double_clicked.connect([this](uint32_t) { server_doubleclicked(); });
+	opengames_list_.selected.connect([this](uint32_t /* value */) { server_selected(); });
+	opengames_list_.double_clicked.connect([this](uint32_t /* value */) { server_doubleclicked(); });
 
 	// try to connect to the metaserver
 	if (!InternetGaming::ref().error() && !InternetGaming::ref().logged_in()) {
@@ -185,6 +184,8 @@ InternetLobby::InternetLobby(MenuCapsule& fsmm,
 	layout();
 	// set focus to chat input
 	chat_.focus_edit();
+
+	initialization_complete();
 }
 
 InternetLobby::~InternetLobby() {
@@ -413,10 +414,8 @@ void InternetLobby::change_servername() {
 				hostgame_.set_enabled(false);
 				servername_.set_warning(true);
 				servername_.set_tooltip(
-				   (boost::format(
-				       _("The game %s is already running. Please choose a different name.")) %
-				    g_style_manager->font_style(UI::FontStyle::kWarning).as_font_tag(game.name))
-				      .str());
+				   format(_("The game %s is already running. Please choose a different name."),
+				          g_style_manager->font_style(UI::FontStyle::kWarning).as_font_tag(game.name)));
 			}
 		}
 	}
@@ -450,9 +449,17 @@ void InternetLobby::clicked_joingame() {
 		}
 		const std::pair<NetAddress, NetAddress>& ips = InternetGaming::ref().ips();
 
-		running_game_.reset(new GameClient(capsule_, running_game_, ips,
-		                                   InternetGaming::ref().get_local_clientname(), true,
-		                                   opengames_list_.get_selected().name));
+		try {
+			running_game_.reset(new GameClient(capsule_, running_game_, ips,
+			                                   InternetGaming::ref().get_local_clientname(), true,
+			                                   opengames_list_.get_selected().name));
+		} catch (const std::exception& e) {
+			running_game_.reset();
+			UI::WLMessageBox mbox(&capsule_.menu(), UI::WindowStyle::kFsMenu, _("Network Error"),
+			                      e.what(), UI::WLMessageBox::MBoxType::kOk);
+			mbox.run<UI::Panel::Returncodes>();
+			return;
+		}
 	} else {
 		throw wexception("No server selected! That should not happen!");
 	}
@@ -472,7 +479,7 @@ void InternetLobby::clicked_hostgame() {
 				do {
 					/** TRANSLATORS: This is shown for multiplayer games when no host */
 					/** TRANSLATORS: server to connect to has been specified yet. */
-					servername_ui = (boost::format(_("unnamed %u")) % i++).str();
+					servername_ui = format(_("unnamed %u"), i++);
 				} while (servername_ui == game.name);
 			} else if (game.name == servername_ui) {
 				change_servername();
@@ -501,7 +508,16 @@ void InternetLobby::clicked_hostgame() {
 	}
 
 	// Start our relay host
-	running_game_.reset(new GameHost(
-	   capsule_, running_game_, InternetGaming::ref().get_local_clientname(), tribeinfos_, true));
+	try {
+		running_game_.reset(new GameHost(&capsule_, running_game_,
+		                                 InternetGaming::ref().get_local_clientname(), tribeinfos_,
+		                                 true));
+	} catch (const std::exception& e) {
+		running_game_.reset();
+		UI::WLMessageBox mbox(&capsule_.menu(), UI::WindowStyle::kFsMenu, _("Network Error"),
+		                      e.what(), UI::WLMessageBox::MBoxType::kOk);
+		mbox.run<UI::Panel::Returncodes>();
+		return;
+	}
 }
 }  // namespace FsMenu
