@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -41,7 +40,7 @@
 #include "sound/sound_handler.h"
 #include "ui_basic/messagebox.h"
 #include "ui_fsmenu/about.h"
-#include "ui_fsmenu/addons.h"
+#include "ui_fsmenu/addons/manager.h"
 #include "ui_fsmenu/campaign_select.h"
 #include "ui_fsmenu/internet_lobby.h"
 #include "ui_fsmenu/loadgame.h"
@@ -71,6 +70,8 @@ int16_t MainMenu::calc_desired_window_width(const UI::Window::WindowLayoutID id)
 	case UI::Window::WindowLayoutID::kFsMenuOptions:
 	case UI::Window::WindowLayoutID::kFsMenuAbout:
 		return std::max(600, get_w() / 2);
+	case UI::Window::WindowLayoutID::kFsMenuKeyboardOptions:
+		return std::max(720, get_w() * 3 / 4);
 	default:
 		NEVER_HERE();
 	}
@@ -84,6 +85,8 @@ int16_t MainMenu::calc_desired_window_height(const UI::Window::WindowLayoutID id
 		return std::max(500, get_h() * 4 / 5);
 	case UI::Window::WindowLayoutID::kFsMenuOptions:
 		return std::max(450, get_h() / 2);
+	case UI::Window::WindowLayoutID::kFsMenuKeyboardOptions:
+		return std::max(360, get_h() * 3 / 8);
 	default:
 		NEVER_HERE();
 	}
@@ -115,7 +118,8 @@ MainMenu::MainMenu(const bool skip_init)
                    "",
                    UI::DropdownType::kTextualMenu,
                    UI::PanelStyle::kFsMenu,
-                   UI::ButtonStyle::kFsMenuMenu),
+                   UI::ButtonStyle::kFsMenuMenu,
+                   [this](MenuTarget t) { action(t); }),
      multiplayer_(&vbox1_,
                   "multiplayer",
                   0,
@@ -126,7 +130,8 @@ MainMenu::MainMenu(const bool skip_init)
                   "",
                   UI::DropdownType::kTextualMenu,
                   UI::PanelStyle::kFsMenu,
-                  UI::ButtonStyle::kFsMenuMenu),
+                  UI::ButtonStyle::kFsMenuMenu,
+                  [this](MenuTarget t) { action(t); }),
      replay_(&vbox1_, "replay", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
      editor_(&vbox1_,
              "editor",
@@ -138,7 +143,8 @@ MainMenu::MainMenu(const bool skip_init)
              "",
              UI::DropdownType::kTextualMenu,
              UI::PanelStyle::kFsMenu,
-             UI::ButtonStyle::kFsMenuMenu),
+             UI::ButtonStyle::kFsMenuMenu,
+             [this](MenuTarget t) { action(t); }),
      addons_(&vbox2_, "addons", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
      options_(&vbox2_, "options", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
      about_(&vbox2_, "about", 0, 0, butw_, buth_, UI::ButtonStyle::kFsMenuMenu, ""),
@@ -209,11 +215,29 @@ MainMenu::MainMenu(const bool skip_init)
 
 	r_login_.open_window = [this]() { new LoginBox(*this, r_login_); };
 	r_about_.open_window = [this]() { new About(*this, r_about_); };
-	r_addons_.open_window = [this]() { new AddOnsCtrl(*this, r_about_); };
+	r_addons_.open_window = [this]() { new AddOnsUI::AddOnsCtrl(*this, r_about_); };
 
 	focus();
 	set_labels();
 	layout();
+
+	initialization_complete();
+}
+
+Widelands::Game* MainMenu::create_safe_game(const bool show_error) {
+	try {
+		return new Widelands::Game;
+	} catch (const std::exception& e) {
+		log_err("Error allocating game: %s", e.what());
+		if (show_error) {
+			UI::WLMessageBox m(
+			   this, UI::WindowStyle::kFsMenu, _("Error"),
+			   format(_("Unable to create a Game instance!\nError message:\n%s"), e.what()),
+			   UI::WLMessageBox::MBoxType::kOk);
+			m.run<UI::Panel::Returncodes>();
+		}
+	}
+	return nullptr;
 }
 
 void MainMenu::update_template() {
@@ -228,10 +252,10 @@ void MainMenu::update_template() {
 	}
 	if (images_.empty()) {
 		log_warn("No main menu backgrounds found, using fallback image");
-		images_.push_back("images/logos/wl-ico-128.png");
+		images_.emplace_back("images/logos/wl-ico-128.png");
 	}
 
-	last_image_ = draw_image_ = std::rand() % images_.size();  // NOLINT
+	last_image_ = draw_image_ = RNG::static_rand(images_.size());
 	last_image_exchange_time_ = 0;
 }
 
@@ -264,10 +288,10 @@ void MainMenu::find_maps(const std::string& directory, std::vector<MapEntry>& re
 				ml->preload_map(true, nullptr);
 				if (map.version().map_version_timestamp > 0) {
 					MapData::MapType type = map.scenario_types() == Map::SP_SCENARIO ?
-					                           MapData::MapType::kScenario :
-					                           MapData::MapType::kNormal;
-					results.push_back(MapEntry(
-					   MapData(map, file, type, MapData::DisplayType::kFilenames), map.version()));
+                                          MapData::MapType::kScenario :
+                                          MapData::MapType::kNormal;
+					results.emplace_back(
+					   MapData(map, file, type, MapData::DisplayType::kFilenames), map.version());
 				}
 			} catch (...) {
 				// invalid file – silently ignore
@@ -294,88 +318,86 @@ void MainMenu::set_labels() {
 	editor_.clear();
 
 	singleplayer_.add(_("New Game"), MenuTarget::kNewGame, nullptr, false, _("Begin a new game"),
-	                  shortcut_string_for(KeyboardShortcut::kMainMenuNew));
+	                  shortcut_string_for(KeyboardShortcut::kMainMenuNew, false));
 	singleplayer_.add(_("New Random Game"), MenuTarget::kRandomGame, nullptr, false,
 	                  _("Create a new random match"),
-	                  shortcut_string_for(KeyboardShortcut::kMainMenuRandomMatch));
+	                  shortcut_string_for(KeyboardShortcut::kMainMenuRandomMatch, false));
 	singleplayer_.add(_("Campaigns"), MenuTarget::kCampaign, nullptr, false, _("Play a campaign"),
-	                  shortcut_string_for(KeyboardShortcut::kMainMenuCampaign));
+	                  shortcut_string_for(KeyboardShortcut::kMainMenuCampaign, false));
 	singleplayer_.add(_("Tutorials"), MenuTarget::kTutorial, nullptr, false,
 	                  _("Play one of our beginners’ tutorials"),
-	                  shortcut_string_for(KeyboardShortcut::kMainMenuTutorial));
+	                  shortcut_string_for(KeyboardShortcut::kMainMenuTutorial, false));
 	singleplayer_.add(_("Load Game"), MenuTarget::kLoadGame, nullptr, false,
 	                  _("Continue a saved game"),
-	                  shortcut_string_for(KeyboardShortcut::kMainMenuLoad));
+	                  shortcut_string_for(KeyboardShortcut::kMainMenuLoad, false));
 
 	// Refresh the Continue tooltip. The SavegameData must be reloaded after
 	// every language switch because it contains localized strings.
 	{
 		filename_for_continue_playing_ = "";
-		Widelands::Game game;
-		SinglePlayerLoader loader(game);
-		std::vector<SavegameData> games = loader.load_files(kSaveDir);
-		SavegameData* newest_singleplayer = nullptr;
-		for (SavegameData& data : games) {
-			if (!data.is_directory() && data.is_singleplayer() &&
-			    (newest_singleplayer == nullptr || newest_singleplayer->compare_save_time(data))) {
-				newest_singleplayer = &data;
+		std::unique_ptr<Widelands::Game> game(create_safe_game(false));
+		if (game != nullptr) {
+			SinglePlayerLoader loader(*game);
+			std::vector<SavegameData> games = loader.load_files(kSaveDir);
+			SavegameData* newest_singleplayer = nullptr;
+			for (SavegameData& data : games) {
+				if (!data.is_directory() && data.is_singleplayer() &&
+				    (newest_singleplayer == nullptr || newest_singleplayer->compare_save_time(data))) {
+					newest_singleplayer = &data;
+				}
+			}
+			if (newest_singleplayer != nullptr) {
+				filename_for_continue_playing_ = newest_singleplayer->filename;
+				singleplayer_.add(
+				   _("Continue Playing"), MenuTarget::kContinueLastsave, nullptr, false,
+				   format("%s<br>%s<br>%s<br>%s<br>%s<br>%s",
+				          g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader)
+				             .as_font_tag(
+				                /* strip leading "save/" and trailing ".wgf" */
+				                filename_for_continue_playing_.substr(
+				                   kSaveDir.length() + 1, filename_for_continue_playing_.length() -
+				                                             kSaveDir.length() -
+				                                             kSavegameExtension.length() - 1)),
+				          format(_("Map: %s"),
+				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+				                    .as_font_tag(newest_singleplayer->mapname)),
+				          format(_("Win Condition: %s"),
+				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+				                    .as_font_tag(newest_singleplayer->wincondition)),
+				          format(_("Players: %s"),
+				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+				                    .as_font_tag(newest_singleplayer->nrplayers)),
+				          format(_("Gametime: %s"),
+				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+				                    .as_font_tag(newest_singleplayer->gametime)),
+				          /** TRANSLATORS: Information about when a game was saved, e.g. 'Saved: Today,
+				           * 10:30'
+				           */
+				          format(_("Saved: %s"),
+				                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+				                    .as_font_tag(newest_singleplayer->savedatestring))),
+				   shortcut_string_for(KeyboardShortcut::kMainMenuContinuePlaying, false));
 			}
-		}
-		if (newest_singleplayer) {
-			filename_for_continue_playing_ = newest_singleplayer->filename;
-			singleplayer_.add(
-			   _("Continue Playing"), MenuTarget::kContinueLastsave, nullptr, false,
-			   (boost::format("%s<br>%s<br>%s<br>%s<br>%s<br>%s") %
-			    g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader)
-			       .as_font_tag(
-			          /* strip leading "save/" and trailing ".wgf" */
-			          filename_for_continue_playing_.substr(
-			             kSaveDir.length() + 1, filename_for_continue_playing_.length() -
-			                                       kSaveDir.length() - kSavegameExtension.length() -
-			                                       1)) %
-			    (boost::format(_("Map: %s")) %
-			     g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			        .as_font_tag(newest_singleplayer->mapname))
-			       .str() %
-			    (boost::format(_("Win Condition: %s")) %
-			     g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			        .as_font_tag(newest_singleplayer->wincondition))
-			       .str() %
-			    (boost::format(_("Players: %s")) %
-			     g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			        .as_font_tag(newest_singleplayer->nrplayers))
-			       .str() %
-			    (boost::format(_("Gametime: %s")) %
-			     g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			        .as_font_tag(newest_singleplayer->gametime))
-			       .str() %
-			    /** TRANSLATORS: Information about when a game was saved, e.g. 'Saved: Today, 10:30' */
-			    (boost::format(_("Saved: %s")) %
-			     g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			        .as_font_tag(newest_singleplayer->savedatestring))
-			       .str())
-			      .str(),
-			   shortcut_string_for(KeyboardShortcut::kMainMenuContinuePlaying));
 		}
 	}
 
 	multiplayer_.add(_("Online Game"), MenuTarget::kMetaserver, nullptr, false,
 	                 _("Join the Widelands lobby"),
-	                 shortcut_string_for(KeyboardShortcut::kMainMenuLobby));
+	                 shortcut_string_for(KeyboardShortcut::kMainMenuLobby, false));
 	multiplayer_.add(_("Online Game Settings"), MenuTarget::kOnlineGameSettings, nullptr, false,
 	                 _("Log in as a registered user"),
-	                 shortcut_string_for(KeyboardShortcut::kMainMenuLogin));
+	                 shortcut_string_for(KeyboardShortcut::kMainMenuLogin, false));
 	multiplayer_.add(_("LAN / Direct IP"), MenuTarget::kLan, nullptr, false,
 	                 _("Play a private online game"),
-	                 shortcut_string_for(KeyboardShortcut::kMainMenuLAN));
+	                 shortcut_string_for(KeyboardShortcut::kMainMenuLAN, false));
 
 	editor_.add(_("New Map"), MenuTarget::kEditorNew, nullptr, false, _("Create a new empty map"),
-	            shortcut_string_for(KeyboardShortcut::kMainMenuEditorNew));
+	            shortcut_string_for(KeyboardShortcut::kMainMenuEditorNew, false));
 	editor_.add(_("Random Map"), MenuTarget::kEditorRandom, nullptr, false,
 	            _("Create a new random map"),
-	            shortcut_string_for(KeyboardShortcut::kMainMenuEditorRandom));
+	            shortcut_string_for(KeyboardShortcut::kMainMenuEditorRandom, false));
 	editor_.add(_("Load Map"), MenuTarget::kEditorLoad, nullptr, false, _("Edit an existing map"),
-	            shortcut_string_for(KeyboardShortcut::kMainMenuEditorLoad));
+	            shortcut_string_for(KeyboardShortcut::kMainMenuEditorLoad, false));
 
 	{
 		filename_for_continue_editing_ = "";
@@ -388,35 +410,30 @@ void MainMenu::set_labels() {
 				last_edited = &m;
 			}
 		}
-		if (last_edited) {
+		if (last_edited != nullptr) {
 			filename_for_continue_editing_ = last_edited->first.filename;
-			editor_.add(_("Continue Editing"), MenuTarget::kEditorContinue, nullptr, false,
-			            (boost::format("%s<br>%s<br>%s<br>%s<br>%s") %
-			             g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader)
-			                .as_font_tag(
-			                   /* strip leading "maps/My_Maps/" and trailing ".wgf" */
-			                   filename_for_continue_editing_.substr(
-			                      13, filename_for_continue_editing_.length() - 17)) %
-			             (boost::format(_("Name: %s")) %
-			              g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                 .as_font_tag(last_edited->first.localized_name))
-			                .str() %
-			             (boost::format(_("Size: %s")) %
-			              g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                 .as_font_tag((boost::format(_("%1$u×%2$u")) % last_edited->first.width %
-			                               last_edited->first.height)
-			                                 .str()))
-			                .str() %
-			             (boost::format(_("Players: %s")) %
-			              g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                 .as_font_tag(std::to_string(last_edited->first.nrplayers)))
-			                .str() %
-			             (boost::format(_("Description: %s")) %
-			              g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
-			                 .as_font_tag(last_edited->first.description))
-			                .str())
-			               .str(),
-			            shortcut_string_for(KeyboardShortcut::kMainMenuContinueEditing));
+			editor_.add(
+			   _("Continue Editing"), MenuTarget::kEditorContinue, nullptr, false,
+			   format("%s<br>%s<br>%s<br>%s<br>%s",
+			          g_style_manager->font_style(UI::FontStyle::kFsTooltipHeader)
+			             .as_font_tag(
+			                /* strip leading "maps/My_Maps/" and trailing ".wgf" */
+			                filename_for_continue_editing_.substr(
+			                   13, filename_for_continue_editing_.length() - 17)),
+			          format(_("Name: %s"),
+			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+			                    .as_font_tag(last_edited->first.localized_name)),
+			          format(_("Size: %s"),
+			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+			                    .as_font_tag(format(_("%1$u×%2$u"), last_edited->first.width,
+			                                        last_edited->first.height))),
+			          format(_("Players: %s"),
+			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+			                    .as_font_tag(std::to_string(last_edited->first.nrplayers))),
+			          format(_("Description: %s"),
+			                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+			                    .as_font_tag(last_edited->first.description))),
+			   shortcut_string_for(KeyboardShortcut::kMainMenuContinueEditing, false));
 		}
 	}
 
@@ -425,45 +442,44 @@ void MainMenu::set_labels() {
 	editor_.set_label(_("Editor…"));
 	singleplayer_.set_tooltip(as_tooltip_text_with_hotkey(
 	   _("Begin or load a single-player campaign or free game"),
-	   shortcut_string_for(KeyboardShortcut::kMainMenuSP), UI::PanelStyle::kFsMenu));
+	   shortcut_string_for(KeyboardShortcut::kMainMenuSP, true), UI::PanelStyle::kFsMenu));
 	multiplayer_.set_tooltip(as_tooltip_text_with_hotkey(
 	   _("Play with your friends over the internet"),
-	   shortcut_string_for(KeyboardShortcut::kMainMenuMP), UI::PanelStyle::kFsMenu));
+	   shortcut_string_for(KeyboardShortcut::kMainMenuMP, true), UI::PanelStyle::kFsMenu));
 	editor_.set_tooltip(as_tooltip_text_with_hotkey(
-	   _("Launch the map editor"), shortcut_string_for(KeyboardShortcut::kMainMenuE),
+	   _("Launch the map editor"), shortcut_string_for(KeyboardShortcut::kMainMenuE, true),
 	   UI::PanelStyle::kFsMenu));
 
 	replay_.set_title(_("Watch Replay"));
 	replay_.set_tooltip(as_tooltip_text_with_hotkey(
-	   _("Watch the replay of an old game"), shortcut_string_for(KeyboardShortcut::kMainMenuReplay),
-	   UI::PanelStyle::kFsMenu));
+	   _("Watch the replay of an old game"),
+	   shortcut_string_for(KeyboardShortcut::kMainMenuReplay, true), UI::PanelStyle::kFsMenu));
 
 	addons_.set_title(_("Add-Ons"));
 	addons_.set_tooltip(as_tooltip_text_with_hotkey(
-	   _("Install and manage add-ons"), shortcut_string_for(KeyboardShortcut::kMainMenuAddons),
+	   _("Install and manage add-ons"), shortcut_string_for(KeyboardShortcut::kMainMenuAddons, true),
 	   UI::PanelStyle::kFsMenu));
 	options_.set_title(_("Options"));
 	options_.set_tooltip(as_tooltip_text_with_hotkey(
 	   _("Technical and game-related settings"),
-	   shortcut_string_for(KeyboardShortcut::kMainMenuOptions), UI::PanelStyle::kFsMenu));
+	   shortcut_string_for(KeyboardShortcut::kMainMenuOptions, true), UI::PanelStyle::kFsMenu));
 	about_.set_title(_("About Widelands"));
 	about_.set_tooltip(as_tooltip_text_with_hotkey(
-	   _("Readme, License, and Credits"), shortcut_string_for(KeyboardShortcut::kMainMenuAbout),
-	   UI::PanelStyle::kFsMenu));
+	   _("Readme, License, and Credits"),
+	   shortcut_string_for(KeyboardShortcut::kMainMenuAbout, true), UI::PanelStyle::kFsMenu));
 	exit_.set_title(_("Exit Widelands"));
 	exit_.set_tooltip(as_tooltip_text_with_hotkey(
-	   _("Quit the game"), shortcut_string_for(KeyboardShortcut::kMainMenuQuit),
+	   _("Quit the game"), shortcut_string_for(KeyboardShortcut::kMainMenuQuit, true),
 	   UI::PanelStyle::kFsMenu));
 
 	version_.set_text(
-	   /** TRANSLATORS: %1$s = version string, %2%s = "Debug" or "Release" */
-	   (boost::format(_("Version %1$s (%2$s)")) % build_id().c_str() % build_type().c_str()).str());
+	   /** TRANSLATORS: %1$s = version string and build type  ("Debug" or "Release") */
+	   format(_("Version %1$s"), build_ver_details()));
 	copyright_.set_text(
 	   /** TRANSLATORS: Placeholders are the copyright years */
-	   (boost::format(_("(C) %1%-%2% by the Widelands Development Team · Licensed under "
-	                    "the GNU General Public License V2.0")) %
-	    kWidelandsCopyrightStart % kWidelandsCopyrightEnd)
-	      .str());
+	   format(_("(C) %1%-%2% by the Widelands Development Team · Licensed under "
+	            "the GNU General Public License V2.0"),
+	          kWidelandsCopyrightStart, kWidelandsCopyrightEnd));
 }
 
 void MainMenu::set_button_visibility(const bool v) {
@@ -477,7 +493,7 @@ void MainMenu::set_button_visibility(const bool v) {
 	version_.set_visible(v);
 }
 
-bool MainMenu::handle_mousepress(uint8_t, int32_t, int32_t) {
+bool MainMenu::handle_mousepress(uint8_t /*btn*/, int32_t /*x*/, int32_t /*y*/) {
 	if (init_time_ != kNoSplash) {
 		init_time_ = kNoSplash;
 		return true;
@@ -489,7 +505,7 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 	// Forward all keys to the open window if there is one
 	bool has_open_window = false;
 	for (UI::UniqueWindow::Registry* r : {&r_login_, &r_about_, &r_addons_}) {
-		if (r->window) {
+		if (r->window != nullptr) {
 			has_open_window = true;
 			if (r->window->handle_key(down, code)) {
 				return true;
@@ -598,19 +614,19 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 			show_internet_login();
 			return true;
 		}
-		if (code.sym == SDLK_BACKSPACE && (code.mod & (KMOD_CTRL | KMOD_SHIFT))) {
+		if (code.sym == SDLK_BACKSPACE && ((code.mod & (KMOD_CTRL | KMOD_SHIFT)) != 0)) {
 			// Easter egg: Press Ctrl/Shift+Backspace to exchange the background immediately :-)
 			last_image_exchange_time_ -=
 			   (last_image_exchange_time_ > kImageExchangeInterval ? kImageExchangeInterval :
-			                                                         last_image_exchange_time_);
+                                                                  last_image_exchange_time_);
 			return true;
 		}
 	}
 	return UI::Panel::handle_key(down, code);
 }
 
-inline Rectf MainMenu::image_pos(const Image& i) {
-	return UI::fit_image(i.width(), i.height(), get_w(), get_h());
+inline Rectf MainMenu::image_pos(const Image& i, const bool crop) {
+	return UI::fit_image(i.width(), i.height(), get_w(), get_h(), crop);
 }
 
 static inline void
@@ -619,10 +635,10 @@ do_draw_image(RenderTarget& r, const Rectf& dest, const Image& img, const float
 	   dest, &img, Recti(0, 0, img.width(), img.height()), opacity, BlendMode::UseAlpha);
 }
 
-inline float MainMenu::calc_opacity(const uint32_t time) {
+inline float MainMenu::calc_opacity(const uint32_t time) const {
 	return last_image_ == draw_image_ ?
-	          1.f :
-	          std::max(0.f, std::min(1.f, static_cast<float>(time - last_image_exchange_time_) /
+             1.f :
+             std::max(0.f, std::min(1.f, static_cast<float>(time - last_image_exchange_time_) /
 	                                         kImageExchangeDuration));
 }
 
@@ -664,7 +680,7 @@ void MainMenu::draw(RenderTarget& r) {
 		last_image_ = draw_image_;
 		if (images_.size() > 1) {
 			do {
-				draw_image_ = std::rand() % images_.size();  // NOLINT
+				draw_image_ = RNG::static_rand(images_.size());
 			} while (draw_image_ == last_image_);
 		}
 		last_image_exchange_time_ = time;
@@ -712,10 +728,10 @@ void MainMenu::draw_overlay(RenderTarget& r) {
 
 	if (time - init_time_ < kInitialFadeoutDelay + kInitialFadeoutDuration) {
 		const float opacity = time - init_time_ > kInitialFadeoutDelay ?
-		                         1.f - static_cast<float>(time - init_time_ - kInitialFadeoutDelay) /
+                               1.f - static_cast<float>(time - init_time_ - kInitialFadeoutDelay) /
 		                                  kInitialFadeoutDuration :
-		                         1.f;
-		do_draw_image(r, image_pos(*splashscreen_), *splashscreen_, opacity);
+                               1.f;
+		do_draw_image(r, image_pos(*splashscreen_, false), *splashscreen_, opacity);
 	} else {
 		const unsigned opacity =
 		   255 - 255.f * (time - init_time_ - kInitialFadeoutDelay - kInitialFadeoutDuration) /
@@ -761,7 +777,7 @@ void MainMenu::layout() {
 	vbox2_.set_size((box_rect_.w - padding_) / 2, box_rect_.h);
 
 	// Tell child windows to update their size if necessary
-	for (UI::Panel* p = get_first_child(); p; p = p->get_next_sibling()) {
+	for (UI::Panel* p = get_first_child(); p != nullptr; p = p->get_next_sibling()) {
 		if (upcast(UI::Window, w, p)) {
 			if (w->window_layout_id() == UI::Window::WindowLayoutID::kNone) {
 				continue;
@@ -791,7 +807,7 @@ void MainMenu::layout() {
 
 bool MainMenu::check_desyncing_addon() {
 	for (const auto& pair : AddOns::g_addons) {
-		if (!pair.first.sync_safe && pair.second) {
+		if (!pair.first->sync_safe && pair.second) {
 			UI::WLMessageBox mmb(
 			   this, UI::WindowStyle::kFsMenu, _("Desyncing Add-On Found"),
 			   _("An enabled add-on is known to cause desyncs. Proceed at your own risk."),
@@ -828,21 +844,26 @@ void MainMenu::action(const MenuTarget t) {
 		break;
 
 	case MenuTarget::kReplay:
-		menu_capsule_.clear_content();
-		new LoadGame(menu_capsule_, *new Widelands::Game(), *new SinglePlayerGameSettingsProvider(),
-		             true, true);
+		if (Widelands::Game* g = create_safe_game()) {
+			menu_capsule_.clear_content();
+			new LoadGame(menu_capsule_, *g, *new SinglePlayerGameSettingsProvider(), true, true);
+		}
 		break;
 	case MenuTarget::kLoadGame:
-		menu_capsule_.clear_content();
-		new LoadGame(menu_capsule_, *new Widelands::Game(), *new SinglePlayerGameSettingsProvider(),
-		             true, false);
+		if (Widelands::Game* g = create_safe_game()) {
+			menu_capsule_.clear_content();
+			new LoadGame(menu_capsule_, *g, *new SinglePlayerGameSettingsProvider(), true, false);
+		}
 		break;
 
-	case MenuTarget::kNewGame:
-		menu_capsule_.clear_content();
-		new MapSelect(menu_capsule_, nullptr, new SinglePlayerGameSettingsProvider(), nullptr,
-		              *new Widelands::Game());
+	case MenuTarget::kNewGame: {
+		if (Widelands::Game* g = create_safe_game()) {
+			menu_capsule_.clear_content();
+			new MapSelect(menu_capsule_, nullptr, new SinglePlayerGameSettingsProvider(), nullptr,
+			              std::shared_ptr<Widelands::Game>(g));
+		}
 		break;
+	}
 
 	case MenuTarget::kRandomGame:
 		menu_capsule_.clear_content();
@@ -851,13 +872,15 @@ void MainMenu::action(const MenuTarget t) {
 
 	case MenuTarget::kContinueLastsave:
 		if (!filename_for_continue_playing_.empty()) {
-			Widelands::Game game;
-			game.set_ai_training_mode(get_config_bool("ai_training", false));
-			SinglePlayerGameSettingsProvider sp;
-			try {
-				game.run_load_game(filename_for_continue_playing_, "");
-			} catch (const std::exception& e) {
-				WLApplication::emergency_save(this, game, e.what());
+			std::unique_ptr<Widelands::Game> game(create_safe_game(true));
+			if (game != nullptr) {
+				game->set_ai_training_mode(get_config_bool("ai_training", false));
+				SinglePlayerGameSettingsProvider sp;
+				try {
+					game->run_load_game(filename_for_continue_playing_, "");
+				} catch (const std::exception& e) {
+					WLApplication::emergency_save(this, *game, e.what());
+				}
 			}
 			// Update the Continue button in case a new savegame was created
 			set_labels();
@@ -878,7 +901,7 @@ void MainMenu::action(const MenuTarget t) {
 			break;
 		}
 		menu_capsule_.clear_content();
-		g_sh->change_music("ingame", 1000);
+		g_sh->change_music(Songset::kIngame, 1000);
 		new NetSetupLAN(menu_capsule_);
 		break;
 	case MenuTarget::kMetaserver: {
@@ -902,7 +925,7 @@ void MainMenu::action(const MenuTarget t) {
 			get_config_string("password_sha1", password_);
 		}
 
-		g_sh->change_music("ingame", 1000);
+		g_sh->change_music(Songset::kIngame, 1000);
 		new InternetLobby(menu_capsule_, nickname_, password_, register_, tribeinfos);
 	} break;
 
