@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2021 by the Widelands Development Team
+ * Copyright (C) 2010-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -21,10 +20,9 @@
 
 #include <memory>
 
-#include <boost/algorithm/string.hpp>
-
 #include "ai/computer_player.h"
 #include "base/i18n.h"
+#include "base/string.h"
 #include "base/wexception.h"
 #include "graphic/image_cache.h"
 #include "graphic/playercolor.h"
@@ -34,22 +32,22 @@
 #include "ui_basic/button.h"
 #include "ui_basic/color_chooser.h"
 #include "ui_basic/dropdown.h"
+#include "ui_basic/messagebox.h"
 #include "ui_basic/mouse_constants.h"
 
-#define AI_NAME_PREFIX "ai" AI_NAME_SEPARATOR
-
 constexpr int kPadding = 4;
 
 /// Holds the info and dropdown menu for a connected client
 struct MultiPlayerClientGroup : public UI::Box {
-	MultiPlayerClientGroup(UI::Panel* const parent,
-	                       int32_t const,
+	MultiPlayerClientGroup(UI::Panel* const grand_parent,
+	                       UI::Panel* const parent,
+	                       int32_t const /* w */,
 	                       int32_t const h,
 	                       PlayerSlot id,
 	                       GameSettingsProvider* const settings)
 	   : UI::Box(parent, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal, 0, 0, kPadding),
 	     slot_dropdown_(this,
-	                    (boost::format("dropdown_slot%d") % static_cast<unsigned int>(id)).str(),
+	                    format("dropdown_slot%d", static_cast<unsigned int>(id)),
 	                    0,
 	                    0,
 	                    0,
@@ -64,7 +62,8 @@ struct MultiPlayerClientGroup : public UI::Box {
 	     name(this, UI::PanelStyle::kFsMenu, UI::FontStyle::kFsMenuLabel, 0, 0, 0, 0),
 	     settings_(settings),
 	     id_(id),
-	     slot_selection_locked_(false) {
+	     slot_selection_locked_(false),
+	     menu_parent_(grand_parent) {
 		add(&slot_dropdown_);
 		add(&name, UI::Box::Resizing::kAlign, UI::Align::kCenter);
 
@@ -89,7 +88,29 @@ struct MultiPlayerClientGroup : public UI::Box {
 		if (slot_dropdown_.has_selection()) {
 			const uint8_t new_slot = slot_dropdown_.get_selected();
 			if (new_slot != settings.users.at(id_).position) {
-				settings_->set_player_number(slot_dropdown_.get_selected());
+				const std::string& slotname =
+				   new_slot == UserSettings::none() ? "" : settings.players.at(new_slot).name;
+				const std::string& username = settings.users.at(id_).name;
+
+				if (new_slot != UserSettings::none() && settings.savegame && !slotname.empty() &&
+				    slotname != username) {
+					slot_dropdown_.toggle();
+					UI::WLMessageBox m(
+					   menu_parent_, UI::WindowStyle::kFsMenu, _("Divergent Player Name"),
+					   format(
+					      _("The player name of the selected slot (%1%) does not match your own (%2%). "
+					        "Are you sure that you want to occupy this slot?"),
+					      slotname, username),
+					   UI::WLMessageBox::MBoxType::kOkCancel, UI::Align::kLeft);
+
+					if (m.run<UI::Panel::Returncodes>() == UI::Panel::Returncodes::kOk) {
+						settings_->set_player_number(new_slot);
+					} else {
+						slot_dropdown_.select(settings.users.at(id_).position);
+					}
+				} else {
+					settings_->set_player_number(new_slot);
+				}
 			}
 		}
 		slot_selection_locked_ = false;
@@ -107,11 +128,10 @@ struct MultiPlayerClientGroup : public UI::Box {
 		for (PlayerSlot slot = 0; slot < settings.players.size(); ++slot) {
 			if (settings.players.at(slot).state == PlayerSettings::State::kHuman ||
 			    settings.players.at(slot).state == PlayerSettings::State::kOpen) {
-				slot_dropdown_.add(
-				   (boost::format(_("Player %u")) % static_cast<unsigned int>(slot + 1)).str(), slot,
-				   playercolor_image(
-				      settings.players[slot].color, "images/players/genstats_player.png"),
-				   slot == user_setting.position);
+				slot_dropdown_.add(format(_("Player %u"), static_cast<unsigned int>(slot + 1)), slot,
+				                   playercolor_image(settings.players[slot].color,
+				                                     "images/players/genstats_player.png"),
+				                   slot == user_setting.position);
 			}
 		}
 		slot_dropdown_.add(_("Spectator"), UserSettings::none(),
@@ -129,6 +149,8 @@ struct MultiPlayerClientGroup : public UI::Box {
 
 		name.set_text(user_setting.name);
 		rebuild_slot_dropdown(settings);
+
+		initialization_complete();
 	}
 
 	UI::Dropdown<uintptr_t> slot_dropdown_;  /// Select the player slot.
@@ -136,12 +158,13 @@ struct MultiPlayerClientGroup : public UI::Box {
 	GameSettingsProvider* const settings_;
 	uint8_t const id_;            /// User number
 	bool slot_selection_locked_;  // Ensure that dropdowns will close on selection.
+	UI::Panel* menu_parent_;
 };
 
 /// Holds the dropdown menus for a player slot
 struct MultiPlayerPlayerGroup : public UI::Box {
 	MultiPlayerPlayerGroup(UI::Panel* const parent,
-	                       int32_t const,
+	                       int32_t const /* w */,
 	                       int32_t const h,
 	                       PlayerSlot id,
 	                       GameSettingsProvider* const settings,
@@ -159,9 +182,9 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	             UI::ButtonStyle::kFsMenuSecondary,
 	             playercolor_image(settings_->settings().players[id].color,
 	                               "images/players/player_position_menu.png"),
-	             (boost::format(_("Player %u")) % static_cast<unsigned int>(id_ + 1)).str()),
+	             format(_("Player %u"), static_cast<unsigned int>(id_ + 1))),
 	     type_dropdown_(this,
-	                    (boost::format("dropdown_type%d") % static_cast<unsigned int>(id)).str(),
+	                    format("dropdown_type%d", static_cast<unsigned int>(id)),
 	                    0,
 	                    0,
 	                    h,
@@ -172,7 +195,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	                    UI::PanelStyle::kFsMenu,
 	                    UI::ButtonStyle::kFsMenuSecondary),
 	     tribes_dropdown_(this,
-	                      (boost::format("dropdown_tribes%d") % static_cast<unsigned int>(id)).str(),
+	                      format("dropdown_tribes%d", static_cast<unsigned int>(id)),
 	                      0,
 	                      0,
 	                      h,
@@ -183,7 +206,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	                      UI::PanelStyle::kFsMenu,
 	                      UI::ButtonStyle::kFsMenuSecondary),
 	     init_dropdown_(this,
-	                    (boost::format("dropdown_init%d") % static_cast<unsigned int>(id)).str(),
+	                    format("dropdown_init%d", static_cast<unsigned int>(id)),
 	                    0,
 	                    0,
 	                    h,
@@ -194,7 +217,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	                    UI::PanelStyle::kFsMenu,
 	                    UI::ButtonStyle::kFsMenuSecondary),
 	     team_dropdown_(this,
-	                    (boost::format("dropdown_team%d") % static_cast<unsigned int>(id)).str(),
+	                    format("dropdown_team%d", static_cast<unsigned int>(id)),
 	                    0,
 	                    0,
 	                    h,
@@ -249,12 +272,12 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 			} else if (selected == "shared_in") {
 				state = PlayerSettings::State::kShared;
 			} else {
-				if (selected == AI_NAME_PREFIX "random") {
+				if (selected == kRandomAiName) {
 					n->set_player_ai(id_, "", true);
 				} else {
-					if (boost::starts_with(selected, AI_NAME_PREFIX)) {
+					if (starts_with(selected, kAiNamePrefix)) {
 						std::vector<std::string> parts;
-						boost::split(parts, selected, boost::is_any_of(AI_NAME_SEPARATOR));
+						split(parts, selected, {kAiNameSeparator});
 						assert(parts.size() == 2);
 						n->set_player_ai(id_, parts[1], false);
 					} else {
@@ -294,12 +317,11 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 		    (player_setting.tribe.empty() ||
 		     settings.get_tribeinfo(player_setting.tribe).suited_for_ai || can_change_hidden_tribe)) {
 			for (const auto* impl : AI::ComputerPlayer::get_implementations()) {
-				type_dropdown_.add(_(impl->descname),
-				                   (boost::format(AI_NAME_PREFIX "%s") % impl->name).str(),
+				type_dropdown_.add(_(impl->descname), format("%s%s", kAiNamePrefix, impl->name),
 				                   g_image_cache->get(impl->icon_filename), false, _(impl->descname));
 			}
 			/** TRANSLATORS: This is the name of an AI used in the game setup screens */
-			type_dropdown_.add(_("Random AI"), AI_NAME_PREFIX "random",
+			type_dropdown_.add(_("Random AI"), kRandomAiName,
 			                   g_image_cache->get("images/ai/ai_random.png"), false, _("Random AI"));
 		}
 
@@ -324,7 +346,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 		// Now select the entry according to server settings
 		if (player_setting.state == PlayerSettings::State::kHuman) {
 			type_dropdown_.set_image(g_image_cache->get("images/wui/stats/genstats_nrworkers.png"));
-			type_dropdown_.set_tooltip((boost::format(_("%1%: %2%")) % _("Type") % _("Human")).str());
+			type_dropdown_.set_tooltip(format(_("%1%: %2%"), _("Type"), _("Human")));
 		} else if (player_setting.state == PlayerSettings::State::kClosed) {
 			type_dropdown_.select("closed");
 		} else if (player_setting.state == PlayerSettings::State::kOpen) {
@@ -337,11 +359,11 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 					type_dropdown_.set_errored(_("No AI"));
 				} else {
 					if (player_setting.random_ai) {
-						type_dropdown_.select(AI_NAME_PREFIX "random");
+						type_dropdown_.select(kRandomAiName);
 					} else {
 						const AI::ComputerPlayer::Implementation* impl =
 						   AI::ComputerPlayer::get_implementation(player_setting.ai);
-						type_dropdown_.select((boost::format(AI_NAME_PREFIX "%s") % impl->name).str());
+						type_dropdown_.select(format("%s%s", kAiNamePrefix, impl->name));
 					}
 				}
 			}
@@ -351,8 +373,8 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	/// Whether the client who is running the UI is allowed to change the tribe for this player slot.
 	bool has_tribe_access() const {
 		return settings_->settings().players[id_].state == PlayerSettings::State::kShared ?
-		          settings_->can_change_player_init(id_) :
-		          settings_->can_change_player_tribe(id_);
+                settings_->can_change_player_init(id_) :
+                settings_->can_change_player_tribe(id_);
 	}
 
 	/// This will update the game settings for the tribe or shared_in with the value
@@ -364,12 +386,11 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 		const PlayerSettings& player_settings = settings_->settings().players[id_];
 		tribe_selection_locked_ = true;
 		tribes_dropdown_.set_disable_style(player_settings.state == PlayerSettings::State::kShared ?
-		                                      UI::ButtonDisableStyle::kPermpressed :
-		                                      UI::ButtonDisableStyle::kFlat);
+                                            UI::ButtonDisableStyle::kPermpressed :
+                                            UI::ButtonDisableStyle::kFlat);
 		if (tribes_dropdown_.has_selection()) {
 			if (player_settings.state == PlayerSettings::State::kShared) {
-				n->set_player_shared(
-				   id_, boost::lexical_cast<unsigned int>(tribes_dropdown_.get_selected()));
+				n->set_player_shared(id_, stoul(tribes_dropdown_.get_selected()));
 			} else {
 				n->set_player_tribe(id_, tribes_dropdown_.get_selected());
 			}
@@ -401,11 +422,9 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 					const std::string player_name =
 					   /** TRANSLATORS: This is an option in multiplayer setup for sharing
 					      another player's starting position. */
-					   (boost::format(_("Shared in Player %u")) % static_cast<unsigned int>(i + 1))
-					      .str();
-					tribes_dropdown_.add(
-					   player_name, boost::lexical_cast<std::string>(static_cast<unsigned int>(i + 1)),
-					   player_image, (i + 1) == player_setting.shared_in, player_name);
+					   format(_("Shared in Player %u"), static_cast<unsigned int>(i + 1));
+					tribes_dropdown_.add(player_name, as_string(static_cast<unsigned int>(i + 1)),
+					                     player_image, (i + 1) == player_setting.shared_in, player_name);
 				}
 			}
 			tribes_dropdown_.set_enabled(tribes_dropdown_.size() > 1);
@@ -418,11 +437,11 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 				}
 			}
 
-			tribes_dropdown_.add(pgettext("tribe", "Random"), "random",
+			tribes_dropdown_.add(pgettext("tribe", "Random"), kRandom,
 			                     g_image_cache->get("images/ui_fsmenu/random.png"), false,
 			                     _("The tribe will be selected at random"));
 			if (player_setting.random_tribe) {
-				tribes_dropdown_.select("random");
+				tribes_dropdown_.select(kRandom);
 			} else {
 				tribes_dropdown_.select(player_setting.tribe);
 			}
@@ -484,13 +503,13 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 				const Widelands::TribeBasicInfo::Initialization& addme = tribeinfo.initializations[i];
 				bool matches_tags = true;
 				for (const std::string& tag : addme.required_map_tags) {
-					if (!tags.count(tag)) {
+					if (tags.count(tag) == 0u) {
 						matches_tags = false;
 						break;
 					}
 				}
-				if (matches_tags &&
-				    !addme.incompatible_win_conditions.count(settings_->get_win_condition_script())) {
+				if (matches_tags && (addme.incompatible_win_conditions.count(
+				                        settings_->get_win_condition_script()) == 0u)) {
 					init_dropdown_.add(_(addme.descname), i, nullptr,
 					                   i == player_setting.initialization_index, _(addme.tooltip));
 				}
@@ -512,7 +531,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 
 	void set_color() {
 		Panel* p = this;
-		while (p->get_parent()) {
+		while (p->get_parent() != nullptr) {
 			p = p->get_parent();
 		}
 		UI::ColorChooser c(p, UI::WindowStyle::kFsMenu, settings_->settings().players[id_].color,
@@ -542,7 +561,7 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 #endif
 		for (Widelands::TeamNumber t = 1; t <= settings.players.size() / 2; ++t) {
 			assert(t < no_of_team_colors);
-			team_dropdown_.add((boost::format(_("Team %d")) % static_cast<unsigned int>(t)).str(), t,
+			team_dropdown_.add(format(_("Team %d"), static_cast<unsigned int>(t)), t,
 			                   playercolor_image(kTeamColors[t], "images/players/team.png"));
 		}
 		team_dropdown_.select(player_setting.team);
@@ -590,6 +609,8 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 				}
 			}
 		}
+
+		initialization_complete();
 	}
 
 	void force_new_dimensions(uint32_t height) {
@@ -622,20 +643,22 @@ struct MultiPlayerPlayerGroup : public UI::Box {
 	std::unique_ptr<Notifications::Subscriber<NoteGameSettings>> subscriber_;
 };
 
-MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const parent,
+MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const launchgame,
+                                             UI::Panel* const parent,
                                              int32_t const x,
                                              int32_t const y,
-                                             int32_t const,
-                                             int32_t const,
+                                             int32_t const /* w */,
+                                             int32_t const /* h */,
                                              GameSettingsProvider* const settings,
                                              uint32_t buth)
    : UI::Box(parent, UI::PanelStyle::kFsMenu, x, y, UI::Box::Horizontal),
      settings_(settings),
-     npsb(new NetworkPlayerSettingsBackend(settings_)),
-     clientbox(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     playerbox(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
-     scrollable_playerbox(&playerbox, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
-     clients_(&clientbox,
+     npsb_(new NetworkPlayerSettingsBackend(settings_)),
+     launchgame_(launchgame),
+     clientbox_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
+     playerbox_(this, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical, 0, 0, kPadding),
+     scrollable_playerbox_(&playerbox_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Vertical),
+     clients_(&clientbox_,
               UI::PanelStyle::kFsMenu,
               UI::FontStyle::kFsGameSetupHeadings,
               0,
@@ -644,7 +667,7 @@ MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const parent,
               0,
               _("Clients"),
               UI::Align::kCenter),
-     players_(&playerbox,
+     players_(&playerbox_,
               UI::PanelStyle::kFsMenu,
               UI::FontStyle::kFsGameSetupHeadings,
               0,
@@ -654,18 +677,18 @@ MultiPlayerSetupGroup::MultiPlayerSetupGroup(UI::Panel* const parent,
               _("Players"),
               UI::Align::kCenter),
      buth_(buth) {
-	clientbox.add(&clients_, Resizing::kAlign, UI::Align::kCenter);
-	clientbox.add_space(3 * kPadding);
-	clientbox.set_scrolling(true);
+	clientbox_.add(&clients_, Resizing::kAlign, UI::Align::kCenter);
+	clientbox_.add_space(3 * kPadding);
+	clientbox_.set_scrolling(true);
 
-	add(&clientbox);
+	add(&clientbox_);
 	add_space(8 * kPadding);
-	add(&playerbox, Resizing::kExpandBoth);
-	playerbox.add(&players_, Resizing::kAlign, UI::Align::kCenter);
-	scrollable_playerbox.set_scrolling(true);
-	playerbox.add_space(kPadding);
+	add(&playerbox_, Resizing::kExpandBoth);
+	playerbox_.add(&players_, Resizing::kAlign, UI::Align::kCenter);
+	scrollable_playerbox_.set_scrolling(true);
+	playerbox_.add_space(kPadding);
 
-	playerbox.add(&scrollable_playerbox, Resizing::kExpandBoth);
+	playerbox_.add(&scrollable_playerbox_, Resizing::kExpandBoth);
 
 	subscriber_ = Notifications::subscribe<NoteGameSettings>([this](const NoteGameSettings& n) {
 		if (n.action == NoteGameSettings::Action::kMap) {
@@ -680,26 +703,26 @@ MultiPlayerSetupGroup::~MultiPlayerSetupGroup() = default;
 
 /// Update which slots are available based on current settings.
 void MultiPlayerSetupGroup::reset() {
-	for (auto& p : multi_player_player_groups) {
+	for (auto& p : multi_player_player_groups_) {
 		p->die();
 	}
-	multi_player_player_groups.clear();
-	for (auto& c : multi_player_client_groups) {
+	multi_player_player_groups_.clear();
+	for (auto& c : multi_player_client_groups_) {
 		c->die();
 	}
-	multi_player_client_groups.clear();
+	multi_player_client_groups_.clear();
 }
 void MultiPlayerSetupGroup::update_players() {
 	const GameSettings& settings = settings_->settings();
 	const size_t number_of_players = settings.players.size();
 
-	for (PlayerSlot i = multi_player_player_groups.size(); i < number_of_players; ++i) {
-		multi_player_player_groups.push_back(
-		   new MultiPlayerPlayerGroup(&scrollable_playerbox, playerbox.get_w() - UI::Scrollbar::kSize,
-		                              buth_, i, settings_, npsb.get()));
-		scrollable_playerbox.add(multi_player_player_groups.at(i), Resizing::kFullSize);
+	for (PlayerSlot i = multi_player_player_groups_.size(); i < number_of_players; ++i) {
+		multi_player_player_groups_.push_back(new MultiPlayerPlayerGroup(
+		   &scrollable_playerbox_, playerbox_.get_w() - UI::Scrollbar::kSize, buth_, i, settings_,
+		   npsb_.get()));
+		scrollable_playerbox_.add(multi_player_player_groups_.at(i), Resizing::kFullSize);
 	}
-	for (auto& p : multi_player_player_groups) {
+	for (auto& p : multi_player_player_groups_) {
 		p->update();
 	}
 }
@@ -707,34 +730,34 @@ void MultiPlayerSetupGroup::update_clients() {
 	const GameSettings& settings = settings_->settings();
 	const size_t number_of_users = settings.users.size();
 
-	if (number_of_users > multi_player_client_groups.size()) {
-		for (uint32_t i = multi_player_client_groups.size(); i < number_of_users; ++i) {
-			multi_player_client_groups.push_back(
-			   new MultiPlayerClientGroup(&clientbox, clientbox.get_w(), buth_, i, settings_));
-			clientbox.add(multi_player_client_groups.at(i), Resizing::kFullSize);
+	if (number_of_users > multi_player_client_groups_.size()) {
+		for (uint32_t i = multi_player_client_groups_.size(); i < number_of_users; ++i) {
+			multi_player_client_groups_.push_back(new MultiPlayerClientGroup(
+			   launchgame_, &clientbox_, clientbox_.get_w(), buth_, i, settings_));
+			clientbox_.add(multi_player_client_groups_.at(i), Resizing::kFullSize);
 		}
 	}
-	for (auto& c : multi_player_client_groups) {
+	for (auto& c : multi_player_client_groups_) {
 		c->update();
 	}
 }
 
 void MultiPlayerSetupGroup::draw(RenderTarget& dst) {
-	const int32_t total_box_height = scrollable_playerbox.get_y() + scrollable_playerbox.get_h();
+	const int32_t total_box_height = scrollable_playerbox_.get_y() + scrollable_playerbox_.get_h();
 
-	for (MultiPlayerPlayerGroup* current_player : multi_player_player_groups) {
+	for (MultiPlayerPlayerGroup* current_player : multi_player_player_groups_) {
 		if (current_player->get_y() < 0 && current_player->get_y() > -current_player->get_h()) {
 			dst.brighten_rect(
-			   Recti(playerbox.get_x(), scrollable_playerbox.get_y(), scrollable_playerbox.get_w(),
+			   Recti(playerbox_.get_x(), scrollable_playerbox_.get_y(), scrollable_playerbox_.get_w(),
 			         current_player->get_h() + current_player->get_y()),
 			   -MOUSE_OVER_BRIGHT_FACTOR);
 		} else if (current_player->get_y() >= 0) {
 			auto rect_height =
-			   std::min(total_box_height - (scrollable_playerbox.get_y() + current_player->get_y()),
+			   std::min(total_box_height - (scrollable_playerbox_.get_y() + current_player->get_y()),
 			            current_player->get_h());
 			dst.brighten_rect(
-			   Recti(playerbox.get_x(), scrollable_playerbox.get_y() + current_player->get_y(),
-			         scrollable_playerbox.get_w(), rect_height < 0 ? 0 : rect_height),
+			   Recti(playerbox_.get_x(), scrollable_playerbox_.get_y() + current_player->get_y(),
+			         scrollable_playerbox_.get_w(), rect_height < 0 ? 0 : rect_height),
 			   -MOUSE_OVER_BRIGHT_FACTOR);
 		}
 	}
@@ -744,16 +767,16 @@ void MultiPlayerSetupGroup::force_new_dimensions(uint32_t max_width,
                                                  uint32_t max_height,
                                                  uint32_t standard_element_height) {
 	buth_ = standard_element_height;
-	clientbox.set_min_desired_breadth(max_width / 3);
-	clientbox.set_max_size(max_width / 3, max_height);
-	playerbox.set_max_size(max_width / 2, max_height);
-	scrollable_playerbox.set_max_size(max_width / 2, max_height - players_.get_h() - 4 * kPadding);
+	clientbox_.set_min_desired_breadth(max_width / 3);
+	clientbox_.set_max_size(max_width / 3, max_height);
+	playerbox_.set_max_size(max_width / 2, max_height);
+	scrollable_playerbox_.set_max_size(max_width / 2, max_height - players_.get_h() - 4 * kPadding);
 
-	for (auto& multiPlayerClientGroup : multi_player_client_groups) {
+	for (auto& multiPlayerClientGroup : multi_player_client_groups_) {
 		multiPlayerClientGroup->force_new_dimensions(standard_element_height);
 	}
 
-	for (auto& multiPlayerPlayerGroup : multi_player_player_groups) {
+	for (auto& multiPlayerPlayerGroup : multi_player_player_groups_) {
 		multiPlayerPlayerGroup->force_new_dimensions(standard_element_height);
 	}
 }
