@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -21,8 +20,6 @@
 
 #include <memory>
 
-#include <boost/algorithm/string.hpp>
-
 #include "base/i18n.h"
 #include "base/log.h"
 #include "base/wexception.h"
@@ -30,7 +27,6 @@
 #include "graphic/font_handler.h"
 #include "graphic/graphic.h"
 #include "graphic/mouse_cursor.h"
-#include "graphic/style_manager.h"
 #include "graphic/text/bidi.h"
 #include "graphic/text/font_set.h"
 #include "graphic/text_layout.h"
@@ -58,15 +54,15 @@ constexpr int kDropdownMaximized = -1;
 void find_selected_locale(std::string* selected_locale, const std::string& current_locale) {
 	if (selected_locale->empty()) {
 		std::vector<std::string> parts;
-		boost::split(parts, current_locale, boost::is_any_of("."));
+		split(parts, current_locale, {'.'});
 		if (current_locale == parts[0]) {
 			*selected_locale = current_locale;
 		} else {
-			boost::split(parts, parts[0], boost::is_any_of("@"));
+			split(parts, parts[0], {'@'});
 			if (current_locale == parts[0]) {
 				*selected_locale = current_locale;
 			} else {
-				boost::split(parts, parts[0], boost::is_any_of("_"));
+				split(parts, parts[0], {'_'});
 				if (current_locale == parts[0]) {
 					*selected_locale = current_locale;
 				}
@@ -131,17 +127,6 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                           UI::DropdownType::kTextual,
                           UI::PanelStyle::kFsMenu,
                           UI::ButtonStyle::kFsMenuMenu),
-     theme_dropdown_(&box_interface_vbox_,
-                     "dropdown_theme",
-                     0,
-                     0,
-                     100,  // 100 is arbitrary, will be resized in layout().
-                     50,
-                     24,
-                     _("UI Theme"),
-                     UI::DropdownType::kTextual,
-                     UI::PanelStyle::kFsMenu,
-                     UI::ButtonStyle::kFsMenuMenu),
 
      inputgrab_(&box_interface_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Grab Input"), "", 0),
      sdl_cursor_(&box_interface_,
@@ -209,7 +194,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                          0,
                          0,
                          UI::ButtonStyle::kFsMenuSecondary,
-                         _("Edit keyboard shortcuts…")),
+                         _("Edit keyboard and mouse actions…")),
 
      // Sound options
      sound_options_(box_sound_, UI::SliderStyle::kFsMenu),
@@ -244,7 +229,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
      zip_(&box_saving_,
           UI::PanelStyle::kFsMenu,
           Vector2i::zero(),
-          _("Compress widelands data files (maps, replays and savegames)"),
+          _("Compress Widelands data files (maps, replays, and savegames)"),
           "",
           0),
      write_syncstreams_(&box_saving_,
@@ -257,8 +242,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
      show_buildhelp_(
         &box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Building Spaces")),
      show_census_(&box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Census")),
-     show_statistics_(
-        &box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Statistics")),
+     show_statistics_(&box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Status")),
      show_soldier_levels_(
         &box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Soldier Levels")),
      show_buildings_(&box_newgame_, UI::PanelStyle::kFsMenu, Vector2i::zero(), _("Show Buildings")),
@@ -284,11 +268,6 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                          or a map region,*/
                       /** TRANSLATORS: and it also lets you jump to it on the map. */
                       _("Use single watchwindow mode")),
-     ctrl_zoom_(&box_ingame_,
-                UI::PanelStyle::kFsMenu,
-                Vector2i::zero(),
-                /** TRANSLATORS: This refers to to zooming with the scrollwheel.*/
-                _("Zoom only when Ctrl is pressed")),
      game_clock_(&box_ingame_,
                  UI::PanelStyle::kFsMenu,
                  Vector2i::zero(),
@@ -301,6 +280,10 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                      UI::PanelStyle::kFsMenu,
                      Vector2i::zero(),
                      _("Scroll when the mouse cursor is near the screen edge")),
+     invert_movement_(&box_ingame_,
+                      UI::PanelStyle::kFsMenu,
+                      Vector2i::zero(),
+                      _("Invert click-and-drag map movement direction")),
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
      training_wheels_box_(&box_ingame_, UI::PanelStyle::kFsMenu, 0, 0, UI::Box::Horizontal),
      training_wheels_(&training_wheels_box_,
@@ -346,7 +329,6 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	// Interface
 	box_interface_vbox_.add(&language_dropdown_, UI::Box::Resizing::kFullSize);
 	box_interface_vbox_.add(&resolution_dropdown_, UI::Box::Resizing::kFullSize);
-	box_interface_vbox_.add(&theme_dropdown_, UI::Box::Resizing::kFullSize);
 	box_interface_hbox_.add(&box_interface_vbox_, UI::Box::Resizing::kExpandBoth);
 	box_interface_hbox_.add(&translation_info_, UI::Box::Resizing::kExpandBoth);
 
@@ -384,10 +366,10 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	box_ingame_.add(&auto_roadbuild_mode_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&transparent_chat_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&single_watchwin_, UI::Box::Resizing::kFullSize);
-	box_ingame_.add(&ctrl_zoom_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&game_clock_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&numpad_diagonalscrolling_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add(&edge_scrolling_, UI::Box::Resizing::kFullSize);
+	box_ingame_.add(&invert_movement_, UI::Box::Resizing::kFullSize);
 	box_ingame_.add_space(kPadding);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	box_ingame_.add(&training_wheels_box_, UI::Box::Resizing::kFullSize);
@@ -432,75 +414,14 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 #endif
 	cancel_.sigclicked.connect([this]() { clicked_cancel(); });
 	apply_.sigclicked.connect([this]() { clicked_apply(); });
-	ok_.sigclicked.connect([this]() { end_modal<MenuTarget>(MenuTarget::kOk); });
+	ok_.sigclicked.connect([this]() { clicked_ok(); });
 
 	/** TRANSLATORS: Options: Save game automatically every: */
 	sb_autosave_.add_replacement(0, _("Off"));
 
 	// Fill in data
 	// Interface options
-	for (int modes = 0; modes < SDL_GetNumDisplayModes(0); ++modes) {
-		SDL_DisplayMode mode;
-		SDL_GetDisplayMode(0, modes, &mode);
-		if (kMinimumResolutionW <= mode.w && kMinimumResolutionH <= mode.h &&
-		    (SDL_BITSPERPIXEL(mode.format) == 32 || SDL_BITSPERPIXEL(mode.format) == 24)) {
-			ScreenResolution this_res = {
-			   mode.w, mode.h, static_cast<int32_t>(SDL_BITSPERPIXEL(mode.format))};
-			if (this_res.depth == 24) {
-				this_res.depth = 32;
-			}
-			if (resolutions_.empty() || this_res.xres != resolutions_.rbegin()->xres ||
-			    this_res.yres != resolutions_.rbegin()->yres) {
-				resolutions_.push_back(this_res);
-			}
-		}
-	}
-
-	int cur_win_res_x = g_gr->get_window_mode_xres();
-	int cur_win_res_y = g_gr->get_window_mode_yres();
-
-	/** TRANSLATORS: Entry in the window size dropdown*/
-	resolution_dropdown_.add(_("Fullscreen"), kDropdownFullscreen, nullptr, opt.fullscreen);
-#ifdef RESIZABLE_WINDOW
-	/** TRANSLATORS: Entry in the window size dropdown*/
-	resolution_dropdown_.add(_("Maximized"), kDropdownMaximized, nullptr,
-	                         !resolution_dropdown_.has_selection() && opt.maximized);
-#endif
-
-	for (uint32_t i = 0; i < resolutions_.size(); ++i) {
-		const bool selected = !resolution_dropdown_.has_selection() &&
-		                      resolutions_[i].xres == cur_win_res_x &&
-		                      resolutions_[i].yres == cur_win_res_y;
-		resolution_dropdown_.add(
-		   /** TRANSLATORS: Screen resolution, e.g. 800 x 600*/
-		   (boost::format(_("%1% x %2%")) % resolutions_[i].xres % resolutions_[i].yres).str(), i,
-		   nullptr, selected);
-	}
-
-	if (!resolution_dropdown_.has_selection()) {
-		int entry = resolutions_.size();
-		resolutions_.resize(entry + 1);
-		resolutions_[entry].xres = cur_win_res_x;
-		resolutions_[entry].yres = cur_win_res_y;
-		resolution_dropdown_.add(
-		   /** TRANSLATORS: Screen resolution, e.g. 800 x 600*/
-		   (boost::format(_("%1% x %2%")) % cur_win_res_x % cur_win_res_y).str(), entry, nullptr,
-		   true);
-	}
-
-	for (const std::string& theme : g_fs->list_directory("templates")) {
-		if (g_fs->is_directory(theme)) {
-			const std::string descname =
-			   (theme == "templates/default") ? _("Default") : FileSystem::fs_filename(theme.c_str());
-			theme_dropdown_.add(descname, theme, nullptr, (theme + '/') == template_dir());
-		}
-	}
-	for (auto& addon : AddOns::g_addons) {
-		if (addon.first.category == AddOns::AddOnCategory::kTheme && addon.second) {
-			const std::string path = kAddOnDir + '/' + addon.first.internal_name + '/';
-			theme_dropdown_.add(addon.first.descname(), path, nullptr, path == template_dir());
-		}
-	}
+	add_screen_resolutions(opt);
 
 	inputgrab_.set_state(opt.inputgrab);
 	sdl_cursor_.set_state(opt.sdl_cursor);
@@ -519,26 +440,64 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	auto_roadbuild_mode_.set_state(opt.auto_roadbuild_mode);
 	transparent_chat_.set_state(opt.transparent_chat);
 	single_watchwin_.set_state(opt.single_watchwin);
-	ctrl_zoom_.set_state(opt.ctrl_zoom);
 	game_clock_.set_state(opt.game_clock);
 	numpad_diagonalscrolling_.set_state(opt.numpad_diagonalscrolling);
 	edge_scrolling_.set_state(opt.edge_scrolling);
+	invert_movement_.set_state(opt.invert_movement);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	training_wheels_.set_state(opt.training_wheels);
 #endif
 
 	// New Game options
-	show_buildhelp_.set_state(opt.display_flags & InteractiveBase::dfShowBuildhelp);
-	show_census_.set_state(opt.display_flags & InteractiveBase::dfShowCensus);
-	show_statistics_.set_state(opt.display_flags & InteractiveBase::dfShowStatistics);
-	show_soldier_levels_.set_state(opt.display_flags & InteractiveBase::dfShowSoldierLevels);
-	show_buildings_.set_state(opt.display_flags & InteractiveBase::dfShowBuildings);
-	show_workarea_overlap_.set_state(opt.display_flags & InteractiveBase::dfShowWorkareaOverlap);
+	show_buildhelp_.set_state((opt.display_flags & InteractiveBase::dfShowBuildhelp) != 0);
+	show_census_.set_state((opt.display_flags & InteractiveBase::dfShowCensus) != 0);
+	show_statistics_.set_state((opt.display_flags & InteractiveBase::dfShowStatistics) != 0);
+	show_soldier_levels_.set_state((opt.display_flags & InteractiveBase::dfShowSoldierLevels) != 0);
+	show_buildings_.set_state((opt.display_flags & InteractiveBase::dfShowBuildings) != 0);
+	show_workarea_overlap_.set_state((opt.display_flags & InteractiveBase::dfShowWorkareaOverlap) !=
+	                                 0);
 
 	// Language options
 	add_languages_to_list(opt.language);
 	update_language_stats();
 	layout();
+
+	initialization_complete();
+}
+void Options::add_screen_resolutions(const OptionsCtrl::OptionsStruct& opt) {
+	ScreenResolution current_res = {g_gr->get_window_mode_xres(), g_gr->get_window_mode_yres()};
+
+	resolution_dropdown_.add(
+	   /** TRANSLATORS: Entry in the window size dropdown*/
+	   _("Fullscreen"), {kDropdownFullscreen, kDropdownFullscreen}, nullptr, opt.fullscreen);
+#ifdef RESIZABLE_WINDOW
+	/** TRANSLATORS: Entry in the window size dropdown*/
+	resolution_dropdown_.add(_("Maximized"), {kDropdownMaximized, kDropdownMaximized}, nullptr,
+	                         !resolution_dropdown_.has_selection() && opt.maximized);
+#endif
+
+	ScreenResolution previous{0, 0};
+	for (int modes = 0; modes < SDL_GetNumDisplayModes(0); ++modes) {
+		SDL_DisplayMode mode;
+		SDL_GetDisplayMode(0, modes, &mode);
+		if (kMinimumResolutionW <= mode.w && kMinimumResolutionH <= mode.h &&
+		    (SDL_BITSPERPIXEL(mode.format) == 32 || SDL_BITSPERPIXEL(mode.format) == 24)) {
+			ScreenResolution this_res = {mode.w, mode.h};
+			if (this_res != previous) {
+				previous = this_res;
+				const bool selected = !resolution_dropdown_.has_selection() && this_res == current_res;
+				resolution_dropdown_.add(
+				   /** TRANSLATORS: Screen resolution, e.g. 800 × 600*/
+				   format(_("%1% × %2%"), this_res.xres, this_res.yres), this_res, nullptr, selected);
+			}
+		}
+	}
+
+	if (!resolution_dropdown_.has_selection()) {
+		resolution_dropdown_.add(
+		   /** TRANSLATORS: Screen resolution, e.g. 800 × 600*/
+		   format(_("%1% × %2%"), current_res.xres, current_res.yres), current_res, nullptr, true);
+	}
 }
 
 void Options::layout() {
@@ -556,7 +515,7 @@ void Options::layout() {
 		// Tabs
 		tabs_.set_size(get_inner_w(), get_inner_h() - buth - 2 * kPadding);
 
-		const int tab_panel_width = get_inner_w() - 2 * kPadding;
+		const int tab_panel_width = get_inner_w() - 3 * kPadding;
 		const int unit_w = tab_panel_width / 3;
 
 		// Interface
@@ -611,8 +570,8 @@ void Options::add_languages_to_list(const std::string& current_locale) {
 
 		for (const std::string& localename : files) {  // Begin scan locales directory
 			const char* path = localename.c_str();
-			if (!strcmp(FileSystem::fs_filename(path), ".") ||
-			    !strcmp(FileSystem::fs_filename(path), "..") || !fs->is_directory(path)) {
+			if ((strcmp(FileSystem::fs_filename(path), ".") == 0) ||
+			    (strcmp(FileSystem::fs_filename(path), "..") == 0) || !fs->is_directory(path)) {
 				continue;
 			}
 
@@ -650,8 +609,12 @@ void Options::add_languages_to_list(const std::string& current_locale) {
 
 /**
  * Updates the language statistics message according to the currently selected locale.
- * @param include_system_lang We only want to include the system lang if it matches the Widelands
- * locale.
+ *
+ * i18n::get_locale() is parsed,
+ *  this can either be "C"
+ *  or something like "en_EN.UTF-8"
+ *
+ *  See https://en.wikipedia.org/wiki/ISO_639
  */
 void Options::update_language_stats() {
 	int percent = 100;
@@ -661,17 +624,19 @@ void Options::update_language_stats() {
 		// Empty locale means try system locale
 		if (locale.empty()) {
 			std::vector<std::string> parts;
-			boost::split(parts, i18n::get_locale(), boost::is_any_of("."));
+			split(parts, i18n::get_locale(), {'.'});  // split of encoding
 			if (language_entries_.count(parts[0]) == 1) {
 				locale = parts[0];
 			} else {
-				boost::split(parts, parts[0], boost::is_any_of("@"));
-				if (language_entries_.count(parts[0]) == 1) {
-					locale = parts[0];
+				std::vector<std::string> parts2;
+				split(parts2, parts[0], {'@'});  // no idea where that @ may come from
+				if (language_entries_.count(parts2[0]) == 1) {
+					locale = parts2[0];
 				} else {
-					boost::split(parts, parts[0], boost::is_any_of("_"));
-					if (language_entries_.count(parts[0]) == 1) {
-						locale = parts[0];
+					std::vector<std::string> parts3;
+					split(parts3, parts2[0], {'_'});  // Split of language from Country
+					if (language_entries_.count(parts3[0]) == 1) {
+						locale = parts3[0];
 					}
 				}
 			}
@@ -689,12 +654,11 @@ void Options::update_language_stats() {
 				if (percent == 100) {
 					message =
 					   /** TRANSLATORS: %s = language name */
-					   (boost::format(_("The translation into %s is complete.")) % entry.descname).str();
+					   format(_("The translation into %s is complete."), entry.descname);
 				} else {
-					/** TRANSLATORS: %1% = language name, %2% = percentage */
-					message = (boost::format(_("The translation into %1% is %2%%% complete.")) %
-					           entry.descname % percent)
-					             .str();
+					message = format(
+					   /** TRANSLATORS: %1% = language name, %2% = percentage */
+					   _("The translation into %1% is %2%%% complete."), entry.descname, percent);
 				}
 			} catch (...) {
 			}
@@ -706,15 +670,18 @@ void Options::update_language_stats() {
 	// will catch up with the work later.
 	if (percent <= 90) {
 		message = message + " " +
-		          (boost::format(_("If you wish to help us translate, please visit %s")) %
-		           "<font underline=1>widelands.org/wiki/TranslatingWidelands</font>")
-		             .str();
+		          format(_("If you wish to help us translate, please visit %s"),
+		                 "<font underline=1>widelands.org/wiki/TranslatingWidelands</font>");
 	}
 	// Make font a bit smaller so the link will fit at 800x600 resolution.
 	translation_info_.set_text(
 	   as_richtext_paragraph(message, UI::FontStyle::kFsMenuTranslationInfo));
 }
 
+void Options::clicked_ok() {
+	end_modal<MenuTarget>(MenuTarget::kOk);
+}
+
 void Options::clicked_apply() {
 	end_modal<MenuTarget>(MenuTarget::kApplyOptions);
 }
@@ -727,12 +694,11 @@ void Options::clicked_cancel() {
 bool Options::handle_key(bool down, SDL_Keysym code) {
 	if (down) {
 		switch (code.sym) {
-		case SDLK_KP_ENTER:
 		case SDLK_RETURN:
-			end_modal<MenuTarget>(MenuTarget::kOk);
+			clicked_ok();
 			return true;
 		case SDLK_ESCAPE:
-			end_modal<MenuTarget>(MenuTarget::kBack);
+			clicked_cancel();
 			return true;
 		default:
 			break;
@@ -747,16 +713,13 @@ OptionsCtrl::OptionsStruct Options::get_values() {
 	if (language_dropdown_.has_selection()) {
 		os_.language = language_dropdown_.get_selected();
 	}
-	if (theme_dropdown_.has_selection()) {
-		os_.theme = theme_dropdown_.get_selected();
-	}
 	if (resolution_dropdown_.has_selection()) {
-		const int res_index = resolution_dropdown_.get_selected();
-		os_.fullscreen = res_index == kDropdownFullscreen;
-		os_.maximized = res_index == kDropdownMaximized;
-		if (res_index != kDropdownFullscreen && res_index != kDropdownMaximized) {
-			os_.xres = resolutions_[res_index].xres;
-			os_.yres = resolutions_[res_index].yres;
+		const ScreenResolution& res = resolution_dropdown_.get_selected();
+		os_.fullscreen = res.xres == kDropdownFullscreen;
+		os_.maximized = res.xres == kDropdownMaximized;
+		if (!os_.fullscreen && !os_.maximized) {
+			os_.xres = res.xres;
+			os_.yres = res.yres;
 		}
 	}
 	os_.inputgrab = inputgrab_.get_state();
@@ -781,10 +744,10 @@ OptionsCtrl::OptionsStruct Options::get_values() {
 	os_.auto_roadbuild_mode = auto_roadbuild_mode_.get_state();
 	os_.transparent_chat = transparent_chat_.get_state();
 	os_.single_watchwin = single_watchwin_.get_state();
-	os_.ctrl_zoom = ctrl_zoom_.get_state();
 	os_.game_clock = game_clock_.get_state();
 	os_.numpad_diagonalscrolling = numpad_diagonalscrolling_.get_state();
 	os_.edge_scrolling = edge_scrolling_.get_state();
+	os_.invert_movement = invert_movement_.get_state();
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	os_.training_wheels = training_wheels_.get_state();
 #endif
@@ -862,10 +825,10 @@ OptionsCtrl::OptionsStruct OptionsCtrl::options_struct(uint32_t active_tab) {
 	opt.auto_roadbuild_mode = opt_section_.get_bool("auto_roadbuild_mode", true);
 	opt.transparent_chat = opt_section_.get_bool("transparent_chat", true);
 	opt.single_watchwin = opt_section_.get_bool("single_watchwin", false);
-	opt.ctrl_zoom = opt_section_.get_bool("ctrl_zoom", false);
 	opt.game_clock = opt_section_.get_bool("game_clock", true);
 	opt.numpad_diagonalscrolling = opt_section_.get_bool("numpad_diagonalscrolling", false);
 	opt.edge_scrolling = opt_section_.get_bool("edge_scrolling", false);
+	opt.invert_movement = opt_section_.get_bool("invert_movement", false);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	opt.training_wheels = opt_section_.get_bool("training_wheels", true);
 #endif
@@ -876,8 +839,6 @@ OptionsCtrl::OptionsStruct OptionsCtrl::options_struct(uint32_t active_tab) {
 	// Language options
 	opt.language = opt_section_.get_string("language", "");
 
-	opt.theme = opt_section_.get_string("theme", "");
-
 	// Last tab for reloading the options menu
 	opt.active_tab = active_tab;
 	return opt;
@@ -913,10 +874,10 @@ void OptionsCtrl::save_options() {
 	opt_section_.set_bool("auto_roadbuild_mode", opt.auto_roadbuild_mode);
 	opt_section_.set_bool("transparent_chat", opt.transparent_chat);
 	opt_section_.set_bool("single_watchwin", opt.single_watchwin);
-	opt_section_.set_bool("ctrl_zoom", opt.ctrl_zoom);
 	opt_section_.set_bool("game_clock", opt.game_clock);
 	opt_section_.set_bool("numpad_diagonalscrolling", opt.numpad_diagonalscrolling);
 	opt_section_.set_bool("edge_scrolling", opt.edge_scrolling);
+	opt_section_.set_bool("invert_movement", opt.invert_movement);
 #if 0  // TODO(Nordfriese): Re-add training wheels code after v1.0
 	opt_section_.set_bool("training_wheels", opt.training_wheels);
 #endif
@@ -927,14 +888,10 @@ void OptionsCtrl::save_options() {
 	// Language options
 	opt_section_.set_string("language", opt.language);
 
-	opt_section_.set_string("theme", opt.theme);
-
 	WLApplication::get()->set_input_grab(opt.inputgrab);
 	g_mouse_cursor->set_use_sdl(opt_dialog_->get_values().sdl_cursor);
 	i18n::set_locale(opt.language);
 	UI::g_fh->reinitialize_fontset(i18n::get_locale());
-	set_template_dir(opt.theme);
-	parent_.get_topmost_forefather().template_directory_changed();
 
 	// Sound options
 	g_sh->save_config();
