@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,17 +12,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "wui/building_statistics_menu.h"
 
-#include <boost/algorithm/string.hpp>
-
 #include "base/i18n.h"
 #include "graphic/style_manager.h"
+#include "logic/game_data_error.h"
 #include "logic/map_objects/descriptions.h"
 #include "logic/map_objects/tribes/militarysite.h"
 #include "logic/map_objects/tribes/productionsite.h"
@@ -37,7 +35,7 @@ constexpr int kLabelHeight = 18;
 constexpr int kSpinboxWidth = 4 * kBuildGridCellWidth;
 constexpr int32_t kWindowWidth = kColumns * kBuildGridCellWidth;
 
-constexpr Duration kUpdateTimeInGametimeMs = Duration(1000);  //  1 second, gametime
+constexpr Duration kUpdateTimeInGametimeMs(1000);  //  1 second, gametime
 
 inline InteractivePlayer& BuildingStatisticsMenu::iplayer() const {
 	return dynamic_cast<InteractivePlayer&>(*get_parent());
@@ -346,6 +344,8 @@ void BuildingStatisticsMenu::init(int last_selected_tab) {
 	        "images/wui/fieldaction/menu_tab_buildport.png", _("Ports"));
 
 	update();
+
+	initialization_complete();
 }
 
 bool BuildingStatisticsMenu::own_building_is_valid(const Widelands::Player& player,
@@ -387,19 +387,19 @@ int BuildingStatisticsMenu::find_tab_for_building(const Widelands::BuildingDescr
 	}
 	if (descr.get_isport()) {
 		return BuildingTab::Ports;
-	} else {
-		switch (descr.get_size()) {
-		case Widelands::BaseImmovable::SMALL:
-			return BuildingTab::Small;
-		case Widelands::BaseImmovable::MEDIUM:
-			return BuildingTab::Medium;
-		case Widelands::BaseImmovable::BIG:
-			return BuildingTab::Big;
-		default:
-			throw wexception(
-			   "Building statictics: Found building without a size: %s", descr.name().c_str());
-		}
 	}
+	switch (descr.get_size()) {
+	case Widelands::BaseImmovable::SMALL:
+		return BuildingTab::Small;
+	case Widelands::BaseImmovable::MEDIUM:
+		return BuildingTab::Medium;
+	case Widelands::BaseImmovable::BIG:
+		return BuildingTab::Big;
+	default:
+		throw wexception(
+		   "Building statistics: Found building without a size: %s", descr.name().c_str());
+	}
+
 	NEVER_HERE();
 }
 
@@ -431,8 +431,8 @@ void BuildingStatisticsMenu::add_button(Widelands::DescriptionIndex id,
                                         UI::Box* row) {
 	UI::Box* button_box = new UI::Box(row, UI::PanelStyle::kWui, 0, 0, UI::Box::Vertical);
 	building_buttons_[id] =
-	   new UI::Button(button_box, (boost::format("building_button%s") % id).str(), 0, 0,
-	                  kBuildGridCellWidth, kBuildGridCellHeight, UI::ButtonStyle::kWuiBuildingStats,
+	   new UI::Button(button_box, format("building_button%d", id), 0, 0, kBuildGridCellWidth,
+	                  kBuildGridCellHeight, UI::ButtonStyle::kWuiBuildingStats,
 	                  descr.representative_image(&iplayer().get_player()->get_playercolor()), "",
 	                  UI::Button::VisualState::kFlat);
 	building_buttons_[id]->set_disable_style(UI::ButtonDisableStyle::kMonochrome |
@@ -512,7 +512,7 @@ void BuildingStatisticsMenu::jump_building(JumpTarget target, bool reverse) {
 				if (!stats_vector[last_building_index_].is_constructionsite) {
 					if (upcast(Widelands::MilitarySite, militarysite,
 					           map[stats_vector[last_building_index_].pos].get_immovable())) {
-						auto* soldier_control = militarysite->soldier_control();
+						const auto* soldier_control = militarysite->soldier_control();
 						assert(soldier_control != nullptr);
 						if (soldier_control->stationed_soldiers().size() <
 						    soldier_control->soldier_capacity()) {
@@ -534,7 +534,7 @@ void BuildingStatisticsMenu::jump_building(JumpTarget target, bool reverse) {
 				if (!stats_vector[last_building_index_].is_constructionsite) {
 					if (upcast(Widelands::MilitarySite, militarysite,
 					           map[stats_vector[last_building_index_].pos].get_immovable())) {
-						auto* soldier_control = militarysite->soldier_control();
+						const auto* soldier_control = militarysite->soldier_control();
 						assert(soldier_control != nullptr);
 						if (soldier_control->stationed_soldiers().size() <
 						    soldier_control->soldier_capacity()) {
@@ -555,7 +555,7 @@ void BuildingStatisticsMenu::jump_building(JumpTarget target, bool reverse) {
 		if (!found) {  // Now look at the old
 			if (upcast(Widelands::MilitarySite, militarysite,
 			           map[stats_vector[last_building_index_].pos].get_immovable())) {
-				auto* soldier_control = militarysite->soldier_control();
+				const auto* soldier_control = militarysite->soldier_control();
 				assert(soldier_control != nullptr);
 				if (soldier_control->stationed_soldiers().size() <
 				    soldier_control->soldier_capacity()) {
@@ -674,47 +674,44 @@ void BuildingStatisticsMenu::update() {
 
 		if (building.type() == Widelands::MapObjectType::PRODUCTIONSITE ||
 		    building.type() == Widelands::MapObjectType::TRAININGSITE) {
-			if (nr_owned) {
+			if (nr_owned != 0u) {
 				int const percent =
 				   static_cast<int>(static_cast<float>(total_prod) / static_cast<float>(nr_owned));
 
 				const RGBColor& color =
-				   (percent < low_production_) ?
-				      style_.low_color() :
-				      (percent < ((low_production_ < 50) ?
-				                     2 * low_production_ :
-				                     low_production_ + ((100 - low_production_) / 2))) ?
-				      style_.medium_color() :
-				      style_.high_color();
+				   (percent < low_production_) ? style_.low_color() :
+				   (percent < ((low_production_ < 50) ?
+                              2 * low_production_ :
+                              low_production_ + ((100 - low_production_) / 2))) ?
+                                             style_.medium_color() :
+                                             style_.high_color();
 
 				/** TRANSLATORS: Percent in building statistics window, e.g. 85% */
 				/** TRANSLATORS: If you wish to add a space, translate as '%i %%' */
-				const std::string perc_str = (boost::format(_("%i%%")) % percent).str();
+				const std::string perc_str = format(_("%i%%"), percent);
 				set_labeltext(productivity_labels_[id], perc_str, color);
 			}
 			if (has_selection_ && id == current_building_type_) {
 				label_nr_unproductive_.set_text(nr_unproductive > 0 ? std::to_string(nr_unproductive) :
-				                                                      "");
+                                                                  "");
 				b_next_unproductive_.set_enabled(nr_unproductive > 0);
 				b_prev_unproductive_.set_enabled(nr_unproductive > 0);
 				hbox_unproductive_.set_visible(true);
 				label_unproductive_.set_text(_("Low productivity:"));
 			}
 		} else if (building.type() == Widelands::MapObjectType::MILITARYSITE) {
-			if (nr_owned) {
-				const RGBColor& color = (total_stationed_soldiers < total_soldier_capacity / 2) ?
-				                           style_.low_color() :
-				                           (total_stationed_soldiers < total_soldier_capacity) ?
-				                           style_.medium_color() :
-				                           style_.high_color();
+			if (nr_owned != 0u) {
+				const RGBColor& color =
+				   (total_stationed_soldiers < total_soldier_capacity / 2) ? style_.low_color() :
+				   (total_stationed_soldiers < total_soldier_capacity)     ? style_.medium_color() :
+                                                                         style_.high_color();
 				const std::string perc_str =
-				   (boost::format(_("%1%/%2%")) % total_stationed_soldiers % total_soldier_capacity)
-				      .str();
+				   format(_("%1%/%2%"), total_stationed_soldiers, total_soldier_capacity);
 				set_labeltext(productivity_labels_[id], perc_str, color);
 			}
 			if (has_selection_ && id == current_building_type_) {
 				label_nr_unproductive_.set_text(nr_unproductive > 0 ? std::to_string(nr_unproductive) :
-				                                                      "");
+                                                                  "");
 				b_next_unproductive_.set_enabled(total_soldier_capacity > total_stationed_soldiers);
 				b_prev_unproductive_.set_enabled(total_soldier_capacity > total_stationed_soldiers);
 				hbox_unproductive_.set_visible(true);
@@ -728,9 +725,9 @@ void BuildingStatisticsMenu::update() {
 		   player.tribe().has_building(id) && (building.is_buildable() || building.is_enhanced());
 		if (can_construct_this_building) {
 			/** TRANSLATORS: Buildings: owned / under construction */
-			owned_text = (boost::format(_("%1%/%2%")) % nr_owned % nr_build).str();
+			owned_text = format(_("%1%/%2%"), nr_owned, nr_build);
 		} else {
-			owned_text = (boost::format(_("%1%/%2%")) % nr_owned % "–").str();
+			owned_text = format(_("%1%/%2%"), nr_owned, "–");
 		}
 		set_labeltext(
 		   owned_labels_[id], owned_text, style_.building_statistics_details_font().color());
@@ -786,3 +783,41 @@ void BuildingStatisticsMenu::low_production_changed() {
 	low_production_ = unproductive_threshold_.get_value();
 	update();
 }
+
+constexpr uint16_t kCurrentPacketVersion = 1;
+UI::Window& BuildingStatisticsMenu::load(FileRead& fr, InteractiveBase& ib) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersion) {
+			UI::UniqueWindow::Registry& r =
+			   dynamic_cast<InteractivePlayer&>(ib).menu_windows_.stats_buildings;
+			r.create();
+			assert(r.window);
+			BuildingStatisticsMenu& m = dynamic_cast<BuildingStatisticsMenu&>(*r.window);
+			m.unproductive_threshold_.set_value(fr.unsigned_8());
+			m.low_production_changed();
+			m.tab_panel_.activate(fr.unsigned_8());
+			const std::string sel = fr.string();
+			if (!sel.empty()) {
+				m.set_current_building_type(ib.egbase().descriptions().safe_building_index(sel));
+			}
+			m.last_building_index_ = fr.signed_32();
+			return m;
+		}
+		throw Widelands::UnhandledVersionError(
+		   "Building Statistics Menu", packet_version, kCurrentPacketVersion);
+
+	} catch (const WException& e) {
+		throw Widelands::GameDataError("building statistics menu: %s", e.what());
+	}
+}
+void BuildingStatisticsMenu::save(FileWrite& fw, Widelands::MapObjectSaver& /* mos */) const {
+	fw.unsigned_16(kCurrentPacketVersion);
+	fw.unsigned_8(low_production_);
+	fw.unsigned_8(tab_panel_.active());
+	fw.string(
+	   current_building_type_ == Widelands::INVALID_INDEX ?
+         "" :
+         iplayer().egbase().descriptions().get_building_descr(current_building_type_)->name());
+	fw.signed_32(last_building_index_);
+}
