@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,17 +12,18 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #include "wui/buildingwindow.h"
 
 #include "base/macros.h"
+#include "base/multithreading.h"
 #include "graphic/image.h"
 #include "graphic/rendertarget.h"
 #include "graphic/style_manager.h"
+#include "logic/game_data_error.h"
 #include "logic/map_objects/tribes/constructionsite.h"
 #include "logic/map_objects/tribes/dismantlesite.h"
 #include "logic/map_objects/tribes/militarysite.h"
@@ -88,8 +89,12 @@ void BuildingWindow::on_building_note(const Widelands::NoteBuilding& note) {
 		case Widelands::NoteBuilding::Action::kChanged:
 			if (!is_dying_) {
 				const std::string active_tab = tabs_->tabs()[tabs_->active()]->get_name();
-				init(true, showing_workarea_);
-				tabs_->activate(active_tab);
+				NoteThreadSafeFunction::instantiate(
+				   [this, active_tab]() {
+					   init(true, showing_workarea_);
+					   tabs_->activate(active_tab);
+				   },
+				   true);
 			}
 			break;
 		// The building is no more. Next think() will call die().
@@ -186,14 +191,14 @@ void BuildingWindow::think() {
 		caps_setup_ = true;
 	}
 
-	if (mute_this_) {
-		if (!mute_all_) {
+	if (mute_this_ != nullptr) {
+		if (mute_all_ == nullptr) {
 			NEVER_HERE();
 		}
 		mute_this_->set_pic(
 		   g_image_cache->get(building->mute_messages() ? kImgUnmuteThis : kImgMuteThis));
 		mute_this_->set_tooltip(building->mute_messages() ? _("Muted – click to unmute") :
-		                                                    _("Mute this building’s messages"));
+                                                          _("Mute this building’s messages"));
 		if (building->owner().is_muted(
 		       building->owner().tribe().safe_building_index(building->descr().name()))) {
 			mute_this_->set_enabled(false);
@@ -265,10 +270,10 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 			   g_image_cache->get(
 			      (is_stopped ? "images/ui_basic/continue.png" : "images/ui_basic/stop.png")),
 			   is_stopped ?
-			      /** TRANSLATORS: Stop/Continue toggle button for production sites. */
-			      _("Continue") :
-			      /** TRANSLATORS: Stop/Continue toggle button for production sites. */
-			      _("Stop"));
+               /** TRANSLATORS: Stop/Continue toggle button for production sites. */
+               _("Continue") :
+               /** TRANSLATORS: Stop/Continue toggle button for production sites. */
+               _("Stop"));
 			stopbtn->sigclicked.connect([this]() { act_start_stop(); });
 			capsbuttons->add(stopbtn);
 
@@ -281,17 +286,16 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 		}  // upcast to productionsite
 
 		upcast(Widelands::ConstructionSite, cs, building);
-		if ((capscache_ & Widelands::Building::PCap_Enhancable) ||
-		    (cs && cs->get_info().becomes->enhancement() != Widelands::INVALID_INDEX)) {
+		if (((capscache_ & Widelands::Building::PCap_Enhancable) != 0u) ||
+		    ((cs != nullptr) && cs->get_info().becomes->enhancement() != Widelands::INVALID_INDEX)) {
 			const Widelands::DescriptionIndex& enhancement =
-			   cs ? cs->get_info().becomes->enhancement() : building->descr().enhancement();
+			   cs != nullptr ? cs->get_info().becomes->enhancement() : building->descr().enhancement();
 			const Widelands::TribeDescr& tribe = owner.tribe();
 			if (owner.is_building_type_allowed(enhancement) &&
 			    owner.tribe().has_building(enhancement)) {
 				const Widelands::BuildingDescr& building_descr = *tribe.get_building_descr(enhancement);
 				std::string enhance_tooltip =
-				   (boost::format(_("Enhance to %s")) % building_descr.descname().c_str()).str() +
-				   "<br>" +
+				   format(_("Enhance to %s"), building_descr.descname()) + "<br>" +
 				   g_style_manager->ware_info_style(UI::WareInfoStyle::kNormal)
 				      .header_font()
 				      .as_font_tag(_("Construction costs:")) +
@@ -303,13 +307,13 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 
 				//  button id = building id
 				enhancebtn->sigclicked.connect(
-				   [this, enhancement, cs] { act_enhance(enhancement, cs); });
+				   [this, enhancement, cs] { act_enhance(enhancement, cs != nullptr); });
 				capsbuttons->add(enhancebtn);
 				requires_destruction_separator = true;
 			}
 		}
 
-		if (capscache_ & Widelands::Building::PCap_Bulldoze) {
+		if ((capscache_ & Widelands::Building::PCap_Bulldoze) != 0u) {
 			UI::Button* destroybtn =
 			   new UI::Button(capsbuttons, "destroy", 0, 0, 34, 34, UI::ButtonStyle::kWuiMenu,
 			                  g_image_cache->get(kImgBulldoze), _("Destroy"));
@@ -319,7 +323,7 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 			requires_destruction_separator = true;
 		}
 
-		if (capscache_ & Widelands::Building::PCap_Dismantle) {
+		if ((capscache_ & Widelands::Building::PCap_Dismantle) != 0u) {
 			if (building->descr().can_be_dismantled()) {
 				const Widelands::Buildcost wares =
 				   Widelands::DismantleSite::count_returned_wares(building);
@@ -390,7 +394,7 @@ void BuildingWindow::create_capsbuttons(UI::Box* capsbuttons, Widelands::Buildin
 		if (building->owner().tribe().safe_building_index(building->descr().name()) ==
 		    building->owner().tribe().scouts_house()) {
 			watch_button_ = new UI::Button(
-			   capsbuttons, "goto", 0, 0, 34, 34, UI::ButtonStyle::kWuiMenu,
+			   capsbuttons, "watchscout", 0, 0, 34, 34, UI::ButtonStyle::kWuiMenu,
 			   g_image_cache->get("images/wui/menus/watch_follow.png"), _("Watch the scout"));
 			watch_button_->sigclicked.connect([this]() { clicked_watch(); });
 			capsbuttons->add(watch_button_);
@@ -442,8 +446,8 @@ void BuildingWindow::act_bulldoze() {
 		return;
 	}
 
-	if (SDL_GetModState() & KMOD_CTRL) {
-		if (building->get_playercaps() & Widelands::Building::PCap_Bulldoze) {
+	if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+		if ((building->get_playercaps() & Widelands::Building::PCap_Bulldoze) != 0u) {
 			game_->send_player_bulldoze(*building);
 		}
 	} else if (upcast(InteractivePlayer, ipl, ibase())) {
@@ -458,9 +462,13 @@ Callback for dismantling request
 */
 void BuildingWindow::act_dismantle() {
 	Widelands::Building* building = building_.get(parent_->egbase());
-	if (SDL_GetModState() & KMOD_CTRL) {
-		if (building->get_playercaps() & Widelands::Building::PCap_Dismantle) {
-			if (game_) {
+	if (building == nullptr) {
+		return;
+	}
+
+	if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+		if ((building->get_playercaps() & Widelands::Building::PCap_Dismantle) != 0u) {
+			if (game_ != nullptr) {
 				game_->send_player_dismantle(*building, (SDL_GetModState() & KMOD_SHIFT) == 0);
 			} else {
 				NEVER_HERE();  // TODO(Nordfriese / Scenario Editor): implement
@@ -484,7 +492,7 @@ void BuildingWindow::act_start_stop() {
 	}
 
 	if (building->descr().type() >= Widelands::MapObjectType::PRODUCTIONSITE) {
-		if (game_) {
+		if (game_ != nullptr) {
 			game_->send_player_start_stop_building(*building);
 		} else {
 			NEVER_HERE();  // TODO(Nordfriese / Scenario Editor): implement
@@ -504,9 +512,9 @@ void BuildingWindow::act_start_or_cancel_expedition() {
 	}
 
 	if (upcast(Widelands::Warehouse const, warehouse, building)) {
-		if (warehouse->get_portdock()) {
+		if (warehouse->get_portdock() != nullptr) {
 			expeditionbtn_->set_enabled(false);
-			if (game_) {
+			if (game_ != nullptr) {
 				game_->send_player_start_or_cancel_expedition(*building);
 			} else {
 				NEVER_HERE();  // TODO(Nordfriese / Scenario Editor): implement
@@ -532,8 +540,8 @@ void BuildingWindow::act_enhance(Widelands::DescriptionIndex id, bool csite) {
 	if (csite) {
 		upcast(Widelands::ConstructionSite, construction_site, building);
 		assert(construction_site);
-		if (SDL_GetModState() & KMOD_CTRL) {
-			if (game_) {
+		if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+			if (game_ != nullptr) {
 				game_->send_player_enhance_building(*construction_site,
 				                                    construction_site->building().enhancement(),
 				                                    (SDL_GetModState() & KMOD_SHIFT) == 0);
@@ -547,9 +555,9 @@ void BuildingWindow::act_enhance(Widelands::DescriptionIndex id, bool csite) {
 		return;
 	}
 
-	if (SDL_GetModState() & KMOD_CTRL) {
-		if (building->get_playercaps() & Widelands::Building::PCap_Enhancable) {
-			if (game_) {
+	if ((SDL_GetModState() & KMOD_CTRL) != 0) {
+		if ((building->get_playercaps() & Widelands::Building::PCap_Enhancable) != 0u) {
+			if (game_ != nullptr) {
 				game_->send_player_enhance_building(
 				   *building, id, (SDL_GetModState() & KMOD_SHIFT) == 0);
 			} else {
@@ -563,7 +571,7 @@ void BuildingWindow::act_enhance(Widelands::DescriptionIndex id, bool csite) {
 
 void BuildingWindow::act_mute(bool all) {
 	if (Widelands::Building* building = building_.get(parent_->egbase())) {
-		if (game_) {
+		if (game_ != nullptr) {
 			game_->send_player_toggle_mute(*building, all);
 		} else {
 			NEVER_HERE();  // TODO(Nordfriese / Scenario Editor): implement
@@ -627,7 +635,7 @@ void BuildingWindow::hide_workarea(bool configure_button) {
  * Sets the perpressed_ state and the tooltip.
  */
 void BuildingWindow::configure_workarea_button() {
-	if (toggle_workarea_) {
+	if (toggle_workarea_ != nullptr) {
 		if (showing_workarea_) {
 			toggle_workarea_->set_tooltip(_("Hide work area"));
 			toggle_workarea_->set_perm_pressed(true);
@@ -666,3 +674,31 @@ void BuildingWindow::update_expedition_button(bool expedition_was_canceled) {
 	}
 	expeditionbtn_->set_enabled(true);
 }
+
+constexpr uint16_t kCurrentPacketVersion = 1;
+UI::Window& BuildingWindow::load(FileRead& fr, InteractiveBase& ib) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersion) {
+			const int32_t x = fr.signed_32();
+			const int32_t y = fr.signed_32();
+			const bool workarea = fr.unsigned_8() != 0u;
+			BuildingWindow& bw = dynamic_cast<BuildingWindow&>(
+			   *ib.show_building_window(Widelands::Coords(x, y), true, workarea));
+			bw.tabs_->activate(fr.unsigned_8());
+			return bw;
+		}
+		throw Widelands::UnhandledVersionError(
+		   "Building Window", packet_version, kCurrentPacketVersion);
+
+	} catch (const WException& e) {
+		throw Widelands::GameDataError("building window: %s", e.what());
+	}
+}
+void BuildingWindow::save(FileWrite& fw, Widelands::MapObjectSaver& /* mos */) const {
+	fw.unsigned_16(kCurrentPacketVersion);
+	fw.signed_32(building_position_.x);
+	fw.signed_32(building_position_.y);
+	fw.unsigned_8(showing_workarea_ ? 1 : 0);
+	fw.unsigned_8(tabs_->active());
+}
