@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2021 by the Widelands Development Team
+ * Copyright (C) 2008-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -21,15 +20,20 @@
 
 #include <memory>
 
+#include "base/log.h"
 #include "graphic/font_handler.h"
 #include "graphic/style_manager.h"
 #include "io/profile.h"
 #include "logic/filesystem_constants.h"
+#include "logic/game_data_error.h"
 #include "logic/map_objects/tribes/ware_descr.h"
 #include "logic/map_objects/tribes/worker_descr.h"
 #include "logic/player.h"
 #include "logic/playercommand.h"
+#include "map_io/map_object_loader.h"
+#include "map_io/map_object_saver.h"
 #include "ui_basic/messagebox.h"
+#include "wui/interactive_base.h"
 
 static const char pic_tab_wares[] = "images/wui/buildings/menu_tab_wares.png";
 static const char pic_tab_workers[] = "images/wui/buildings/menu_tab_workers.png";
@@ -147,13 +151,16 @@ EconomyOptionsWindow::EconomyOptionsWindow(Panel* parent,
 			   return;
 		   }
 		   read_targets();
-		   if (save_profile_dialog_) {
+		   if (save_profile_dialog_ != nullptr) {
 			   save_profile_dialog_->update_table();
 		   }
 	   });
 
 	read_targets();
+
 	activate_tab(type);
+
+	initialization_complete();
 }
 
 EconomyOptionsWindow::~EconomyOptionsWindow() {
@@ -163,42 +170,44 @@ EconomyOptionsWindow::~EconomyOptionsWindow() {
 	if (Widelands::Economy* e_wo = player_->get_economy(worker_serial_)) {
 		e_wo->set_options_window(nullptr);
 	}
-	if (save_profile_dialog_) {
+	if (save_profile_dialog_ != nullptr) {
 		save_profile_dialog_->unset_parent();
 	}
 }
 
-void EconomyOptionsWindow::create(Panel* parent,
-                                  Widelands::Descriptions* descriptions,
-                                  const Widelands::Flag& flag,
-                                  Widelands::WareWorker type,
-                                  bool can_act) {
+EconomyOptionsWindow& EconomyOptionsWindow::create(Panel* parent,
+                                                   Widelands::Descriptions* descriptions,
+                                                   const Widelands::Flag& flag,
+                                                   Widelands::WareWorker type,
+                                                   bool can_act) {
 	Widelands::Economy* ware_economy = flag.get_economy(Widelands::wwWARE);
 	Widelands::Economy* worker_economy = flag.get_economy(Widelands::wwWORKER);
-	bool window_open = false;
-	if (ware_economy->get_options_window()) {
-		window_open = true;
+	EconomyOptionsWindow* window_open = nullptr;
+	if (ware_economy->get_options_window() != nullptr) {
 		EconomyOptionsWindow& window =
 		   *static_cast<EconomyOptionsWindow*>(ware_economy->get_options_window());
+		window_open = &window;
 		window.activate_tab(type);
 		if (window.is_minimal()) {
 			window.restore();
 		}
 		window.move_to_top();
 	}
-	if (worker_economy->get_options_window()) {
-		window_open = true;
+	if (worker_economy->get_options_window() != nullptr) {
 		EconomyOptionsWindow& window =
 		   *static_cast<EconomyOptionsWindow*>(worker_economy->get_options_window());
+		window_open = &window;
 		window.activate_tab(type);
 		if (window.is_minimal()) {
 			window.restore();
 		}
 		window.move_to_top();
 	}
-	if (!window_open) {
-		new EconomyOptionsWindow(parent, descriptions, ware_economy, worker_economy, type, can_act);
+	if (window_open != nullptr) {
+		return *window_open;
 	}
+	return *new EconomyOptionsWindow(
+	   parent, descriptions, ware_economy, worker_economy, type, can_act);
 }
 void EconomyOptionsWindow::activate_tab(Widelands::WareWorker type) {
 	tabpanel_.activate(type == Widelands::WareWorker::wwWARE ? "wares" : "workers");
@@ -216,10 +225,10 @@ std::string EconomyOptionsWindow::localize_profile_name(const std::string& name)
 }
 
 void EconomyOptionsWindow::on_economy_note(const Widelands::NoteEconomy& note) {
-	Widelands::Serial* serial = note.old_economy == ware_serial_ ?
-	                               &ware_serial_ :
-	                               note.old_economy == worker_serial_ ? &worker_serial_ : nullptr;
-	if (serial) {
+	Widelands::Serial* serial = note.old_economy == ware_serial_   ? &ware_serial_ :
+	                            note.old_economy == worker_serial_ ? &worker_serial_ :
+                                                                    nullptr;
+	if (serial != nullptr) {
 		switch (note.action) {
 		case Widelands::NoteEconomy::Action::kMerged: {
 			*serial = note.new_economy;
@@ -241,7 +250,8 @@ void EconomyOptionsWindow::on_economy_note(const Widelands::NoteEconomy& note) {
 }
 
 void EconomyOptionsWindow::layout() {
-	int w, h;
+	int w;
+	int h;
 	tabpanel_.get_desired_size(&w, &h);
 	main_box_.set_desired_size(w, h + 78);
 	update_desired_size();
@@ -298,7 +308,7 @@ EconomyOptionsWindow::TargetWaresDisplay::info_for_ware(Widelands::DescriptionIn
 		/** TRANSLATORS: Infinite number of wares or workers */
 		return g_style_manager->font_style(UI::FontStyle::kWuiLabel).as_font_tag(_("∞"));
 	}
-	return boost::lexical_cast<std::string>(amount);
+	return as_string(amount);
 }
 
 /**
@@ -356,7 +366,7 @@ void EconomyOptionsWindow::reset_target() {
 
 void EconomyOptionsWindow::EconomyOptionsPanel::toggle_infinite() {
 	Widelands::Economy* economy = player_->get_economy(serial_);
-	if (!economy) {
+	if (economy == nullptr) {
 		return die();
 	}
 	assert(economy->type() == type_);
@@ -375,7 +385,7 @@ void EconomyOptionsWindow::EconomyOptionsPanel::toggle_infinite() {
 					new_quantity = is_wares ? economy_options_window_->get_predefined_targets()
 					                             .at(kDefaultEconomyProfile)
 					                             .wares.at(index) :
-					                          economy_options_window_->get_predefined_targets()
+                                         economy_options_window_->get_predefined_targets()
 					                             .at(kDefaultEconomyProfile)
 					                             .workers.at(index);
 				} else {
@@ -404,7 +414,7 @@ void EconomyOptionsWindow::EconomyOptionsPanel::change_target(int delta) {
 		return;
 	}
 	Widelands::Economy* economy = player_->get_economy(serial_);
-	if (!economy) {
+	if (economy == nullptr) {
 		return die();
 	}
 	assert(economy->type() == type_);
@@ -454,24 +464,31 @@ void EconomyOptionsWindow::EconomyOptionsPanel::reset_target() {
 		if (display_.ware_selected(index) ||
 		    (!anything_selected && !display_.is_ware_hidden(index))) {
 			if (is_wares) {
+				auto setting = settings.wares.find(index);
+				if (setting == settings.wares.end()) {
+					continue;
+				}
 				game.send_player_command(new Widelands::CmdSetWareTargetQuantity(
-				   game.get_gametime(), player_->player_number(), serial_, index,
-				   settings.wares.at(index)));
+				   game.get_gametime(), player_->player_number(), serial_, index, setting->second));
 			} else {
+				auto setting = settings.workers.find(index);
+				if (setting == settings.workers.end()) {
+					continue;
+				}
 				game.send_player_command(new Widelands::CmdSetWorkerTargetQuantity(
-				   game.get_gametime(), player_->player_number(), serial_, index,
-				   settings.workers.at(index)));
+				   game.get_gametime(), player_->player_number(), serial_, index, setting->second));
 			}
 		}
 	}
 }
 
-constexpr Duration kThinkInterval = Duration(200);
+constexpr Duration kThinkInterval(200);
 
 void EconomyOptionsWindow::think() {
 	const Time& time = player_->egbase().get_gametime();
-	if (time - time_last_thought_ < kThinkInterval || !player_->get_economy(ware_serial_) ||
-	    !player_->get_economy(worker_serial_)) {
+	if (time - time_last_thought_ < kThinkInterval ||
+	    (player_->get_economy(ware_serial_) == nullptr) ||
+	    (player_->get_economy(worker_serial_) == nullptr)) {
 		// If our economy has been deleted, die() was already called, no need to do anything
 		return;
 	}
@@ -560,7 +577,7 @@ void EconomyOptionsWindow::SaveProfileWindow::update_save_enabled() {
 	if (text.empty() || text == kDefaultEconomyProfile) {
 		save_.set_enabled(false);
 		save_.set_tooltip(text.empty() ? _("The profile name cannot be empty") :
-		                                 _("The default profile cannot be overwritten"));
+                                       _("The default profile cannot be overwritten"));
 	} else {
 		save_.set_enabled(true);
 		save_.set_tooltip(_("Save the profile under this name"));
@@ -593,7 +610,7 @@ void EconomyOptionsWindow::SaveProfileWindow::update_table() {
 	layout();
 }
 
-void EconomyOptionsWindow::SaveProfileWindow::save() {
+void EconomyOptionsWindow::SaveProfileWindow::save_profile() {
 	const std::string name = profile_name_.text();
 	assert(!name.empty());
 	assert(name != kDefaultEconomyProfile);
@@ -636,7 +653,7 @@ void EconomyOptionsWindow::SaveProfileWindow::unset_parent() {
 }
 
 void EconomyOptionsWindow::SaveProfileWindow::think() {
-	if (!economy_options_) {
+	if (economy_options_ == nullptr) {
 		die();
 	}
 	UI::Window::think();
@@ -664,11 +681,11 @@ EconomyOptionsWindow::SaveProfileWindow::SaveProfileWindow(UI::Panel* parent,
 	table_.add_column(200, _("Existing Profiles"));
 	update_table();
 
-	table_.selected.connect([this](uint32_t) { table_selection_changed(); });
+	table_.selected.connect([this](uint32_t /* index */) { table_selection_changed(); });
 	profile_name_.changed.connect([this] { update_save_enabled(); });
-	profile_name_.ok.connect([this] { save(); });
+	profile_name_.ok.connect([this] { save_profile(); });
 	profile_name_.cancel.connect([this] { die(); });
-	save_.sigclicked.connect([this] { save(); });
+	save_.sigclicked.connect([this] { save_profile(); });
 	cancel_.sigclicked.connect([this] { die(); });
 	delete_.sigclicked.connect([this] { delete_selected(); });
 
@@ -685,16 +702,18 @@ EconomyOptionsWindow::SaveProfileWindow::SaveProfileWindow(UI::Panel* parent,
 
 	table_selection_changed();
 	update_save_enabled();
+
+	initialization_complete();
 }
 
 EconomyOptionsWindow::SaveProfileWindow::~SaveProfileWindow() {
-	if (economy_options_) {
+	if (economy_options_ != nullptr) {
 		economy_options_->close_save_profile_window();
 	}
 }
 
 void EconomyOptionsWindow::create_target() {
-	if (save_profile_dialog_) {
+	if (save_profile_dialog_ != nullptr) {
 		// Already open
 		return;
 	}
@@ -804,7 +823,7 @@ void EconomyOptionsWindow::read_targets() {
 	profile.read(complete_filename.c_str());
 
 	Section* global_section = profile.get_section(kDefaultEconomyProfile);
-	if (global_section) {
+	if (global_section != nullptr) {
 		std::map<std::string, std::string> serials;
 		while (Section::Value* v = global_section->get_next_val()) {
 			serials.insert(std::make_pair(v->get_name(), v->get_string()));
@@ -815,16 +834,21 @@ void EconomyOptionsWindow::read_targets() {
 			PredefinedTargets t;
 			while (Section::Value* v = section->get_next_val()) {
 				const std::string name(v->get_name());
-				const std::pair<Widelands::WareWorker, Widelands::DescriptionIndex> wareworker =
-				   descriptions_->load_ware_or_worker(name);
-				assert(wareworker.second != Widelands::INVALID_INDEX);
-				switch (wareworker.first) {
-				case Widelands::WareWorker::wwWARE:
-					t.wares.insert(std::make_pair(wareworker.second, v->get_natural()));
-					break;
-				case Widelands::WareWorker::wwWORKER:
-					t.workers.insert(std::make_pair(wareworker.second, v->get_natural()));
-					break;
+				try {
+					const std::pair<Widelands::WareWorker, Widelands::DescriptionIndex> wareworker =
+					   descriptions_->load_ware_or_worker(name);
+					assert(wareworker.second != Widelands::INVALID_INDEX);
+					switch (wareworker.first) {
+					case Widelands::WareWorker::wwWARE:
+						t.wares.insert(std::make_pair(wareworker.second, v->get_natural()));
+						break;
+					case Widelands::WareWorker::wwWORKER:
+						t.workers.insert(std::make_pair(wareworker.second, v->get_natural()));
+						break;
+					}
+				} catch (const Widelands::GameDataError&) {
+					log_warn("Unknown ware or worker '%s' in economy profile '%s'", name.c_str(),
+					         pair.second.c_str());
 				}
 			}
 			predefined_targets_.insert(std::make_pair(pair.second, t));
@@ -840,3 +864,47 @@ void EconomyOptionsWindow::read_targets() {
 
 	update_profiles();
 }
+
+constexpr uint16_t kCurrentPacketVersion = 1;
+UI::Window*
+EconomyOptionsWindow::load(FileRead& fr, InteractiveBase& ib, Widelands::MapObjectLoader& mol) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersion) {
+			const uint32_t flag_serial = fr.unsigned_32();
+			if (flag_serial == 0) {
+				return nullptr;
+			}
+			Widelands::Flag& flag = mol.get<Widelands::Flag>(flag_serial);
+			return &create(&ib, ib.egbase().mutable_descriptions(), flag,
+			               fr.unsigned_8() > 0 ? Widelands::wwWORKER : Widelands::wwWARE,
+			               ib.can_act(flag.owner().player_number()));
+		}
+		throw Widelands::UnhandledVersionError(
+		   "Economy Options Window", packet_version, kCurrentPacketVersion);
+
+	} catch (const WException& e) {
+		throw Widelands::GameDataError("economy options window: %s", e.what());
+	}
+}
+void EconomyOptionsWindow::save(FileWrite& fw, Widelands::MapObjectSaver& mos) const {
+	fw.unsigned_16(kCurrentPacketVersion);
+	Widelands::Economy* e_wa = player_->get_economy(ware_serial_);
+	Widelands::Economy* e_wo = player_->get_economy(worker_serial_);
+	if ((e_wa == nullptr) || (e_wo == nullptr)) {
+		fw.unsigned_32(0);
+		return;
+	}
+	Widelands::Flag* f = e_wa->get_arbitrary_flag(e_wo);
+	if (f == nullptr) {
+		log_warn("EconomyOptionsWindow::save: No flag exists in both economies (%u & %u)",
+		         ware_serial_, worker_serial_);
+		f = e_wa->get_arbitrary_flag();
+	}
+	if (f == nullptr) {
+		fw.unsigned_32(0);
+		return;
+	}
+	fw.unsigned_32(mos.get_object_file_index(*f));
+	fw.unsigned_8(tabpanel_.active());
+}
