@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,7 +27,6 @@
 #include "graphic/style_manager.h"
 #include "logic/cmd_queue.h"
 #include "logic/map_objects/checkstep.h"
-#include "logic/map_objects/tribes/attack_target.h"
 #include "logic/map_objects/tribes/soldier.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/map_objects/tribes/warehouse.h"
@@ -41,7 +39,6 @@
 #include "ui_basic/tabpanel.h"
 #include "ui_basic/unique_window.h"
 #include "wui/actionconfirm.h"
-#include "wui/attack_box.h"
 #include "wui/economy_options_window.h"
 #include "wui/game_debug_ui.h"
 #include "wui/interactive_player.h"
@@ -61,18 +58,17 @@ constexpr int kBuildGridCellSize = 50;
 struct BuildGrid : public UI::IconGrid {
 	BuildGrid(UI::Panel* parent, Widelands::Player* plr, int32_t x, int32_t y, int32_t cols);
 
-	boost::signals2::signal<void(Widelands::DescriptionIndex)> buildclicked;
-	boost::signals2::signal<void(Widelands::DescriptionIndex)> buildmouseout;
-	boost::signals2::signal<void(Widelands::DescriptionIndex)> buildmousein;
+	Notifications::Signal<Widelands::DescriptionIndex> buildclicked;
+	Notifications::Signal<Widelands::DescriptionIndex> buildmouseout;
+	Notifications::Signal<Widelands::DescriptionIndex> buildmousein;
 
-	void add(Widelands::DescriptionIndex);
+	void add(Widelands::DescriptionIndex /*id*/);
 
 private:
 	void click_slot(int32_t idx);
 	void mouseout_slot(int32_t idx);
 	void mousein_slot(int32_t idx);
 
-private:
 	Widelands::Player* plr_;
 };
 
@@ -163,7 +159,6 @@ public:
 	void add_buttons_build(int32_t buildcaps, int32_t max_nodecaps);
 	void add_buttons_road(bool flag);
 	void add_buttons_waterway(bool flag);
-	void add_buttons_attack();
 
 	void act_watch();
 	void act_debug();
@@ -178,24 +173,20 @@ public:
 	void act_abort_buildwaterway();
 	void act_abort_buildwaterway_and_start_buildroad();
 	void act_removewaterway();
-	void act_build(Widelands::DescriptionIndex);
-	void building_icon_mouse_out(Widelands::DescriptionIndex);
-	void building_icon_mouse_in(Widelands::DescriptionIndex);
+	void act_build(Widelands::DescriptionIndex /* idx */);
+	void building_icon_mouse_out(Widelands::DescriptionIndex /* idx */);
+	void building_icon_mouse_in(Widelands::DescriptionIndex /* idx */);
 	void act_geologist();
 	void act_scout();
 	void act_mark_removal();
 	void act_unmark_removal();
-	void act_attack();  /// Launch the attack
-
-	/// Total number of attackers available for a specific enemy flag
-	uint32_t get_max_attackers();
 
 private:
 	uint32_t add_tab(const std::string& name,
 	                 const char* picname,
 	                 UI::Panel* panel,
 	                 const std::string& tooltip_text = "");
-	UI::Button& add_button(UI::Box*,
+	UI::Button& add_button(UI::Box* /*box*/,
 	                       const char* name,
 	                       const char* picname,
 	                       void (FieldActionWindow::*fn)(),
@@ -218,9 +209,6 @@ private:
 	std::set<Widelands::Coords> overlapping_workareas_;
 	bool is_showing_workarea_overlaps_;
 	Widelands::DescriptionIndex building_under_mouse_;
-
-	/// Variables to use with attack dialog.
-	AttackBox* attack_box_;
 };
 
 constexpr const char* const kImgTabBuildroad = "images/wui/fieldaction/menu_tab_buildroad.png";
@@ -254,7 +242,6 @@ constexpr const char* const kImgButtonUnmarkRemoval =
    "images/wui/fieldaction/menu_unmark_removal.png";
 
 constexpr const char* const kImgTabTarget = "images/wui/fieldaction/menu_tab_target.png";
-constexpr const char* const kImgTabAttack = "images/wui/fieldaction/menu_tab_attack.png";
 
 /*
 ===============
@@ -273,8 +260,7 @@ FieldActionWindow::FieldActionWindow(InteractiveBase* const ib,
      best_tab_(0),
      showing_workarea_preview_(false),
      is_showing_workarea_overlaps_(ib->get_display_flag(InteractiveBase::dfShowWorkareaOverlap)),
-     building_under_mouse_(Widelands::INVALID_INDEX),
-     attack_box_(nullptr) {
+     building_under_mouse_(Widelands::INVALID_INDEX) {
 	ib->set_sel_freeze(true);
 	set_center_panel(&tabpanel_);
 }
@@ -286,7 +272,6 @@ FieldActionWindow::~FieldActionWindow() {
 	showing_workarea_preview_ = false;
 	clear_overlapping_workareas();
 	ibase().set_sel_freeze(false);
-	delete attack_box_;
 }
 
 void FieldActionWindow::think() {
@@ -304,7 +289,7 @@ void FieldActionWindow::think() {
 			}
 		}
 	}
-	if (player_ && !player_->is_seeing(node_.field - &ibase().egbase().map()[0]) &&
+	if ((player_ != nullptr) && !player_->is_seeing(node_.field - &ibase().egbase().map()[0]) &&
 	    !player_->see_all()) {
 		die();
 	}
@@ -333,6 +318,8 @@ void FieldActionWindow::init() {
 	// Will only do something if we explicitly set another fast click panel
 	// than the first button
 	warp_mouse_to_fastclick_panel();
+
+	initialization_complete();
 }
 
 static bool suited_for_targeting(Widelands::PlayerNumber p,
@@ -353,10 +340,10 @@ static bool suited_for_targeting(Widelands::PlayerNumber p,
 
 			const Widelands::BuildingDescr& descr =
 			   mo->descr().type() == Widelands::MapObjectType::CONSTRUCTIONSITE ?
-			      dynamic_cast<const Widelands::ConstructionSite&>(*mo).building() :
-			      dynamic_cast<const Widelands::Building&>(*mo).descr();
+               dynamic_cast<const Widelands::ConstructionSite&>(*mo).building() :
+               dynamic_cast<const Widelands::Building&>(*mo).descr();
 
-			if (i.descr().collected_by().count(descr.name())) {
+			if (i.descr().collected_by().count(descr.name()) != 0u) {
 				upcast(const Widelands::Building, b, mo);
 				assert(b);
 				assert(!descr.workarea_info().empty());
@@ -407,9 +394,9 @@ void FieldActionWindow::add_buttons_auto() {
 
 	const Widelands::PlayerNumber owner = node_.field->get_owned_by();
 
-	if (!igbase || igbase->can_see(owner)) {
+	if ((igbase == nullptr) || igbase->can_see(owner)) {
 		Widelands::BaseImmovable* const imm = map_.get_immovable(node_);
-		const bool can_act = igbase ? igbase->can_act(owner) : true;
+		const bool can_act = igbase != nullptr ? igbase->can_act(owner) : true;
 
 		// The box with road-building buttons
 		buildbox = new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
@@ -427,7 +414,8 @@ void FieldActionWindow::add_buttons_auto() {
 
 				Building* const building = flag->get_building();
 
-				if (!building || (building->get_playercaps() & Building::PCap_Bulldoze)) {
+				if ((building == nullptr) ||
+				    ((building->get_playercaps() & Building::PCap_Bulldoze) != 0u)) {
 					add_button(buildbox, "rip_flag", kImgButtonRipFlag, &FieldActionWindow::act_ripflag,
 					           _("Destroy this flag"));
 				}
@@ -462,53 +450,42 @@ void FieldActionWindow::add_buttons_auto() {
 						} else {
 							tooltip = it->second;
 						}
-						tooltip =
-						   (boost::format("<rt><p>%s</p><p>%s</p></rt>") %
-						    g_style_manager->font_style(UI::FontStyle::kDisabled)
-						       .as_font_tag(_("Send scout to explore surroundings")) %
-						    g_style_manager->font_style(UI::FontStyle::kWuiTooltip).as_font_tag(tooltip))
-						      .str();
+						tooltip = format(
+						   "<rt><p>%s</p><p>%s</p></rt>",
+						   g_style_manager->font_style(UI::FontStyle::kDisabled)
+						      .as_font_tag(_("Send scout to explore surroundings")),
+						   g_style_manager->font_style(UI::FontStyle::kWuiTooltip).as_font_tag(tooltip));
 					}
 					add_button(buildbox, "scout", kImgButtonScout, &FieldActionWindow::act_scout,
 					           tooltip, false, enabled);
 				}
 			}
 		} else {
-			const int32_t buildcaps = player_ ? player_->get_buildcaps(node_) : 0;
+			const int32_t buildcaps = player_ != nullptr ? player_->get_buildcaps(node_) : 0;
 			const int32_t nodecaps = map_.get_max_nodecaps(ibase().egbase(), node_);
 
 			// Add house building
-			if (player_ && ((nodecaps & Widelands::BUILDCAPS_SIZEMASK) ||
-			                (nodecaps & Widelands::BUILDCAPS_MINE))) {
+			if ((player_ != nullptr) && (((nodecaps & Widelands::BUILDCAPS_SIZEMASK) != 0) ||
+			                             ((nodecaps & Widelands::BUILDCAPS_MINE) != 0))) {
 				add_buttons_build(buildcaps, nodecaps);
 			}
 
 			// Add build actions
-			if (buildcaps & Widelands::BUILDCAPS_FLAG) {
+			if ((buildcaps & Widelands::BUILDCAPS_FLAG) != 0) {
 				add_button(buildbox, "build_flag", kImgButtonBuildFlag,
 				           &FieldActionWindow::act_buildflag, _("Place a flag"));
 			}
 
-			if (can_act && dynamic_cast<const Widelands::Road*>(imm)) {
+			if (can_act && (dynamic_cast<const Widelands::Road*>(imm) != nullptr)) {
 				add_button(buildbox, "destroy_road", kImgButtonRemoveRoad,
 				           &FieldActionWindow::act_removeroad, _("Destroy a road"));
 			}
 
-			if (can_act && dynamic_cast<const Widelands::Waterway*>(imm)) {
+			if (can_act && (dynamic_cast<const Widelands::Waterway*>(imm) != nullptr)) {
 				add_button(buildbox, "destroy_waterway", kImgButtonRemoveWaterway,
 				           &FieldActionWindow::act_removewaterway, _("Destroy a waterway"));
 			}
 		}
-	} else if (player_) {
-		if (upcast(Building, building, map_.get_immovable(node_))) {
-			for (Widelands::Coords& coords : building->get_positions(igbase->egbase())) {
-				if (player_->is_seeing(
-				       Widelands::Map::get_index(coords, ibase().egbase().map().get_width()))) {
-					add_buttons_attack();
-					break;
-				}
-			}
-		}
 	}
 
 	//  Watch actions, only when in game (no use in editor).
@@ -523,42 +500,20 @@ void FieldActionWindow::add_buttons_auto() {
 	}
 
 	// Add tabs
-	if (buildbox && buildbox->get_nritems()) {
+	if ((buildbox != nullptr) && (buildbox->get_nritems() != 0)) {
 		add_tab("roads", kImgTabBuildroad, buildbox, _("Build road"));
 	}
 
 	add_tab("watch", kImgTabWatch, &watchbox, _("Watch"));
 }
 
-void FieldActionWindow::add_buttons_attack() {
-	UI::Box& a_box = *new UI::Box(&tabpanel_, UI::PanelStyle::kWui, 0, 0, UI::Box::Horizontal);
-
-	if (upcast(Widelands::Building, building, map_.get_immovable(node_))) {
-		if (const Widelands::AttackTarget* attack_target = building->attack_target()) {
-			if (player_ && player_->is_hostile(building->owner()) &&
-			    attack_target->can_be_attacked()) {
-				attack_box_ = new AttackBox(&a_box, player_, &node_, 0, 0);
-				a_box.add(attack_box_);
-
-				UI::Button* attack_button = attack_box_->get_attack_button();
-				attack_button->sigclicked.connect([this]() { act_attack(); });
-				set_fastclick_panel(attack_button);
-			}
-		}
-	}
-
-	if (a_box.get_nritems()) {  //  add tab
-		add_tab("attack", kImgTabAttack, &a_box, _("Attack"));
-	}
-}
-
 /*
 ===============
 Add buttons for house building.
 ===============
 */
 void FieldActionWindow::add_buttons_build(int32_t buildcaps, int32_t max_nodecaps) {
-	if (!player_) {
+	if (player_ == nullptr) {
 		return;
 	}
 	const Widelands::FCoords brn = map_.br_n(node_);
@@ -566,9 +521,9 @@ void FieldActionWindow::add_buttons_build(int32_t buildcaps, int32_t max_nodecap
 	    !brn.field->is_interior(player_->player_number())) {
 		return;
 	}
-	if (!(brn.field->get_immovable() &&
+	if (!((brn.field->get_immovable() != nullptr) &&
 	      brn.field->get_immovable()->descr().type() == Widelands::MapObjectType::FLAG) &&
-	    !(player_->get_buildcaps(brn) & Widelands::BUILDCAPS_FLAG)) {
+	    ((player_->get_buildcaps(brn) & Widelands::BUILDCAPS_FLAG) == 0)) {
 		return;
 	}
 	BuildGrid* bbg_house[4] = {nullptr, nullptr, nullptr, nullptr};
@@ -599,16 +554,17 @@ void FieldActionWindow::add_buttons_build(int32_t buildcaps, int32_t max_nodecap
 		// TODO(Nordfriese): Use Player::check_can_build to simplify the code
 
 		if (building_descr->get_built_over_immovable() != Widelands::INVALID_INDEX &&
-		    !(node_.field->get_immovable() && node_.field->get_immovable()->has_attribute(
-		                                         building_descr->get_built_over_immovable()))) {
+		    !((node_.field->get_immovable() != nullptr) &&
+		      node_.field->get_immovable()->has_attribute(
+		         building_descr->get_built_over_immovable()))) {
 			continue;
 		}
 		// Figure out if we can build it here, and in which tab it belongs
 		if (building_descr->get_ismine()) {
-			if (!((building_descr->get_built_over_immovable() == Widelands::INVALID_INDEX ?
-			          buildcaps :
-			          max_nodecaps) &
-			      Widelands::BUILDCAPS_MINE)) {
+			if (((building_descr->get_built_over_immovable() == Widelands::INVALID_INDEX ?
+                  buildcaps :
+                  max_nodecaps) &
+			     Widelands::BUILDCAPS_MINE) == 0) {
 				continue;
 			}
 
@@ -617,12 +573,12 @@ void FieldActionWindow::add_buttons_build(int32_t buildcaps, int32_t max_nodecap
 			int32_t size = building_descr->get_size() - Widelands::BaseImmovable::SMALL;
 
 			if (((building_descr->get_built_over_immovable() == Widelands::INVALID_INDEX ?
-			         buildcaps :
-			         max_nodecaps) &
+                  buildcaps :
+                  max_nodecaps) &
 			     Widelands::BUILDCAPS_SIZEMASK) < size + 1) {
 				continue;
 			}
-			if (building_descr->get_isport() && !(buildcaps & Widelands::BUILDCAPS_PORT)) {
+			if (building_descr->get_isport() && ((buildcaps & Widelands::BUILDCAPS_PORT) == 0)) {
 				continue;
 			}
 			if (building_descr->get_size() >= Widelands::BaseImmovable::BIG &&
@@ -640,7 +596,7 @@ void FieldActionWindow::add_buttons_build(int32_t buildcaps, int32_t max_nodecap
 		}
 
 		// Allocate the tab's grid if necessary
-		if (!*ppgrid) {
+		if (*ppgrid == nullptr) {
 			*ppgrid = new BuildGrid(&tabpanel_, player_, 0, 0, 5);
 			(*ppgrid)->buildclicked.connect([this](Widelands::DescriptionIndex i) { act_build(i); });
 			(*ppgrid)->buildmouseout.connect(
@@ -656,14 +612,14 @@ void FieldActionWindow::add_buttons_build(int32_t buildcaps, int32_t max_nodecap
 
 	// Add all necessary tabs
 	for (int32_t i = 0; i < 4; ++i) {
-		if (bbg_house[i]) {
+		if (bbg_house[i] != nullptr) {
 			tabpanel_.activate(best_tab_ =
 			                      add_tab(name_tab_build[i], pic_tab_buildhouse[i], bbg_house[i],
 			                              i18n::translate(tooltip_tab_build[i])));
 		}
 	}
 
-	if (bbg_mine) {
+	if (bbg_mine != nullptr) {
 		tabpanel_.activate(best_tab_ =
 		                      add_tab("mines", kImgTabBuildmine, bbg_mine, _("Build mines")));
 	}
@@ -782,7 +738,7 @@ Build a flag at this field
 */
 void FieldActionWindow::act_buildflag() {
 	upcast(Game, game, &ibase().egbase());
-	if (game) {
+	if (game != nullptr) {
 		game->send_player_build_flag(player_->player_number(), node_);
 	} else {
 		player_->build_flag(node_);
@@ -790,7 +746,7 @@ void FieldActionWindow::act_buildflag() {
 
 	if (ibase().in_road_building_mode()) {
 		ibase().finish_build_road();
-	} else if (game) {
+	} else if (game != nullptr) {
 		upcast(InteractivePlayer, iaplayer, &ibase());
 		iaplayer->set_flag_to_connect(node_);
 	}
@@ -822,9 +778,9 @@ void FieldActionWindow::act_ripflag() {
 	upcast(InteractivePlayer, iaplayer, &ibase());
 
 	if (upcast(Widelands::Flag, flag, node_.field->get_immovable())) {
-		const bool ctrl_pressed = SDL_GetModState() & KMOD_CTRL;
+		const bool ctrl_pressed = (SDL_GetModState() & KMOD_CTRL) != 0;
 		if (Building* const building = flag->get_building()) {
-			if (building->get_playercaps() & Building::PCap_Bulldoze) {
+			if ((building->get_playercaps() & Building::PCap_Bulldoze) != 0u) {
 				if (ctrl_pressed) {
 					game->send_player_bulldoze(*flag, ctrl_pressed);
 				} else {
@@ -912,7 +868,7 @@ void FieldActionWindow::act_removeroad() {
 	Widelands::EditorGameBase& egbase = ibase().egbase();
 	if (upcast(Widelands::Road, road, egbase.map().get_immovable(node_))) {
 		upcast(Game, game, &ibase().egbase());
-		game->send_player_bulldoze(*road, SDL_GetModState() & KMOD_CTRL);
+		game->send_player_bulldoze(*road, (SDL_GetModState() & KMOD_CTRL) != 0);
 	}
 	reset_mouse_and_die();
 }
@@ -921,7 +877,7 @@ void FieldActionWindow::act_removewaterway() {
 	Widelands::EditorGameBase& egbase = ibase().egbase();
 	if (upcast(Widelands::Waterway, waterway, egbase.map().get_immovable(node_))) {
 		upcast(Game, game, &ibase().egbase());
-		game->send_player_bulldoze(*waterway, SDL_GetModState() & KMOD_CTRL);
+		game->send_player_bulldoze(*waterway, (SDL_GetModState() & KMOD_CTRL) != 0);
 	}
 	reset_mouse_and_die();
 }
@@ -940,7 +896,7 @@ void FieldActionWindow::act_build(Widelands::DescriptionIndex idx) {
 	reset_mouse_and_die();
 }
 
-void FieldActionWindow::building_icon_mouse_out(Widelands::DescriptionIndex) {
+void FieldActionWindow::building_icon_mouse_out(Widelands::DescriptionIndex /* idx */) {
 	if (showing_workarea_preview_) {
 		ibase().hide_workarea(node_, false);
 		showing_workarea_preview_ = false;
@@ -983,36 +939,31 @@ void FieldActionWindow::building_icon_mouse_in(const Widelands::DescriptionIndex
 		do {
 			if (player_->is_seeing(map.get_index(mr.location()))) {
 				if (Widelands::BaseImmovable* imm = mr.location().field->get_immovable()) {
-					const Widelands::MapObjectType imm_type = imm->descr().type();
+					Widelands::MapObjectType imm_type = imm->descr().type();
+					std::string imm_name = imm->descr().name();
 					if (imm_type < Widelands::MapObjectType::BUILDING) {
 						// We are not interested in trees and pebbles
 						continue;
 					}
 					const Widelands::BuildingDescr* d = nullptr;
 					bool positive = false;  // unused default value to make g++ happy
+					bool is_milsite_or_whouse = descr.type() == Widelands::MapObjectType::MILITARYSITE ||
+					                            descr.type() == Widelands::MapObjectType::WAREHOUSE;
 					if (imm_type == Widelands::MapObjectType::CONSTRUCTIONSITE) {
 						upcast(Widelands::ConstructionSite, cs, imm);
 						d = cs->get_info().becomes;
 						assert(d);
-						if ((descr.type() == Widelands::MapObjectType::PRODUCTIONSITE &&
-						     (d->type() != Widelands::MapObjectType::PRODUCTIONSITE ||
-						      !dynamic_cast<const Widelands::ProductionSiteDescr&>(descr)
-						          .highlight_overlapping_workarea_for(d->name(), &positive))) ||
-						    ((descr.type() == Widelands::MapObjectType::MILITARYSITE ||
-						      descr.type() == Widelands::MapObjectType::WAREHOUSE) &&
-						     d->type() != Widelands::MapObjectType::MILITARYSITE &&
-						     d->type() != Widelands::MapObjectType::WAREHOUSE)) {
-							continue;
-						}
-					} else if (descr.type() == Widelands::MapObjectType::PRODUCTIONSITE) {
+						imm_type = d->type();
+						imm_name = d->name();
+					}
+					if (descr.type() == Widelands::MapObjectType::PRODUCTIONSITE) {
 						if (imm_type != Widelands::MapObjectType::PRODUCTIONSITE ||
 						    imm->get_owner() != player_ ||
 						    !dynamic_cast<const Widelands::ProductionSiteDescr&>(descr)
-						        .highlight_overlapping_workarea_for(imm->descr().name(), &positive)) {
+						        .highlight_overlapping_workarea_for(imm_name, &positive)) {
 							continue;
 						}
-					} else if (descr.type() == Widelands::MapObjectType::WAREHOUSE ||
-					           descr.type() == Widelands::MapObjectType::MILITARYSITE) {
+					} else if (is_milsite_or_whouse) {
 						if (imm_type != Widelands::MapObjectType::MILITARYSITE &&
 						    imm_type != Widelands::MapObjectType::WAREHOUSE) {
 							continue;
@@ -1023,7 +974,7 @@ void FieldActionWindow::building_icon_mouse_in(const Widelands::DescriptionIndex
 						// Don't count big buildings more than once
 						continue;
 					}
-					if (!d) {
+					if (d == nullptr) {
 						d = &bld->descr();
 					}
 					const WorkareaInfo& wa = d->workarea_info();
@@ -1034,19 +985,27 @@ void FieldActionWindow::building_icon_mouse_in(const Widelands::DescriptionIndex
 					if (wa_radius == 0) {
 						continue;
 					}
+					if (overlapping_workareas_.count(mr.location()) != 0) {
+						// mr.advance() may reach the same location several times during an iteration,
+						// but we only want the workarea once
+						continue;
+					}
 					if (map.calc_distance(node_, mr.location()) <= workarea_radius + wa_radius) {
 						std::map<Widelands::TCoords<>, uint32_t> colors;
 						for (const Widelands::TCoords<>& t : map.triangles_in_region(
 						        map.to_set(Widelands::Area<>(mr.location(), wa_radius)))) {
-							colors[t] = mr.location() == t.node || mr.location() == map.br_n(t.node) ||
-							                  mr.location() == (t.t == Widelands::TriangleIndex::D ?
-							                                       map.bl_n(t.node) :
-							                                       map.r_n(t.node)) ||
-							                  main_region.count(t) ?
-							               descr.type() == Widelands::MapObjectType::PRODUCTIONSITE ?
-							               positive ? kOverlapColorPositive : kOverlapColorNegative :
-							               kOverlapColorDefault :
-							               kOverlapColorPale;
+							Widelands::Coords coords =
+							   (t.t == Widelands::TriangleIndex::D) ? map.bl_n(t.node) : map.r_n(t.node);
+							if (mr.location() == t.node || mr.location() == map.br_n(t.node) ||
+							    mr.location() == coords || main_region.count(t) != 0u) {
+								if (descr.type() == Widelands::MapObjectType::PRODUCTIONSITE) {
+									colors[t] = positive ? kOverlapColorPositive : kOverlapColorNegative;
+								} else {
+									colors[t] = kOverlapColorDefault;
+								}
+							} else {
+								colors[t] = kOverlapColorPale;
+							}
 						}
 						ibase().show_workarea(wa, mr.location(), colors);
 						overlapping_workareas_.insert(mr.location());
@@ -1094,23 +1053,6 @@ void FieldActionWindow::act_unmark_removal() {
 	reset_mouse_and_die();
 }
 
-/**
- * Here there are a problem: the sender of an event is always the owner of
- * were is done this even. But for attacks, the owner of an event is the
- * player who start an attack, so is needed to get an extra parameter to
- * the send_player_enemyflagaction, the player number
- */
-void FieldActionWindow::act_attack() {
-	assert(attack_box_);
-	upcast(Game, game, &ibase().egbase());
-	if (upcast(Building, building, game->map().get_immovable(node_))) {
-		upcast(InteractivePlayer const, iaplayer, &ibase());
-		game->send_player_enemyflagaction(building->base_flag(), iaplayer->player_number(),
-		                                  attack_box_->soldiers(), attack_box_->get_allow_conquer());
-	}
-	reset_mouse_and_die();
-}
-
 /*
 ===============
 show_field_action
@@ -1132,10 +1074,11 @@ void show_field_action(InteractiveBase* const ibase,
 			FieldActionWindow& w = *new FieldActionWindow(ibase, player, registry);
 			if (ibase->in_road_building_mode(RoadBuildingType::kRoad)) {
 				w.add_buttons_road(target != ibase->get_build_road_start() &&
-				                   (player->get_buildcaps(target) & Widelands::BUILDCAPS_FLAG));
+				                   ((player->get_buildcaps(target) & Widelands::BUILDCAPS_FLAG) != 0));
 			} else {
-				w.add_buttons_waterway(target != ibase->get_build_road_start() &&
-				                       (player->get_buildcaps(target) & Widelands::BUILDCAPS_FLAG));
+				w.add_buttons_waterway(
+				   target != ibase->get_build_road_start() &&
+				   ((player->get_buildcaps(target) & Widelands::BUILDCAPS_FLAG) != 0));
 			}
 			w.init();
 			return;
@@ -1162,7 +1105,7 @@ void show_field_action(InteractiveBase* const ibase,
 				finish = true;
 			} else if (i->descr().type() == Widelands::MapObjectType::ROAD ||
 			           i->descr().type() == Widelands::MapObjectType::WATERWAY) {
-				if (player->get_buildcaps(target) & Widelands::BUILDCAPS_FLAG) {
+				if ((player->get_buildcaps(target) & Widelands::BUILDCAPS_FLAG) != 0) {
 					upcast(Game, game, &player->egbase());
 					game->send_player_build_flag(player->player_number(), target);
 					finish = true;
