@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2021 by the Widelands Development Team
+ * Copyright (C) 2008-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -102,7 +101,7 @@ GameChatPanel::GameChatPanel(UI::Panel* parent,
 		recipient_dropdown_.select(chat_.last_recipient_);
 		// Insert "@playername " into the edit field if the dropdown currently has a selection
 		set_recipient();
-		update_signal_connection = chat_.participants_->participants_updated.connect([this]() {
+		update_signal_connection_ = chat_.participants_->participants_updated.subscribe([this]() {
 			// When the participants change, create new contents for dropdown
 			has_team_ = chat_.participants_->needs_teamchat();
 			prepare_recipients();
@@ -114,8 +113,8 @@ GameChatPanel::GameChatPanel(UI::Panel* parent,
 
 	hbox_.add(&editbox, UI::Box::Resizing::kFillSpace);
 
-	chat_message_subscriber_ =
-	   Notifications::subscribe<ChatMessage>([this](const ChatMessage&) { recalculate(true); });
+	chat_message_subscriber_ = Notifications::subscribe<ChatMessage>(
+	   [this](const ChatMessage& /* msg */) { recalculate(true); });
 	recalculate(false);
 	layout();
 }
@@ -124,12 +123,6 @@ void GameChatPanel::layout() {
 	vbox_.set_size(get_inner_w(), get_inner_h());
 }
 
-GameChatPanel::~GameChatPanel() {
-	if (chat_.participants_ != nullptr) {
-		update_signal_connection.disconnect();
-	}
-}
-
 /**
  * If Ctrl+Space is pressed, try to autocomplete the player name
  */
@@ -140,7 +133,7 @@ bool GameChatPanel::handle_key(const bool down, const SDL_Keysym code) {
 		return false;
 	}
 
-	if (!down || code.sym != SDLK_SPACE || !(SDL_GetModState() & KMOD_CTRL)) {
+	if (!down || code.sym != SDLK_SPACE || ((SDL_GetModState() & KMOD_CTRL) == 0)) {
 		return false;
 	}
 
@@ -498,7 +491,7 @@ bool GameChatPanel::select_recipient() {
 /**
  * The mouse was clicked on this chatbox
  */
-bool GameChatPanel::handle_mousepress(const uint8_t btn, int32_t, int32_t) {
+bool GameChatPanel::handle_mousepress(const uint8_t btn, int32_t /*x*/, int32_t /*y*/) {
 	if (btn == SDL_BUTTON_LEFT && get_can_focus()) {
 		focus_edit();
 		clicked();
