@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,7 +27,11 @@
  */
 class GameExitConfirmBox : public UI::WLMessageBox {
 public:
-	GameExitConfirmBox(UI::Panel& parent, InteractiveGameBase& gb);
+	GameExitConfirmBox(UI::Panel& parent, InteractiveGameBase& i);
+	GameExitConfirmBox(UI::Panel& parent,
+	                   InteractiveGameBase& gb,
+	                   const std::string& title,
+	                   const std::string& message);
 
 	void clicked_ok() override;
 
