@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -22,6 +21,7 @@
 #include "base/time_string.h"
 #include "base/wexception.h"
 #include "graphic/text_layout.h"
+#include "logic/game_data_error.h"
 #include "logic/message_queue.h"
 #include "logic/player.h"
 #include "logic/playercommand.h"
@@ -65,8 +65,6 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	list = new UI::Table<uintptr_t>(this, kPadding, kButtonSize + 2 * kPadding,
 	                                kWindowWidth - 2 * kPadding, kTableHeight, UI::PanelStyle::kWui,
 	                                UI::TableRows::kMulti);
-	list->selected.connect([this](uint32_t a) { selected(a); });
-	list->double_clicked.connect([this](uint32_t a) { double_clicked(a); });
 	list->add_column(kWindowWidth - 2 * kPadding - 60 - 60 - 75, _("Title"));
 	list->add_column(60, pgettext("message", "Type"), "", UI::Align::kCenter);
 	list->add_column(60, _("Status"), "", UI::Align::kCenter);
@@ -81,33 +79,23 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	geologistsbtn_ = new UI::Button(this, "filter_geologists_messages", kPadding, kPadding,
 	                                kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                                g_image_cache->get("images/wui/fieldaction/menu_geologist.png"));
-	geologistsbtn_->sigclicked.connect(
-	   [this]() { filter_messages(Widelands::Message::Type::kGeologists); });
 
 	economybtn_ = new UI::Button(this, "filter_economy_messages", 2 * kPadding + kButtonSize,
 	                             kPadding, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                             g_image_cache->get("images/wui/stats/genstats_nrwares.png"));
-	economybtn_->sigclicked.connect(
-	   [this]() { filter_messages(Widelands::Message::Type::kEconomy); });
 
 	seafaringbtn_ =
 	   new UI::Button(this, "filter_seafaring_messages", 3 * kPadding + 2 * kButtonSize, kPadding,
 	                  kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                  g_image_cache->get("images/wui/buildings/start_expedition.png"));
-	seafaringbtn_->sigclicked.connect(
-	   [this]() { filter_messages(Widelands::Message::Type::kSeafaring); });
 
 	warfarebtn_ = new UI::Button(this, "filter_warfare_messages", 4 * kPadding + 3 * kButtonSize,
 	                             kPadding, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                             g_image_cache->get("images/wui/messages/messages_warfare.png"));
-	warfarebtn_->sigclicked.connect(
-	   [this]() { filter_messages(Widelands::Message::Type::kWarfare); });
 
 	scenariobtn_ = new UI::Button(this, "filter_scenario_messages", 5 * kPadding + 4 * kButtonSize,
 	                              kPadding, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                              g_image_cache->get("images/wui/menus/objectives.png"));
-	scenariobtn_->sigclicked.connect(
-	   [this]() { filter_messages(Widelands::Message::Type::kScenario); });
 
 	message_filter_ = Widelands::Message::Type::kAllMessages;
 	set_filter_messages_tooltips();
@@ -118,14 +106,12 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	                             UI::ButtonStyle::kWuiPrimary,
 	                             g_image_cache->get("images/wui/messages/message_archive.png"));
 	update_archive_button_tooltip();
-	archivebtn_->sigclicked.connect([this]() { archive_or_restore(); });
 
 	togglemodebtn_ = new UI::Button(
 	   this, "toggle_between_inbox_or_archive",
 	   archivebtn_->get_x() + archivebtn_->get_w() + kPadding, archivebtn_->get_y(), kButtonSize,
 	   kButtonSize, UI::ButtonStyle::kWuiPrimary,
 	   g_image_cache->get("images/wui/messages/message_archived.png"), _("Show Archive"));
-	togglemodebtn_->sigclicked.connect([this]() { toggle_mode(); });
 
 	centerviewbtn_ = new UI::Button(
 	   this, "center_main_mapview_on_location", kWindowWidth - kPadding - kButtonSize,
@@ -134,8 +120,7 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	   as_tooltip_text_with_hotkey(
 	      /** TRANSLATORS: Tooltip in the messages window */
 	      _("Center main mapview on location"),
-	      shortcut_string_for(KeyboardShortcut::kInGameMessagesGoto), UI::PanelStyle::kWui));
-	centerviewbtn_->sigclicked.connect([this]() { center_view(); });
+	      shortcut_string_for(KeyboardShortcut::kInGameMessagesGoto, true), UI::PanelStyle::kWui));
 	centerviewbtn_->set_enabled(false);
 
 	if (get_usedefaultpos()) {
@@ -154,6 +139,24 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	list->layout();
 
 	list->focus();
+
+	geologistsbtn_->sigclicked.connect(
+	   [this]() { filter_messages(Widelands::Message::Type::kGeologists); });
+	economybtn_->sigclicked.connect(
+	   [this]() { filter_messages(Widelands::Message::Type::kEconomy); });
+	seafaringbtn_->sigclicked.connect(
+	   [this]() { filter_messages(Widelands::Message::Type::kSeafaring); });
+	warfarebtn_->sigclicked.connect(
+	   [this]() { filter_messages(Widelands::Message::Type::kWarfare); });
+	scenariobtn_->sigclicked.connect(
+	   [this]() { filter_messages(Widelands::Message::Type::kScenario); });
+	list->selected.connect([this](uint32_t a) { selected(a); });
+	list->double_clicked.connect([this](uint32_t a) { double_clicked(a); });
+	archivebtn_->sigclicked.connect([this]() { archive_or_restore(); });
+	togglemodebtn_->sigclicked.connect([this]() { toggle_mode(); });
+	centerviewbtn_->sigclicked.connect([this]() { center_view(); });
+
+	initialization_complete();
 }
 
 /**
@@ -165,7 +168,7 @@ bool GameMessageMenu::compare_title(uint32_t a, uint32_t b) {
 	const Message* msga = mq[MessageId((*list)[a])];
 	const Message* msgb = mq[MessageId((*list)[b])];
 
-	if (msga && msgb) {
+	if ((msga != nullptr) && (msgb != nullptr)) {
 		if (msga->title() == msgb->title()) {
 			return compare_time_sent(a, b);
 		}
@@ -183,7 +186,7 @@ bool GameMessageMenu::compare_status(uint32_t a, uint32_t b) {
 	const Message* msga = mq[MessageId((*list)[a])];
 	const Message* msgb = mq[MessageId((*list)[b])];
 
-	if (msga && msgb) {
+	if ((msga != nullptr) && (msgb != nullptr)) {
 		if (msga->status() == msgb->status()) {
 			return compare_time_sent(a, b);
 		}
@@ -201,7 +204,7 @@ bool GameMessageMenu::compare_type(uint32_t a, uint32_t b) {
 	const Message* msga = mq[MessageId((*list)[a])];
 	const Message* msgb = mq[MessageId((*list)[b])];
 
-	if (msga && msgb) {
+	if ((msga != nullptr) && (msgb != nullptr)) {
 		const Widelands::Message::Type cat_a = msga->message_type_category();
 		const Widelands::Message::Type cat_b = msgb->message_type_category();
 		if (cat_a == cat_b) {
@@ -220,7 +223,7 @@ bool GameMessageMenu::compare_time_sent(uint32_t a, uint32_t b) {
 	const Message* msga = mq[MessageId((*list)[a])];
 	const Message* msgb = mq[MessageId((*list)[b])];
 
-	if (msga && msgb) {
+	if ((msga != nullptr) && (msgb != nullptr)) {
 		return msga->sent() > msgb->sent();
 	}
 	return false;  // shouldn't happen
@@ -281,7 +284,7 @@ void GameMessageMenu::think() {
 	uint32_t removed = 0;
 	const auto& sel = list->selections();
 	const uint32_t max_index = (sel.empty() ? 0 : *sel.rbegin());
-	for (uint32_t j = list->size(); j; --j) {
+	for (uint32_t j = list->size(); j != 0u; --j) {
 		MessageId id_((*list)[j - 1]);
 		if (Message const* const message = mq[id_]) {
 			if (should_be_hidden(*message)) {
@@ -309,7 +312,7 @@ void GameMessageMenu::think() {
 	for (const auto& temp_message : mq) {
 		MessageId const id = temp_message.first;
 		const Message& message = *temp_message.second;
-		if (!should_be_hidden(message) && !list->find(id.value())) {
+		if (!should_be_hidden(message) && (list->find(id.value()) == nullptr)) {
 			UI::Table<uintptr_t>::EntryRecord& er = list->add(id.value());
 			update_record(er, message);
 			list->sort();
@@ -421,7 +424,7 @@ bool GameMessageMenu::handle_key(bool down, SDL_Keysym code) {
 		}
 	}
 
-	return UI::Panel::handle_key(down, code);
+	return UI::UniqueWindow::handle_key(down, code);
 }
 
 void GameMessageMenu::archive_or_restore() {
@@ -523,7 +526,8 @@ void GameMessageMenu::toggle_filter_messages_button(UI::Button& button,
 		/** TRANSLATORS: %1% is a tooltip, %2% is the corresponding hotkey */
 		button.set_tooltip(as_tooltip_text_with_hotkey(
 		   /** TRANSLATORS: Tooltip in the messages window */
-		   _("Show all messages"), shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterAll),
+		   _("Show all messages"),
+		   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterAll, true),
 		   UI::PanelStyle::kWui));
 	}
 }
@@ -535,24 +539,28 @@ void GameMessageMenu::set_filter_messages_tooltips() {
 	geologistsbtn_->set_tooltip(as_tooltip_text_with_hotkey(
 	   /** TRANSLATORS: Tooltip in the messages window */
 	   _("Show geologists’ messages only"),
-	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterGeologists),
+	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterGeologists, true),
 	   UI::PanelStyle::kWui));
 	economybtn_->set_tooltip(as_tooltip_text_with_hotkey(
 	   /** TRANSLATORS: Tooltip in the messages window */
 	   _("Show economy messages only"),
-	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterEconomy), UI::PanelStyle::kWui));
+	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterEconomy, true),
+	   UI::PanelStyle::kWui));
 	seafaringbtn_->set_tooltip(as_tooltip_text_with_hotkey(
 	   /** TRANSLATORS: Tooltip in the messages window */
 	   _("Show seafaring messages only"),
-	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterSeafaring), UI::PanelStyle::kWui));
+	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterSeafaring, true),
+	   UI::PanelStyle::kWui));
 	warfarebtn_->set_tooltip(as_tooltip_text_with_hotkey(
 	   /** TRANSLATORS: Tooltip in the messages window */
 	   _("Show warfare messages only"),
-	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterWarfare), UI::PanelStyle::kWui));
+	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterWarfare, true),
+	   UI::PanelStyle::kWui));
 	scenariobtn_->set_tooltip(as_tooltip_text_with_hotkey(
 	   /** TRANSLATORS: Tooltip in the messages window */
 	   _("Show scenario messages only"),
-	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterScenario), UI::PanelStyle::kWui));
+	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterScenario, true),
+	   UI::PanelStyle::kWui));
 }
 
 /**
@@ -622,10 +630,9 @@ void GameMessageMenu::update_archive_button_tooltip() {
 			    * message.
 			    * DO NOT omit the placeholder in your translation.
 			    */
-			   (boost::format(ngettext("Restore the selected %d message",
-			                           "Restore the selected %d messages", no_selections)) %
-			    no_selections)
-			      .str();
+			   format(ngettext("Restore the selected %d message", "Restore the selected %d messages",
+			                   no_selections),
+			          no_selections);
 		} else {
 			/** TRANSLATORS: Tooltip in the messages window */
 			button_tooltip = _("Restore selected message");
@@ -638,10 +645,9 @@ void GameMessageMenu::update_archive_button_tooltip() {
 			    * message.
 			    * DO NOT omit the placeholder in your translation.
 			    */
-			   (boost::format(ngettext("Archive the selected %d message",
-			                           "Archive the selected %d messages", no_selections)) %
-			    no_selections)
-			      .str();
+			   format(ngettext("Archive the selected %d message", "Archive the selected %d messages",
+			                   no_selections),
+			          no_selections);
 		} else {
 			/** TRANSLATORS: Tooltip in the messages window */
 			button_tooltip = _("Archive selected message");
@@ -649,6 +655,50 @@ void GameMessageMenu::update_archive_button_tooltip() {
 		break;
 	}
 	archivebtn_->set_tooltip(as_tooltip_text_with_hotkey(
-	   button_tooltip, shortcut_string_for(KeyboardShortcut::kCommonDeleteItem),
+	   button_tooltip, shortcut_string_for(KeyboardShortcut::kCommonDeleteItem, true),
 	   UI::PanelStyle::kWui));
 }
+
+constexpr uint16_t kCurrentPacketVersion = 1;
+UI::Window& GameMessageMenu::load(FileRead& fr, InteractiveBase& ib) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersion) {
+			UI::UniqueWindow::Registry& r = dynamic_cast<InteractivePlayer&>(ib).message_menu_;
+			r.create();
+			assert(r.window);
+			GameMessageMenu& m = dynamic_cast<GameMessageMenu&>(*r.window);
+			m.think();  // Fills the list
+			if ((m.mode == Mode::kInbox) ^ (fr.unsigned_8() == 1)) {
+				m.toggle_mode();
+			}
+			m.filter_messages(static_cast<Widelands::Message::Type>(fr.unsigned_8()));
+			size_t nr_sel = fr.unsigned_32();
+			if (nr_sel != 0u) {
+				for (; nr_sel != 0u; --nr_sel) {
+					m.list->multiselect(fr.unsigned_32(), true);
+				}
+				m.list->multiselect(fr.unsigned_32(), true);
+			}
+			return m;
+		}
+		throw Widelands::UnhandledVersionError(
+		   "Messages Menu", packet_version, kCurrentPacketVersion);
+
+	} catch (const WException& e) {
+		throw Widelands::GameDataError("messages menu: %s", e.what());
+	}
+}
+void GameMessageMenu::save(FileWrite& fw, Widelands::MapObjectSaver& /* mos */) const {
+	fw.unsigned_16(kCurrentPacketVersion);
+	fw.unsigned_8(mode == Mode::kInbox ? 1 : 0);
+	fw.unsigned_8(static_cast<uint8_t>(message_filter_));
+	const size_t nr_sel = list->selections().size();
+	fw.unsigned_32(nr_sel);
+	if (nr_sel != 0u) {
+		for (const uint32_t& s : list->selections()) {
+			fw.unsigned_32(s);
+		}
+		fw.unsigned_32(list->selection_index());
+	}
+}
