@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2021 by the Widelands Development Team
+ * Copyright (C) 2010-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -23,6 +22,7 @@
 #include "graphic/style_manager.h"
 #include "graphic/text_layout.h"
 #include "logic/player.h"
+#include "wlapplication_mousewheel_options.h"
 #include "wui/interactive_base.h"
 
 constexpr int8_t kButtonSize = 25;
@@ -83,7 +83,7 @@ static std::string priority_tooltip(const size_t priority) {
 
 void ensure_box_can_hold_input_queues(UI::Box& b) {
 	UI::Panel* p = &b;
-	while (p->get_parent()) {
+	while (p->get_parent() != nullptr) {
 		p = p->get_parent();
 	}
 	b.set_max_size(p->get_w() - 200, p->get_h() - 200);
@@ -116,31 +116,30 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 }
 
 static inline std::string create_tooltip(const bool increase) {
-	return (boost::format("<p>%s%s%s</p>") %
-	        g_style_manager->font_style(UI::FontStyle::kWuiTooltipHeader)
-	           .as_font_tag(
-	              increase ?
-	                 /** TRANSLATORS: Button tooltip in in a building's wares input queue */
-	                 _("Increase the number of wares you want to be stored here") :
-	                 /** TRANSLATORS: Button tooltip in in a building's wares input queue */
-	                 _("Decrease the number of wares you want to be stored here")) %
-	        as_listitem(increase ?
-	                       /** TRANSLATORS: Button tooltip in in a building's wares input queue -
-	                          option explanation */
-	                       _("Hold down Shift to increase all ware types at the same time") :
-	                       /** TRANSLATORS: Button tooltip in in a building's wares input queue -
-	                          option explanation */
-	                       _("Hold down Shift to decrease all ware types at the same time"),
-	                    UI::FontStyle::kWuiTooltip) %
-	        as_listitem(increase ?
-	                       /** TRANSLATORS: Button tooltip in in a building's wares input queue -
-	                          option explanation */
-	                       _("Hold down Ctrl to allow all of this ware") :
-	                       /** TRANSLATORS: Button tooltip in in a building's wares input queue -
-	                          option explanation */
-	                       _("Hold down Ctrl to allow none of this ware"),
-	                    UI::FontStyle::kWuiTooltip))
-	   .str();
+	return format(
+	   "<p>%s%s%s</p>",
+	   g_style_manager->font_style(UI::FontStyle::kWuiTooltipHeader)
+	      .as_font_tag(increase ?
+                         /** TRANSLATORS: Button tooltip in in a building's wares input queue */
+                         _("Increase the number of wares you want to be stored here") :
+                         /** TRANSLATORS: Button tooltip in in a building's wares input queue */
+                         _("Decrease the number of wares you want to be stored here")),
+	   as_listitem(increase ?
+                        /** TRANSLATORS: Button tooltip in in a building's wares input queue -
+                           option explanation */
+                        _("Hold down Shift to increase all ware types at the same time") :
+                        /** TRANSLATORS: Button tooltip in in a building's wares input queue -
+                           option explanation */
+                        _("Hold down Shift to decrease all ware types at the same time"),
+	               UI::FontStyle::kWuiTooltip),
+	   as_listitem(increase ?
+                        /** TRANSLATORS: Button tooltip in in a building's wares input queue -
+                           option explanation */
+                        _("Hold down Ctrl to allow all of this ware") :
+                        /** TRANSLATORS: Button tooltip in in a building's wares input queue -
+                           option explanation */
+                        _("Hold down Ctrl to allow none of this ware"),
+	               UI::FontStyle::kWuiTooltip));
 }
 
 InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
@@ -157,7 +156,7 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
      can_act_(!show_only && ibase_.can_act(bld.owner().player_number())),
      show_only_(show_only),
      has_priority_(has_priority && ww == Widelands::wwWARE),
-     building_(bld),
+     building_(&bld),
      type_(ww),
      index_(idx),
      queue_(q),
@@ -218,10 +217,10 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
                kButtonSize,
                0,
                4,
-               has_priority_ ?
-                  priority_to_index(settings_ ? settings_->ware_queues.at(index_).priority :
-                                                building_.get_priority(type_, index_)) :
-                  2,
+               has_priority_ ? priority_to_index(settings_ != nullptr ?
+                                                    settings_->ware_queues.at(index_).priority :
+                                                    bld.get_priority(type_, index_)) :
+                               2,
                UI::SliderStyle::kWuiLight,
                "",
                kButtonSize,
@@ -229,17 +228,16 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
      spacer_(&hbox_, UI::PanelStyle::kWui, 0, 0, priority_.get_w(), priority_.get_h()),
      slider_was_moved_(nullptr),
      collapsed_(false),
-     nr_icons_(queue_ ?
-                  queue_->get_max_size() :
-                  type_ == Widelands::wwWORKER ? settings_->worker_queues.at(index_).max_fill :
-                                                 settings_->ware_queues.at(index_).max_fill),
+     nr_icons_(queue_ != nullptr            ? queue_->get_max_size() :
+               type_ == Widelands::wwWORKER ? settings_->worker_queues.at(index_).max_fill :
+                                              settings_->ware_queues.at(index_).max_fill),
      icons_(nr_icons_, nullptr),
      fill_index_under_mouse_(-1) {
 
 	assert((queue_ == nullptr) ^ (settings_ == nullptr));
 
-	b_decrease_real_fill_.set_visible(queue_ && ibase_.omnipotent());
-	b_increase_real_fill_.set_visible(queue_ && ibase_.omnipotent());
+	b_decrease_real_fill_.set_visible((queue_ != nullptr) && ibase_.omnipotent());
+	b_increase_real_fill_.set_visible((queue_ != nullptr) && ibase_.omnipotent());
 
 	b_decrease_desired_fill_.set_visible(!show_only_);
 	b_increase_desired_fill_.set_visible(!show_only_);
@@ -259,8 +257,8 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 	for (size_t i = 0; i < nr_icons_; ++i) {
 		icons_[i] = new UI::Icon(&hbox_, UI::PanelStyle::kWui, 0, 0, kButtonSize, kButtonSize,
 		                         type_ == Widelands::wwWARE ?
-		                            building_.owner().tribe().get_ware_descr(index_)->icon() :
-		                            building_.owner().tribe().get_worker_descr(index_)->icon());
+                                  bld.owner().tribe().get_ware_descr(index_)->icon() :
+                                  bld.owner().tribe().get_worker_descr(index_)->icon());
 		hbox_.add(icons_[i]);
 	}
 
@@ -288,32 +286,36 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 	if (can_act_) {
 		collapse_.sigclicked.connect([this]() {
 			const bool c = !collapsed_;
-			recurse([c](InputQueueDisplay& i) { i.set_collapsed(c); });
+			recurse([c](InputQueueDisplay& i) {
+				if (i.can_act_) {
+					i.set_collapsed(c);
+				}
+			});
 		});
 
 		b_decrease_desired_fill_.sigclicked.connect([this]() {
-			if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 				recurse([](InputQueueDisplay& i) { i.clicked_desired_fill(-1); });
 			} else {
 				clicked_desired_fill(-1);
 			}
 		});
 		b_increase_desired_fill_.sigclicked.connect([this]() {
-			if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 				recurse([](InputQueueDisplay& i) { i.clicked_desired_fill(1); });
 			} else {
 				clicked_desired_fill(1);
 			}
 		});
 		b_decrease_real_fill_.sigclicked.connect([this]() {
-			if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 				recurse([](InputQueueDisplay& i) { i.clicked_real_fill(-1); });
 			} else {
 				clicked_real_fill(-1);
 			}
 		});
 		b_increase_real_fill_.sigclicked.connect([this]() {
-			if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 				recurse([](InputQueueDisplay& i) { i.clicked_real_fill(1); });
 			} else {
 				clicked_real_fill(1);
@@ -322,7 +324,7 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 		priority_.changedto.connect([this](size_t i) {
 			const Widelands::WarePriority& p = index_to_priority(i);
 			slider_was_moved_ = &p;
-			if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 				recurse([&p](InputQueueDisplay& iqd) { iqd.set_priority(p); });
 			} else {
 				set_priority(p);
@@ -334,15 +336,20 @@ InputQueueDisplay::InputQueueDisplay(UI::Panel* parent,
 	}
 
 	set_tooltip(type_ == Widelands::wwWARE ?
-	               building_.owner().tribe().get_ware_descr(index_)->descname() :
-	               building_.owner().tribe().get_worker_descr(index_)->descname());
+                  bld.owner().tribe().get_ware_descr(index_)->descname() :
+                  bld.owner().tribe().get_worker_descr(index_)->descname());
 
-	// Initialize tooltips, icon colours and stuff
-	think();
+	if (nr_icons_ == 0) {
+		// Can happen when this is a dropout queue that has already been emptied.
+		assert(queue_ != nullptr);
+		hide_from_view();
+	}
+
+	// Do not call think() yet, it might deadlock
 }
 
 void InputQueueDisplay::recurse(const std::function<void(InputQueueDisplay&)>& functor) {
-	for (UI::Panel* p = get_parent()->get_first_child(); p; p = p->get_next_sibling()) {
+	for (UI::Panel* p = get_parent()->get_first_child(); p != nullptr; p = p->get_next_sibling()) {
 		if (upcast(InputQueueDisplay, i, p)) {
 			functor(*i);
 		}
@@ -368,7 +375,7 @@ bool InputQueueDisplay::handle_mousepress(const uint8_t btn, const int32_t x, co
 		return UI::Box::handle_mousepress(btn, x, y);
 	}
 
-	if (SDL_GetModState() & KMOD_SHIFT) {
+	if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 		recurse([fill](InputQueueDisplay& i) { i.set_desired_fill(fill); });
 	} else {
 		set_desired_fill(fill);
@@ -378,22 +385,59 @@ bool InputQueueDisplay::handle_mousepress(const uint8_t btn, const int32_t x, co
 }
 
 bool InputQueueDisplay::handle_mousemove(
-   uint8_t, const int32_t x, const int32_t y, int32_t, int32_t) {
+   uint8_t /*state*/, const int32_t x, const int32_t y, int32_t /*xdiff*/, int32_t /*ydiff*/) {
 	fill_index_under_mouse_ = fill_index_at(x, y);
 	return true;
 }
 
+bool InputQueueDisplay::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
+	if (show_only_ || !can_act_) {
+		return false;
+	}
+	int32_t change = get_mousewheel_change(MousewheelHandlerConfigID::kChangeValue, x, y,
+	                                       // shift has special meaning, prevent it to work
+	                                       // as part of modifier
+	                                       modstate & ~KMOD_SHIFT);
+	if (change != 0) {
+		if (get_mouse_position().x < priority_.get_x() - kButtonSize / 4) {
+			// Mouse is over desired fill
+			if ((modstate & KMOD_SHIFT) != 0) {
+				recurse([change](InputQueueDisplay& i) { i.change_desired_fill(change); });
+			} else {
+				change_desired_fill(change);
+			}
+			return true;
+		}
+		if (has_priority_) {
+			// Mouse is over priority or collapse button
+			// Can't just use method from Slider, because of the special
+			// meaning of shift to change all input priorities together.
+
+			// KMOD_SHIFT + changedto is already connected to recurse.
+			priority_.change_value_by(change);
+			return true;
+		}
+	}
+	return false;
+}
+
 void InputQueueDisplay::set_priority(const Widelands::WarePriority& priority) {
+	MutexLock m(MutexLock::ID::kObjects);
+	Widelands::Building* b = building_.get(ibase_.egbase());
+	if (b == nullptr) {
+		return;
+	}
+
 	if (!can_act_ || !has_priority_ ||
-	    priority == (queue_ ? building_.get_priority(type_, index_) : get_setting()->priority)) {
+	    priority == (queue_ != nullptr ? b->get_priority(type_, index_) : get_setting()->priority)) {
 		return;
 	}
 
 	if (Widelands::Game* game = ibase_.get_game()) {
-		game->send_player_set_ware_priority(building_, type_, index_, priority, settings_ != nullptr);
+		game->send_player_set_ware_priority(*b, type_, index_, priority, settings_ != nullptr);
 	} else {
-		if (queue_) {
-			building_.set_priority(type_, index_, priority);
+		if (queue_ != nullptr) {
+			b->set_priority(type_, index_, priority);
 		} else {
 			get_setting()->priority = priority;
 		}
@@ -402,23 +446,28 @@ void InputQueueDisplay::set_priority(const Widelands::WarePriority& priority) {
 
 void InputQueueDisplay::clicked_desired_fill(const int8_t delta) {
 	assert(delta == 1 || delta == -1);
+	MutexLock m(MutexLock::ID::kObjects);
+	Widelands::Building* b = building_.get(ibase_.egbase());
+	if (b == nullptr) {
+		return;
+	}
 
-	const unsigned desired_fill = queue_ ? queue_->get_max_fill() : get_setting()->desired_fill;
-	const unsigned max_fill = queue_ ? queue_->get_max_size() : get_setting()->max_fill;
+	const unsigned desired_fill =
+	   queue_ != nullptr ? queue_->get_max_fill() : get_setting()->desired_fill;
+	const unsigned max_fill = queue_ != nullptr ? queue_->get_max_size() : get_setting()->max_fill;
 	assert(desired_fill <= max_fill);
 
 	if (!can_act_ || desired_fill == (delta < 0 ? 0 : max_fill)) {
 		return;
 	}
 
-	const bool ctrl_down = SDL_GetModState() & KMOD_CTRL;
+	const bool ctrl_down = (SDL_GetModState() & KMOD_CTRL) != 0;
 	const unsigned new_fill = ctrl_down ? delta < 0 ? 0 : max_fill : desired_fill + delta;
 
 	if (Widelands::Game* game = ibase_.get_game()) {
-		game->send_player_set_input_max_fill(
-		   building_, index_, type_, new_fill, settings_ != nullptr);
+		game->send_player_set_input_max_fill(*b, index_, type_, new_fill, settings_ != nullptr);
 	} else {
-		if (queue_) {
+		if (queue_ != nullptr) {
 			queue_->set_max_fill(new_fill);
 		} else {
 			get_setting()->desired_fill = new_fill;
@@ -426,9 +475,54 @@ void InputQueueDisplay::clicked_desired_fill(const int8_t delta) {
 	}
 }
 
+void InputQueueDisplay::change_desired_fill(const int8_t delta) {
+	if (delta == 0) {
+		return;
+	}
+	MutexLock m(MutexLock::ID::kObjects);
+	Widelands::Building* b = building_.get(ibase_.egbase());
+	if (b == nullptr) {
+		return;
+	}
+
+	unsigned desired_fill = queue_ != nullptr ? queue_->get_max_fill() : get_setting()->desired_fill;
+	const unsigned max_fill = queue_ != nullptr ? queue_->get_max_size() : get_setting()->max_fill;
+	assert(desired_fill <= max_fill);
+
+	if (!can_act_ || desired_fill == (delta < 0 ? 0 : max_fill)) {
+		return;
+	}
+
+	if (delta < 0 && static_cast<int>(desired_fill) <= -delta) {
+		desired_fill = 0;
+	} else {
+		desired_fill += delta;
+		if (desired_fill > max_fill) {
+			desired_fill = max_fill;
+		}
+	}
+
+	if (Widelands::Game* game = ibase_.get_game()) {
+		game->send_player_set_input_max_fill(*b, index_, type_, desired_fill, settings_ != nullptr);
+	} else {
+		if (queue_ != nullptr) {
+			queue_->set_max_fill(desired_fill);
+		} else {
+			get_setting()->desired_fill = desired_fill;
+		}
+	}
+}
+
 void InputQueueDisplay::set_desired_fill(unsigned new_fill) {
-	const unsigned desired_fill = queue_ ? queue_->get_max_fill() : get_setting()->desired_fill;
-	const unsigned max_fill = queue_ ? queue_->get_max_size() : get_setting()->max_fill;
+	MutexLock m(MutexLock::ID::kObjects);
+	Widelands::Building* b = building_.get(ibase_.egbase());
+	if (b == nullptr) {
+		return;
+	}
+
+	const unsigned desired_fill =
+	   queue_ != nullptr ? queue_->get_max_fill() : get_setting()->desired_fill;
+	const unsigned max_fill = queue_ != nullptr ? queue_->get_max_size() : get_setting()->max_fill;
 	assert(desired_fill <= max_fill);
 	new_fill = std::min(new_fill, max_fill);
 
@@ -437,10 +531,9 @@ void InputQueueDisplay::set_desired_fill(unsigned new_fill) {
 	}
 
 	if (Widelands::Game* game = ibase_.get_game()) {
-		game->send_player_set_input_max_fill(
-		   building_, index_, type_, new_fill, settings_ != nullptr);
+		game->send_player_set_input_max_fill(*b, index_, type_, new_fill, settings_ != nullptr);
 	} else {
-		if (queue_) {
+		if (queue_ != nullptr) {
 			queue_->set_max_fill(new_fill);
 		} else {
 			get_setting()->desired_fill = new_fill;
@@ -450,8 +543,9 @@ void InputQueueDisplay::set_desired_fill(unsigned new_fill) {
 
 void InputQueueDisplay::clicked_real_fill(const int8_t delta) {
 	assert(delta == 1 || delta == -1);
+	MutexLock m(MutexLock::ID::kObjects);
 
-	if (!queue_ || !ibase_.omnipotent()) {
+	if ((queue_ == nullptr) || !ibase_.omnipotent() || (building_.get(ibase_.egbase()) == nullptr)) {
 		return;
 	}
 
@@ -463,7 +557,7 @@ void InputQueueDisplay::clicked_real_fill(const int8_t delta) {
 		return;
 	}
 
-	const bool ctrl_down = SDL_GetModState() & KMOD_CTRL;
+	const bool ctrl_down = (SDL_GetModState() & KMOD_CTRL) != 0;
 	const unsigned new_fill = ctrl_down ? delta < 0 ? 0 : max_fill : real_fill + delta;
 
 	queue_->set_filled(new_fill);
@@ -482,28 +576,39 @@ void InputQueueDisplay::set_collapsed(const bool c) {
 
 inline Widelands::ProductionsiteSettings::InputQueueSetting*
 InputQueueDisplay::get_setting() const {
-	return settings_ ? type_ == Widelands::wwWARE ? &settings_->ware_queues.at(index_) :
-	                                                &settings_->worker_queues.at(index_) :
-	                   nullptr;
+	return settings_ != nullptr ? type_ == Widelands::wwWARE ? &settings_->ware_queues.at(index_) :
+                                                              &settings_->worker_queues.at(index_) :
+                                 nullptr;
 }
 
 static const RGBAColor kColorComing(127, 127, 127, 191);
 static const RGBAColor kColorMissing(191, 191, 191, 127);
 
+/** Prevent the queue from being drawn and receiving UI events. */
+void InputQueueDisplay::hide_from_view() {
+	set_visible(false);
+	set_thinks(false);
+}
+
 void InputQueueDisplay::think() {
-	if (queue_ && queue_->get_max_size() == 0) {
-		set_visible(false);
+	MutexLock m(MutexLock::ID::kObjects);
+	Widelands::Building* b = building_.get(ibase_.egbase());
+	if (b == nullptr) {
+		return;
+	}
+
+	if ((queue_ != nullptr) && queue_->get_max_size() == 0) {
+		hide_from_view();
 		return;
 	}
-	set_visible(true);
 
 	const Widelands::ProductionsiteSettings::InputQueueSetting* setting = get_setting();
-	const unsigned max_fill = queue_ ? queue_->get_max_size() : setting->max_fill;
-	const unsigned real_fill = queue_ ? queue_->get_filled() : setting->desired_fill;
-	const unsigned desired_fill = queue_ ? queue_->get_max_fill() : setting->desired_fill;
-	const unsigned nr_missing = queue_ ? queue_->get_missing() : 0;
+	const unsigned max_fill = queue_ != nullptr ? queue_->get_max_size() : setting->max_fill;
+	const unsigned real_fill = queue_ != nullptr ? queue_->get_filled() : setting->desired_fill;
+	const unsigned desired_fill = queue_ != nullptr ? queue_->get_max_fill() : setting->desired_fill;
+	const unsigned nr_missing = queue_ != nullptr ? queue_->get_missing() : 0;
 	const unsigned nr_coming =
-	   queue_ && real_fill < desired_fill ? desired_fill - real_fill - nr_missing : 0;
+	   (queue_ != nullptr) && real_fill < desired_fill ? desired_fill - real_fill - nr_missing : 0;
 
 	assert(desired_fill <= max_fill);
 	assert(real_fill <= max_fill);
@@ -518,11 +623,11 @@ void InputQueueDisplay::think() {
 	}
 
 	if (has_priority_) {
-		const Widelands::WarePriority& p =                                            // NOLINT
-		   queue_ ? building_.get_priority(type_, index_) : get_setting()->priority;  // NOLINT
+		const Widelands::WarePriority& p =                                     // NOLINT
+		   queue_ ? b->get_priority(type_, index_) : get_setting()->priority;  // NOLINT
 		// The purpose of this check is to prevent the slider from snapping back directly after
 		// the user dragged it, because the playercommand is not executed immediately of course
-		if (!slider_was_moved_ || *slider_was_moved_ == p) {
+		if ((slider_was_moved_ == nullptr) || *slider_was_moved_ == p) {
 			priority_.set_value(priority_to_index(p));
 			slider_was_moved_ = nullptr;
 		}
@@ -531,7 +636,7 @@ void InputQueueDisplay::think() {
 
 	collapse_.set_tooltip(collapsed_ ? _("Show controls") : _("Hide controls"));
 	collapse_.set_pic(g_image_cache->get(collapsed_ ? "images/ui_basic/scrollbar_right.png" :
-	                                                  "images/ui_basic/scrollbar_left.png"));
+                                                     "images/ui_basic/scrollbar_left.png"));
 }
 
 static const RGBAColor kPriorityColors[] = {RGBAColor(0, 0, 255, 127), RGBAColor(63, 127, 255, 127),
@@ -553,19 +658,26 @@ void InputQueueDisplay::draw(RenderTarget& r) {
 }
 
 void InputQueueDisplay::draw_overlay(RenderTarget& r) {
+	MutexLock m(MutexLock::ID::kObjects);
+	Widelands::Building* b = building_.get(ibase_.egbase());
+	if (b == nullptr) {
+		return;
+	}
+
 	// Draw max fill indicator
 	if (!show_only_) {
-		assert(nr_icons_);
-		const unsigned desired_fill = queue_ ? queue_->get_max_fill() : get_setting()->desired_fill;
+		assert(nr_icons_ > 0);
+		const unsigned desired_fill =
+		   queue_ != nullptr ? queue_->get_max_fill() : get_setting()->desired_fill;
 		assert(desired_fill <= nr_icons_);
 
 		auto calc_xpos = [this](const size_t fill) {
 			return (fill == 0 ? icons_[0]->get_x() :
-			                    fill == nr_icons_ ?
-			                    icons_[nr_icons_ - 1]->get_x() + icons_[nr_icons_ - 1]->get_w() :
-			                    (icons_[fill - 1]->get_x() + icons_[fill - 1]->get_w() +
+			        fill == nr_icons_ ?
+                             icons_[nr_icons_ - 1]->get_x() + icons_[nr_icons_ - 1]->get_w() :
+                             (icons_[fill - 1]->get_x() + icons_[fill - 1]->get_w() +
 			                     icons_[fill]->get_x()) /
-			                          2) +
+			                       2) +
 			       hbox_.get_x() - max_fill_indicator_.width() / 2;
 		};
 
@@ -584,8 +696,8 @@ void InputQueueDisplay::draw_overlay(RenderTarget& r) {
 
 	// Draw priority indicator
 	if (has_priority_ && collapsed_) {
-		const size_t p = priority_to_index(queue_ ? building_.get_priority(type_, index_) :
-		                                            get_setting()->priority);
+		const size_t p = priority_to_index(queue_ != nullptr ? b->get_priority(type_, index_) :
+                                                             get_setting()->priority);
 		const int w = kButtonSize / 5;
 		const int x = hbox_.get_x() + collapse_.get_x() - w;
 		r.brighten_rect(Recti(x, hbox_.get_y(), w, kButtonSize), -32);
