@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -22,11 +21,12 @@
 #include <memory>
 
 #include <SDL_timer.h>
-#include <boost/algorithm/string.hpp>
 
 #include "base/log.h"
 #include "base/macros.h"
 #include "base/math.h"
+#include "base/multithreading.h"
+#include "base/string.h"
 #include "base/time_string.h"
 #include "economy/flag.h"
 #include "economy/road.h"
@@ -38,6 +38,7 @@
 #include "logic/cmd_queue.h"
 #include "logic/game.h"
 #include "logic/game_controller.h"
+#include "logic/game_data_error.h"
 #include "logic/map_objects/checkstep.h"
 #include "logic/map_objects/immovable.h"
 #include "logic/map_objects/tribes/productionsite.h"
@@ -51,19 +52,31 @@
 #include "scripting/lua_interface.h"
 #include "sound/sound_handler.h"
 #include "wlapplication_options.h"
+#include "wui/attack_window.h"
+#include "wui/building_statistics_menu.h"
 #include "wui/constructionsitewindow.h"
 #include "wui/dismantlesitewindow.h"
+#include "wui/economy_options_window.h"
+#include "wui/encyclopedia_window.h"
 #include "wui/game_chat_menu.h"
 #include "wui/game_debug_ui.h"
+#include "wui/game_diplomacy_menu.h"
+#include "wui/game_message_menu.h"
+#include "wui/game_objectives_menu.h"
 #include "wui/info_panel.h"
 #include "wui/mapviewpixelfunctions.h"
 #include "wui/militarysitewindow.h"
 #include "wui/minimap.h"
+#include "wui/seafaring_statistics_menu.h"
 #include "wui/shipwindow.h"
+#include "wui/soldier_statistics_menu.h"
+#include "wui/stock_menu.h"
 #include "wui/toolbar.h"
 #include "wui/trainingsitewindow.h"
 #include "wui/unique_window_handler.h"
+#include "wui/ware_statistics_menu.h"
 #include "wui/warehousewindow.h"
+#include "wui/watchwindow.h"
 
 namespace {
 
@@ -77,11 +90,11 @@ using Widelands::MapObject;
 using Widelands::TCoords;
 
 int caps_to_buildhelp(const Widelands::NodeCaps caps) {
-	if (caps & Widelands::BUILDCAPS_MINE) {
+	if ((caps & Widelands::BUILDCAPS_MINE) != 0) {
 		return Widelands::Field::Buildhelp_Mine;
 	}
 	if ((caps & Widelands::BUILDCAPS_SIZEMASK) == Widelands::BUILDCAPS_BIG) {
-		if (caps & Widelands::BUILDCAPS_PORT) {
+		if ((caps & Widelands::BUILDCAPS_PORT) != 0) {
 			return Widelands::Field::Buildhelp_Port;
 		}
 		return Widelands::Field::Buildhelp_Big;
@@ -92,7 +105,7 @@ int caps_to_buildhelp(const Widelands::NodeCaps caps) {
 	if ((caps & Widelands::BUILDCAPS_SIZEMASK) == Widelands::BUILDCAPS_SMALL) {
 		return Widelands::Field::Buildhelp_Small;
 	}
-	if (caps & Widelands::BUILDCAPS_FLAG) {
+	if ((caps & Widelands::BUILDCAPS_FLAG) != 0) {
 		return Widelands::Field::Buildhelp_Flag;
 	}
 	return Widelands::Field::Buildhelp_None;
@@ -119,7 +132,8 @@ InteractiveBase::InteractiveBase(EditorGameBase& the_egbase, Section& global_s,
                   _("Map View"),
                   UI::DropdownType::kPictorialMenu,
                   UI::PanelStyle::kWui,
-                  UI::ButtonStyle::kWuiPrimary),
+                  UI::ButtonStyle::kWuiPrimary,
+                  [this](MapviewMenuEntry t) { mapview_menu_selected(t); }),
      quick_navigation_(&map_view_),
      workareas_cache_(nullptr),
      egbase_(the_egbase),
@@ -131,9 +145,14 @@ InteractiveBase::InteractiveBase(EditorGameBase& the_egbase, Section& global_s,
      lastframe_(SDL_GetTicks()),
      frametime_(0),
      avg_usframetime_(0),
+     last_frame_realtime_(0),
+     previous_frame_realtime_(0),
+     last_frame_gametime_(0),
+     previous_frame_gametime_(0),
      road_building_mode_(nullptr),
      unique_window_handler_(new UniqueWindowHandler()),
-     cheat_mode_enabled_(false) {
+     cheat_mode_enabled_(false),
+     screenshot_failed_(false) {
 
 	// Load the buildhelp icons.
 	{
@@ -170,6 +189,9 @@ InteractiveBase::InteractiveBase(EditorGameBase& the_egbase, Section& global_s,
 	   [this](const GraphicResolutionChanged& message) {
 		   set_size(message.new_width, message.new_height);
 		   map_view_.set_size(message.new_width, message.new_height);
+		   map_view_.pan_by(Vector2i((message.old_width - message.new_width) / 2,
+		                             (message.old_height - message.new_height) / 2),
+		                    MapView::Transition::Jump);
 		   resize_chat_overlay();
 		   finalize_toolbar();
 		   info_panel_.layout();
@@ -227,6 +249,9 @@ InteractiveBase::InteractiveBase(EditorGameBase& the_egbase, Section& global_s,
 
 	setDefaultCommand([this](const std::vector<std::string>& str) { cmd_lua(str); });
 	addCommand("mapobject", [this](const std::vector<std::string>& str) { cmd_map_object(str); });
+
+	// Inform panel code that we have logic-related code
+	set_logic_think();
 }
 
 InteractiveBase::~InteractiveBase() {
@@ -262,22 +287,22 @@ void InteractiveBase::rebuild_mapview_menu() {
 	mapviewmenu_.add(minimap_registry_.window != nullptr ? _("Hide Minimap") : _("Show Minimap"),
 	                 MapviewMenuEntry::kMinimap,
 	                 g_image_cache->get("images/wui/menus/toggle_minimap.png"), false, "",
-	                 shortcut_string_for(KeyboardShortcut::kCommonMinimap));
+	                 shortcut_string_for(KeyboardShortcut::kCommonMinimap, false));
 
 	/** TRANSLATORS: An entry in the game's map view menu */
 	mapviewmenu_.add(_("Zoom +"), MapviewMenuEntry::kIncreaseZoom,
 	                 g_image_cache->get("images/wui/menus/zoom_increase.png"), false, "",
-	                 shortcut_string_for(KeyboardShortcut::kCommonZoomIn));
+	                 shortcut_string_for(KeyboardShortcut::kCommonZoomIn, false));
 
 	/** TRANSLATORS: An entry in the game's map view menu */
 	mapviewmenu_.add(_("Reset zoom"), MapviewMenuEntry::kResetZoom,
 	                 g_image_cache->get("images/wui/menus/zoom_reset.png"), false, "",
-	                 shortcut_string_for(KeyboardShortcut::kCommonZoomReset));
+	                 shortcut_string_for(KeyboardShortcut::kCommonZoomReset, false));
 
 	/** TRANSLATORS: An entry in the game's map view menu */
 	mapviewmenu_.add(_("Zoom –"), MapviewMenuEntry::kDecreaseZoom,
 	                 g_image_cache->get("images/wui/menus/zoom_decrease.png"), false, "",
-	                 shortcut_string_for(KeyboardShortcut::kCommonZoomOut));
+	                 shortcut_string_for(KeyboardShortcut::kCommonZoomOut, false));
 
 	mapviewmenu_.select(last_selection);
 }
@@ -296,7 +321,6 @@ void InteractiveBase::mapview_menu_selected(MapviewMenuEntry entry) {
 		map_view()->increase_zoom();
 		mapviewmenu_.toggle();
 	} break;
-
 	case MapviewMenuEntry::kResetZoom: {
 		map_view()->reset_zoom();
 		mapviewmenu_.toggle();
@@ -315,7 +339,9 @@ InteractiveBase::get_buildhelp_overlay(const Widelands::NodeCaps caps) const {
 
 bool InteractiveBase::has_workarea_preview(const Widelands::Coords& coords,
                                            const Widelands::Map* map) const {
-	if (!map) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
+	if (map == nullptr) {
 		for (const auto& preview : workarea_previews_) {
 			if (preview->coords == coords) {
 				return true;
@@ -375,7 +401,7 @@ InfoToDraw InteractiveBase::get_info_to_draw(bool show) const {
 	const auto display_flags = get_display_flags();
 	InfoToDraw info_to_draw = InfoToDraw::kNone;
 
-	if (display_flags & InteractiveBase::dfShowBuildings) {
+	if ((display_flags & InteractiveBase::dfShowBuildings) != 0u) {
 		info_to_draw = info_to_draw | InfoToDraw::kShowBuildings;
 	}
 
@@ -383,13 +409,13 @@ InfoToDraw InteractiveBase::get_info_to_draw(bool show) const {
 		return info_to_draw;
 	}
 
-	if (display_flags & InteractiveBase::dfShowCensus) {
+	if ((display_flags & InteractiveBase::dfShowCensus) != 0u) {
 		info_to_draw = info_to_draw | InfoToDraw::kCensus;
 	}
-	if (display_flags & InteractiveBase::dfShowStatistics) {
+	if ((display_flags & InteractiveBase::dfShowStatistics) != 0u) {
 		info_to_draw = info_to_draw | InfoToDraw::kStatistics;
 	}
-	if (display_flags & InteractiveBase::dfShowSoldierLevels) {
+	if ((display_flags & InteractiveBase::dfShowSoldierLevels) != 0u) {
 		info_to_draw = info_to_draw | InfoToDraw::kSoldierLevels;
 	}
 
@@ -398,6 +424,7 @@ InfoToDraw InteractiveBase::get_info_to_draw(bool show) const {
 
 void InteractiveBase::unset_sel_picture() {
 	set_sel_picture(g_image_cache->get("images/ui_basic/fsel.png"));
+	set_tooltip("");
 }
 
 bool InteractiveBase::buildhelp() const {
@@ -422,7 +449,7 @@ UI::Button* InteractiveBase::add_toolbar_button(const std::string& image_basenam
 	                  UI::ButtonStyle::kWuiPrimary,
 	                  g_image_cache->get("images/" + image_basename + ".png"), tooltip_text);
 	toolbar_.box.add(button);
-	if (window) {
+	if (window != nullptr) {
 		window->opened.connect([button] { button->set_perm_pressed(true); });
 		window->closed.connect([button] { button->set_perm_pressed(false); });
 
@@ -452,12 +479,14 @@ InteractiveBase::road_building_steepness_overlays() const {
 void InteractiveBase::show_workarea(const WorkareaInfo& workarea_info,
                                     Widelands::Coords coords,
                                     std::map<Widelands::TCoords<>, uint32_t>& extra_data) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
 	workarea_previews_.insert(
 	   std::unique_ptr<WorkareaPreview>(new WorkareaPreview{coords, &workarea_info, extra_data}));
 	workareas_cache_.reset(nullptr);
 }
 
 void InteractiveBase::show_workarea(const WorkareaInfo& workarea_info, Widelands::Coords coords) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
 	std::map<Widelands::TCoords<>, uint32_t> empty;
 	show_workarea(workarea_info, coords, empty);
 }
@@ -505,12 +534,15 @@ static uint8_t workarea_max(uint8_t a, uint8_t b, uint8_t c) {
 }
 
 Workareas InteractiveBase::get_workarea_overlays(const Widelands::Map& map) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	if (!workareas_cache_) {
 		workareas_cache_.reset(new Workareas());
 		for (const auto& preview : workarea_previews_) {
 			workareas_cache_->push_back(get_workarea_overlay(map, *preview));
 		}
 	}
+
 	return Workareas(*workareas_cache_);
 }
 
@@ -625,8 +657,10 @@ WorkareasEntry InteractiveBase::get_workarea_overlay(const Widelands::Map& map,
 }
 
 void InteractiveBase::hide_workarea(const Widelands::Coords& coords, bool is_additional) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	for (auto it = workarea_previews_.begin(); it != workarea_previews_.end(); ++it) {
-		if (it->get()->coords == coords && (is_additional ^ it->get()->data.empty())) {
+		if (it->get()->coords == coords && ((is_additional ^ it->get()->data.empty()) != 0)) {
 			workarea_previews_.erase(it);
 			workareas_cache_.reset(nullptr);
 			return;
@@ -644,6 +678,8 @@ void InteractiveBase::postload() {
 }
 
 void InteractiveBase::draw_road_building(FieldsToDraw::Field& field) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	const auto rpo = road_building_preview_overlays();
 	const auto rinfo = rpo.find(field.fcoords);
 	if (rinfo != rpo.end()) {
@@ -651,18 +687,18 @@ void InteractiveBase::draw_road_building(FieldsToDraw::Field& field) {
 			switch (dir) {
 			case Widelands::WALK_E:
 				field.road_e = in_road_building_mode(RoadBuildingType::kRoad) ?
-				                  Widelands::RoadSegment::kNormal :
-				                  Widelands::RoadSegment::kWaterway;
+                              Widelands::RoadSegment::kNormal :
+                              Widelands::RoadSegment::kWaterway;
 				break;
 			case Widelands::WALK_SE:
 				field.road_se = in_road_building_mode(RoadBuildingType::kRoad) ?
-				                   Widelands::RoadSegment::kNormal :
-				                   Widelands::RoadSegment::kWaterway;
+                               Widelands::RoadSegment::kNormal :
+                               Widelands::RoadSegment::kWaterway;
 				break;
 			case Widelands::WALK_SW:
 				field.road_sw = in_road_building_mode(RoadBuildingType::kRoad) ?
-				                   Widelands::RoadSegment::kNormal :
-				                   Widelands::RoadSegment::kWaterway;
+                               Widelands::RoadSegment::kNormal :
+                               Widelands::RoadSegment::kWaterway;
 				break;
 			default:
 				throw wexception("Attempt to set road-building overlay for invalid direction %i", dir);
@@ -675,15 +711,29 @@ void InteractiveBase::info_panel_fast_forward_message_queue() {
 	info_panel_.fast_forward_message_queue();
 }
 
-/*
-===============
-Called once per frame by the UI code
-===============
-*/
-void InteractiveBase::think() {
-	egbase().think();  // Call game logic here. The game advances.
+// Called one per logic frame by the logic thread's main loop.
+// This is where the actual game logic (and from it, the game
+// controller's and the AI's `think()` functions) are called.
+// Also updates the stats about the logic FPS and real gamespeed.
+void InteractiveBase::game_logic_think() {
+	previous_frame_realtime_ = last_frame_realtime_;
+	previous_frame_gametime_ = last_frame_gametime_;
+	last_frame_realtime_ = SDL_GetTicks();
+	last_frame_gametime_ = egbase().get_gametime();
 
+	egbase().think();  // Call game logic here. The game advances.
+}
+
+void InteractiveBase::think() {
 	UI::Panel::think();
+	if (in_road_building_mode()) {
+		if (in_road_building_mode(RoadBuildingType::kRoad)) {
+			set_tooltip(format(_("Road length: %u"), get_build_road_path().get_nsteps()));
+		} else {
+			set_tooltip(format(_("Waterway length: %1$u/%2$u"), get_build_road_path().get_nsteps(),
+			                   egbase().map().get_waterway_max_length()));
+		}
+	}
 }
 
 double InteractiveBase::average_fps() const {
@@ -695,7 +745,7 @@ double InteractiveBase::average_fps() const {
 Draw debug overlay when appropriate.
 ===============
 */
-void InteractiveBase::draw_overlay(RenderTarget&) {
+void InteractiveBase::draw_overlay(RenderTarget& /* rt */) {
 	// Timing
 	uint32_t curframe = SDL_GetTicks();
 
@@ -703,54 +753,18 @@ void InteractiveBase::draw_overlay(RenderTarget&) {
 	avg_usframetime_ = ((avg_usframetime_ * 15) + (frametime_ * 1000)) / 16;
 	lastframe_ = curframe;
 
-	Game* game = dynamic_cast<Game*>(&egbase());
-
-	if (in_road_building_mode() && tooltip().empty()) {
-		draw_tooltip(
-		   in_road_building_mode(RoadBuildingType::kRoad) ?
-		      (boost::format(_("Road length: %u")) % get_build_road_path().get_nsteps()).str() :
-		      (boost::format(_("Waterway length: %1$u/%2$u")) % get_build_road_path().get_nsteps() %
-		       egbase().map().get_waterway_max_length())
-		         .str(),
-		   UI::PanelStyle::kWui);
-	}
-
-	// This portion of code keeps the speed of game so that FPS are kept within
-	// range 13 - 15, this is used for training of AI
-	if (game != nullptr) {
-		if (game->is_auto_speed()) {
-			const uint32_t cur_fps = average_fps();
-			int32_t speed_diff = 0;
-			if (cur_fps < 13) {
-				speed_diff = -100;
-			}
-			if (cur_fps > 15) {
-				speed_diff = +100;
-			}
-			if (speed_diff != 0) {
-				if (GameController* const ctrl = game->game_controller()) {
-					if ((ctrl->desired_speed() > 950 && ctrl->desired_speed() < 30000) ||
-					    (ctrl->desired_speed() < 1000 && speed_diff > 0) ||
-					    (ctrl->desired_speed() > 29999 && speed_diff < 0)) {
-						ctrl->set_desired_speed(ctrl->desired_speed() + speed_diff);
-					}
-				}
-			}
-		}
-	}
-
 	// Node information
 	std::string node_text;
-	if (game == nullptr || get_display_flag(dfDebug)) {
+	if (!egbase().is_game() || get_display_flag(dfDebug)) {
 		// Blit node information in the editor, and in debug mode also for games
-		static boost::format node_format("(%i, %i, %i)");
 		const int32_t height = egbase().map()[sel_.pos.node].get_height();
-		node_text = (node_format % sel_.pos.node.x % sel_.pos.node.y % height).str();
+		node_text = format("(%i, %i, %i)", sel_.pos.node.x, sel_.pos.node.y, height);
 	}
 	info_panel_.set_coords_string(node_text);
 
 	// In-game clock and FPS
-	info_panel_.set_time_string(game ? gametimestring(egbase().get_gametime().get(), true) : "");
+	info_panel_.set_time_string(
+	   egbase().is_game() ? gametimestring(egbase().get_gametime().get(), true) : "");
 	info_panel_.set_fps_string(
 	   get_display_flag(dfDebug), cheat_mode_enabled_, 1000.0 / frametime_, average_fps());
 }
@@ -808,7 +822,7 @@ void InteractiveBase::mainview_move() {
 
 // Open the minimap or close it if it's open
 void InteractiveBase::toggle_minimap() {
-	if (!minimap_registry_.window) {
+	if (minimap_registry_.window == nullptr) {
 		// Don't store the MiniMap pointer, we can access it via 'minimap_registry_.get_window()'.
 		// A MiniMap is a UniqueWindow, its parent will delete it.
 		new MiniMap(*this, &minimap_registry_);
@@ -837,7 +851,7 @@ void InteractiveBase::hide_minimap() {
 	minimap_registry_.destroy();
 }
 
-void InteractiveBase::resize_minimap() {
+void InteractiveBase::resize_minimap() const {
 	if (MiniMap* const minimap = minimap_registry_.get_window()) {
 		minimap->check_boundaries();
 	}
@@ -871,7 +885,7 @@ Get and set one individual flag of the display flags.
 ===============
 */
 bool InteractiveBase::get_display_flag(uint32_t const flag) const {
-	return display_flags_ & flag;
+	return (display_flags_ & flag) != 0u;
 }
 
 void InteractiveBase::set_display_flag(uint32_t const flag, bool const on) {
@@ -882,7 +896,125 @@ void InteractiveBase::set_display_flag(uint32_t const flag, bool const on) {
 		display_flags_ |= flag;
 	}
 	if (old_value != display_flags_) {
-		rebuild_showhide_menu();
+		NoteThreadSafeFunction::instantiate([this]() { rebuild_showhide_menu(); }, false);
+	}
+}
+
+/*
+===============
+Saveloading support for open unique windows.
+===============
+*/
+constexpr uint16_t kCurrentPacketVersionUniqueWindows = 1;
+
+void InteractiveBase::load_windows(FileRead& fr, Widelands::MapObjectLoader& mol) {
+	try {
+		const uint16_t packet_version = fr.unsigned_16();
+		if (packet_version == kCurrentPacketVersionUniqueWindows) {
+			for (;;) {
+				const UI::Panel::SaveType type = static_cast<UI::Panel::SaveType>(fr.unsigned_8());
+				if (type == UI::Panel::SaveType::kNone) {
+					break;
+				}
+
+				const int32_t x = fr.signed_32();
+				const int32_t y = fr.signed_32();
+				const bool pin = fr.unsigned_8() != 0u;
+				UI::Window* w = nullptr;
+
+				switch (type) {
+				case UI::Panel::SaveType::kBuildingWindow:
+					w = &BuildingWindow::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kShipWindow:
+					w = &ShipWindow::load(fr, *this, mol);
+					break;
+				case UI::Panel::SaveType::kWatchWindow:
+					w = &WatchWindow::load(fr, *this, mol);
+					break;
+				case UI::Panel::SaveType::kStockMenu:
+					w = &StockMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kGeneralStats:
+					w = &GeneralStatisticsMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kSoldierStats:
+					w = &SoldierStatisticsMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kWareStats:
+					w = &WareStatisticsMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kSeafaringStats:
+					w = &SeafaringStatisticsMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kBuildingStats:
+					w = &BuildingStatisticsMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kObjectives:
+					w = &GameObjectivesMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kDiplomacy:
+					w = &GameDiplomacyMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kMessages:
+					w = &GameMessageMenu::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kMinimap:
+					w = &MiniMap::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kEncyclopedia:
+					w = &UI::EncyclopediaWindow::load(fr, *this);
+					break;
+				case UI::Panel::SaveType::kConfigureEconomy:
+					w = EconomyOptionsWindow::load(fr, *this, mol);
+					break;
+				case UI::Panel::SaveType::kAttackWindow:
+					w = &AttackWindow::load(fr, *this, mol);
+					break;
+				default:
+					throw Widelands::GameDataError(
+					   "Invalid panel save type %u", static_cast<unsigned>(type));
+				}
+
+				if (w != nullptr) {
+					w->set_pinned(pin);
+					w->set_pos(Vector2i(x, y));
+					w->move_inside_parent();  // In case the game was loaded at a smaller resolution.
+				}
+			}
+		} else {
+			throw Widelands::UnhandledVersionError(
+			   "Unique Windows", packet_version, kCurrentPacketVersionUniqueWindows);
+		}
+	} catch (const WException& e) {
+		throw Widelands::GameDataError("unique windows: %s", e.what());
+	}
+}
+
+void InteractiveBase::save_windows(FileWrite& fw, Widelands::MapObjectSaver& mos) {
+	fw.unsigned_16(kCurrentPacketVersionUniqueWindows);
+	for (UI::Panel* child = get_first_child(); child != nullptr; child = child->get_next_sibling()) {
+		const UI::Panel::SaveType t = child->save_type();
+		if (t != UI::Panel::SaveType::kNone) {
+			fw.unsigned_8(static_cast<uint8_t>(t));
+			fw.signed_32(child->get_x());
+			fw.signed_32(child->get_y());
+			fw.unsigned_8(dynamic_cast<UI::Window&>(*child).is_pinned() ? 1 : 0);
+			child->save(fw, mos);
+		}
+	}
+	fw.unsigned_8(0);
+}
+
+void InteractiveBase::cleanup_for_load() {
+	std::set<UI::Panel*> panels_to_kill;
+	for (UI::Panel* child = get_first_child(); child != nullptr; child = child->get_next_sibling()) {
+		if (dynamic_cast<UI::Window*>(child) != nullptr) {
+			panels_to_kill.insert(child);
+		}
+	}
+	for (UI::Panel* p : panels_to_kill) {
+		delete p;
 	}
 }
 
@@ -894,13 +1026,15 @@ Begin building a road
 void InteractiveBase::start_build_road(Coords road_start,
                                        Widelands::PlayerNumber const player,
                                        RoadBuildingType t) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	assert(!road_building_mode_);
 	road_building_mode_.reset(new RoadBuildingMode(player, road_start, t));
 
 	road_building_add_overlay();
 	set_sel_picture(g_image_cache->get(t == RoadBuildingType::kWaterway ?
-	                                      "images/ui_basic/fsel_waterwaybuilding.png" :
-	                                      "images/ui_basic/fsel_roadbuilding.png"));
+                                         "images/ui_basic/fsel_waterwaybuilding.png" :
+                                         "images/ui_basic/fsel_roadbuilding.png"));
 
 	if (t == RoadBuildingType::kWaterway) {
 		// Show workarea to visualise length limit
@@ -949,6 +1083,8 @@ Stop building the road
 ===============
 */
 void InteractiveBase::abort_build_road() {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	assert(road_building_mode_);
 	if (road_building_mode_->type == RoadBuildingType::kWaterway) {
 		assert(road_building_mode_->work_area);
@@ -971,6 +1107,8 @@ Finally build the road
 ===============
 */
 void InteractiveBase::finish_build_road() {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	assert(road_building_mode_);
 
 	if (road_building_mode_->type == RoadBuildingType::kWaterway) {
@@ -991,11 +1129,11 @@ void InteractiveBase::finish_build_road() {
 		log_warn_time(egbase().get_gametime(),
 		              "Refusing to finish waterway building: length is %" PRIuS " but limit is %d\n",
 		              length, egbase().map().get_waterway_max_length());
-	} else if (length) {
+	} else if (length != 0u) {
 		upcast(Game, g, &egbase());
 
 		// Build the path as requested
-		if (g) {
+		if (g != nullptr) {
 			if (road_building_mode_->type == RoadBuildingType::kWaterway) {
 				g->send_player_build_waterway(
 				   road_building_mode_->player, *new Widelands::Path(road_building_mode_->path));
@@ -1015,7 +1153,7 @@ void InteractiveBase::finish_build_road() {
 			}
 		}
 
-		if (allow_user_input() && (SDL_GetModState() & KMOD_CTRL)) {
+		if (allow_user_input() && ((SDL_GetModState() & KMOD_CTRL) != 0)) {
 			//  place flags
 			const Map& map = egbase().map();
 			const std::vector<Coords>& c_vector = road_building_mode_->path.get_coords();
@@ -1023,14 +1161,14 @@ void InteractiveBase::finish_build_road() {
 			std::vector<Coords>::const_iterator const last = c_vector.end() - 2;
 
 			auto place_flag = [this, g](const Widelands::FCoords& coords) {
-				if (g) {
+				if (g != nullptr) {
 					g->send_player_build_flag(road_building_mode_->player, coords);
 				} else {
 					egbase().get_player(road_building_mode_->player)->build_flag(coords);
 				}
 			};
 
-			if (SDL_GetModState() & KMOD_SHIFT) {
+			if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
 				//  start to end
 				for (std::vector<Coords>::const_iterator it = first; it <= last; ++it) {
 					place_flag(map.get_fcoords(*it));
@@ -1055,6 +1193,8 @@ Otherwise append if possible or return false.
 ===============
 */
 bool InteractiveBase::append_build_road(Coords const field) {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	assert(road_building_mode_);
 
 	const Map& map = egbase().map();
@@ -1164,8 +1304,8 @@ void InteractiveBase::play_sound_effect(const NoteSound& note) const {
 		               kSoundDistanceDivisor;
 
 		distance = (note.priority == kFxMaximumPriority) ?
-		              (math::clamp(distance, 0, kSoundMaxDistance) / 2) :
-		              distance;
+                    (math::clamp(distance, 0, kSoundMaxDistance) / 2) :
+                    distance;
 
 		if (distance < kSoundMaxDistance) {
 			g_sh->play_fx(note.type, note.fx, note.priority, note.allow_multiple,
@@ -1186,6 +1326,8 @@ Add road building data to the road overlay
 ===============
 */
 void InteractiveBase::road_building_add_overlay() {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
+
 	assert(road_building_mode_);
 	assert(road_building_mode_->overlay_road_previews.empty());
 	assert(road_building_mode_->overlay_steepness_indicators.empty());
@@ -1237,16 +1379,16 @@ void InteractiveBase::road_building_add_overlay() {
 			if (!next_to && road_building_mode_->path.get_nsteps() >= map.get_waterway_max_length()) {
 				continue;  // exceeds length limit
 			}
-		} else if (!(caps & Widelands::MOVECAPS_WALK)) {
+		} else if ((caps & Widelands::MOVECAPS_WALK) == 0) {
 			continue;  // need to be able to walk there
 		}
 
 		//  can't build on robusts
 		const Widelands::BaseImmovable* imm = map.get_immovable(neighb);
-		if (imm && imm->get_size() >= Widelands::BaseImmovable::SMALL &&
-		    (!(dynamic_cast<const Widelands::Flag*>(imm) ||
-		       (dynamic_cast<const Widelands::RoadBase*>(imm) &&
-		        (caps & Widelands::BUILDCAPS_FLAG))))) {
+		if ((imm != nullptr) && imm->get_size() >= Widelands::BaseImmovable::SMALL &&
+		    (!((dynamic_cast<const Widelands::Flag*>(imm) != nullptr) ||
+		       ((dynamic_cast<const Widelands::RoadBase*>(imm) != nullptr) &&
+		        ((caps & Widelands::BUILDCAPS_FLAG) != 0))))) {
 			continue;
 		}
 		if (road_building_mode_->path.get_index(neighb) >= 0) {
@@ -1298,6 +1440,7 @@ Remove road building data from road overlay
 ===============
 */
 void InteractiveBase::road_building_remove_overlay() {
+	MutexLock m(MutexLock::ID::kIBaseVisualizations);
 	assert(road_building_mode_);
 	road_building_mode_->overlay_road_previews.clear();
 	road_building_mode_->overlay_steepness_indicators.clear();
@@ -1319,7 +1462,7 @@ UI::UniqueWindow* InteractiveBase::show_building_window(const Widelands::Coords&
 	upcast(Widelands::Building, building, immovable);
 	assert(building);
 	UI::UniqueWindow::Registry& registry =
-	   unique_windows().get_registry((boost::format("building_%d") % building->serial()).str());
+	   unique_windows().get_registry(format("building_%d", building->serial()));
 
 	switch (building->descr().type()) {
 	case Widelands::MapObjectType::CONSTRUCTIONSITE:
@@ -1366,24 +1509,25 @@ UI::UniqueWindow* InteractiveBase::show_building_window(const Widelands::Coords&
 		             building->descr().name().c_str(), to_string(building->descr().type()).c_str());
 		NEVER_HERE();
 	}
-	registry.create();
+	NoteThreadSafeFunction::instantiate([&registry]() { registry.create(); }, true);
 	return registry.window;
 }
 
-void InteractiveBase::show_ship_window(Widelands::Ship* ship) {
+UI::UniqueWindow& InteractiveBase::show_ship_window(Widelands::Ship* ship) {
 	UI::UniqueWindow::Registry& registry =
-	   unique_windows().get_registry((boost::format("ship_%d") % ship->serial()).str());
+	   unique_windows().get_registry(format("ship_%d", ship->serial()));
 	registry.open_window = [this, &registry, ship] { new ShipWindow(*this, registry, ship); };
 	registry.create();
+	return *registry.window;
 }
 
-void InteractiveBase::broadcast_cheating_message() {
-	if (!get_game()) {
+void InteractiveBase::broadcast_cheating_message() const {
+	if (get_game() == nullptr) {
 		return;  // Editor
 	}
 	if (upcast(GameHost, h, game().game_controller())) {
 		h->send_system_message_code(
-		   "CHEAT", player_number() ? game().player(player_number()).get_name() : "");
+		   "CHEAT", player_number() != 0u ? game().player(player_number()).get_name() : "");
 	} else if (upcast(GameClient, c, game().game_controller())) {
 		c->send_cheating_info();
 	}
@@ -1394,6 +1538,20 @@ bool InteractiveBase::handle_key(bool const down, SDL_Keysym const code) {
 		return true;
 	}
 
+	if (matches_shortcut(KeyboardShortcut::kCommonScreenshot, code)) {
+		// Screenshot taken by topmost handler, just show a notification
+		if (down) {
+			log_message(_("Failed saving screenshot!"));
+			screenshot_failed_ = true;
+		} else {
+			if (!screenshot_failed_) {
+				log_message(_("Screenshot saved"));
+			}
+			screenshot_failed_ = false;
+		}
+		return true;
+	}
+
 	if (down) {
 		if (matches_shortcut(KeyboardShortcut::kCommonBuildhelp, code)) {
 			toggle_buildhelp();
@@ -1407,14 +1565,14 @@ bool InteractiveBase::handle_key(bool const down, SDL_Keysym const code) {
 		switch (code.sym) {
 #ifndef NDEBUG  //  only in debug builds
 		case SDLK_SPACE:
-			if ((code.mod & KMOD_CTRL) && (code.mod & KMOD_SHIFT)) {
+			if (((code.mod & KMOD_CTRL) != 0) && ((code.mod & KMOD_SHIFT) != 0)) {
 				GameChatMenu::create_script_console(
 				   this, debugconsole_, *DebugConsole::get_chat_provider());
 				return true;
 			}
 			break;
 		case SDLK_BACKSPACE:
-			if ((code.mod & KMOD_CTRL) && (code.mod & KMOD_SHIFT)) {
+			if (((code.mod & KMOD_CTRL) != 0) && ((code.mod & KMOD_SHIFT) != 0)) {
 				if (cheat_mode_enabled_) {
 					cheat_mode_enabled_ = false;
 				} else if (!omnipotent()) {
@@ -1440,7 +1598,7 @@ bool InteractiveBase::handle_key(bool const down, SDL_Keysym const code) {
 }
 
 void InteractiveBase::cmd_lua(const std::vector<std::string>& args) {
-	const std::string cmd = boost::algorithm::join(args, " ");
+	const std::string cmd = join(args, " ");
 
 	broadcast_cheating_message();
 
@@ -1463,11 +1621,11 @@ void InteractiveBase::cmd_map_object(const std::vector<std::string>& args) {
 		return;
 	}
 
-	uint32_t serial = boost::lexical_cast<uint32_t>(args[1]);
+	uint32_t serial = stoul(args[1]);
 	MapObject* obj = egbase().objects().get_object(serial);
 
-	if (!obj) {
-		DebugConsole::write(str(boost::format("No MapObject with serial number %1%") % serial));
+	if (obj == nullptr) {
+		DebugConsole::write(format("No MapObject with serial number %1%", serial));
 		return;
 	}
 
