@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2021 by the Widelands Development Team
+ * Copyright (C) 2002-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -61,9 +60,7 @@ enum class Units {
 };
 
 std::string ytick_text_style(const std::string& text, const UI::FontStyleInfo& style) {
-	static boost::format f("<rt keep_spaces=1><p>%s</p></rt>");
-	f % style.as_font_tag(text);
-	return f.str();
+	return format("<rt keep_spaces=1><p>%s</p></rt>", style.as_font_tag(text));
 }
 
 std::string xtick_text_style(const std::string& text) {
@@ -88,28 +85,27 @@ Units get_suggested_unit(uint32_t game_time, bool is_generic = false) {
 			return Units::kHourGeneric;
 		}
 		return Units::kMinutesGeneric;
-	} else {
-		if (game_time > 4 * kDays) {
-			return Units::kDayNarrow;
-		}
-		if (game_time > 4 * kHours) {
-			return Units::kHourNarrow;
-		}
-		return Units::kMinutesNarrow;
 	}
+	if (game_time > 4 * kDays) {
+		return Units::kDayNarrow;
+	}
+	if (game_time > 4 * kHours) {
+		return Units::kHourNarrow;
+	}
+	return Units::kMinutesNarrow;
 }
 
 std::string get_value_with_unit(Units unit, int value) {
 	switch (unit) {
 	case Units::kDayNarrow:
 		/** TRANSLATORS: day(s). Keep this as short as possible. Used in statistics. */
-		return (boost::format(npgettext("unit_narrow", "%1%d", "%1%d", value)) % value).str();
+		return format(npgettext("unit_narrow", "%1%d", "%1%d", value), value);
 	case Units::kHourNarrow:
 		/** TRANSLATORS: hour(s). Keep this as short as possible. Used in statistics. */
-		return (boost::format(npgettext("unit_narrow", "%1%h", "%1%h", value)) % value).str();
+		return format(npgettext("unit_narrow", "%1%h", "%1%h", value), value);
 	case Units::kMinutesNarrow:
 		/** TRANSLATORS: minute(s). Keep this as short as possible. Used in statistics. */
-		return (boost::format(npgettext("unit_narrow", "%1%m", "%1%m", value)) % value).str();
+		return format(npgettext("unit_narrow", "%1%m", "%1%m", value), value);
 	case Units::kMinutesGeneric:
 	case Units::kHourGeneric:
 	case Units::kDayGeneric:
@@ -199,7 +195,8 @@ void draw_diagram(uint32_t time_ms,
                   RenderTarget& dst) {
 	const RGBColor& axis_line_color = g_style_manager->statistics_plot_style().axis_line_color();
 
-	uint32_t how_many_ticks, max_x;
+	uint32_t how_many_ticks;
+	uint32_t max_x;
 
 	Units unit = get_suggested_unit(time_ms, true);
 	max_x = ms_to_unit(unit, time_ms);
@@ -265,8 +262,8 @@ void draw_diagram(uint32_t time_ms,
 		// The space at the end is intentional to have the tick centered
 		// over the number, not to the left
 		if (how_many_ticks != 0 && i != 0) {
-			std::shared_ptr<const UI::RenderedText> xtick = UI::g_fh->render(
-			   xtick_text_style((boost::format("-%u ") % (max_x / how_many_ticks * i)).str()));
+			std::shared_ptr<const UI::RenderedText> xtick =
+			   UI::g_fh->render(xtick_text_style(format("-%u ", (max_x / how_many_ticks * i))));
 			Vector2i pos(posx, inner_h - kSpaceBottom + 10);
 			UI::center_vertically(xtick->height(), &pos);
 			xtick->draw(dst, pos, UI::Align::kCenter);
@@ -283,24 +280,26 @@ void draw_diagram(uint32_t time_ms,
 	dst.draw_line_strip(
 	   {Vector2f(
 	       inner_w - kSpaceRight + 2,
-	       kSpacing * 3 + ((((inner_h - kSpaceBottom) + kSpacing * 3) / 2) - kSpacing * 3) / 2),
+	       kSpacing * 3 + ((((inner_h - kSpaceBottom) + kSpacing * 3) / 2.f) - kSpacing * 3) / 2.f),
 	    Vector2f(
 	       inner_w - kSpaceRight,
-	       kSpacing * 3 + ((((inner_h - kSpaceBottom) + kSpacing * 3) / 2) - kSpacing * 3) / 2)},
+	       kSpacing * 3 + ((((inner_h - kSpaceBottom) + kSpacing * 3) / 2.f) - kSpacing * 3) / 2.f)},
 	   axis_line_color, kAxisLinesWidth);
 
 	dst.draw_line_strip(
-	   {Vector2f(inner_w - kSpaceRight + 3, ((inner_h - kSpaceBottom) + kSpacing * 3) / 2),
-	    Vector2f(inner_w - kSpaceRight, ((inner_h - kSpaceBottom) + kSpacing * 3) / 2)},
+	   {Vector2f(inner_w - kSpaceRight + 3, ((inner_h - kSpaceBottom) + kSpacing * 3) / 2.f),
+	    Vector2f(inner_w - kSpaceRight, ((inner_h - kSpaceBottom) + kSpacing * 3) / 2.f)},
 	   axis_line_color, kAxisLinesWidth);
 
 	dst.draw_line_strip(
-	   {Vector2f(inner_w - kSpaceRight + 2,
-	             inner_h - kSpaceBottom -
-	                (inner_h - kSpaceBottom - ((inner_h - kSpaceBottom) + kSpacing * 3) / 2) / 2),
-	    Vector2f(inner_w - kSpaceRight,
-	             inner_h - kSpaceBottom -
-	                (inner_h - kSpaceBottom - ((inner_h - kSpaceBottom) + kSpacing * 3) / 2) / 2)},
+	   {Vector2f(
+	       inner_w - kSpaceRight + 2,
+	       inner_h - kSpaceBottom -
+	          (inner_h - kSpaceBottom - ((inner_h - kSpaceBottom) + kSpacing * 3) / 2.f) / 2.f),
+	    Vector2f(
+	       inner_w - kSpaceRight,
+	       inner_h - kSpaceBottom -
+	          (inner_h - kSpaceBottom - ((inner_h - kSpaceBottom) + kSpacing * 3) / 2.f) / 2.f)},
 	   axis_line_color, kAxisLinesWidth);
 
 	dst.draw_line_strip({Vector2f(inner_w - kSpaceRight + 3, inner_h - kSpaceBottom),
@@ -357,7 +356,7 @@ std::vector<std::string> WuiPlotArea::get_labels() const {
 		Units unit = get_suggested_unit(time_in_ms[i], false);
 		labels.push_back(get_value_with_unit(unit, ms_to_unit(unit, time_in_ms[i])));
 	}
-	labels.push_back(_("game"));
+	labels.emplace_back(_("game"));
 	return labels;
 }
 
@@ -373,13 +372,13 @@ uint32_t WuiPlotArea::get_plot_time() const {
 		// or a multiple of 20h
 		// or a multiple of 4 days
 		if (time_ms > 8 * kDays) {
-			time_ms += -(time_ms % (4 * kDays)) + 4 * kDays;
+			time_ms += 4 * kDays - (time_ms % (4 * kDays));
 		} else if (time_ms > 40 * kHours) {
-			time_ms += -(time_ms % (20 * kHours)) + 20 * kHours;
+			time_ms += 20 * kHours - (time_ms % (20 * kHours));
 		} else if (time_ms > 4 * kHours) {
-			time_ms += -(time_ms % (2 * kHours)) + 2 * kHours;
+			time_ms += 2 * kHours - (time_ms % (2 * kHours));
 		} else {
-			time_ms += -(time_ms % (15 * kMinutes)) + 15 * kMinutes;
+			time_ms += 15 * kMinutes - (time_ms % (15 * kMinutes));
 		}
 		return time_ms;
 	}
@@ -496,12 +495,11 @@ void WuiPlotArea::draw(RenderTarget& dst) {
 		update();
 		needs_update_ = false;
 	}
-	if (highest_scale_) {
+	if (highest_scale_ != 0u) {
 		draw_plot(dst, get_inner_h() - kSpaceBottom, std::to_string(highest_scale_), highest_scale_);
 	}
 	// Print the 0
-	draw_value((boost::format("%u") % (0)).str(),
-	           g_style_manager->statistics_plot_style().x_tick_font(),
+	draw_value("0", g_style_manager->statistics_plot_style().x_tick_font(),
 	           Vector2i(get_inner_w() - kSpaceRight + 3, get_inner_h() - kSpaceBottom + 10), dst);
 }
 
@@ -515,7 +513,7 @@ void WuiPlotArea::draw_plot(RenderTarget& dst,
 		if (plot.second.showplot) {
 			draw_plot_line(dst,
 			               (plotmode_ == Plotmode::kRelative) ? plot.second.relative_data.get() :
-			                                                    plot.second.absolute_data,
+                                                             plot.second.absolute_data,
 			               highest_scale, sub_, plot.second.plotcolor, yoffset);
 		}
 	}
@@ -723,7 +721,7 @@ void DifferentialPlotArea::draw(RenderTarget& dst) {
 	         Vector2i::zero());
 
 	// yoffset of the zero line
-	float const yoffset = ((get_inner_h() - kSpaceBottom) + kSpacing * 3) / 2;
+	float const yoffset = ((get_inner_h() - kSpaceBottom) + kSpacing * 3) / 2.f;
 
 	// draw zero line
 	dst.draw_line_strip({Vector2f(get_inner_w() - kSpaceRight, yoffset),
@@ -733,7 +731,7 @@ void DifferentialPlotArea::draw(RenderTarget& dst) {
 	// Draw data and diagram
 	draw_plot(dst, yoffset, std::to_string(highest_scale_), 2 * highest_scale_);
 	// Print the min value
-	draw_value((boost::format("-%u") % (highest_scale_)).str(),
+	draw_value(format("-%u", (highest_scale_)),
 	           g_style_manager->statistics_plot_style().y_min_value_font(),
 	           Vector2i(get_inner_w() - kSpaceRight + 3, get_inner_h() - kSpaceBottom + 10), dst);
 }
