@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2021 by the Widelands Development Team
+ * Copyright (C) 2003-2022 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -82,7 +81,7 @@ AbstractWaresDisplay::AbstractWaresDisplay(
 	curware_.set_text(_("Stock"));
 
 	graphic_resolution_changed_subscriber_ = Notifications::subscribe<GraphicResolutionChanged>(
-	   [this](const GraphicResolutionChanged&) { recalc_desired_size(true); });
+	   [this](const GraphicResolutionChanged& /* note */) { recalc_desired_size(true); });
 
 	recalc_desired_size(false);
 }
@@ -132,9 +131,9 @@ void AbstractWaresDisplay::recalc_desired_size(bool relayout) {
 		// TODO(GunChleoc): Window::on_resolution_changed_note can't shift these properly due to the
 		// changing dimensions.
 		UI::Panel* p = this;
-		while (p->get_parent()) {
+		while (p->get_parent() != nullptr) {
 			p = p->get_parent();
-			if (dynamic_cast<UI::Window*>(p)) {
+			if (dynamic_cast<UI::Window*>(p) != nullptr) {
 				p->layout();
 				return;
 			}
@@ -142,19 +141,20 @@ void AbstractWaresDisplay::recalc_desired_size(bool relayout) {
 	}
 }
 
-bool AbstractWaresDisplay::handle_mousemove(uint8_t state, int32_t x, int32_t y, int32_t, int32_t) {
+bool AbstractWaresDisplay::handle_mousemove(
+   uint8_t state, int32_t x, int32_t y, int32_t /*xdiff*/, int32_t /*ydiff*/) {
 	const Widelands::DescriptionIndex index = ware_at_point(x, y);
 
 	curware_.set_fixed_width(get_inner_w());
 
 	curware_.set_text(index != Widelands::INVALID_INDEX ?
-	                     (type_ == Widelands::wwWORKER ? tribe_.get_worker_descr(index)->descname() :
-	                                                     tribe_.get_ware_descr(index)->descname()) :
-	                     "");
+                        (type_ == Widelands::wwWORKER ? tribe_.get_worker_descr(index)->descname() :
+                                                        tribe_.get_ware_descr(index)->descname()) :
+                        "");
 	if (selection_anchor_ != Widelands::INVALID_INDEX) {
 		// Ensure mouse button is still pressed as some
 		// mouse release events do not reach us
-		if (state ^ SDL_BUTTON_LMASK) {
+		if ((state ^ SDL_BUTTON_LMASK) != 0) {
 			// TODO(unknown): We should call another function that will not pass that events
 			// to our Panel superclass
 			handle_mouserelease(SDL_BUTTON_LEFT, x, y);
@@ -436,7 +436,7 @@ void AbstractWaresDisplay::draw_ware(RenderTarget& dst, Widelands::DescriptionIn
 	//  draw a background
 	const UI::WareInfoStyleInfo& style =
 	   draw_selected ? g_style_manager->ware_info_style(UI::WareInfoStyle::kHighlight) :
-	                   g_style_manager->ware_info_style(UI::WareInfoStyle::kNormal);
+                      g_style_manager->ware_info_style(UI::WareInfoStyle::kNormal);
 
 	uint16_t w = style.icon_background_image()->width();
 
@@ -446,7 +446,7 @@ void AbstractWaresDisplay::draw_ware(RenderTarget& dst, Widelands::DescriptionIn
 	              draw_ware_background_overlay(id), BlendMode::Default);
 
 	const Image* icon = type_ == Widelands::wwWORKER ? tribe_.get_worker_descr(id)->icon() :
-	                                                   tribe_.get_ware_descr(id)->icon();
+                                                      tribe_.get_ware_descr(id)->icon();
 
 	dst.blit(p + Vector2i((w - kWareMenuPicWidth) / 2, 1), icon);
 
@@ -517,7 +517,7 @@ StockMenuWaresDisplay::StockMenuWaresDisplay(UI::Panel* const parent,
 }
 
 std::string StockMenuWaresDisplay::info_for_ware(const Widelands::DescriptionIndex di) {
-	const std::string text = WaresDisplay::info_for_ware(di);
+	std::string text = WaresDisplay::info_for_ware(di);
 	if (solid_icon_backgrounds_) {
 		return text;
 	}
@@ -531,11 +531,11 @@ std::string StockMenuWaresDisplay::info_for_ware(const Widelands::DescriptionInd
 
 	// Indicate trend over the last 5 minutes
 	const std::vector<uint32_t>& history = get_type() == Widelands::wwWARE ?
-	                                          *player_.get_ware_stock_statistics(di) :
-	                                          *player_.get_worker_stock_statistics(di);
+                                             *player_.get_ware_stock_statistics(di) :
+                                             *player_.get_worker_stock_statistics(di);
 	const size_t nr_entries = history.size();
 
-	if (!nr_entries) {
+	if (nr_entries == 0u) {
 		// No records yet
 		return text;
 	}
@@ -543,18 +543,18 @@ std::string StockMenuWaresDisplay::info_for_ware(const Widelands::DescriptionInd
 	const size_t kSampleEntriesForTrend = 5 * 60 * 1000 / Widelands::kStatisticsSampleTime.get();
 	const uint32_t last_amount =
 	   history[nr_entries < kSampleEntriesForTrend ? nr_entries - 1 :
-	                                                 nr_entries - kSampleEntriesForTrend];
+                                                    nr_entries - kSampleEntriesForTrend];
 
 	const UI::BuildingStatisticsStyleInfo& colors = g_style_manager->building_statistics_style();
 	const std::string indicator =
 	   current_amount < last_amount ?
-	      StyleManager::color_tag(_("↓"), colors.alternative_low_color()) :
-	      current_amount > last_amount ?
-	      StyleManager::color_tag(_("↑"), colors.alternative_high_color()) :
-	      StyleManager::color_tag(_("="), colors.alternative_medium_color());
+         StyleManager::color_tag(_("↓"), colors.alternative_low_color()) :
+	   current_amount > last_amount ?
+         StyleManager::color_tag(_("↑"), colors.alternative_high_color()) :
+         StyleManager::color_tag(_("="), colors.alternative_medium_color());
 	/** TRANSLATORS: The first placeholder is the stock amount of a ware/worker, and the second is an
 	 * icon indicating a trend. Very little space is available. */
-	return (boost::format(_("%1$s%2$s")) % text % indicator).str();
+	return format(_("%1$s%2$s"), text, indicator);
 }
 
 RGBAColor
@@ -575,10 +575,9 @@ StockMenuWaresDisplay::draw_ware_background_overlay(const Widelands::Description
 	const uint32_t amount = amount_of(di);
 	const uint32_t target = player_.get_total_economy_target(get_type(), di);
 	const RGBColor& color =
-	   amount < target ?
-	      g_style_manager->building_statistics_style().alternative_low_color() :
-	      amount > target ? g_style_manager->building_statistics_style().alternative_high_color() :
-	                        g_style_manager->building_statistics_style().alternative_medium_color();
+	   amount < target ? g_style_manager->building_statistics_style().alternative_low_color() :
+	   amount > target ? g_style_manager->building_statistics_style().alternative_high_color() :
+                        g_style_manager->building_statistics_style().alternative_medium_color();
 	return RGBAColor(color.r, color.g, color.b, 80);
 }
 
@@ -603,11 +602,11 @@ static const char* unit_suffixes[] = {
    _("%1%G")};
 std::string get_amount_string(uint32_t amount, bool cutoff1k) {
 	uint8_t size = 0;
-	while (amount >= (size || cutoff1k ? 1000 : 10000)) {
+	while (amount >= ((size != 0u) || cutoff1k ? 1000 : 10000)) {
 		amount /= 1000;
 		size++;
 	}
-	return (boost::format(unit_suffixes[size]) % amount).str();
+	return format(unit_suffixes[size], amount);
 }
 
 uint32_t WaresDisplay::amount_of(const Widelands::DescriptionIndex ware) {
