@@ -1,6 +1,6 @@
 dirname = "test/maps/market_trading.wmf/" .. path.dirname(__file__)
 
-descriptions:new_productionsite_type {
+wl.Descriptions():new_productionsite_type {
    name = "barbarians_custom_building",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext("custom_building", "Custom Building"),
@@ -38,7 +38,7 @@ descriptions:new_productionsite_type {
    programs = {
       work = {
          -- TRANSLATORS: Completed/Skipped/Did not start felling trees because ...
-         descname = _"felling trees",
+         descname = _("felling trees"),
          actions = {
             "callworker=harvest",
             "sleep=duration:20s"
